Package: cffr
Title: Generate Citation File Format ('CFF') Metadata
Version: 1.4.2
Authors@R: c(
    person("Diego", "Hernangómez", , "diego.hernangomezherrero@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8457-4658")),
    person("João", "Martins", role = "rev",
           comment = c(ORCID = "0000-0001-7961-4280")),
    person("Scott", "Chamberlain", role = "rev",
           comment = c(ORCID = "0000-0003-1444-9135"))
  )
Description: Citation File Format ('CFF') version 1.2.0
    <doi:10.5281/zenodo.5171937> is a human- and machine-readable file
    format for software citation metadata. Core utilities generate, read,
    write and validate Citation File Format metadata for 'R' packages.
License: GPL (>= 3)
URL: https://docs.ropensci.org/cffr/, https://github.com/ropensci/cffr
BugReports: https://github.com/ropensci/cffr/issues
Depends: R (>= 4.1.0)
Imports: cli (>= 2.0.0), desc (>= 1.3.0), jsonlite (>= 1.7.2),
        jsonvalidate (>= 1.1.0), tools, utils, yaml (>= 2.2.1)
Suggests: bibtex (>= 0.5.0), knitr, lifecycle, quarto, rmarkdown,
        testthat (>= 3.3.0), usethis, withr
VignetteBuilder: quarto
Config/Needs/website: devtools
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.1.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
LazyData: true
X-schema.org-isPartOf: https://ropensci.org
X-schema.org-keywords: attribution, citation, credit, citation-files,
        cff, metadata, citation-file-format, cran, r, r-package,
        ropensci, rstats, r-cran
NeedsCompilation: no
Packaged: 2026-08-24 11:32:20 UTC; diego
Author: Diego Hernangómez [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-8457-4658>),
  João Martins [rev] (ORCID: <https://orcid.org/0000-0001-7961-4280>),
  Scott Chamberlain [rev] (ORCID:
    <https://orcid.org/0000-0003-1444-9135>)
Maintainer: Diego Hernangómez <diego.hernangomezherrero@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-24 12:10:02 UTC
Built: R 4.6.1; ; 2026-08-24 13:01:41 UTC; unix

Package: basicdescdate
Type: Package
Title: A Basic Description with Date
Version: 0.1.6
Authors@R:
    person(given = "Marc",
             family = "Basic",
             role = c("aut", "cre", "cph"),
             email = "marcbasic@gmail.com")
Description: A very basic description. Should parse without problems. I have a Date
License: GPL-3
URL: https://github.com/basic/package, https://basic.github.io/package
BugReports: https://github.com/basic/package/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1.9000
Date: 1999-01-01

Type: Package
Package: manypersons
Title: A lot of persons
Version: 0.1.6
Authors@R:
    c(person(given = "Diego",
             family = "Hernangómez",
             role = c("aut", "cre"),
             email = "fake@gmail.com",
             comment = c(ORCID = "0000-0001-8457-4658", email = "error",
             "not-valid" = "error")),
      person(given = "Joe",
             family = "Doe",
             role = "aut",
             email = "I am a wrong email",
             comment = c(affiliation = "This One", country = "ES",
             "date-end" = "error")),
      person(given = "Pepe",
             family = "Doe",
             role = "aut",
             email = "fake@gmail.com",
             comment = "error"),
      person(given = "I am an entity",
             role = c("cre"),
             comment = c("date-end" = "2020-01-01", affiliation = "error")),
      person(given = "ERROR entity",
             role = "cph",
             comment = "for the administrative boundaries."),
      person(given = "ERROR",
             family = "person",
             role = "cph",
             comment = c(ORCID = "0000-0003-2042-7063", "for the gisco_countrycode dataset.")))
Description: Overkill desc with many persons. Try this
License: GPL-3
URL: https://github.com/many/persons,
    https://many.github.io/persons
BugReports: https://github.com/many/persons/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1.9000
X-schema.org-keywords: metadata, cffr, ropensci, citation,
    credit, linked-data, one, one, two

Type: Package
Package: manyurls
Title: A lot of urls
Version: 0.1.6
BugReports: https://github.com/test/package/issues
URL: https://github.com/test/package, https://test.github.io/package/,
     https://r-forge.r-project.org/projects/test/,
     http://google.ru, https://gitlab.com/r-packages/behaviorchange,
     this.is.not.an.url
Encoding: UTF-8
Authors@R:
    person(given = "Marc",
             family = "Basic",
             role = c("aut", "cre", "cph"),
             email = "marcbasic@gmail.com")
Description: This package has many urls. Specifically, 1 Bug Reports and
           6 URLs. Expected is to have 1 repository-code, 1 url and 3 URLs,
           since there is 1 duplicate and 1 invalid url.
License: GPL-3

Package: rgeos
Title: Interface to Geometry Engine - Open Source ('GEOS')
Version: 0.5-7
Date: 2020-09-07
Depends: R (>= 3.3.0), methods, sp (>= 1.1-0)
Imports: utils, stats, graphics
LinkingTo: sp
Suggests: maptools (>= 0.8-5), testthat, XML, maps, rgdal
NeedsCompilation: yes
LazyLoad: yes
Description: Interface to Geometry Engine - Open Source ('GEOS') using the C 'API' for topology operations on geometries. Please note that 'rgeos' will be retired by the end of 2023, plan transition to sf functions using 'GEOS' at your earliest convenience. The 'GEOS' library is external to the package, and, when installing the package from source, must be correctly installed first. Windows and Mac Intel OS X binaries are provided on 'CRAN'. ('rgeos' >= 0.5-1): Up to and including 'GEOS' 3.7.1, topological operations succeeded with some invalid geometries for which the same operations fail from and including 'GEOS' 3.7.2. The 'checkValidity=' argument defaults and structure have been changed, from default FALSE to integer default '0L' for 'GEOS' < 3.7.2 (no check), '1L' 'GEOS' >= 3.7.2 (check and warn). A value of '2L' is also provided that may be used, assigned globally using 'set_RGEOS_CheckValidity(2L)', or locally using the 'checkValidity=2L' argument, to attempt zero-width buffer repair if invalid geometries are found. The previous default (FALSE, now '0L') is fastest and used for 'GEOS' < 3.7.2, but will not warn users of possible problems before the failure of topological operations that previously succeeded. From 'GEOS' 3.8.0, repair of geometries may also be attempted using 'gMakeValid()', which may, however, return a collection of geometries of different types.
Authors@R: c(
	person("Roger", "Bivand", role = c("cre", "aut"), email = "Roger.Bivand@nhh.no", comment=c(ORCID="0000-0003-2392-6140")),
	person("Colin", "Rundel", role = "aut"),
	person("Edzer", "Pebesma", role = "ctb"),
	person("Rainer", "Stuetz", role = "ctb"),
	person("Karl Ove", "Hufthammer", role = "ctb"),
        person("Patrick", "Giraudoux", role = "ctb"),
        person("Martin", "Davis", role = c("cph", "ctb")),
        person("Sandro", "Santilli", role = c("cph", "ctb")))
License: GPL (>= 2)
URL: https://r-forge.r-project.org/projects/rgeos/
        https://trac.osgeo.org/geos/
        http://rgeos.r-forge.r-project.org/index.html
SystemRequirements: GEOS (>= 3.2.0); for building from source: GEOS
        from http://trac.osgeo.org/geos/; GEOS OSX frameworks built by
        William Kyngesburye at http://www.kyngchaos.com/ may be used
        for source installs on OSX.
RoxygenNote: 6.1.1
Packaged: 2021-09-07 13:14:38 UTC; rsb
Author: Roger Bivand [cre, aut] (<https://orcid.org/0000-0003-2392-6140>),
  Colin Rundel [aut],
  Edzer Pebesma [ctb],
  Rainer Stuetz [ctb],
  Karl Ove Hufthammer [ctb],
  Patrick Giraudoux [ctb],
  Martin Davis [cph, ctb],
  Sandro Santilli [cph, ctb]
Maintainer: Roger Bivand <Roger.Bivand@nhh.no>
Repository: CRAN
Date/Publication: 2020-09-07 14:00:05 UTC
Built: R 4.1.1; x86_64-w64-mingw32; 2021-09-09 08:49:58 UTC; windows
Archs: i386, x64

Type: Package
Package: codemetar
Title: Generate 'CodeMeta' Metadata for R Packages
Version: 0.3.5
Authors@R:
    c(person(given = "Carl",
             family = "Boettiger",
             role = c("aut", "cre", "cph"),
             email = "cboettig@gmail.com",
             comment = c(ORCID = "0000-0002-1642-628X")),
      person(given = "Anna",
             family = "Krystalli",
             role = c("rev", "ctb"),
             comment = c(ORCID = "0000-0002-2378-4915")),
      person(given = "Toph",
             family = "Allen",
             role = "rev",
             comment = c(ORCID = "0000-0003-4580-091X")),
      person(given = "Maëlle",
             family = "Salmon",
             role = c("ctb", "aut"),
             comment = c(ORCID = "0000-0002-2815-0399")),
      person(given = "rOpenSci",
             role = "fnd",
             comment = "https://ropensci.org/"),
      person(given = "Katrin",
             family = "Leinweber",
             role = "ctb",
             comment = c(ORCID = "0000-0001-5135-5758")),
      person(given = "Noam",
             family = "Ross",
             role = "ctb",
             comment = c(ORCID = "0000-0002-2136-0000")),
      person(given = "Arfon",
             family = "Smith",
             role = "ctb"),
      person(given = "Jeroen",
             family = "Ooms",
             role = "ctb",
             comment = c(ORCID = "0000-0002-4035-0289")),
      person(given = "Sebastian",
             family = "Meyer",
             role = "ctb",
             comment = c(ORCID = "0000-0002-1791-9449")),
      person(given = "Michael",
             family = "Rustler",
             role = "ctb",
             comment = c(ORCID = "0000-0003-0647-7726")),
      person(given = "Hauke",
             family = "Sonnenberg",
             role = "ctb",
             comment = c(ORCID = "0000-0001-9134-2871")),
      person(given = "Sebastian",
             family = "Kreutzer",
             role = "ctb",
             comment = c(ORCID = "0000-0002-0734-2199")),
      person(given = "Thierry",
             family = "Onkelinx",
             role = "ctb",
             comment = c(ORCID = "0000-0001-8804-4216"))
      )
Description: The 'Codemeta' Project defines a 'JSON-LD' format for
        describing software metadata, as detailed at
        <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files
        automatically for R packages, as well as tools and examples for
        working with 'codemeta.json' 'JSON-LD' more generally.
License: GPL-3
URL: https://github.com/ropensci/codemetar,
        https://docs.ropensci.org/codemetar/
BugReports: https://github.com/ropensci/codemetar/issues
Depends: R (>= 3.2.0)
Imports: commonmark, crul, desc, gert, gh, jsonlite (>= 1.6), magrittr,
        memoise, methods, pingr, purrr, remotes, sessioninfo, stats,
        urltools, xml2, cli, codemeta
Suggests: withr, covr, details, dplyr (>= 0.7.0), jsonld, jsonvalidate,
        knitr, printr, rmarkdown, testthat (>= 3.0.0), usethis
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.2.1.9000
X-schema.org-isPartOf: https://ropensci.org
X-schema.org-keywords: metadata, codemeta, ropensci, citation, credit,
        linked-data
Roxygen: list(markdown = TRUE)
Config/testthat/edition: 3
Repository: https://ropensci.r-universe.dev
RemoteUrl: https://github.com/ropensci/codemetar
RemoteRef: main
RemoteSha: bdd9a29d7eabcc43c3195fe461f884932eba763c
NeedsCompilation: no
Packaged: 2024-02-09 05:55:58 UTC; root
Author: Carl Boettiger [aut, cre, cph]
    (<https://orcid.org/0000-0002-1642-628X>),
  Anna Krystalli [rev, ctb] (<https://orcid.org/0000-0002-2378-4915>),
  Toph Allen [rev] (<https://orcid.org/0000-0003-4580-091X>),
  Maëlle Salmon [ctb, aut] (<https://orcid.org/0000-0002-2815-0399>),
  rOpenSci [fnd] (https://ropensci.org/),
  Katrin Leinweber [ctb] (<https://orcid.org/0000-0001-5135-5758>),
  Noam Ross [ctb] (<https://orcid.org/0000-0002-2136-0000>),
  Arfon Smith [ctb],
  Jeroen Ooms [ctb] (<https://orcid.org/0000-0002-4035-0289>),
  Sebastian Meyer [ctb] (<https://orcid.org/0000-0002-1791-9449>),
  Michael Rustler [ctb] (<https://orcid.org/0000-0003-0647-7726>),
  Hauke Sonnenberg [ctb] (<https://orcid.org/0000-0001-9134-2871>),
  Sebastian Kreutzer [ctb] (<https://orcid.org/0000-0002-0734-2199>),
  Thierry Onkelinx [ctb] (<https://orcid.org/0000-0001-8804-4216>)
Maintainer: Carl Boettiger <cboettig@gmail.com>
Built: R 4.3.2; ; 2024-02-09 05:58:05 UTC; windows

Package: resmush
Title: Optimize and Compress Image Files with 'reSmush.it'
Version: 0.1.0
Authors@R: 
    person("Diego", "Hernangómez", , "diego.hernangomezherrero@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8457-4658"))
Description: Compress local and online images using the 'reSmush.it' API
    service <https://resmush.it/>.
License: MIT + file LICENSE
URL: https://dieghernan.github.io/resmush/,
        https://github.com/dieghernan/resmush
BugReports: https://github.com/dieghernan/resmush/issues
Depends: R (>= 3.6.0)
Imports: cli, curl, httr2 (>= 1.0.0), tools, utils
Suggests: grid, knitr, png, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/Needs/website: dieghernan/gitdevr, xfun, dplyr, tibble, devtools
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
RoxygenNote: 7.3.1
X-schema.org-keywords: r, compress-images, optimize-images, resmushit,
        api
NeedsCompilation: no
Packaged: 2024-02-01 12:16:47 UTC; diego
Author: Diego Hernangómez [aut, cre, cph]
    (<https://orcid.org/0000-0001-8457-4658>)
Maintainer: Diego Hernangómez <diego.hernangomezherrero@gmail.com>
Repository: RSPM
Date/Publication: 2024-02-02 19:50:02 UTC
Built: R 4.3.0; ; 2024-02-05 12:08:28 UTC; windows

Package: basicdesc
Type: Package
Title: A Basic Description
Version: 0.1.6
Author: Marc Basic
Maintainer: Marc Basic <marcbasic@gmail.com>
Description: A very basic description. Should parse without problems.
License: GPL-3
URL: https://github.com/basic/package, https://basic.github.io/package
BugReports: https://github.com/basic/package/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1.9000

Package: codemetar
Type: Package
Title: Generate 'CodeMeta' Metadata for R Packages
Version: 0.1.6
Authors@R:
    c(person(given = "Carl",
             family = "Boettiger",
             role = c("aut", "cre", "cph"),
             email = "cboettig@gmail.com",
             comment = c(ORCID = "0000-0002-1642-628X")),
      person(given = "Anna",
             family = "Krystalli",
             role = c("rev", "ctb"),
             comment = c(ORCID = "0000-0002-2378-4915")),
      person(given = "Toph",
             family = "Allen",
             role = "rev",
             comment = c(ORCID = "0000-0003-4580-091X")),
      person(given = "Maëlle",
             family = "Salmon",
             role = c("ctb", "aut"),
             comment = c(ORCID = "0000-0002-2815-0399")),
      person(given = "rOpenSci",
             role = "fnd",
             comment = "https://ropensci.org/"),
      person(given = "Katrin",
             family = "Leinweber",
             role = "ctb"),
      person(given = "Noam",
             family = "Ross",
             role = "ctb"),
      person(given = "Arfon",
             family = "Smith",
             role = "ctb"))
Description: The 'Codemeta' Project defines a 'JSON-LD' format for describing
  software metadata, as detailed at <https://codemeta.github.io>. This package
  provides utilities to generate, parse, and modify 'codemeta.json' files
  automatically for R packages, as well as tools and examples for working with
  'codemeta.json' 'JSON-LD' more generally.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1.9000
Depends: R (>= 3.0.0)
Imports:
    jsonlite (>= 1.3),
    jsonld,
    git2r,
    devtools,
    memoise,
    methods,
    stats,
    stringi,
    readr,
    desc,
    usethis,
    whisker,
    tibble,
    crul,
    gh,
    stringr,
    sessioninfo,
    purrr
Suggests:
    testthat,
    jsonvalidate,
    covr,
    knitr,
    rmarkdown,
    magrittr,
    xml2,
    dplyr (>= 0.7.0),
    printr
VignetteBuilder: knitr
X-schema.org-keywords: metadata, codemeta, ropensci, citation, credit, linked-data
X-schema.org-isPartOf: https://ropensci.org

Package: basicdesc
Type: Package
Title: A Basic Description
Version: 0.1.6
Authors@R:
    person(given = "Marc",
             family = "Basic",
             role = c("aut", "cre", "cph"),
             email = "marcbasic@gmail.com")
Description: A very basic description. Should parse without problems.
License: GPL-3
URL: https://github.com/basic/package, https://basic.github.io/package
BugReports: https://github.com/basic/package/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1.9000

Package: surveillance
Title: Temporal and Spatio-Temporal Modeling and Monitoring of Epidemic
        Phenomena
Version: 1.19.1
Date: 2021-03-30
Authors@R: c(MH = person("Michael", "Höhle",
                  email = "hoehle@math.su.se",
                  role = c("aut", "ths"),
                  comment = c(ORCID = "0000-0002-0423-6702")),
             SM = person("Sebastian", "Meyer",
                  email = "seb.meyer@fau.de",
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0002-1791-9449")),
             MP = person("Michaela", "Paul", role = "aut"),
             LH = person("Leonhard", "Held",
                  email = "Leonhard.Held@uzh.ch",
                  role = c("ctb", "ths")),
             person("Howard", "Burkom", role = "ctb"),
             person("Thais", "Correa", role = "ctb"),
             person("Mathias", "Hofmann", role = "ctb"),
             person("Christian", "Lang", role = "ctb"),
             person("Juliane", "Manitz", role = "ctb"),
             person("Andrea", "Riebler", role = "ctb"),
             person("Daniel", "Sabanés Bové", role = "ctb"),
             MS = person("Maëlle", "Salmon", role = "ctb"),
             DS = person("Dirk", "Schumacher", role = "ctb"),
             person("Stefan", "Steiner", role = "ctb"),
             person("Mikko", "Virtanen", role = "ctb"),
             person("Wei", "Wei", role = "ctb"),
             person("Valentin", "Wimmer", role = "ctb"),
             person("R Core Team", role = "ctb",
                    comment = "A few code segments are modified versions of
                               code from base R"))
Author: Michael Höhle [aut, ths] (<https://orcid.org/0000-0002-0423-6702>),
  Sebastian Meyer [aut, cre] (<https://orcid.org/0000-0002-1791-9449>),
  Michaela Paul [aut],
  Leonhard Held [ctb, ths],
  Howard Burkom [ctb],
  Thais Correa [ctb],
  Mathias Hofmann [ctb],
  Christian Lang [ctb],
  Juliane Manitz [ctb],
  Andrea Riebler [ctb],
  Daniel Sabanés Bové [ctb],
  Maëlle Salmon [ctb],
  Dirk Schumacher [ctb],
  Stefan Steiner [ctb],
  Mikko Virtanen [ctb],
  Wei Wei [ctb],
  Valentin Wimmer [ctb],
  R Core Team [ctb] (A few code segments are modified versions of code
    from base R)
Maintainer: Sebastian Meyer <seb.meyer@fau.de>
Depends: R (>= 3.6.0), methods, grDevices, graphics, stats, utils, sp
        (>= 1.0-15), xtable (>= 1.7-0)
Imports: Rcpp (>= 0.11.1), polyCub (>= 0.8.0), MASS, Matrix, nlme,
        spatstat.geom, spatstat (>= 2.0)
LinkingTo: Rcpp, polyCub
Suggests: parallel, grid, gridExtra (>= 2.0.0), lattice, colorspace,
        scales, animation, msm, spc, quadprog, memoise, polyclip,
        rgeos, gpclib, maptools, intervals, spdep, numDeriv, maxLik,
        gsl, fanplot, hhh4contacts, tinytest (>= 1.2.4), coda, splancs,
        gamlss, INLA, runjags, MGLM (>= 0.1.0), knitr
Enhances: xts, ggplot2
Description: Statistical methods for the modeling and monitoring of time series
        of counts, proportions and categorical data, as well as for the modeling
        of continuous-time point processes of epidemic phenomena.
        The monitoring methods focus on aberration detection in count data time
        series from public health surveillance of communicable diseases, but
        applications could just as well originate from environmetrics,
        reliability engineering, econometrics, or social sciences. The package
        implements many typical outbreak detection procedures such as the
        (improved) Farrington algorithm, or the negative binomial GLR-CUSUM
        method of Höhle and Paul (2008) <doi:10.1016/j.csda.2008.02.015>.
        A novel CUSUM approach combining logistic and multinomial logistic
        modeling is also included. The package contains several real-world data
        sets, the ability to simulate outbreak data, and to visualize the
        results of the monitoring in a temporal, spatial or spatio-temporal
        fashion. A recent overview of the available monitoring procedures is
        given by Salmon et al. (2016) <doi:10.18637/jss.v070.i10>.
        For the retrospective analysis of epidemic spread, the package provides
        three endemic-epidemic modeling frameworks with tools for visualization,
        likelihood inference, and simulation. hhh4() estimates models for
        (multivariate) count time series following Paul and Held (2011)
        <doi:10.1002/sim.4177> and Meyer and Held (2014) <doi:10.1214/14-AOAS743>.
        twinSIR() models the susceptible-infectious-recovered (SIR) event
        history of a fixed population, e.g, epidemics across farms or networks,
        as a multivariate point process as proposed by Höhle (2009)
        <doi:10.1002/bimj.200900050>. twinstim() estimates self-exciting point
        process models for a spatio-temporal point pattern of infective events,
        e.g., time-stamped geo-referenced surveillance data, as proposed by
        Meyer et al. (2012) <doi:10.1111/j.1541-0420.2011.01684.x>.
        A recent overview of the implemented space-time modeling frameworks
        for epidemic phenomena is given by Meyer et al. (2017)
        <doi:10.18637/jss.v077.i11>.
License: GPL-2
URL: https://surveillance.R-Forge.R-project.org/
Additional_repositories: https://inla.r-inla-download.org/R/stable/
Encoding: latin1
VignetteBuilder: utils, knitr
NeedsCompilation: yes
Packaged: 2021-03-30 12:11:05 UTC; smeyer
Repository: CRAN
Date/Publication: 2021-03-31 05:10:03 UTC
Built: R 4.1.1; x86_64-w64-mingw32; 2021-09-03 23:33:41 UTC; windows
Archs: i386, x64

Package: wrongurls
Type: Package
Title: Generate CodeMeta Metadata for R Packages
Version: 0.1.0
Author: Carl Boettiger [aut, cre, cph] <cboettig@gmail.com> (https://orcid.org/0000-0002-1642-628X)
Maintainer: Carl Boettiger <cboettig@gmail.com> (https://orcid.org/0000-0002-1642-628X)
Description: Codemeta defines a 'JSON-LD' format for describing software metadata.
    This package provides utilities to generate, parse, and modify codemeta.jsonld
    files automatically for R packages.
License: MIT + file LICENSE
URL: thisisnotandurl,
     https://httpbin.org/status/429,
     https://www.github.es/ropensci/codemeta,
BugReports: https://httpbin.org/status/404
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1
Depends: R (>= 3.0.0)
Imports: jsonlite (>= 1.3),
    jsonld,
    git2r,
    devtools,
    methods
Suggests: testthat,
    jsonvalidate,
    covr,
    knitr,
    rmarkdown,
    httr,
    magrittr,
    readr,
    xml2
VignetteBuilder: knitr
X-schema.org-keywords: metadata, codemeta, ropensci, citation, credit, linked-data
X-schema.org-isPartOf: https://ropensci.org

Type: Package
Package: codemetar
Title: Generate 'CodeMeta' Metadata for R Packages
Version: 0.1.6
Author: Jane Doo
Maintainer: John Ok <email@email.edu>, Jane Doe <email2@email.edu>
Description: The 'Codemeta' Project defines a 'JSON-LD' format for
    describing software metadata, as detailed at
    <https://codemeta.github.io>. This package provides utilities to
    generate, parse, and modify 'codemeta.json' files automatically for R
    packages, as well as tools and examples for working with
    'codemeta.json' 'JSON-LD' more generally.
License: GPL-3
URL: https://github.com/ropensci/codemetar,
    https://ropensci.github.io/codemetar
BugReports: https://github.com/ropensci/codemetar/issues
Depends:
    R (>= 3.0.0)
Imports:
    crul,
    desc,
    devtools,
    gh,
    git2r,
    jsonld,
    jsonlite (>= 1.3),
    memoise,
    methods,
    purrr,
    readr,
    sessioninfo,
    stats,
    stringi,
    stringr,
    tibble,
    usethis,
    whisker
Suggests:
    covr,
    dplyr (>= 0.7.0),
    jsonvalidate,
    knitr,
    magrittr,
    printr,
    rmarkdown,
    testthat,
    xml2
VignetteBuilder:
    knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1.9000

Package: MomTrunc
Type: Package
Title: Moments of Folded and Doubly Truncated Multivariate
        Distributions
Version: 6.0
Date: 2022-06-15
Authors@R: 
  c(person(given = "Christian E.",
           family = "Galarza",
           role = c("aut", "cre","trl"),
           email = c("cgalarza88@gmail.com","chedgala@espol.edu.ec"),
           comment = c(ORCID = "0000-0002-4818-6006")),
    person(given = "Raymond",
           family = "Kan",
           role = "ctb",
           comment = c(ORCID = "0000-0002-0578-9974")),
    person(given = "Victor H.",
           family = "Lachos",
           role = c("aut","ths"),
           email = c("hlachos@gmail.com","hlachos@uconn.edu"),
           comment = c(ORCID = "0000-0002-7239-2459")))
Author: Christian E. Galarza [aut, cre, trl]
    (<https://orcid.org/0000-0002-4818-6006>),
  Raymond Kan [ctb] (<https://orcid.org/0000-0002-0578-9974>),
  Victor H. Lachos [aut, ths] (<https://orcid.org/0000-0002-7239-2459>)
Maintainer: Christian E. Galarza <cgalarza88@gmail.com>
Description: It computes arbitrary products moments (mean vector and variance-covariance matrix), for some double truncated (and folded) multivariate distributions. These distributions belong to the family of selection elliptical distributions, which includes well known skewed distributions as the unified skew-t distribution (SUT) and its particular cases as the extended skew-t (EST), skew-t (ST) and the symmetric student-t (T) distribution. Analogous normal cases unified skew-normal (SUN), extended skew-normal (ESN), skew-normal (SN), and symmetric normal (N) are also included. Density, probabilities and random deviates are also offered for these members.
License: GPL (>= 2)
Depends: R (>= 3.6.0)
Imports: Rcpp (>= 1.0.1), mvtnorm (>= 1.0.11), tlrmvnmvt (>= 1.1.0),
        hypergeo
LinkingTo: Rcpp (>= 1.0.1), RcppArmadillo, mvtnorm
Suggests: tmvtnorm
NeedsCompilation: yes
Packaged: 2022-06-15 18:51:55 UTC; cgala
Repository: CRAN
Date/Publication: 2022-06-15 23:10:05 UTC

Package: idonthavencoding
Type: Package
Title: No Encoding
Version: 0.1.6
Authors@R:
    person(given = "Marc",
             family = "Basic",
             role = c("aut", "cre", "cph"),
             email = "marcbasic@gmail.com",
             comment = c(ORCID = "0000-0000-0000-001X"))
Description: I don't have Encoding field. Should parse without problems. Maybe
        some warning, but because the missing Encoding field.
License: GPL-3
URL: https://github.com/idonthavencoding/package, https://idonthavencoding.github.io/package
BugReports: https://github.com/idonthavencoding/package/issues
LazyData: true
RoxygenNote: 6.0.1.9000

Package: options
Title: Simple, Consistent Package Options
Version: 0.3.0
Authors@R:
    person(
      "Doug",
      "Kelkhoff",
      email = "doug.kelkhoff@gmail.com",
      role = c("aut", "cre")
    )
Description:
    Simple mechanisms for defining and interpreting package options. Provides
    helpers for interpreting environment variables, global options, defining
    default values and more.
License:
    MIT + file LICENSE
URL:
    https://dgkf.github.io/options/,
    https://codeberg.org/dgkf/options
BugReports:
    https://codeberg.org/dgkf/options/issues
Imports:
    utils
Suggests:
    crayon,
    knitr,
    rmarkdown,
    roxygen2,
    rcmdcheck,
    pkgload,
    withr,
    testthat (>= 3.0.0)
Config/Needs/website:
    pkgdown
Roxygen:
    list(markdown = TRUE)
VignetteBuilder:
    knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Config/testthat/edition: 3

Package: quint
Type: Package
Title: Qualitative Interaction Trees
Version: 2.2.2
Date: 2022-07-01
Authors@R: c(person(given = "Elise", family = "Dusseldorp", role =
        c("aut", "cre","cph"), email = "elise.dusseldorp@fsw.leidenuniv.nl"),
        person("Lisa","Doove",role=c("aut"),email="lisa.doove@kuleuven.be"),
	person("Jeanne", "van de  Put", role = "aut", email="jmms1991@gmail.com"),
	person("Cor","Ninaber",comment="supported with the plot function",role="ctb"),
        person(given = "Iven", family = "Van Mechelen", role = c("aut","cph"), email
        = "iven.vanmechelen@kuleuven.be"),
        person(given = "Juan", family = "Claramunt Gonzalez", role = c("aut"), email
        = "j.claramunt.gonzalez@fsw.leidenuniv.nl"))
Maintainer: Elise Dusseldorp <elise.dusseldorp@fsw.leidenuniv.nl>
Description: Grows a qualitative interaction tree. Quint is a tool for subgroup analysis, suitable for data from a two-arm randomized controlled trial. More information in Dusseldorp, E., Doove, L., & Van Mechelen, I. (2016) <doi:10.3758/s13428-015-0594-z>.
Depends: R (>= 3.0.2)
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
License: GPL (>= 3)
LazyData: TRUE
RoxygenNote: 7.1.1
Author: Elise Dusseldorp [aut, cre, cph],
  Lisa Doove [aut],
  Jeanne van de Put [aut],
  Cor Ninaber [ctb] (supported with the plot function),
  Iven Van Mechelen [aut, cph],
  Juan Claramunt Gonzalez [aut]
NeedsCompilation: no
Packaged: 2022-07-01 11:24:25 UTC; jclaramunt
Repository: CRAN
Date/Publication: 2022-07-01 21:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 18:25:19 UTC; windows
RemoteType: standard
RemotePkgRef: quint
RemoteRef: quint
RemoteRepos: http://cran.rstudio.com
RemoteSha: 2.2.2

Package: GenomicRanges
Title: Representation and manipulation of genomic intervals
Description: The ability to efficiently represent and manipulate genomic
	annotations and alignments is playing a central role when it comes
	to analyzing high-throughput sequencing data (a.k.a. NGS data).
	The GenomicRanges package defines general purpose containers for
	storing and manipulating genomic intervals and variables defined along
	a genome. More specialized containers for representing and manipulating
	short alignments against a reference genome, or a matrix-like
	summarization of an experiment, are defined in the GenomicAlignments
	and SummarizedExperiment packages, respectively. Both packages build
	on top of the GenomicRanges infrastructure.
biocViews: Genetics, Infrastructure, DataRepresentation, Sequencing,
        Annotation, GenomeAnnotation, Coverage
URL: https://bioconductor.org/packages/GenomicRanges
BugReports: https://github.com/Bioconductor/GenomicRanges/issues
Version: 1.51.4
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Patrick", "Aboyoun", role="aut"),
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("Michael", "Lawrence", role="aut"))
Depends: R (>= 4.0.0), methods, stats4, BiocGenerics (>= 0.37.0),
        S4Vectors (>= 0.27.12), IRanges (>= 2.31.2), GenomeInfoDb (>=
        1.15.2)
Imports: utils, stats, XVector (>= 0.29.2)
LinkingTo: S4Vectors, IRanges
Suggests: Matrix, Biobase, AnnotationDbi, annotate, Biostrings (>=
        2.25.3), SummarizedExperiment (>= 0.1.5), Rsamtools (>=
        1.13.53), GenomicAlignments, rtracklayer, BSgenome,
        GenomicFeatures, Gviz, VariantAnnotation, AnnotationHub,
        DESeq2, DEXSeq, edgeR, KEGGgraph, RNAseqData.HNRNPC.bam.chr14,
        pasillaBamSubset, KEGGREST, hgu95av2.db, hgu95av2probe,
        BSgenome.Scerevisiae.UCSC.sacCer2, BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Mmusculus.UCSC.mm10,
        TxDb.Athaliana.BioMart.plantsmart22,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene, RUnit, digest, knitr,
        rmarkdown, BiocStyle
VignetteBuilder: knitr
Collate: normarg-utils.R utils.R phicoef.R transcript-utils.R
        constraint.R strand-utils.R genomic-range-squeezers.R
        GenomicRanges-class.R GenomicRanges-comparison.R
        GRanges-class.R GPos-class.R GRangesFactor-class.R
        DelegatingGenomicRanges-class.R GNCList-class.R
        GenomicRangesList-class.R GRangesList-class.R
        makeGRangesFromDataFrame.R makeGRangesListFromDataFrame.R
        RangedData-methods.R findOverlaps-methods.R
        intra-range-methods.R inter-range-methods.R coverage-methods.R
        setops-methods.R subtract-methods.R nearest-methods.R
        absoluteRanges.R tileGenome.R tile-methods.R genomicvars.R
        zzz.R
git_url: https://git.bioconductor.org/packages/GenomicRanges
git_branch: master
git_last_commit: f1a44dc
git_last_commit_date: 2022-12-15
Date/Publication: 2022-12-15
NeedsCompilation: yes
Packaged: 2022-12-15 21:38:20 UTC; biocbuild
Author: Patrick Aboyoun [aut],
  Hervé Pagès [aut, cre],
  Michael Lawrence [aut]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
Built: R 4.3.0; x86_64-pc-linux-gnu; 2023-04-26 07:23:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",cff_pers_lst)
S3method("[",cff_ref_lst)
S3method(as.data.frame,cff)
S3method(as.data.frame,cff_pers)
S3method(as.data.frame,cff_pers_lst)
S3method(as.data.frame,cff_ref)
S3method(as.data.frame,cff_ref_lst)
S3method(as.list,cff)
S3method(as.person,cff)
S3method(as.person,cff_pers)
S3method(as.person,cff_pers_lst)
S3method(as.person,cff_ref)
S3method(as.person,cff_ref_lst)
S3method(as_bibentry,"NULL")
S3method(as_bibentry,cff)
S3method(as_bibentry,cff_ref)
S3method(as_bibentry,cff_ref_lst)
S3method(as_bibentry,character)
S3method(as_bibentry,default)
S3method(as_bibentry,list)
S3method(as_cff,Bibtex)
S3method(as_cff,bibentry)
S3method(as_cff,default)
S3method(as_cff,list)
S3method(as_cff,person)
S3method(as_cff_person,character)
S3method(as_cff_person,default)
S3method(as_cff_person,person)
S3method(c,cff)
S3method(head,cff)
S3method(print,cff)
S3method(tail,cff)
S3method(toBibtex,cff)
S3method(toBibtex,cff_pers)
S3method(toBibtex,cff_pers_lst)
S3method(toBibtex,cff_ref)
S3method(toBibtex,cff_ref_lst)
export(as.cff)
export(as_bibentry)
export(as_cff)
export(as_cff_person)
export(cff)
export(cff_create)
export(cff_extract_to_bibtex)
export(cff_from_bibtex)
export(cff_gha_update)
export(cff_git_hook_install)
export(cff_git_hook_remove)
export(cff_modify)
export(cff_parse_citation)
export(cff_parse_person)
export(cff_parse_person_bibtex)
export(cff_read)
export(cff_read_bib)
export(cff_read_bib_text)
export(cff_read_cff_citation)
export(cff_read_citation)
export(cff_read_description)
export(cff_schema_definitions_entity)
export(cff_schema_definitions_person)
export(cff_schema_definitions_refs)
export(cff_schema_keys)
export(cff_schema_keys_license)
export(cff_to_bibtex)
export(cff_validate)
export(cff_write)
export(cff_write_bib)
export(cff_write_citation)
export(encoded_utf_to_latex)
export(write_bib)
export(write_citation)
importFrom(tools,encoded_text_to_latex)
importFrom(utils,
  as.person,
  available.packages,
  bibentry,
  capture.output,
  citation,
  download.file,
  head,
  installed.packages,
  modifyList,
  packageDescription,
  person,
  read.csv,
  tail,
  toBibtex
)

<!-- README.md is generated from README.qmd. Please edit that file -->

# cffr <a href="https://docs.ropensci.org/cffr/"><img src="man/figures/logo.png" alt="cffr website" align="right" height="139"/></a>

<!-- badges: start -->

[![CRAN-status](https://www.r-pkg.org/badges/version/cffr)](https://CRAN.R-project.org/package=cffr)
[![CRAN-results](https://badges.cranchecks.info/worst/cffr.svg)](https://cran.r-project.org/web/checks/check_results_cffr.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/cffr?color=blue)](https://cran.r-project.org/package=cffr)
[![R-CMD-check](https://github.com/ropensci/cffr/actions/workflows/check-full.yaml/badge.svg)](https://github.com/ropensci/cffr/actions/workflows/check-full.yaml)
[![codecov](https://codecov.io/gh/ropensci/cffr/branch/main/graph/badge.svg?token=YRO3XL8RWK)](https://app.codecov.io/gh/ropensci/cffr)
[![Coverage
Status](https://coveralls.io/repos/github/ropensci/cffr/badge.svg?branch=main)](https://coveralls.io/github/ropensci/cffr?branch=main)
[![r-universe](https://ropensci.r-universe.dev/badges/cffr)](https://ropensci.r-universe.dev/cffr)
[![CITATION-cff](https://github.com/ropensci/cffr/actions/workflows/cff-validator.yaml/badge.svg)](https://github.com/ropensci/cffr/actions/workflows/cff-validator.yaml)
[![DOI](https://joss.theoj.org/papers/10.21105/joss.03900/status.svg)](https://doi.org/10.21105/joss.03900)
[![Project Status: Active - The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
![GitHub code size in
bytes](https://img.shields.io/github/languages/code-size/ropensci/cffr)
[![peer-review](https://badges.ropensci.org/463_status.svg)](https://github.com/ropensci/software-review/issues/463)

<!-- badges: end -->

**cffr** provides utilities to generate, read, write, modify and
validate Citation File Format metadata for **R** packages. It also
includes tools and examples for working with `*.cff` files more
generally.

## What is a `CITATION.cff` file?

A `CITATION.cff` file follows the [Citation File Format
(CFF)](https://citation-file-format.github.io/) ([Druskat et al.
2021](#ref-druskat_citation_2021)) (v1.2.0). It contains human- and
machine-readable citation metadata for software and datasets. Developers
can include it in a repository to explain how to cite their software.

This format is increasingly used in the software citation ecosystem.
[**GitHub**](https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/customizing-your-repository/about-citation-files),
[**Zenodo**](https://citation-file-format.github.io/#/supported-by-zenodo-)
and
[**Zotero**](https://citation-file-format.github.io/#/supported-by-zotero-)
support this citation format ([Druskat
2021](#ref-druskat_stephan_making_2021)). **GitHub** support is of
special interest:

<figure>

<img src="vignettes/tweet-1.png" class="mx-auto d-block"
data-fig-align="center" width="400"
alt="GitHub citation support announcement" />

<figcaption class="blockquote-footer">Nat Friedman (@natfriedman) July 27, 2021</figcaption>
</figure>

See [Enhanced support for citations on
**GitHub**](https://github.blog/news-insights/company-news/enhanced-support-citations-github/)
([Smith 2021](#ref-smith2021)) for more information.

### Related projects

[The CodeMeta Project](https://codemeta.github.io/) ([Jones et al.
2017](#ref-jones2017)) creates a concept vocabulary that can be used to
standardize the exchange of software metadata across repositories and
organizations. One of the many uses of a `codemeta.json` file (created
following the standards defined by The CodeMeta Project) is to provide
citation metadata such as title, authors, publication year and venue
([Fenner 2021](#ref-fenner2021)). The packages **codemeta** ([Boettiger
and Salmon 2021](#ref-codemeta)) and **codemetar** ([Boettiger and
Salmon 2026](#ref-codemetar2021)) can generate `codemeta.json` files
from **R** package metadata.

## The **cffr** package

**cffr** uses metadata from both your package’s `DESCRIPTION` file and
its `CITATION` file, if present. **cffr** works best if your package
passes `R CMD check` or `devtools::check()`.

As of 2026-08-24 there are at least 562 repositories on GitHub using
**cffr**. [Browse the search
results](https://github.com/search?q=cffr%20path%3A**%2FCITATION.cff&type=code).

### Installation

Install **cffr** from [CRAN](https://CRAN.R-project.org/package=cffr):

``` r
install.packages("cffr")
```

You can install the development version of **cffr** with:

``` r
pak::pak("ropensci/cffr")
```

Alternatively, you can install **cffr** from
[**r-universe**](https://ropensci.r-universe.dev/cffr):

``` r
# Install **cffr** in R.
install.packages(
  "cffr",
  repos = c("https://ropensci.r-universe.dev", "https://cloud.r-project.org")
)
```

### Example

Most commonly, from within your package directory, you run
`cff_write()`. It creates a `cff` object, writes it to a `CITATION.cff`
file and validates it in a single command:

``` r
library(cffr)

# For in-development packages.
cff_write()
#>
#> CITATION.cff generated.
#>
#> ══ Validating CFF ══════════════════════════════════════════════════════════════
#> ✔ CITATION.cff is valid.
```

**cffr** also provides custom print methods and helper functions for
customizing `CITATION.cff` files and integrating them into your
workflows.

This basic example shows how to create a `cff` object (see `?cff` for
more information). Here, we create a `cff` object from the metadata of
the **knitr** package:

``` r
library(cffr)

# Example with an installed package.
test <- cff_create("knitr")
```

<details>

<summary>

<code>CITATION.cff</code> for <strong>knitr</strong>
</summary>

    cff-version: 1.2.0
    message: 'To cite package "knitr" in publications use:'
    type: software
    license: GPL-1.0-only
    title: 'knitr: A General-Purpose Package for Dynamic Report Generation in R'
    version: '1.51'
    identifiers:
    - type: doi
      value: 10.32614/CRAN.package.knitr
    abstract: Provides a general-purpose tool for dynamic report generation in R using
      Literate Programming techniques.
    authors:
    - family-names: Xie
      given-names: Yihui
      email: xie@yihui.name
      orcid: https://orcid.org/0000-0003-0645-5666
    preferred-citation:
      type: manual
      title: 'knitr: A General-Purpose Package for Dynamic Report Generation in R'
      authors:
      - family-names: Xie
        given-names: Yihui
        orcid: https://orcid.org/0000-0003-0645-5666
        email: xie@yihui.name
      year: '2025'
      notes: R package version 1.51
      url: https://yihui.org/knitr/
    repository: https://CRAN.R-project.org/package=knitr
    repository-code: https://github.com/yihui/knitr
    url: https://yihui.org/knitr/
    date-released: '2025-12-20'
    contact:
    - family-names: Xie
      given-names: Yihui
      email: xie@yihui.name
      orcid: https://orcid.org/0000-0003-0645-5666
    keywords:
    - dynamic-documents
    - knitr
    - literate-programming
    - r
    - r-package
    - rmarkdown
    - sweave
    references:
    - type: book
      title: Dynamic Documents with R and knitr
      authors:
      - family-names: Xie
        given-names: Yihui
      publisher:
        name: Chapman and Hall/CRC
        address: Boca Raton, Florida
      year: '2015'
      edition: 2nd
      notes: ISBN 978-1498716963
      url: https://yihui.org/knitr/
    - type: generic
      title: 'knitr: A Comprehensive Tool for Reproducible Research in R'
      authors:
      - family-names: Xie
        given-names: Yihui
      collection-title: Implementing Reproducible Computational Research
      collection-type: collection
      editors:
      - family-names: Stodden
        given-names: Victoria
      - family-names: Leisch
        given-names: Friedrich
      - family-names: Peng
        given-names: Roger D.
      publisher:
        name: Chapman and Hall/CRC
      year: '2014'
      notes: ISBN 978-1466561595
    - type: software
      title: 'R: A Language and Environment for Statistical Computing'
      notes: Depends
      url: https://www.R-project.org/
      authors:
      - name: R Core Team
        website: https://ror.org/02zz1nj61
      institution:
        name: R Foundation for Statistical Computing
        website: https://ror.org/05qewa988
        address: Vienna, Austria
      year: '2026'
      doi: 10.32614/R.manuals
      version: '>= 3.6.0'
    - type: software
      title: evaluate
      abstract: 'evaluate: Parsing and Evaluation Tools that Provide More Details than
        the Default'
      notes: Imports
      url: https://evaluate.r-lib.org/
      repository: https://CRAN.R-project.org/package=evaluate
      authors:
      - family-names: Wickham
        given-names: Hadley
        email: hadley@posit.co
      - family-names: Xie
        given-names: Yihui
        orcid: https://orcid.org/0000-0003-0645-5666
      year: '2026'
      doi: 10.32614/CRAN.package.evaluate
      version: '>= 0.15'
    - type: software
      title: highr
      abstract: 'highr: Syntax Highlighting for R Source Code'
      notes: Imports
      url: https://github.com/yihui/highr
      repository: https://CRAN.R-project.org/package=highr
      authors:
      - family-names: Xie
        given-names: Yihui
        email: xie@yihui.name
        orcid: https://orcid.org/0000-0003-0645-5666
      - family-names: Qiu
        given-names: Yixuan
      year: '2026'
      doi: 10.32614/CRAN.package.highr
      version: '>= 0.11'
    - type: software
      title: methods
      abstract: 'R: A Language and Environment for Statistical Computing'
      notes: Imports
      authors:
      - name: R Core Team
        website: https://ror.org/02zz1nj61
      institution:
        name: R Foundation for Statistical Computing
        website: https://ror.org/05qewa988
        address: Vienna, Austria
      year: '2026'
      doi: 10.32614/R.manuals
    - type: software
      title: tools
      abstract: 'R: A Language and Environment for Statistical Computing'
      notes: Imports
      authors:
      - name: R Core Team
        website: https://ror.org/02zz1nj61
      institution:
        name: R Foundation for Statistical Computing
        website: https://ror.org/05qewa988
        address: Vienna, Austria
      year: '2026'
      doi: 10.32614/R.manuals
    - type: software
      title: xfun
      abstract: 'xfun: Supporting Functions for Packages Maintained by ''Yihui Xie'''
      notes: Imports
      url: https://github.com/yihui/xfun
      repository: https://CRAN.R-project.org/package=xfun
      authors:
      - family-names: Xie
        given-names: Yihui
        email: xie@yihui.name
        orcid: https://orcid.org/0000-0003-0645-5666
      year: '2026'
      doi: 10.32614/CRAN.package.xfun
      version: '>= 0.52'
    - type: software
      title: yaml
      abstract: 'yaml: Methods to Convert R Data to YAML and Back'
      notes: Imports
      url: https://yaml.r-lib.org
      repository: https://CRAN.R-project.org/package=yaml
      authors:
      - family-names: Stephens
        given-names: Jeremy
      - family-names: Simonov
        given-names: Kirill
      year: '2026'
      doi: 10.32614/CRAN.package.yaml
      version: '>= 2.1.19'
    - type: software
      title: bslib
      abstract: 'bslib: Custom ''Bootstrap'' ''Sass'' Themes for ''shiny'' and ''rmarkdown'''
      notes: Suggests
      url: https://rstudio.github.io/bslib/
      repository: https://CRAN.R-project.org/package=bslib
      authors:
      - family-names: Sievert
        given-names: Carson
        email: carson@posit.co
        orcid: https://orcid.org/0000-0002-4958-2844
      - family-names: Cheng
        given-names: Joe
        email: joe@posit.co
      - family-names: Aden-Buie
        given-names: Garrick
        email: garrick@posit.co
        orcid: https://orcid.org/0000-0002-7111-0077
      year: '2026'
      doi: 10.32614/CRAN.package.bslib
    - type: software
      title: DBI
      abstract: 'DBI: R Database Interface'
      notes: Suggests
      url: https://dbi.r-dbi.org
      repository: https://CRAN.R-project.org/package=DBI
      authors:
      - name: R Special Interest Group on Databases (R-SIG-DB)
      - family-names: Wickham
        given-names: Hadley
      - family-names: Müller
        given-names: Kirill
        email: kirill@cynkra.com
        orcid: https://orcid.org/0000-0002-1416-3412
      year: '2026'
      doi: 10.32614/CRAN.package.DBI
      version: '>= 0.4-1'
    - type: software
      title: digest
      abstract: 'digest: Create Compact Hash Digests of R Objects'
      notes: Suggests
      url: https://eddelbuettel.github.io/digest/
      repository: https://CRAN.R-project.org/package=digest
      authors:
      - family-names: Eddelbuettel
        given-names: Dirk
        email: edd@debian.org
        orcid: https://orcid.org/0000-0001-6419-907X
      year: '2026'
      doi: 10.32614/CRAN.package.digest
    - type: software
      title: gifski
      abstract: 'gifski: Highest Quality GIF Encoder'
      notes: Suggests
      url: https://r-rust.r-universe.dev/gifski
      repository: https://CRAN.R-project.org/package=gifski
      authors:
      - family-names: Ooms
        given-names: Jeroen
        email: jeroenooms@gmail.com
        orcid: https://orcid.org/0000-0002-4035-0289
      - name: Kornel Lesiński
      - name: Authors of the dependency Rust crates
      year: '2026'
      doi: 10.32614/CRAN.package.gifski
    - type: software
      title: htmlwidgets
      abstract: 'htmlwidgets: HTML Widgets for R'
      notes: Suggests
      url: https://github.com/ramnathv/htmlwidgets
      repository: https://CRAN.R-project.org/package=htmlwidgets
      authors:
      - family-names: Vaidyanathan
        given-names: Ramnath
      - family-names: Xie
        given-names: Yihui
      - family-names: Allaire
        given-names: JJ
      - family-names: Cheng
        given-names: Joe
        email: joe@posit.co
      - family-names: Sievert
        given-names: Carson
        email: carson@posit.co
        orcid: https://orcid.org/0000-0002-4958-2844
      - family-names: Russell
        given-names: Kenton
      year: '2026'
      doi: 10.32614/CRAN.package.htmlwidgets
      version: '>= 0.7'
    - type: software
      title: jpeg
      abstract: 'jpeg: Read and write JPEG images'
      notes: Suggests
      url: https://www.rforge.net/jpeg/
      repository: https://CRAN.R-project.org/package=jpeg
      authors:
      - family-names: Urbanek
        given-names: Simon
        email: Simon.Urbanek@r-project.org
        orcid: https://orcid.org/0000-0003-2297-1732
      year: '2026'
      doi: 10.32614/CRAN.package.jpeg
    - type: software
      title: magick
      abstract: 'magick: Advanced Graphics and Image-Processing in R'
      notes: Suggests
      url: https://docs.ropensci.org/magick/
      repository: https://CRAN.R-project.org/package=magick
      authors:
      - family-names: Ooms
        given-names: Jeroen
        email: jeroenooms@gmail.com
        orcid: https://orcid.org/0000-0002-4035-0289
      year: '2026'
      doi: 10.32614/CRAN.package.magick
    - type: software
      title: litedown
      abstract: 'litedown: A Lightweight Version of R Markdown'
      notes: Suggests
      url: https://github.com/yihui/litedown
      repository: https://CRAN.R-project.org/package=litedown
      authors:
      - family-names: Xie
        given-names: Yihui
        email: xie@yihui.name
        orcid: https://orcid.org/0000-0003-0645-5666
      year: '2026'
      doi: 10.32614/CRAN.package.litedown
    - type: software
      title: markdown
      abstract: 'markdown: Render Markdown with ''commonmark'''
      notes: Suggests
      url: https://github.com/rstudio/markdown
      repository: https://CRAN.R-project.org/package=markdown
      authors:
      - family-names: Xie
        given-names: Yihui
        email: xie@yihui.name
        orcid: https://orcid.org/0000-0003-0645-5666
      - family-names: Allaire
        given-names: JJ
      - family-names: Horner
        given-names: Jeffrey
      year: '2026'
      doi: 10.32614/CRAN.package.markdown
      version: '>= 1.3'
    - type: software
      title: otel
      abstract: 'otel: OpenTelemetry R API'
      notes: Suggests
      url: https://otel.r-lib.org
      repository: https://CRAN.R-project.org/package=otel
      authors:
      - family-names: Csárdi
        given-names: Gábor
        email: csardi.gabor@gmail.com
      year: '2026'
      doi: 10.32614/CRAN.package.otel
    - type: software
      title: png
      abstract: 'png: Read and write PNG images'
      notes: Suggests
      url: https://www.rforge.net/png/
      repository: https://CRAN.R-project.org/package=png
      authors:
      - family-names: Urbanek
        given-names: Simon
        email: Simon.Urbanek@r-project.org
        orcid: https://orcid.org/0000-0003-2297-1732
      year: '2026'
      doi: 10.32614/CRAN.package.png
    - type: software
      title: ragg
      abstract: 'ragg: Graphic Devices Based on AGG'
      notes: Suggests
      url: https://ragg.r-lib.org
      repository: https://CRAN.R-project.org/package=ragg
      authors:
      - family-names: Pedersen
        given-names: Thomas Lin
        email: thomas.pedersen@posit.co
        orcid: https://orcid.org/0000-0002-5147-4711
      - family-names: Shemanarev
        given-names: Maxim
      year: '2026'
      doi: 10.32614/CRAN.package.ragg
    - type: software
      title: rlang
      abstract: 'rlang: Functions for Base Types and Core R and ''Tidyverse'' Features'
      notes: Suggests
      url: https://rlang.r-lib.org
      repository: https://CRAN.R-project.org/package=rlang
      authors:
      - family-names: Henry
        given-names: Lionel
        email: lionel@posit.co
      - family-names: Wickham
        given-names: Hadley
        email: hadley@posit.co
      year: '2026'
      doi: 10.32614/CRAN.package.rlang
    - type: software
      title: rmarkdown
      abstract: 'rmarkdown: Dynamic Documents for R'
      notes: Suggests
      url: https://pkgs.rstudio.com/rmarkdown/
      repository: https://CRAN.R-project.org/package=rmarkdown
      authors:
      - family-names: Allaire
        given-names: JJ
        email: jj@posit.co
      - family-names: Xie
        given-names: Yihui
        email: xie@yihui.name
        orcid: https://orcid.org/0000-0003-0645-5666
      - family-names: Dervieux
        given-names: Christophe
        email: cderv@posit.co
        orcid: https://orcid.org/0000-0003-4474-2498
      - family-names: McPherson
        given-names: Jonathan
        email: jonathan@posit.co
      - family-names: Luraschi
        given-names: Javier
      - family-names: Ushey
        given-names: Kevin
        email: kevin@posit.co
      - family-names: Atkins
        given-names: Aron
        email: aron@posit.co
      - family-names: Wickham
        given-names: Hadley
        email: hadley@posit.co
      - family-names: Cheng
        given-names: Joe
        email: joe@posit.co
      - family-names: Chang
        given-names: Winston
        email: winston@posit.co
      - family-names: Iannone
        given-names: Richard
        email: rich@posit.co
        orcid: https://orcid.org/0000-0003-3925-190X
      year: '2026'
      doi: 10.32614/CRAN.package.rmarkdown
    - type: software
      title: sass
      abstract: 'sass: Syntactically Awesome Style Sheets (''Sass'')'
      notes: Suggests
      url: https://rstudio.github.io/sass/
      repository: https://CRAN.R-project.org/package=sass
      authors:
      - family-names: Cheng
        given-names: Joe
        email: joe@rstudio.com
      - family-names: Mastny
        given-names: Timothy
        email: tim.mastny@gmail.com
      - family-names: Iannone
        given-names: Richard
        email: rich@rstudio.com
        orcid: https://orcid.org/0000-0003-3925-190X
      - family-names: Schloerke
        given-names: Barret
        email: barret@rstudio.com
        orcid: https://orcid.org/0000-0001-9986-114X
      - family-names: Sievert
        given-names: Carson
        email: carson@rstudio.com
        orcid: https://orcid.org/0000-0002-4958-2844
      year: '2026'
      doi: 10.32614/CRAN.package.sass
    - type: software
      title: showtext
      abstract: 'showtext: Using Fonts More Easily in R Graphs'
      notes: Suggests
      url: https://github.com/yixuan/showtext
      repository: https://CRAN.R-project.org/package=showtext
      authors:
      - family-names: Qiu
        given-names: Yixuan
        email: yixuan.qiu@cos.name
      year: '2026'
      doi: 10.32614/CRAN.package.showtext
    - type: software
      title: styler
      abstract: 'styler: Non-Invasive Pretty Printing of R Code'
      notes: Suggests
      url: https://styler.r-lib.org
      repository: https://CRAN.R-project.org/package=styler
      authors:
      - family-names: Müller
        given-names: Kirill
        email: kirill@cynkra.com
        orcid: https://orcid.org/0000-0002-1416-3412
      - family-names: Walthert
        given-names: Lorenz
        email: lorenz.walthert@icloud.com
      - family-names: Patil
        given-names: Indrajeet
        email: patilindrajeet.science@gmail.com
        orcid: https://orcid.org/0000-0003-1995-6531
      year: '2026'
      doi: 10.32614/CRAN.package.styler
      version: '>= 1.2.0'
    - type: software
      title: tibble
      abstract: 'tibble: Simple Data Frames'
      notes: Suggests
      url: https://tibble.tidyverse.org/
      repository: https://CRAN.R-project.org/package=tibble
      authors:
      - family-names: Müller
        given-names: Kirill
        email: kirill@cynkra.com
        orcid: https://orcid.org/0000-0002-1416-3412
      - family-names: Wickham
        given-names: Hadley
        email: hadley@rstudio.com
      year: '2026'
      doi: 10.32614/CRAN.package.tibble
    - type: software
      title: tinytex
      abstract: 'tinytex: Helper Functions to Install and Maintain TeX Live, and Compile
        LaTeX Documents'
      notes: Suggests
      url: https://github.com/rstudio/tinytex
      repository: https://CRAN.R-project.org/package=tinytex
      authors:
      - family-names: Xie
        given-names: Yihui
        email: xie@yihui.name
        orcid: https://orcid.org/0000-0003-0645-5666
      year: '2026'
      doi: 10.32614/CRAN.package.tinytex
      version: '>= 0.56'
    - type: software
      title: rstudioapi
      abstract: 'rstudioapi: Safely Access the RStudio API'
      notes: Suggests
      url: https://rstudio.github.io/rstudioapi/
      repository: https://CRAN.R-project.org/package=rstudioapi
      authors:
      - family-names: Ushey
        given-names: Kevin
        email: kevin@rstudio.com
      - family-names: Allaire
        given-names: JJ
        email: jj@posit.co
      - family-names: Wickham
        given-names: Hadley
        email: hadley@posit.co
      - family-names: Ritchie
        given-names: Gary
        email: gary@posit.co
      year: '2026'
      doi: 10.32614/CRAN.package.rstudioapi

</details>

We can validate the result using `cff_validate()`:

``` r
cff_validate(test)
#> ══ Validating CFF ══════════════════════════════════════════════════════════════
```

See the [reference
documentation](https://docs.ropensci.org/cffr/reference/index.html) and
`vignette("cffr", package = "cffr")` to learn more about working with
`cff` objects.

### Keep your `CITATION.cff` file up to date

#### **GitHub Actions**

The easiest way to keep your `CITATION.cff` file up to date is to use
**GitHub Actions**. Use `cff_gha_update()` to install a **GitHub
Actions** workflow that updates your `CITATION.cff` file in the
following cases:

- When you publish a new release of the package on **GitHub**.
- Each time you modify your `DESCRIPTION` or `inst/CITATION` files.
- When you run the workflow manually.

``` r
cff_gha_update()

#> ✔ Installing ./.github/workflows/update-citation-cff.yaml.
#> ℹ Adding .github to .Rbuildignore.
```

See the [example workflow
file](https://github.com/ropensci/cffr/blob/main/.github/workflows/update-citation-cff.yaml).

#### Git pre-commit hook

[![](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)

You can also use a [Git pre-commit
hook](https://git-scm.com/book/en/v2/Customizing-Git-Git-Hooks#_committing_workflow_hooks):

> The `pre-commit` hook is run first, before you even type in a commit
> message. It is used to inspect the snapshot that is about to be
> committed, to see if you have forgotten something, to make sure tests
> run or to examine whatever you need to inspect in the code. Exiting
> non-zero from this hook aborts the commit, although you can bypass it
> with `git commit --no-verify`.

Install a pre-commit hook with `cff_git_hook_install()`. This function
requires the **usethis** package.

### Learn more

Check the following articles to learn more about **cffr**:

- [cffr: Create a CITATION.cff file for your R
  package](https://ropensci.org/blog/2021/11/23/cffr/) ([Hernangómez
  2021a](#ref-hernangomez_cffr_2021))
- [How I test cffr on about 2,000 packages using **GitHub Actions** and
  **r-universe**](https://ropensci.org/blog/2021/11/23/how-i-test-cffr/)
  ([Hernangómez 2021b](#ref-hernangomez_test_2021))

## Related packages

- **citation** ([Dietrich and Leoncio 2025](#ref-citation22)) includes
  an `r2cff()` function that creates a `CITATION.cff` file using the
  information in your `DESCRIPTION` file. It also provides minimal
  validity checks.
- **handlr** ([Chamberlain and Wiernik 2025](#ref-handlr)) is a tool for
  converting among citation formats, including `*.cff` files.
- **codemeta** ([Boettiger and Salmon 2021](#ref-codemeta)) /
  **codemetar** ([Boettiger and Salmon 2026](#ref-codemetar2021))
  provide similar tools for creating `codemeta.json` files, another
  format for storing and sharing software metadata.

## Code of conduct

The **cffr** project is released with a [Contributor Code of
Conduct](https://docs.ropensci.org/cffr/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

## Citation

Hernangómez D (2021). “cffr: Generate Citation File Format Metadata for
R Packages.” *Journal of Open Source Software*, 6(67), 3900.
<https://doi.org/10.21105/joss.03900>

A **BibTeX** entry for **LaTeX** users is:

``` bibtex
@article{hernangomez2021,
    title        = {{cffr}: Generate Citation File Format Metadata for {R} Packages},
    author       = {Diego Hernangómez},
    year         = 2021,
    journal      = {Journal of Open Source Software},
    publisher    = {The Open Journal},
    volume       = 6,
    number       = 67,
    pages        = 3900,
    doi          = {10.21105/joss.03900},
    url          = {https://doi.org/10.21105/joss.03900}
}
```

You can also use the [citation provided by
**GitHub**](https://github.com/ropensci/cffr), which is generated from
the information in a `CITATION.cff` file created with **cffr**. See
[About CITATION
files](https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/customizing-your-repository/about-citation-files)
for more information.

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-codemeta" class="csl-entry">

Boettiger, Carl, and Maëlle Salmon. 2021.
*<span class="nocase">codemeta</span>: A Smaller
<span class="nocase">codemetar</span> Package*.
<https://doi.org/10.32614/CRAN.package.codemeta>.

</div>

<div id="ref-codemetar2021" class="csl-entry">

Boettiger, Carl, and Maëlle Salmon. 2026.
*<span class="nocase">codemetar</span>: Generate ’CodeMeta’ Metadata for
R Packages*. <https://doi.org/10.32614/CRAN.package.codemetar>.

</div>

<div id="ref-handlr" class="csl-entry">

Chamberlain, Scott, and Brenton M. Wiernik. 2025.
*<span class="nocase">handlr</span>: Convert Among Citation Formats*.
<https://doi.org/10.32614/CRAN.package.handlr>.

</div>

<div id="ref-citation22" class="csl-entry">

Dietrich, Jan Philipp, and Waldir Leoncio. 2025.
*<span class="nocase">citation</span>: Software Citation Tools*.
<https://doi.org/10.32614/CRAN.package.citation>.

</div>

<div id="ref-druskat_stephan_making_2021" class="csl-entry">

Druskat, Stephan. 2021. *Making Software Citation Easi(er) - The
Citation File Format and Its Integrations*. Version 1. Zenodo.
<https://doi.org/10.5281/zenodo.5529914>.

</div>

<div id="ref-druskat_citation_2021" class="csl-entry">

Druskat, Stephan, Jurriaan H. Spaaks, Neil Chue Hong, et al. 2021.
*Citation File Format*. Version 1.2.0. Zenodo.
<https://doi.org/10.5281/zenodo.5171937>.

</div>

<div id="ref-fenner2021" class="csl-entry">

Fenner, Martin. 2021. *We Need Your Feedback: Aligning the CodeMeta
Vocabulary for Scientific Software with Schema.org*.
<https://doi.org/10.5438/a49j-x692>.

</div>

<div id="ref-hernangomez_cffr_2021" class="csl-entry">

Hernangómez, Diego. 2021a. *<span class="nocase">cffr</span>: Create a
<span class="nocase">CITATION.cff</span> File for Your R Package*.
rOpenSci. <https://doi.org/10.59350/p37gh-f4s56>.

</div>

<div id="ref-hernangomez_test_2021" class="csl-entry">

Hernangómez, Diego. 2021b. *How i Test <span class="nocase">cffr</span>
on about 2,000 Packages Using GitHub Actions and
<span class="nocase">R-universe</span>*. rOpenSci.
<https://doi.org/10.59350/8bpby-4aw08>.

</div>

<div id="ref-jones2017" class="csl-entry">

Jones, Matthew B, Carl Boettiger, Abby Cabunoc Mayes, et al. 2017.
*CodeMeta: An Exchange Schema for Software Metadata*. Version 2.0. KNB
Data Repository. <https://doi.org/10.5063/schema/codemeta-2.0>.

</div>

<div id="ref-smith2021" class="csl-entry">

Smith, Arfon. 2021. *Enhanced Support for Citations on
GitHub*.[https://github.blog/news-insights/company-news/enhanced-support-citations-github/
](
		https://github.blog/news-insights/company-news/enhanced-support-citations-github/
	).

</div>

</div>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = ""
)

# Load the table of tables

p2file <- system.file("extdata/crosswalk_tables.csv", package = "cffr")

table_master <- read.csv(p2file)

## ----bibentry, comment="#>"---------------------------------------------------
entry <- bibentry("book",
  key = "einstein1921",
  title = "Relativity: The Special and the General Theory",
  author = person("A.", "Einstein"),
  year = 1920,
  publisher = "Henry Holt and Company",
  address = "London, United Kingdom",
  isbn = 9781587340925,
)

toBibtex(entry)

## ----echo=FALSE, results='asis'-----------------------------------------------
entry

## ----cffbibread, comment="#>"-------------------------------------------------
string <- "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925}"

# To cff
library(cffr)
cff_format <- cff_read_bib_text(string)

cff_format

# To BibTeX with S3 method
toBibtex(cff_format)

## ----entry_fields1, echo=FALSE------------------------------------------------
df_table <- table_master[table_master$table == "entry_fields", -1]

nms <- c(
  "**field**", "**\\@article**", "**\\@book**", "**\\@booklet**",
  "**\\@inbook**", "**\\@incollection**", "**\\@conference, \\@inproceedings**",
  "**\\@manual**", "**\\@mastersthesis, phdthesis**", "**\\@misc**",
  "**\\@proceedings**", "**\\@techreport**", "**\\@unpublished**"
)


df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
t1 <- df_table[, c(1:7)]
nm1 <- nms[1:7]

knitr::kable(t1,
  col.names = nm1, row.names = NA, align = c("l", rep("c", 6)),
  caption = "**BibTeX**, required fields by entry"
)

## ----entry_fields2, echo=FALSE------------------------------------------------
t2 <- df_table[, c(1, 8:13)]
nm2 <- nms[c(1, 8:13)]
knitr::kable(t2,
  col.names = nm2, row.names = NA, align = c("l", rep("c", 6)),
  caption = "(cont) **BibTeX**, required fields by entry"
)

## ----refkeys, echo=FALSE, message=FALSE, warning=FALSE, results='asis'--------
library(cffr)

# Fill with whites
init <- paste0("[", cff_schema_definitions_refs(), "]{.underline}")

l <- c(init, rep("", 4))


refkeys <- matrix(l, ncol = 5, byrow = TRUE)

knitr::kable(refkeys,
  row.names = NA,
  caption = "Valid keys on [CFF]{underline} `definition-reference` objects"
)

## ----entry_bib2cff, echo=FALSE, results='asis'--------------------------------
df_table <- table_master[table_master$table == "entry_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("**BibTeX** Entry", "[**CFF key: type**]{.underline}", "Notes"),
  row.names = NA,
  caption = "Entry/Type crosswalk: From **BibTeX** to [CFF]{.underline}"
)

## ----entry_cff2bib, echo=FALSE,results='asis'---------------------------------
df_table <- table_master[table_master$table == "entry_cff2bib", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("[**CFF key: type**]{.underline}", "**BibTeX** Entry", "Notes"),
  caption = "Entry/Type crosswalk: From [CFF]{.underline} to **BibTeX**"
)

## ----fields_bib2cff, echo=FALSE,results='asis'--------------------------------
df_table <- table_master[table_master$table == "fields_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("**BibTeX Field**", "[CFF key]{.underline}", "Notes"),
  caption = "**BibTeX** - [CFF]{.underline} Field/Key crosswalk"
)

## ----fields_biblatex2cff, echo=FALSE,results='asis'---------------------------
df_table <- table_master[table_master$table == "fields_biblatex2cff", c(2:3)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f2 <- gsub("link_to_entry_models", "#entrymodels", df_table$f2)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("**BibLaTeX Field**", "[CFF key]{.underline}"),
  caption = "**BibLaTeX** - [CFF]{.underline} Field/Key crosswalk"
)

## ----model_article, echo=FALSE, results='asis'--------------------------------
df_table <- table_master[table_master$table == "model_article", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@article** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@article{article-full,
    title        = {The Gnats and Gnus Document Preparation System},
    author       = {Leslie A. Aamport},
    year         = 1986,
    month        = jul,
    journal      = {{G-Animal's} Journal},
    volume       = 41,
    number       = 7,
    pages        = {73+},
    note         = {This is a full ARTICLE entry}}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_book, echo=FALSE, results='asis'-----------------------------------
df_table <- table_master[table_master$table == "model_book", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@book / \\@inbook** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@book{book-full,
    title        = {Seminumerical Algorithms},
    author       = {Donald E. Knuth},
    year         = 1981,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 2,
    note         = {This is a full BOOK entry},
    edition      = {Second}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----echo=FALSE---------------------------------------------------------------
bib <- "@inbook{inbook-full,
    title        = {Fundamental Algorithms},
    author       = {Donald E. Knuth},
    year         = 1973,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 1,
    pages        = {10--119},
    note         = {This is a full INBOOK entry},
    edition      = {Second},
    type         = {Section},
    chapter      = {1.2}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_booklet, echo=FALSE, results='asis'--------------------------------
df_table <- table_master[table_master$table == "model_booklet", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@booklet** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@booklet{booklet-full,
    title        = {The Programming of Computer Art},
    author       = {Jill C. Knvth},
    date         = {1988-03-14},
    month        = feb,
    address      = {Stanford, California},
    note         = {This is a full BOOKLET entry},
    howpublished = {Vernier Art Center}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_inproceedings, echo=FALSE, results='asis'--------------------------
df_table <- table_master[table_master$table == "model_inproceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@conference / \\@inproceedings** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@inproceedings{inproceedings-full,
    title        = {On Notions of Information Transfer in {VLSI} Circuits},
    author       = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
    year         = 1983,
    month        = mar,
    booktitle    = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    pages        = {133--139},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_incollection, echo=FALSE, results='asis'---------------------------
df_table <- table_master[table_master$table == "model_incollection", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@incollection** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@incollection{incollection-full,
    title        = {Semigroups of Recurrences},
    author       = {Daniel D. Lincoll},
    year         = 1977,
    month        = sep,
    booktitle    = {High Speed Computer and Algorithm Organization},
    publisher    = {Academic Press},
    address      = {New York},
    series       = {Fast Computers},
    number       = 23,
    pages        = {179--183},
    note         = {This is a full INCOLLECTION entry},
    editor       = {David J. Lipcoll and D. H. Lawrie and A. H. Sameh},
    chapter      = 3,
    type         = {Part},
    edition      = {Third}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_manual, echo=FALSE, results='asis'---------------------------------
df_table <- table_master[table_master$table == "model_manual", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@manual** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@manual{manual-full,
  title        = {The Definitive Computer Manual},
    author       = {Larry Manmaker},
    year         = 1986,
    month        = {apr-may},
    address      = {Silicon Valley},
    note         = {This is a full MANUAL entry},
    organization = {Chips-R-Us},
    edition      = {Silver}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_thesis, echo=FALSE, results='asis'---------------------------------
df_table <- table_master[table_master$table == "model_thesis", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@mastersthesis / \\@phdthesis** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@mastersthesis{mastersthesis-full,
    title        = {Mastering Thesis Writing},
    author       = {Edouard Masterly},
    year         = 1988,
    month        = jun,
    address      = {English Department},
    note         = {This is a full MASTERSTHESIS entry},
    school       = {Stanford University},
    type         = {Master's project}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----echo=FALSE---------------------------------------------------------------
bib <- "@phdthesis{phdthesis-full,
    title        = {Fighting Fire with Fire: Festooning {F}rench Phrases},
    author       = {F. Phidias Phony-Baloney},
    year         = 1988,
    month        = jun,
    address      = {Department of French},
    note         = {This is a full PHDTHESIS entry},
    school       = {Fanstord University},
    type         = {{PhD} Dissertation}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_misc, echo=FALSE, results='asis'-----------------------------------
df_table <- table_master[table_master$table == "model_misc", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@misc** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_proceedings, echo=FALSE, results='asis'----------------------------
df_table <- table_master[table_master$table == "model_proceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@proceedings** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@proceedings{proceedings-full,
    title        = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    year         = 1983,
    month        = mar,
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    note         = {This is a full PROCEEDINGS entry},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_techreport, echo=FALSE, results='asis'-----------------------------
df_table <- table_master[table_master$table == "model_techreport", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@techreport** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@techreport{techreport-full,
    title        = {A Sorting Algorithm},
    author       = {Tom Terrific},
    year         = 1988,
    month        = oct,
    address      = {Computer Science Department, Fanstord, California},
    number       = 7,
    note         = {This is a full TECHREPORT entry},
    institution  = {Fanstord University},
    type         = {Wishful Research Result}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----model_unpublished, echo=FALSE, results='asis'----------------------------
df_table <- table_master[table_master$table == "model_unpublished", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@unpublished** Model"
)

## ----echo=FALSE---------------------------------------------------------------
bib <- "@unpublished{unpublished-minimal,
    title        = {Lower Bounds for Wishful Research Results},
    author       = {Ulrich Underwood and Ned Net and Paul Pot},
    note         = {Talk at Fanstord University (this is a minimal UNPUBLISHED entry)}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----echo=FALSE---------------------------------------------------------------
bib <- "@inbook{inbook-biblatex,
	author       = {Yihui Xie and Christophe Dervieux and Emily Riederer},
	title        = {Bibliographies and citations},
	booktitle    = {{R} Markdown Cookbook},
	date         = {2023-12-30},
	publisher    = {Chapman and Hall/CRC},
	address      = {Boca Raton, Florida},
	series       = {The {R} Series},
	isbn         = 9780367563837,
	url          = {https://bookdown.org/yihui/rmarkdown-cookbook},
	chapter      = {4.5}
}"

cff_read_bib_text(bib)

## ----echo=FALSE---------------------------------------------------------------
toBibtex(cff_read_bib_text(bib))

## ----cff_types, echo=FALSE, results='asis'------------------------------------
df_table <- table_master[table_master$table == "cff_types", c(2:3)]
df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("Reference type string", "Description"),
  row.names = NA,
  caption = "Complete list of [CFF]{.underline} reference types."
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(cffr)

## ----summary , echo=FALSE-----------------------------------------------------
keys <- cff_schema_keys(sorted = TRUE)
origin <- vector(length = length(keys))
origin[keys == "cff-version"] <- "parameter on function"
origin[keys == "type"] <- "Fixed value: 'software'"
origin[keys == "identifiers"] <- "DESCRIPTION/CITATION files"
origin[keys == "references"] <- "DESCRIPTION/CITATION files"

origin[keys %in% c(
  "message",
  "title",
  "version",
  "authors",
  "abstract",
  "repository",
  "repository-code",
  "url",
  "date-released",
  "contact",
  "keywords",
  "license"
)] <- "DESCRIPTION file"

origin[keys %in% c(
  "doi",
  "preferred-citation"
)] <- "CITATION file"


origin[origin == FALSE] <- "Ignored by cffr"

df <- data.frame(
  key = paste0("<a href='#", keys, "'>", keys, "</a>"),
  source = origin
)


knitr::kable(df, escape = FALSE)

## ----abstract-----------------------------------------------------------------
library(cffr)

# Create cffr for yaml

cff_obj <- cff_create("rmarkdown")

# Get DESCRIPTION of rmarkdown to check

pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cat(cff_obj$abstract)

cat(pkg$get("Description"))

## ----authors------------------------------------------------------------------
# An example DESCRIPTION
path <- system.file("examples/DESCRIPTION_many_persons", package = "cffr")
pkg <- desc::desc(path)

# See persons listed
pkg$get_authors()


# Default behaviour, use authors and creators (maintainers)
cff_obj <- cff_create(path)
cff_obj$authors


# Use now Copyright holders and maintainers
cff_obj_alt <- cff_create(path, authors_roles = c("cre", "cph"))
cff_obj_alt$authors

## ----cffversion---------------------------------------------------------------
cff_objv110 <- cff_create("jsonlite", cff_version = "v1.1.0")

cat(cff_objv110$`cff-version`)

## ----contact------------------------------------------------------------------
cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cff_obj$contact

pkg$get_author()

## ----date-released------------------------------------------------------------
# From an installed package

cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))


cat(pkg$get("Date/Publication"))


cat(cff_obj$`date-released`)



# A DESCRIPTION file without a Date
nodate <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp <- tempfile("DESCRIPTION")

# Create a temporary file
file.copy(nodate, tmp)


pkgnodate <- desc::desc(tmp)
cffnodate <- cff_create(tmp)

# Won't appear
cat(cffnodate$`date-released`)

pkgnodate

# Adding a Date

desc::desc_set("Date", "1999-01-01", file = tmp)

cat(cff_create(tmp)$`date-released`)

## ----doi----------------------------------------------------------------------
cff_doi <- cff_create("cffr")

cat(cff_doi$doi)

cat(cff_doi$`preferred-citation`$doi)

## ----identifiers--------------------------------------------------------------
file <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

pkg <- desc::desc(file)

cat(pkg$get_urls())

cat(cff_create(file)$url)

cat(cff_create(file)$`repository-code`)

cff_create(file)$identifiers

## ----keyword------------------------------------------------------------------
# A DESCRIPTION file without keywords
nokeywords <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp2 <- tempfile("DESCRIPTION")

# Create a temporary file
file.copy(nokeywords, tmp2)


pkgnokeywords <- desc::desc(tmp2)
cffnokeywords <- cff_create(tmp2)

# Won't appear
cat(cffnokeywords$keywords)

pkgnokeywords

# Adding Keywords

desc::desc_set("X-schema.org-keywords", "keyword1, keyword2, keyword3",
  file = tmp2
)

cat(cff_create(tmp2)$keywords)

## ----ghkeyword----------------------------------------------------------------
# Get cff object from jsonvalidate

jsonval <- cff_create("jsonvalidate")

# Keywords are retrieved from the GitHub repo

jsonval

# Check keywords
jsonval$keywords

# The repo
jsonval$`repository-code`

## ----license------------------------------------------------------------------
cff_obj <- cff_create("yaml")

cat(cff_obj$license)

pkg <- desc::desc(file.path(find.package("yaml"), "DESCRIPTION"))

cat(pkg$get("License"))

## ----eval=FALSE---------------------------------------------------------------
#  msg <- paste0(
#    'To cite package "',
#    "NAME_OF_THE_PACKAGE",
#    '" in publications use:'
#  )

## ----message------------------------------------------------------------------
cat(cff_create("jsonlite")$message)

## ----preferred-citation-------------------------------------------------------
cffobj <- cff_create("rmarkdown")

cffobj$`preferred-citation`

citation("rmarkdown")[1]

## ----references---------------------------------------------------------------
cffobj <- cff_create("rmarkdown")

cffobj$references

citation("rmarkdown")[-1]

## ----repository---------------------------------------------------------------
# Installed package

inst <- cff_create("jsonlite")

cat(inst$repository)

# Demo file downloaded from the r-universe

runiv <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")
runiv_cff <- cff_create(runiv)

cat(runiv_cff$repository)

desc::desc(runiv)$get("Repository")

# For in development package

norepo <- system.file("examples/DESCRIPTION_basic", package = "cffr")

# No repo
norepo_cff <- cff_create(norepo)

cat(norepo_cff[["repository"]])

# Change the name to a known package on CRAN: ggplot2

tmp <- tempfile("DESCRIPTION")
file.copy(norepo, tmp)


# Change name
desc::desc_set("Package", "ggplot2", file = tmp)

cat(cff_create(tmp)[["repository"]])

## ----repository-code----------------------------------------------------------
# Installed package on GitHub

cff_create("jsonlite")$`repository-code`



# GitLab

gitlab <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")

cat(cff_create(gitlab)$`repository-code`)


# Check

desc::desc(gitlab)

## ----eval=FALSE---------------------------------------------------------------
#  title <- paste0(
#    "NAME_OF_THE_PACKAGE",
#    ": ",
#    "TITLE_OF_THE_PACKAGE"
#  )

## ----title--------------------------------------------------------------------
# Installed package

cat(cff_create("testthat")$title)

## ----url----------------------------------------------------------------------
# Many urls
manyurls <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

cat(cff_create(manyurls)$url)

# Check

desc::desc(manyurls)

## ----version------------------------------------------------------------------
# Should be (>= 3.0.0)
cat(cff_create("testthat")$version)

## -----------------------------------------------------------------------------
#| include: false
# jarl-ignore-file fixed_regex: Not production code.

library(cffr)
# Load the table of tables.

p2file <- system.file("extdata/crosswalk_tables.csv", package = "cffr")

table_master <- read.csv(p2file)


## -----------------------------------------------------------------------------
#| label: tbl-entry_fields1
#| echo: false
#| tbl-cap: BibTeX entries
#| tbl-subcap:
#|   - "BibTeX: required fields by entry"
#|   - "BibTeX: required fields by entry (cont.)"
#|
df_table <- table_master[table_master$table == "entry_fields", -1]

nms <- c(
  "field",
  "\\@article",
  "\\@book",
  "\\@booklet",
  "\\@inbook",
  "\\@incollection",
  "\\@conference, \\@inproceedings",
  "\\@manual",
  "\\@mastersthesis, phdthesis",
  "\\@misc",
  "\\@proceedings",
  "\\@techreport",
  "\\@unpublished"
)

df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
t1 <- df_table[, c(1:7)]
nm1 <- nms[1:7]

knitr::kable(
  t1,
  col.names = nm1,
  row.names = NA,
  align = c("l", rep("c", 6))
)

t2 <- df_table[, c(1, 8:13)]
nm2 <- nms[c(1, 8:13)]
knitr::kable(
  t2,
  col.names = nm2,
  row.names = NA,
  align = c("l", rep("c", 6))
)


## -----------------------------------------------------------------------------
#| label: tbl-refkeys
#| echo: false
#| message: false
#| warning: false
#| results: asis
#| tbl-cap: "Valid keys in CFF `definitions.reference` objects"
library(cffr)

# Fill with blanks.
init <- paste0("`", cff_schema_definitions_refs(), "`")

l <- c(init, rep("", 4))

refkeys <- matrix(l, ncol = 5, byrow = TRUE)

knitr::kable(
  refkeys,
  row.names = NA
)


## -----------------------------------------------------------------------------
#| label: cffbibread
#| comment: '#>'
string <- "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925}"

# Convert to `cff`.
library(cffr)
cff_format <- cff_read_bib_text(string)

cff_format

# Convert to BibTeX with the S3 method.
toBibtex(cff_format)


## -----------------------------------------------------------------------------
#| label: tbl-entry_bib2cff
#| echo: false
#| results: asis
#| tbl-cap: "Entry/Type crosswalk: From BibTeX to CFF"
df_table <- table_master[table_master$table == "entry_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("BibTeX entry", "CFF key: `type`", "Notes"),
  row.names = NA
)


## -----------------------------------------------------------------------------
#| label: tbl-entry_cff2bib
#| echo: false
#| results: asis
#| tbl-cap: "Entry/Key `type` crosswalk: From CFF to BibTeX"
df_table <- table_master[table_master$table == "entry_cff2bib", c(2:4)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("CFF key `type`", "BibTeX entry", "Notes")
)


## -----------------------------------------------------------------------------
#| label: tbl-fields_bib2cff
#| echo: false
#| results: asis
#| tbl-cap: "BibTeX - CFF key crosswalk"
df_table <- table_master[table_master$table == "fields_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("BibTeX **Field**", "CFF key", "Notes")
)


## -----------------------------------------------------------------------------
#| label: tbl-fields_biblatex2cff
#| echo: false
#| results: asis
#| tbl-cap: "BibLaTeX - CFF field/key crosswalk"
df_table <- table_master[table_master$table == "fields_biblatex2cff", c(2:3)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f2 <- gsub("link_to_entry_models", "#entrymodels", df_table$f2)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("**BibLaTeX Field**", "CFF key")
)


## -----------------------------------------------------------------------------
#| label: tbl-model_article
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_article", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@article** model"
)


## -----------------------------------------------------------------------------
bib <- "@article{article-full,
    title        = {The Gnats and Gnus Document Preparation System},
    author       = {Leslie A. Aamport},
    year         = 1986,
    month        = jul,
    journal      = {{G-Animal's} Journal},
    volume       = 41,
    number       = 7,
    pages        = {73+},
    note         = {This is a full ARTICLE entry}}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_book
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_book", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@book / \\@inbook** model"
)


## -----------------------------------------------------------------------------
bib <- "@book{book-full,
    title        = {Seminumerical Algorithms},
    author       = {Donald E. Knuth},
    year         = 1981,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 2,
    note         = {This is a full BOOK entry},
    edition      = {Second}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
bib <- "@inbook{inbook-full,
    title        = {Fundamental Algorithms},
    author       = {Donald E. Knuth},
    year         = 1973,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 1,
    pages        = {10--119},
    note         = {This is a full INBOOK entry},
    edition      = {Second},
    type         = {Section},
    chapter      = {1.2}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_booklet
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_booklet", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@booklet** model"
)


## -----------------------------------------------------------------------------
bib <- "@booklet{booklet-full,
    title        = {The Programming of Computer Art},
    author       = {Jill C. Knvth},
    date         = {1988-03-14},
    month        = feb,
    address      = {Stanford, California},
    note         = {This is a full BOOKLET entry},
    howpublished = {Vernier Art Center}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_inproceedings
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_inproceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@conference / \\@inproceedings** model"
)


## -----------------------------------------------------------------------------
bib <- "@inproceedings{inproceedings-full,
    title        = {On Notions of Information Transfer in {VLSI} Circuits},
    author       = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
    year         = 1983,
    month        = mar,
    booktitle    = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    pages        = {133--139},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_incollection
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_incollection", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@incollection** model"
)


## -----------------------------------------------------------------------------
bib <- "@incollection{incollection-full,
    title        = {Semigroups of Recurrences},
    author       = {Daniel D. Lincoll},
    year         = 1977,
    month        = sep,
    booktitle    = {High Speed Computer and Algorithm Organization},
    publisher    = {Academic Press},
    address      = {New York},
    series       = {Fast Computers},
    number       = 23,
    pages        = {179--183},
    note         = {This is a full INCOLLECTION entry},
    editor       = {David J. Lipcoll and D. H. Lawrie and A. H. Sameh},
    chapter      = 3,
    type         = {Part},
    edition      = {Third}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_manual
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_manual", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@manual** model"
)


## -----------------------------------------------------------------------------
bib <- "@manual{manual-full,
  title        = {The Definitive Computer Manual},
    author       = {Larry Manmaker},
    year         = 1986,
    month        = {apr-may},
    address      = {Silicon Valley},
    note         = {This is a full MANUAL entry},
    organization = {Chips-R-Us},
    edition      = {Silver}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_thesis
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_thesis", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@mastersthesis / \\@phdthesis** model"
)


## -----------------------------------------------------------------------------
bib <- "@mastersthesis{mastersthesis-full,
    title        = {Mastering Thesis Writing},
    author       = {Edouard Masterly},
    year         = 1988,
    month        = jun,
    address      = {English Department},
    note         = {This is a full MASTERSTHESIS entry},
    school       = {Stanford University},
    type         = {Master's project}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
bib <- "@phdthesis{phdthesis-full,
    title        = {Fighting Fire with Fire: Festooning {F}rench Phrases},
    author       = {F. Phidias Phony-Baloney},
    year         = 1988,
    month        = jun,
    address      = {Department of French},
    note         = {This is a full PHDTHESIS entry},
    school       = {Fanstord University},
    type         = {{PhD} Dissertation}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_misc
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_misc", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@misc** model"
)


## -----------------------------------------------------------------------------
bib <- "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_proceedings
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_proceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@proceedings** model"
)


## -----------------------------------------------------------------------------
bib <- "@proceedings{proceedings-full,
    title        = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    year         = 1983,
    month        = mar,
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    note         = {This is a full PROCEEDINGS entry},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_techreport
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_techreport", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@techreport** model"
)


## -----------------------------------------------------------------------------
bib <- "@techreport{techreport-full,
    title        = {A Sorting Algorithm},
    author       = {Tom Terrific},
    year         = 1988,
    month        = oct,
    address      = {Computer Science Department, Fanstord, California},
    number       = 7,
    note         = {This is a full TECHREPORT entry},
    institution  = {Fanstord University},
    type         = {Wishful Research Result}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-model_unpublished
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_unpublished", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@unpublished** model"
)


## -----------------------------------------------------------------------------
bib <- "@unpublished{unpublished-minimal,
    title        = {Lower Bounds for Wishful Research Results},
    author       = {Ulrich Underwood and Ned Net and Paul Pot},
    note         = {Talk at Fanstord University (this is a minimal UNPUBLISHED entry)}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
bib <- "@inbook{inbook-biblatex,
	author       = {Yihui Xie and Christophe Dervieux and Emily Riederer},
	title        = {Bibliographies and citations},
	booktitle    = {{R} Markdown Cookbook},
	date         = {2023-12-30},
	publisher    = {Chapman and Hall/CRC},
	address      = {Boca Raton, Florida},
	series       = {The {R} Series},
	isbn         = 9780367563837,
	url          = {https://yihui.org/rmarkdown-cookbook/},
	chapter      = {4.5}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))


## -----------------------------------------------------------------------------
#| label: tbl-cff_types
#| echo: false
#| results: asis
#| tbl-cap: "Complete list of CFF reference types"
df_table <- table_master[table_master$table == "cff_types", c(2:3)]
df_table[is.na(df_table)] <- ""

row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("Reference type string", "Description"),
  row.names = NA
)

## -----------------------------------------------------------------------------
#| include: false
library(cffr)


## -----------------------------------------------------------------------------
#| label: tbl-summary
#| echo: false
#| tbl-cap: "cffr and R packages: Conversion table"
keys <- cff_schema_keys(sorted = TRUE)
origin <- vector(length = length(keys))
origin[keys == "cff-version"] <- "function argument"
origin[keys == "type"] <- "Fixed value: 'software'"
origin[keys == "identifiers"] <- "DESCRIPTION and inst/CITATION"
origin[keys == "references"] <- "DESCRIPTION and inst/CITATION"

origin[
  keys %in%
    c(
      "message",
      "title",
      "version",
      "authors",
      "abstract",
      "repository",
      "repository-code",
      "url",
      "date-released",
      "contact",
      "keywords",
      "license",
      "commit"
    )
] <- "DESCRIPTION file"

origin[
  keys %in%
    c(
      "doi",
      "preferred-citation"
    )
] <- "inst/CITATION"

origin[origin == "FALSE"] <- "Ignored by cffr"

df <- data.frame(
  key = paste0("<a href='#", keys, "'>", keys, "</a>"),
  source = origin
)

knitr::kable(df, escape = FALSE)


## -----------------------------------------------------------------------------
#| label: abstract
library(cffr)

# Create a `cff` object for YAML.

cff_obj <- cff_create("rmarkdown")

# Get the DESCRIPTION of **rmarkdown** to check.

pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cat(cff_obj$abstract)

cat(pkg$get("Description"))


## -----------------------------------------------------------------------------
#| label: authors
# An example DESCRIPTION.
path <- system.file("examples/DESCRIPTION_many_persons", package = "cffr")
pkg <- desc::desc(path)

# See the listed persons.
pkg$get_authors()

# Default behavior: use authors and creators (maintainers).
cff_obj <- cff_create(path)
cff_obj$authors

# Use copyright holders and maintainers.
cff_obj_alt <- cff_create(path, authors_roles = c("cre", "cph"))
cff_obj_alt$authors


## -----------------------------------------------------------------------------
#| label: cffversion
cff_objv110 <- cff_create("jsonlite", cff_version = "v1.1.0")

cat(cff_objv110$`cff-version`)


## -----------------------------------------------------------------------------
#| label: commit
# An example DESCRIPTION.
path <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")
pkg <- desc::desc(path)

# See `RemoteSha`.
pkg$get("RemoteSha")

cff_read(path)


## -----------------------------------------------------------------------------
#| label: contact
cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cff_obj$contact

pkg$get_author()


## -----------------------------------------------------------------------------
#| label: date-released
# Use an installed package.

cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cat(pkg$get("Date/Publication"))

cat(cff_obj$`date-released`)

# A DESCRIPTION file without a `Date`.
nodate <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp <- tempfile("DESCRIPTION")

# Create a temporary file.
file.copy(nodate, tmp)

pkgnodate <- desc::desc(tmp)
cffnodate <- cff_create(tmp)

# No value is returned.
cat(cffnodate$`date-released`)

pkgnodate

# Add a `Date`.

desc::desc_set("Date", "1999-01-01", file = tmp)

cat(cff_create(tmp)$`date-released`)


## -----------------------------------------------------------------------------
#| label: doi
cff_doi <- cff_create("cffr")

cat(cff_doi$doi)

cat(cff_doi$`preferred-citation`$doi)


## -----------------------------------------------------------------------------
#| label: identifiers
file <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

pkg <- desc::desc(file)

cat(pkg$get_urls())

cat(cff_create(file)$url)

cat(cff_create(file)$`repository-code`)

cff_create(file)$identifiers


## -----------------------------------------------------------------------------
#| label: keyword
# A DESCRIPTION file without keywords
nokeywords <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp2 <- tempfile("DESCRIPTION")

# Create a temporary file.
file.copy(nokeywords, tmp2)

pkgnokeywords <- desc::desc(tmp2)
cffnokeywords <- cff_create(tmp2)

# No value is returned.
cat(cffnokeywords$keywords)

pkgnokeywords

# Add keywords.

desc::desc_set(
  "X-schema.org-keywords",
  "keyword1, keyword2, keyword3",
  file = tmp2
)

cat(cff_create(tmp2)$keywords)


## -----------------------------------------------------------------------------
#| label: ghkeyword
# Get a `cff` object from **jsonvalidate**.

jsonval <- cff_create("jsonvalidate")

# Keywords are retrieved from the **GitHub** repository.

jsonval

# Check keywords.
jsonval$keywords

# The repository.
jsonval$`repository-code`


## -----------------------------------------------------------------------------
#| label: license
cff_obj <- cff_create("yaml")

cat(cff_obj$license)

pkg <- desc::desc(file.path(find.package("yaml"), "DESCRIPTION"))

cat(pkg$get("License"))


## -----------------------------------------------------------------------------
#| eval: false
#| code-fold: false
# msg <- paste0(
#   'To cite package "',
#   "NAME_OF_THE_PACKAGE",
#   '" in publications use:'
# )


## -----------------------------------------------------------------------------
#| label: message
cat(cff_create("jsonlite")$message)


## -----------------------------------------------------------------------------
#| label: preferred-citation
cffobj <- cff_create("rmarkdown")

cffobj$`preferred-citation`

citation("rmarkdown")[1]


## -----------------------------------------------------------------------------
#| label: references
cffobj <- cff_create("rmarkdown")

cffobj$references

citation("rmarkdown")[-1]


## -----------------------------------------------------------------------------
#| label: repository
# Use an installed package.

inst <- cff_create("jsonlite")

cat(inst$repository)

# Demo file downloaded from the r-universe.

runiv <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")
runiv_cff <- cff_create(runiv)

cat(runiv_cff$repository)

desc::desc(runiv)$get("Repository")

# For an in-development package.

norepo <- system.file("examples/DESCRIPTION_basic", package = "cffr")

# No repository is available.
norepo_cff <- cff_create(norepo)

cat(norepo_cff[["repository"]])

# Change the name to a known package on CRAN: ggplot2.

tmp <- tempfile("DESCRIPTION")
file.copy(norepo, tmp)

# Change the package name.
desc::desc_set("Package", "ggplot2", file = tmp)

cat(cff_create(tmp)[["repository"]])


## -----------------------------------------------------------------------------
#| label: repository-code
# Installed package on GitHub.

cat(cff_create("jsonlite")$`repository-code`)

# GitLab

gitlab <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")

cat(cff_create(gitlab)$`repository-code`)

# Codeberg
codeberg <- system.file("examples/DESCRIPTION_codeberg", package = "cffr")

cat(cff_create(codeberg)$`repository-code`)


## -----------------------------------------------------------------------------
#| eval: false
#| code-fold: false
# title <- paste0("NAME_OF_THE_PACKAGE", ": ", "TITLE_OF_THE_PACKAGE")


## -----------------------------------------------------------------------------
#| label: title
# Use an installed package.

cat(cff_create("testthat")$title)


## -----------------------------------------------------------------------------
#| label: url
# Many URLs.
manyurls <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

cat(cff_create(manyurls)$url)

# Check the source metadata.

desc::desc(manyurls)


## -----------------------------------------------------------------------------
#| label: version
#| code-fold: false
# Should be (>= 3.0.0).
cat(cff_create("testthat")$version)

## -----------------------------------------------------------------------------
#| include: false
library(cffr)


## -----------------------------------------------------------------------------
#| label: setup
#| eval: false
# library(cffr)
# 
# cff_write()
# 
# # Done.


## -----------------------------------------------------------------------------
#| label: newfields
newobject <- cff()

newobject


## -----------------------------------------------------------------------------
#| label: validkeys
cff_schema_keys()


## -----------------------------------------------------------------------------
#| label: modify
modobject <- cff_modify(
  newobject,
  url = "https://ropensci.org/",
  version = "0.0.1",
  repository = "https://github.com/ropensci/cffr",
  # If the key is already present, it is overridden.
  title = "Modifying a 'cff' object"
)

modobject

# Validate against the schema.

cff_validate(modobject)


## -----------------------------------------------------------------------------
#| label: includeauthor
# Valid person keys.

cff_schema_definitions_person()

# Create the person.

chiquito <- person(
  "Gregorio",
  "Sánchez Fernández",
  email = "fake@email2.com",
  comment = c(
    alias = "Chiquito de la Calzada",
    city = "Malaga",
    country = "ES",
    ORCID = "0000-0000-0000-0001"
  )
)

chiquito

# Convert to `cff`.
chiquito_cff <- as_cff_person(chiquito)
chiquito_cff

# Append to previous authors.

newauthors <- c(modobject$authors, chiquito_cff)
newauthors

newauthorobject <- cff_modify(modobject, authors = newauthors)

newauthorobject

cff_validate(newauthorobject)


## -----------------------------------------------------------------------------
#| label: parsingcits
# Valid reference keys.

cff_schema_definitions_refs()

# Automatic coercion from another **R** package.
base_r <- citation("base")

bib <- bibentry(
  "Book",
  title = "This is a book",
  author = "Lisa Lee",
  year = 1980,
  publisher = "McGraw Hill",
  volume = 2
)

refs <- c(base_r, bib)

refs

# Convert to `cff`.

refs_cff <- as_cff(refs)

refs_cff


## -----------------------------------------------------------------------------
#| label: references
finalobject <- cff_modify(newauthorobject, references = refs_cff)

finalobject

cff_validate(finalobject)


## -----------------------------------------------------------------------------
#| label: write
# Create a temporary output file.
tmp <- tempfile(fileext = ".cff")

see_res <- cff_write(finalobject, outfile = tmp)

cat(readLines(tmp), sep = "\n")


## -----------------------------------------------------------------------------
#| label: read
reading <- cff_read(tmp)

reading


## -----------------------------------------------------------------------------
allkeys <- list(
  "url" = "https://ropensci.org/",
  "version" = "0.0.1",
  "repository" = "https://github.com/ropensci/cffr",
  # If the key is already present, it is overridden.
  title = "Modifying a 'cff' object",
  authors = newauthors,
  references = refs_cff
)

tmp2 <- tempfile(fileext = ".cff")

res <- cff_write(cff(), outfile = tmp2, keys = allkeys)

res


## -----------------------------------------------------------------------------
#| include: false
# Clean temporary files.
unlink(tmp)
unlink(tmp2)

#' Read BibTeX markup as a [`cff_ref_lst`] object
#'
#' @description
#' Convert a `character` string representing a BibTeX entry into a
#' [`cff_ref_lst`] object.
#'
#' @family bibtex
#' @family reading
#'
#' @seealso
#'
#' [cff_read_bib()] for reading `*.bib` files.
#'
#' @export
#' @encoding UTF-8
#'
#' @param x A `character` vector with one or more complete BibTeX entries.
#' @param encoding Encoding to be assumed for `x`. See [readLines()].
#' @param ... Arguments passed to [cff_read_bib()].
#'
#' @return
#'
#' An object of classes [`cff_ref_lst, cff`][cff_ref_lst] according to the
#' `definitions.references` specified in
#' the
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [Citation File Format schema]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#' Each element of the `cff_ref_lst` object has classes
#' [`cff_ref, cff`][cff_ref].
#'
#' @details
#'
#' This function writes `x` to a temporary `*.bib` file and reads it using
#' [cff_read_bib()].
#'
#' This function requires \CRANpkg{bibtex} (>= 0.5.0) and uses
#' [bibtex::read.bib()] for parsing.
#'
#' @examples
#' if (requireNamespace("bibtex", quietly = TRUE)) {
#'   x <- c(
#'     "@book{einstein1921,
#'       title        = {Relativity: The Special and the General Theory},
#'       author       = {Einstein, Albert},
#'       year         = 1920,
#'       publisher    = {Henry Holt and Company},
#'       address      = {London, United Kingdom},
#'       isbn         = 9781587340925
#'   }",
#'     "@misc{misc-full,
#'       title        = {Handing out random pamphlets in airports},
#'       author       = {Joe-Bob Missilany},
#'       year         = 1984,
#'       month        = oct,
#'       note         = {This is a full MISC entry},
#'       howpublished = {Handed out at O'Hare}
#'   }"
#'   )
#'
#'   cff_read_bib_text(x)
#' }
cff_read_bib_text <- function(x, encoding = "UTF-8", ...) {
  # Validations
  if (!inherits(x, "character")) {
    cli::cli_abort(
      paste0(
        "{.arg x} should be a {.cls character}, not a ",
        "{.cls {class(x)}}."
      )
    )
  }

  if (any(grepl("\\.bib$", x, ignore.case = TRUE))) {
    cli::cli_alert_warning(
      paste0("{.arg x} seems to be a {.val .bib} file, not a BibTeX entry.")
    )
    cli::cli_alert_info("Reading {.arg x} with {.fn cffr:cff_read_bib}")
    the_cff <- cff_read_bib(x, encoding = encoding, ...)
    return(the_cff)
  }

  if (!any(grepl("^@", x))) {
    cli::cli_alert_warning(
      paste0("{.arg x} doesn't look as a BibTeX entry. Check the results.")
    )
  }
  # Write x to a tempfile
  file <- tempfile(fileext = ".bib")
  writeLines(x, file)
  the_cff <- cff_read_bib(file, encoding = encoding, ...)
  unlink(file)
  the_cff
}

test_that("Errors and messages", {
  skip_if_not_installed("bibtex", "0.5.0")
  a_cff <- cff()
  expect_snapshot(cff_read_bib_text(a_cff), error = TRUE)
  expect_snapshot(cff_read_bib_text("a bad line"), error = TRUE)
})

test_that("Read lines", {
  skip_if_not_installed("bibtex", "0.5.0")

  x <- c(
    "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, Albert},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925
}",
    "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"
  )

  list <- cff_read_bib_text(x)

  expect_s3_class(list, "cff")
  expect_length(list, 2)

  expect_snapshot(list)

  # Create a Bibtex file with that lines
  tmpbib <- tempfile(fileext = ".bib")
  writeLines(x, tmpbib)

  expect_snapshot(fromfile <- cff_read_bib_text(tmpbib))
  expect_identical(fromfile, list)
})

#' Create [`bibentry`] objects from several sources
#'
#' @rdname as_bibentry
#' @name as_bibentry
#' @order 1
#' @description
#'
#' This function creates [`bibentry`] objects from different metadata sources
#' ([`cff`] objects, `DESCRIPTION` files and other sources). The inverse
#' transformation (`bibentry` object to [`cff_ref_lst`]) can be done with the
#' corresponding [as_cff.bibentry()] method.
#'
#' With [`toBibtex()`][toBibtex.cff()] it is possible to convert [`cff`] objects
#' to BibTeX markup on the fly, see **Examples**.
#'
#' @seealso
#' [utils::bibentry()] to understand more about the `bibentry` class.
#'
#' - `vignette("crosswalk", package = "cffr")` provides details on how the
#'   metadata of a package is mapped to produce a `cff` object.
#'
#' - `vignette("bibtex_cff", "cffr")` provides detailed information about the
#'   internal mapping performed between `cff` objects and BibTeX markup (
#'   both `cff` to BibTeX and BibTeX to `cff`).
#'
#' Other related functions:
#' - [utils::toBibtex()].
#'
#' @references
#' - Patashnik, Oren. "BIBTEXTING" February 1988.
#'   <https://osl.ugr.es/CTAN/biblio/bibtex/base/btxdoc.pdf>.
#'
#' - Haines, R., & The Ruby Citation File Format Developers. (2021).
#'   *Ruby CFF Library (Version 0.9.0)* (Computer software).
#'   \doi{10.5281/zenodo.1184077}.
#'
#' - Hernangomez D (2022). "BibTeX and CFF, a potential crosswalk."
#'   *The cffr package, Vignettes*. \doi{10.21105/joss.03900},
#'   <https://docs.ropensci.org/cffr/articles/bibtex-cff.html>.
#'
#' @param x The source used to generate
#'   the `bibentry` object via \CRANpkg{cffr}. It can be:
#'   - A missing value, which retrieves the `DESCRIPTION`
#'     file from your in-development package.
#'   - An existing `cff` object created with [cff()], [cff_create()], or
#'     [as_cff()].
#'   - Path to a CITATION.cff file (`"CITATION.cff"`).
#'   - The name of an installed package (`"jsonlite"`).
#'   - Path to a DESCRIPTION file (`"DESCRIPTION"`).
#' @param ... Additional arguments to be passed to or from methods.
#'
#' @param what Fields to extract from a full `cff` object. The value could be:
#'   - `preferred`: This would create a single entry with the main citation
#'      info of the package (key `preferred-citation`).
#'   - `references`: Extract all the entries of `references` key.
#'   - `all`: A combination of the previous two options. This would extract
#'      both the `preferred-citation` and the `references` key.
#'
#'  See `vignette("crosswalk", package = "cffr")`.
#'
#' @family bibtex
#' @family s3method
#'
#' @details
#'
#' An **R** `bibentry` object is the representation of a BibTeX entry. These
#' objects can be converted to BibTeX markup with [toBibtex()], that creates an
#' object of class `Bibtex` and can be printed and exported as a valid BibTeX
#' entry.
#'
#' `as_bibtex()` tries to map the information of the source `x` into a [`cff`]
#' object and performs a mapping of the metadata to BibTeX, according to
#' `vignette("bibtex_cff", "cffr")`.
#'
#' @return
#' `as_bibentry()` returns a `bibentry` object with one or more entries.
#'
#' @export
#' @encoding UTF-8
#'
#' @examples
#' \donttest{
#' # From a cff object ----
#' cff_object <- cff()
#'
#' cff_object
#'
#' # bibentry object
#' bib <- as_bibentry(cff_object)
#'
#' class(bib)
#'
#' bib
#'
#' # Print as bibtex
#' toBibtex(bib)
#'
#' # Thanks to the S3 Method we can also do
#' toBibtex(cff_object)
#'
#' # Other sources ----
#' # From a CITATION.cff
#'
#' path <- system.file("examples/CITATION_complete.cff", package = "cffr")
#' cff_file <- as_bibentry(path)
#'
#' cff_file
#'
#' # For an installed package with options
#' installed_package <- as_bibentry("jsonvalidate", what = "all")
#'
#' installed_package
#'
#' # Use a DESCRIPTION file
#' path2 <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")
#' desc_file <- as_bibentry(path2)
#'
#' toBibtex(desc_file)
#' }
#'
as_bibentry <- function(x, ...) {
  UseMethod("as_bibentry")
}


#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 2
as_bibentry.default <- function(x, ...) {
  dot_list <- list(...)
  as_bibentry(dot_list)
}

#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 3
as_bibentry.character <- function(
  x,
  ...,
  what = c("preferred", "references", "all")
) {
  # A named list
  if (missing(x)) {
    dot_list <- list(...)
    return(as_bibentry(dot_list))
  }

  what <- match_cff_arg(
    what,
    c("preferred", "references", "all"),
    "what",
    environment()
  )

  x <- x[1]
  src_detect <- detect_x_source(x)

  if (src_detect == "dontknow") {
    # nolint start
    msg <- paste0('install.packages("', x, '")')
    # nolint end
    cli::cli_abort(
      paste(
        "Don't know how to extract a {.cls bibentry} from {.val {x}}.",
        "If it is a package run {.run {msg}} first."
      )
    )
  }

  if (src_detect == "package") {
    x <- cff_create(x)
  } else {
    x <- cff_read(x)
  }

  as_bibentry(x, what = what)
}

#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 4
as_bibentry.NULL <- function(x, ...) {
  cff_create()
}


#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 5
as_bibentry.list <- function(x, ...) {
  ## Convert and catch errors ----
  bib <- try(do.call(bibentry, x), silent = TRUE)

  # If key missing
  if (inherits(bib, "try-error")) {
    message <- attributes(bib)$condition$message
    cli::cli_alert_danger(paste("Can't convert to {.fn bibentry}: "))
    cli::cli_alert_info(message)
    cli::cli_alert_warning("Returning empty {.cls bibentry}")
    return(bibentry())
  }

  # Unlist easy to undo the do.call effect
  bib <- bib[[1]]

  bib
}


#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 6
as_bibentry.cff <- function(
  x,
  ...,
  what = c("preferred", "references", "all")
) {
  what <- match_cff_arg(
    what,
    c("preferred", "references", "all"),
    "what",
    environment()
  )

  obj <- x
  # Try to generate preferred if not present
  if (!("preferred-citation" %in% names(obj))) {
    prefcit <- obj
    prefcit$type <- "generic"
    prefcit <- prefcit[names(prefcit) %in% cff_schema_definitions_refs()]
    prefcit <- new_cff(prefcit)
    # And add to the object
    obj$`preferred-citation` <- prefcit
  }

  # Select type to extract
  obj_extract <- switch(what,
    "preferred" = list(obj$`preferred-citation`),
    "references" = obj$references,
    c(list(obj$`preferred-citation`), obj$references)
  )

  if (is.null(obj_extract)) {
    cli::cli_alert_warning(
      paste0(
        "In {.arg x} didn't find anything with {.arg what} = {.val {what}}. ",
        "Returning empty {.cls bibentry}."
      )
    )
    return(bibentry())
  }

  # Prepare for dispatching
  objend <- as_cff(obj_extract)

  as_bibentry(objend)
}

#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 7
as_bibentry.cff_ref_lst <- function(x, ...) {
  ref <- lapply(x, function(y) {
    # Reclass to dispatch method
    as_bibentry(as_cff(y))
  })
  ref <- do.call(c, ref)
  ref
}


#' @export
#' @encoding UTF-8
#' @rdname as_bibentry
#' @order 8
as_bibentry.cff_ref <- function(x, ...) {
  # Relist to cff for dispatching methods on persons
  x <- as_cff(x)

  # Partially based on ruby parser
  # https://github.com/citation-file-format/ruby-cff/blob/main/lib/cff/ >>
  # (cont) formatter/bibtex_formatter.rb

  # Create a initial empty list

  tobibentry <- list()

  # Direct mapping ----

  ## From BibTeX ----
  ### author----
  tobibentry$author <- as.person(x$authors)
  ### chapter----
  tobibentry$chapter <- x$section

  ### edition----
  tobibentry$edition <- x$edition

  ### editor----
  tobibentry$editor <- as.person(x$editors)

  ### howpublished----
  tobibentry$howpublished <- get_bib_howpublised(x)

  ### journal----
  tobibentry$journal <- x$journal

  ### note ----
  tobibentry$note <- get_bib_note(x)

  ### number----
  tobibentry$number <- clean_str(x[["issue"]])

  ### pages ----
  tobibentry$pages <- clean_str(
    paste(unique(c(x$start, x$end)), collapse = "--")
  )

  ### publisher ----
  tobibentry$publisher <- clean_str(x$publisher$name)

  ### title ----
  tobibentry$title <- x$title

  ### volume----
  tobibentry$volume <- x$volume

  ## From BibLaTeX ----

  tobibentry$abstract <- x$abstract
  tobibentry$date <- x$`date-published`
  tobibentry$doi <- x$doi
  tobibentry$file <- x$filename
  tobibentry$isbn <- x$isbn
  tobibentry$issn <- x$issn
  tobibentry$issuetitle <- x$`issue-title`
  tobibentry$keywords <- clean_str(paste0(unique(x$keywords), collapse = ","))
  tobibentry$pagetotal <- x$pages
  tobibentry$translator <- toBibtex(as.person(x$translators))
  tobibentry$url <- x$url
  tobibentry$urldate <- x$`date-accessed`
  tobibentry$version <- x$version

  # BibTeX entry----

  tobibentry$bibtype <- guess_bibtype(x)

  # address----
  tobibentry$address <- get_bib_address(x)

  # booktitle /series ----
  # Map cff collection-title
  tobibentry <- c(tobibentry, get_bib_booktitle(x, tobibentry$bibtype))

  # institution/organization/school ----
  # Map cff institution
  tobibentry <- c(tobibentry, get_bib_inst_org(x, tobibentry$bibtype))

  # month----
  tobibentry$month <- get_bib_month(x)

  # year ----
  tobibentry$year <- get_bib_year(x)

  # Handle anonymous author----
  # If anonymous coming from cff and not needed, then not use it

  is_anon <- identical(clean_str(x$authors[[1]]$name), "anonymous")

  # If unknown remove from bib types that doesn't require it strictly

  if (
    all(
      is_anon,
      tobibentry$bibtype %in%
        c(
          "booklet",
          "manual",
          "book",
          "inbook",
          "misc",
          "proceedings"
        )
    )
  ) {
    tobibentry$author <- NULL
  }
  # BibTeX key----
  tobibentry$key <- make_bibkey(tobibentry)

  # Final steps ----
  ##  Sort ----
  # based on default by
  # https://flamingtempura.github.io/bibtex-tidy/
  tosort <- c(
    "title",
    "author",
    "year",
    "month",
    "day",
    "journal",
    "booktitle",
    "publisher",
    "address",
    "editor",
    "series",
    "volume",
    "number",
    "pages",
    "doi",
    "isbn",
    "issn",
    "url",
    "note"
  )

  unique <- unique(c(tosort, names(tobibentry)))
  sorted <- unique[unique %in% names(tobibentry)]
  tobibentry <- tobibentry[sorted]

  ## Convert

  as_bibentry(tobibentry)
}

# Utils in utils-bib.R

test_that("as_bibentry default", {
  bib_coerced <- as_bibentry(
    bibtype = "Article",
    key = "knuth:1984",
    author = person("R Core Team"),
    title = "Literate Programming",
    journal = "The Computer Journal",
    year = "1984",
    # Optional
    volume = "27",
    number = 2,
    pages = "97--111",
    month = "January",
    keywords = "Some, simple, keywords"
  )

  direct <- bibentry(
    bibtype = "Article",
    key = "knuth:1984",
    author = person("R Core Team"),
    title = "Literate Programming",
    journal = "The Computer Journal",
    year = "1984",
    # Optional
    volume = "27",
    number = 2,
    pages = "97--111",
    month = "January",
    keywords = "Some, simple, keywords"
  )
  expect_identical(bib_coerced, direct)

  with_number_first <- as_bibentry(
    number = 2,
    bibtype = "Article",
    key = "knuth:1984",
    author = person("R Core Team"),
    title = "Literate Programming",
    journal = "The Computer Journal",
    year = "1984",
    # Optional
    volume = "27",
    pages = "97--111",
    month = "January",
    keywords = "Some, simple, keywords"
  )

  direct <- bibentry(
    number = 2,
    bibtype = "Article",
    key = "knuth:1984",
    author = person("R Core Team"),
    title = "Literate Programming",
    journal = "The Computer Journal",
    year = "1984",
    # Optional
    volume = "27",
    pages = "97--111",
    month = "January",
    keywords = "Some, simple, keywords"
  )

  expect_identical(with_number_first, direct)

  # No additional dots
  expect_snapshot(s <- as_bibentry(a = 1))
})

test_that("as_bibentry NULL", {
  skip_on_cran()
  current_dir <- getwd()

  name <- paste0("mock-pack", runif(1) * 10)
  new_dir <- file.path(tempdir(), name)

  dir.create(new_dir, recursive = TRUE)

  expect_true(dir.exists(new_dir))

  setwd(new_dir)

  # Move files
  file.copy(
    system.file("examples/DESCRIPTION_basic", package = "cffr"),
    to = "DESCRIPTION"
  )

  # Get bibentry
  a_bib <- as_bibentry()

  # Revert to initial wd
  setwd(current_dir)

  expect_snapshot(toBibtex(a_bib))

  unlink(new_dir, recursive = TRUE, force = TRUE)
})

test_that("as_bibentry character", {
  skip_on_cran()
  skip_if_not_installed("rmarkdown")

  base <- as_bibentry("rmarkdown")

  expect_s3_class(base, "bibentry")

  expect_length(base, 1)

  refs <- as_bibentry("rmarkdown", what = "references")
  expect_s3_class(refs, "bibentry")

  expect_gte(length(refs), 1)

  all <- as_bibentry("rmarkdown", what = "all")
  expect_s3_class(all, "bibentry")

  expect_length(all, length(base) + length(refs))

  # If an invented package
  expect_snapshot(as_bibentry("invented_package"), error = TRUE)

  # If another kind of file
  f <- system.file("examples/DESCRIPTION_basic", package = "cffr")
  s <- as_bibentry(f)
  expect_s3_class(s, "bibentry")
})

test_that("as_bibentry cff", {
  f <- system.file("examples/CITATION_basic.cff", package = "cffr")

  a_cff <- cff_read(f)
  theref <- a_cff$references[[1]]
  a_cff <- cff_modify(a_cff, references = NULL)
  expect_null(a_cff$references)

  expect_snapshot(end <- as_bibentry(a_cff, what = "references"))
  expect_identical(end, bibentry())

  expect_s3_class(theref, c("cff_ref", "cff"), exact = TRUE)
  mod_ref <- cff_modify(
    theref,
    type = "manual",
    authors = as_cff_person("{anonymous}")
  )

  expect_snapshot(toBibtex(as_bibentry(mod_ref)))
})

# Coerce citation from BibTeX ----

test_that("Article", {
  bib <- bibentry(
    "Article",
    key = "knuth:1984",
    author = person("R Core Team"),
    title = "Literate Programming",
    journal = "The Computer Journal",
    year = "1984",
    # Optional
    volume = "27",
    number = 2,
    pages = "97--111",
    month = "January",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- sort(names(unclass(bib)[[1]]))
  fld2 <- sort(names(unclass(tobib)[[1]]))
  expect_identical(fld1, fld2)
})

test_that("Book", {
  bib <- bibentry(
    "Book",
    key = "latex:companion",
    author = "Frank Mittelbach and Michel Gossens
            and Johannes Braams and David Carlisle
            and Chris Rowley",
    editor = "{Barnes and Noble}",
    title = "The LaTeX Companion",
    publisher = "Addison-Wesley Professional",
    year = "2004",
    # Optional
    volume = "3",
    number = 7,
    series = "The LateX Books",
    address = "Santa Monica",
    edition = "Fourth",
    month = "August",
    note = "Example modified for testing purposes",
    keywords = c("Two, keyword")
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- sort(names(unclass(bib)[[1]]))
  fld2 <- sort(names(unclass(tobib)[[1]]))
  expect_identical(fld1, fld2)
})


test_that("Booklet", {
  bib <- bibentry(
    "Booklet",
    key = "Mustermann2016",
    title = "Java Booklet",
    # Optional
    author = "Max Mustermann",
    howpublished = "Internet",
    address = "Stuttgart",
    month = "feb",
    year = "2016",
    note = "Example modified from Jabref",
    keywords = "java"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- sort(names(unclass(bib)[[1]]))
  fld2 <- sort(names(unclass(tobib)[[1]]))

  # Keyword is not coerced
  expect_identical(setdiff(fld1, fld2), "keywords")
})

test_that("Conference", {
  bib <- bibentry(
    "InProceedings",
    key = "inproceedings-full",
    author = "Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis",
    title = "On Notions of Information Transfer in {VLSI} Circuits",
    booktitle = "Proc. Fifteenth Annual ACM STOC",
    year = "1983",
    # Optional
    editor = "Wizard V. Oz and Mihalis Yannakakis",
    volume = "41",
    number = 17,
    series = "All ACM Conferences",
    pages = "133--139",
    address = "Boston",
    organization = "ACM",
    publisher = "Academic Press",
    note = "Example modified for testing purposes"
  )

  # Hack to convert to conference
  bib_un <- unclass(bib)[[1]]

  attr(bib_un, "bibtype") <- "Conference"

  bib <- list(bib_un)
  class(bib) <- "bibentry"

  bib_cff <- as_cff(bib)

  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "series")
})

test_that("InBook", {
  bib <- bibentry(
    "InBook",
    key = "1326",
    author = "A. King",
    editor = "E Tick and G Succi",
    title = "A Framework for Freeness Analysis",
    chapter = "7, 14",
    pages = "137--149",
    publisher = "Kluwer Academic Publishers",
    year = "1994",
    # Optional
    volume = "27",
    number = 2,
    series = "Implementations of Logic Programming Systems",
    type = "A chapter",
    address = "Dordrecht",
    edition = "Second",
    month = "January",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))
  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "type")
})

test_that("InCollection", {
  bib <- bibentry(
    "InCollection",
    key = "Mihalcea:2006",
    author = "Rada Mihalcea",
    title = "Knowledge-Based Methods for {WSD}",
    booktitle = "Word Sense Disambiguation: Algorithms
               and Applications",
    publisher = "Springer",
    year = "2006",
    # Optional
    editor = "Eneko Agirre and Philip Edmonds",
    volume = "23",
    number = 3,
    series = "The Word Sense Series",
    type = "A random type",
    chapter = "1,2,3",
    pages = "24--57",
    address = "107--132",
    edition = "Third",
    month = "August",
    note = "A note"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), c("series", "type"))
})

test_that("InProceedings", {
  bib <- bibentry(
    "InProceedings",
    key = "inproceedings-full",
    author = "Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis",
    title = "On Notions of Information Transfer in {VLSI} Circuits",
    booktitle = "Proc. Fifteenth Annual ACM STOC",
    year = "1983",
    # Optional
    editor = "Wizard V. Oz and Mihalis Yannakakis",
    volume = "41",
    number = 17,
    series = "All ACM Conferences",
    pages = "133--139",
    address = "Boston",
    organization = "ACM",
    publisher = "Academic Press",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "series")

  # If we remove collection title use conference
  bib_cff[[1]]$`collection-title` <- NULL
  bib_cff[[1]]$conference$name <- "I Am a conference"
  bib <- as_bibentry(bib_cff)
  expect_snapshot(toBibtex(bib))
})

test_that("Manual", {
  bib <- bibentry(
    "Manual",
    title = "A Language and Environment for Statistical Computing",
    # Optional
    author = person("R Core Team"),
    organization = "R Foundation for Statistical Computing",
    address = "Vienna, Austria",
    edition = "Fourth",
    month = "August",
    year = "2021",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(fld1, fld2)
})

test_that("MastersThesis", {
  bib <- bibentry(
    "MastersThesis",
    key = "Master2006",
    author = "Eesa Alsolami",
    title = "An examination of keystroke dynamics
                for continuous user authentication",
    school = "Queensland University of Technology",
    year = "2012",
    # Optional
    type = "Final thesis",
    address = "Queensland, NZ",
    month = "August",
    note = "Example modified for testing purposes"
  )
  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "type")
})

test_that("Misc", {
  bib <- bibentry(
    "Misc",
    # Optional
    author = person("R Core Team"),
    title = "A Language and Environment for Statistical Computing",
    howpublished = "CD-ROM",
    month = 1,
    year = "2021",
    note = "A note"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(fld1, fld2)
})


test_that("PhdThesis", {
  bib <- bibentry(
    "PhdThesis",
    author = "Eesa Alsolami",
    title = "An examination of keystroke dynamics
                for continuous user authentication",
    school = "Queensland University of Technology",
    year = "2012",
    # Optional
    type = "Final PhD thesis",
    address = "Queensland, NZ",
    month = "August",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "type")
})


test_that("Proceedings", {
  bib <- bibentry(
    "Proceedings",
    title = "Proc. Fifteenth Annual STOC",
    year = "1983",
    # Optional
    editor = "Wizard V. Oz and Mihalis Yannakakis",
    volume = 1,
    number = 17,
    series = "All ACM Conferences",
    address = "Boston, US",
    month = "August",
    organization = "The OX Association for Computing Machinery",
    publisher = "Academic Press",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(fld1, fld2)
})

test_that("TechReport", {
  bib <- bibentry(
    "TechReport",
    author = "Matthew C. Jadud and Sally A. Fincher",
    title = "Naive tools for studying compilation histories",
    institution = "University of Kent Canterbury",
    year = "2003",
    # Optional
    type = "techreport",
    number = "3-03",
    address = paste(
      "Computing Laboratory, University of Kent,",
      "Canterbury, Kent, CT2 7NF"
    ),
    month = "mar",
    note = "Example modified for testing purposes"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "type")
})

test_that("Unpublished", {
  bib <- bibentry(
    "Unpublished",
    author = "D. Kaplan",
    title = "Demonstratives",
    note = "Unpublished manuscript, UCLA",
    # Optional
    year = "1977",
    month = "aug",
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(fld1, fld2)

  # With custom note
  bib_cff[[1]]$notes <- NULL
  bib <- as_bibentry(bib_cff)
  expect_snapshot(toBibtex(bib))
})

test_that("InBook with booktitle", {
  bib <- bibentry(
    "InBook",
    title = "Bibliographies and citations",
    year = "2020",
    author = "Yihui Xie and Christophe Dervieux and Emily Riederer",
    booktitle = "{R} Markdown Cookbook",
    publisher = "Chapman and Hall/CRC",
    address = "Boca Raton, Florida",
    series = "The {R} Series",
    isbn = "9780367563837",
    url = "https://yihui.org/rmarkdown-cookbook/",
    chapter = "4.5"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Should be an incollection now
  res <- as_bibentry(bib_cff)
  init_type <- attr(unclass(res)[[1]], "bibtype")
  expect_identical(tolower(init_type), "incollection")

  # Back to bibtex and check names
  tobib <- as_bibentry(bib_cff)

  fld1 <- unique(sort(names(unclass(bib)[[1]])))
  fld2 <- unique(sort(names(unclass(tobib)[[1]])))

  expect_identical(setdiff(fld1, fld2), "series")
})

test_that("Test entry without author", {
  bib <- bibentry(
    "Proceedings",
    editor = "Yolande Berbers and Willy Zwaenepoel",
    title = "Proceedings of the 6th European Conference on Computer Systems",
    booktitle = paste(
      "Proceedings of the 6th European",
      "
                      Conference on Computer Systems"
    ),
    publisher = "ACM",
    venue = "Leuven, Belgium",
    month = "apr",
    year = 2006,
    isbn = "1-59593-322-02",
  )

  bib_cff <- as_cff(bib)

  expect_identical(
    bib_cff[[1]]$authors[[1]]$name,
    "anonymous"
  )

  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))
})


test_that("Test entry without author but has a key", {
  bib <- bibentry(
    "Misc",
    key = "I am the key",
    title = "Proceedings of the 6th European Conference on Computer Systems",
    booktitle = paste(
      "Proceedings of the 6th European Conference",
      "on Computer Systems"
    ),
    publisher = "ACM",
    venue = "Leuven, Belgium",
    month = "apr",
    year = 2006,
    isbn = "1-59593-322-02",
  )

  bib_cff <- as_cff(bib)

  expect_identical(
    bib_cff[[1]]$authors[[1]]$name,
    "anonymous"
  )

  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))
})


test_that("Test entry without author and key", {
  bib <- bibentry(
    "Misc",
    title = "Proceedings of the 6th European Conference on Computer Systems",
    booktitle = paste(
      "Proceedings of the 6th European",
      "
                      Conference on Computer Systems"
    ),
    publisher = "ACM",
    venue = "Leuven, Belgium",
    month = "apr",
    year = 2006,
    isbn = "1-59593-322-02",
  )

  bib_cff <- as_cff(bib)

  expect_identical(
    bib_cff[[1]]$authors[[1]]$name,
    "anonymous"
  )

  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))
})


test_that("Skip misc without title", {
  bib <- bibentry(
    bibtype = "misc",
    author = c(person("SHERPA/RoMEO")),
    url = "http://www.sherpa.ac.uk/romeo/",
    year = 2018
  )

  expect_message(bib_cff <- as_cff(bib), "Skipping")

  expect_null(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_snapshot(cffobj)

  expect_true(cff_validate(cffobj, verbose = FALSE))
})


test_that("Skip misc without title, not skipping the good one", {
  bib <- c(
    bibentry(
      bibtype = "misc",
      author = c(person("SHERPA/RoMEO")),
      url = "http://www.sherpa.ac.uk/romeo/",
      year = 2018
    ),
    bibentry(
      bibtype = "misc",
      title = "{rromeo}: An {R} Client for {SHERPA/RoMEO} {API}",
      author = c(person("Matthias", "Grenié"), person("Hugo", "Gruson")),
      year = 2019,
      header = "To cite this package in publications, please use:",
      url = "https://CRAN.R-project.org/package=rromeo"
    )
  )

  expect_message(bib_cff <- as_cff(bib), "SHERPA/RoMEO")

  expect_length(bib_cff, 1)

  expect_s3_class(bib_cff[[1]], "cff")

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_snapshot(cffobj)

  expect_equal(
    cffobj$references[[1]]$title,
    "rromeo: An R Client for SHERPA/RoMEO API"
  )

  expect_true(cff_validate(cffobj, verbose = FALSE))
})


test_that("Check extended BibLatex Fields", {
  bib <- bibentry(
    "Article",
    author = "M. A. Kastenholz, and Philippe H. Hünenbergerb",
    title = "Computation of methodology hyphen independent ionic solvation
                  free energies from molecular simulations",
    journal = "J. Chem. Phys.",
    year = 2006,
    note = "Example modified for testing purposes",
    pages = "55--65",

    # Additional BibLatex Fields
    date = "2006-03-15",
    file = "a_file.pdf",
    issuetitle = "Semantic {3D} Media and Content",
    translator = "Wicksteed, P. H. and Cornford, F. M.",
    urldate = "2006-10-01",
    pagetotal = 528,
    abstract = "The computation of ionic solvation free energies from
                  atomistic simulations is a surprisingly difficult problem that
                  has found no satisfactory solution for more than 15 years.",
    doi = "10.1063/1.2172593",
    isbn = "0-816-52066-6",
    issn = "0097-8493",
    url = "http://www.ctan.org"
  )

  bib_cff <- as_cff(bib)
  expect_snapshot(bib_cff)

  cffobj <- cff_create(cff(), keys = list(references = bib_cff))

  expect_true(cff_validate(cffobj, verbose = FALSE))
})

test_that("Duplicate entries", {
  bib <- bibentry(
    "InBook",
    title = "Bibliographies and citations",
    year = "2020",
    author = "Yihui Xie and Christophe Dervieux and Emily Riederer",
    booktitle = "{R} Markdown Cookbook",
    publisher = "Chapman and Hall/CRC",
    address = "Boca Raton, Florida",
    series = "The {R} Series",
    isbn = "9780367563837",
    url = "https://yihui.org/rmarkdown-cookbook/",
    chapter = "4.5"
  )

  expect_snapshot(bib_cff <- as_cff(rep(bib, 2)))
  expect_length(bib_cff, 1)
})

test_that("Identifiers and dois", {
  bib <- bibentry(
    bibtype = "Manual",
    title = "A Language and Environment for Statistical Computing",
    year = "2022",
    year = "2023",
    author = person("R Core Team"),
    version = NULL,
    error = "",
    url = "https://www.R-project.org/",
    url = "https://google.com/",
    doi = "10.5281/zenodo.5366600",
    doi = "10.5281/zenodo.5366601",
    doi = "10.5281/zenodo.5366602",
    identifiers = "a,b"
  )

  bib_cff <- as_cff(bib)
  sin <- bib_cff[[1]]
  checf <- sin[names(sin) %in% c("url", "doi", "identifiers")]
  expect_snapshot(as_cff(checf))

  cffobj <- cff_modify(cff(), references = bib_cff)

  expect_snapshot(cffobj)
  expect_true(cff_validate(cffobj, verbose = FALSE))
})

test_that("Test R 2026", {
  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")

  # https://github.com/wch/r-source/blob/trunk/src/library/base/inst/CITATION

  bib <- bibentry(
    "Manual",
    title = "R: A Language and Environment for Statistical Computing",
    author = person("R Core Team", comment = c(ROR = "02zz1nj61")),
    organization = person(
      "R Foundation for Statistical Computing",
      comment = c(ROR = "05qewa988")
    ),
    address = "Vienna, Austria",
    year = 2026,
    doi = "10.32614/R.manuals",
    url = "https://www.R-project.org/"
  )

  bib_cff <- as_cff(bib)
  sin <- bib_cff[[1]]
  expect_snapshot(as_cff(sin))

  cffobj <- cff_modify(cff(), references = bib_cff)

  expect_snapshot(cffobj)
  expect_true(cff_validate(cffobj, verbose = FALSE))
})

# See ´
# https://github.com/citation-file-format/ruby-cff/tree/main/test/files

test_that("preferred-citation-book-missing", {
  x <- system.file(
    "examples/preferred-citation-book-missing.cff",
    package = "cffr"
  )

  expect_snapshot(s <- as_bibentry(x))
})

test_that("preferred-citation-book", {
  x <- system.file("examples/preferred-citation-book.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-conference-paper-2", {
  x <- system.file(
    "examples/preferred-citation-conference-paper-2.cff",
    package = "cffr"
  )

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-conference-paper-missing", {
  x <- system.file(
    "examples/preferred-citation-conference-paper-missing.cff",
    package = "cffr"
  )

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-conference-paper", {
  x <- system.file(
    "examples/preferred-citation-conference-paper.cff",
    package = "cffr"
  )

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-manual", {
  x <- system.file("examples/preferred-citation-manual.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-no-month", {
  x <- system.file("examples/preferred-citation-no-month.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-no-vol", {
  x <- system.file("examples/preferred-citation-no-vol.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-pamphlet", {
  x <- system.file("examples/preferred-citation-pamphlet.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-report-no-institution", {
  x <- system.file(
    "examples/preferred-citation-report-no-institution.cff",
    package = "cffr"
  )

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("preferred-citation-report", {
  x <- system.file("examples/preferred-citation-report.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})


test_that("preferred-citation-unpublished", {
  x <- system.file(
    "examples/preferred-citation-unpublished.cff",
    package = "cffr"
  )

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("reprozip", {
  x <- system.file("examples/reprozip.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})


test_that("smith-et-al", {
  x <- system.file("examples/smith-et-al.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("tidyverse-joss-paper", {
  x <- system.file("examples/tidyverse-joss-paper.cff", package = "cffr")

  bib <- as_bibentry(x)
  expect_snapshot(toBibtex(bib))
})

test_that("Test in mock package", {
  skip_on_cran()
  current_dir <- getwd()

  name <- paste0("mock-pack", runif(1) * 10)
  new_dir <- file.path(tempdir(), name)

  dir.create(new_dir, recursive = TRUE)

  expect_true(dir.exists(new_dir))

  setwd(new_dir)

  # Move files
  file.copy(
    system.file("examples/DESCRIPTION_many_urls", package = "cffr"),
    to = "DESCRIPTION"
  )

  # Create a cff
  expect_silent(cff_write(verbose = FALSE))
  expect_false(file.exists("inst/CITATION"))

  # Create a cff also with auto_citation
  expect_silent(cff_write(verbose = FALSE, r_citation = TRUE))
  expect_true(file.exists("inst/CITATION"))

  auto_cit1 <- utils::readCitationFile(
    "inst/CITATION",
    meta = list(Encoding = "UTF-8")
  )

  # Same but with verbose
  ff <- cff_write(verbose = TRUE, r_citation = TRUE)

  # No backup

  expect_length(list.files("inst"), 1)

  auto_cit2 <- utils::readCitationFile(
    "inst/CITATION",
    meta = list(Encoding = "UTF-8")
  )

  expect_identical(auto_cit1, auto_cit2)

  # Update version and re-check

  d <- desc::desc("DESCRIPTION")

  expect_message(d$bump_version("major"))
  d$write("DESCRIPTION")

  # Check new vers
  cff_write(verbose = FALSE, r_citation = TRUE)

  auto_cit3 <- utils::readCitationFile(
    "inst/CITATION",
    meta = list(Encoding = "UTF-8")
  )

  # Clean
  unlink("inst", recursive = TRUE, force = TRUE)
  expect_false(dir.exists("inst"))

  # Get bibentry
  a_bib <- as_bibentry()
  # Create citation
  cit <- utils::readCitationFile(
    system.file("examples/CITATION_basic", package = "cffr"),
    meta = list(Encoding = "UTF-8")
  )

  expect_silent(cff_write_citation(cit, "./inst/CITATION", verbose = FALSE))
  expect_true(file_exist_abort("./inst/CITATION"))

  # Create Rbuildignore
  file.create(".Rbuildignore", showWarnings = FALSE)
  expect_true(file_exist_abort(".Rbuildignore"))

  # Add action
  expect_message(
    expect_message(
      expect_message(cff_gha_update(), "Creating directory"),
      "Installing"
    ),
    "Adding"
  )

  expect_message(cff_gha_update(), "already installed")
  expect_message(cff_gha_update(overwrite = TRUE), "Installing")

  expect_true(file_exist_abort(file.path(
    ".github",
    "workflows",
    "update-citation-cff.yaml"
  )))

  cffobj <- cff_create()

  expect_output(cff_write())

  expect_true(file_exist_abort("CITATION.cff"))

  expect_true(cff_validate("CITATION.cff", verbose = FALSE))

  ignore <- readLines(".Rbuildignore")

  expect_true(("^CITATION\\.cff$" %in% ignore))
  expect_true(("^\\.github$" %in% ignore))

  # Check citation from package
  cit <- utils::readCitationFile(
    "./inst/CITATION",
    meta = list(Encoding = "UTF-8")
  )

  # Revert to initial wd
  setwd(current_dir)

  unlink(new_dir, recursive = TRUE, force = TRUE)

  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")

  expect_false(identical(auto_cit1, cit))
  expect_true(identical(auto_cit1, auto_cit2))
  expect_snapshot(auto_cit1)
  expect_snapshot(auto_cit3)
  expect_snapshot(cffobj)
  expect_snapshot(toBibtex(cit))
  expect_snapshot(toBibtex(a_bib))
})

# Utils for authors----
make_r_person <- function(x) {
  checknames <- grepl("^name$|given-names|family-names", names(x))
  if (!isTRUE(any(checknames))) {
    return(person())
  }
  # Prepare list
  x <- unclass(x)

  # Family is special key
  # R Core uses for entity 'given', so do I
  # see in citation()
  is_entity <- any(grepl("^name$", names(x)))
  if (is_entity) {
    given <- clean_str(x[["name"]])
    # Extra protect
    family <- NULL
  } else {
    given <- clean_str(x[["given-names"]])
    family <- paste(
      clean_str(x[["name-particle"]]),
      clean_str(x[["family-names"]])
    )
    # Add suffix with comma
    family <- paste0(c(family, clean_str(x[["name-suffix"]])), collapse = ", ")
    family <- clean_str(family)
  }

  role <- clean_str(x$role)

  # Make comments
  x_comments <- x[
    !names(x) %in%
      c(
        "family-names",
        "given-names",
        "name",
        "name-particle",
        "name-suffix",
        "email",
        "role"
      )
  ]

  x_comments <- lapply(x_comments, clean_str)
  x_comments <- unlist(x_comments, use.names = TRUE)

  # Prepare ORCID
  x_comments <- gsub("^https://orcid.org/", "", x_comments)
  x_comments <- gsub("^http://orcid.org/", "", x_comments)
  nm <- gsub("orcid", "ORCID", names(x_comments), fixed = TRUE)
  names(x_comments) <- nm

  pers_lst <- list(
    given = given,
    family = family,
    role = role,
    email = clean_str(x$email),
    comment = x_comments
  )

  do.call(person, pers_lst)
}

#' Read an external file as a [`cff`] object
#'
#' @description
#' Read files and convert them to [`cff`] objects. Files supported
#' are:
#' - `CITATION.cff` files.
#' - `DESCRIPTION` files.
#' - **R** citation files (usually located in `inst/CITATION`).
#' - BibTeX files (with extension `*.bib`).
#'
#' [cff_read()] attempts to guess the type of file provided in `path`. However,
#' we provide aliases for each specific file type:
#' - [cff_read_cff_citation()], which uses [yaml::read_yaml()].
#' - [cff_read_description()], which uses [desc::desc()].
#' - [cff_read_citation()], which uses [utils::readCitationFile()].
#' - [cff_read_bib()], which requires \CRANpkg{bibtex} (>= 0.5.0) and uses
#'   [bibtex::read.bib()].
#'
#' @export
#' @encoding UTF-8
#' @rdname cff_read
#' @family reading
#' @seealso
#'
#' The underlying functions used for reading external files:
#' - [yaml::read_yaml()] for `CITATION.cff` files.
#' - [desc::desc()] for `DESCRIPTION` files.
#' - [utils::readCitationFile()] for **R** citation files.
#' - [bibtex::read.bib()] for BibTeX files (extension `*.bib`).
#'
#' @param path Path to a file.
#' @param encoding Encoding to be assumed for `path`. See [readLines()].
#' @param meta A list of package metadata as obtained by
#'   [utils::packageDescription()] or `NULL` (the default). See **Details**.
#' @param ... Arguments to be passed to other functions, for example to
#'   [yaml::read_yaml()] or [bibtex::read.bib()].
#'
#' @inheritParams cff_create
#'
#' @return
#'
#' * `cff_read_cff_citation()` and `cff_read_description()` return an object
#'   with class `cff`.
#' * `cff_read_citation()` and `cff_read_bib()` return an object of classes
#'   [`cff_ref_lst, cff`][cff_ref_lst] according to the `definitions.references`
#'   specified in the
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [Citation File Format schema]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#'
#' Learn more about the \CRANpkg{cffr} class system in [cff_class].
#'
#' @references
#'
#' - R Core Team (2023). _Writing R Extensions_.
#'   <https://cran.r-project.org/doc/manuals/r-release/R-exts.html>
#'
#' - Hernangomez D (2022). "BibTeX and CFF, a potential crosswalk."
#'   *The cffr package, Vignettes*. \doi{10.21105/joss.03900},
#'   <https://docs.ropensci.org/cffr/articles/bibtex-cff.html>.
#'
#' @details
#'
#' For details of `cff_read_description()` see [cff_create()].
#'
#' ## The `meta` object
#'
#' Section 1.9 CITATION files of *Writing R Extensions* (R Core Team 2023)
#' specifies how to create dynamic `CITATION` files using `meta` object, hence
#' the `meta` argument in [cff_read_citation()] may be needed for reading
#' some files correctly.
#'
#' @examples
#'
#' # Create cff object from cff file
#'
#' from_cff_file <- cff_read(system.file("examples/CITATION_basic.cff",
#'   package = "cffr"
#' ))
#'
#' head(from_cff_file, 7)
#'
#' # Create cff object from DESCRIPTION
#' from_desc <- cff_read(system.file("examples/DESCRIPTION_basic",
#'   package = "cffr"
#' ))
#'
#' from_desc
#'
#' # Create cff object from BibTex
#'
#' if (requireNamespace("bibtex", quietly = TRUE)) {
#'   from_bib <- cff_read(system.file("examples/example.bib",
#'     package = "cffr"
#'   ))
#'
#'   # First item only
#'   from_bib[[1]]
#' }
#' # Create cff object from CITATION
#' from_citation <- cff_read(system.file("CITATION", package = "cffr"))
#'
#' # First item only
#' from_citation[[1]]
#'
cff_read <- function(path, ...) {
  if (length(path) > 1) {
    cli::cli_abort(
      "Use a single value, {.arg path} has length {.val {length(path)}}"
    )
  }

  file_exist_abort(path, abort = TRUE)
  filetype <- detect_x_source(path)

  if (filetype == "dontknow") {
    cli::cli_abort(
      paste0(
        "Don't recognize the file type of {.file {path}}.",
        " Use a specific function (e.g. {.fn cffr:cff_read_description}"
      )
    )
  }

  endobj <- switch(filetype,
    "cff_citation" = cff_read_cff_citation(path, ...),
    "description" = cff_read_description(path, ...),
    "bib" = cff_read_bib(path, ...),
    "citation" = cff_read_citation(path, ...),
    cli::cli_abort("Don't know how to read {.val {x}}.")
  )

  endobj
}

#' @export
#' @encoding UTF-8
#' @rdname cff_read
cff_read_cff_citation <- function(path, ...) {
  file_exist_abort(path, abort = TRUE)

  cffobj <- yaml::read_yaml(
    path,
    ...,
    # Read languages always as list (#105)
    handlers = list(map = function(x) {
      if (length(x$languages) == 1) {
        x$languages <- list(x$languages)
      }
      x
    })
  )
  new_cff(cffobj)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_read
cff_read_description <- function(
  path,
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  authors_roles = c("aut", "cre"),
  ...
) {
  file_exist_abort(path, abort = TRUE)

  pkg <- desc::desc(path)
  pkg$coerce_authors_at_r()

  msg <- paste0(
    'To cite package "',
    pkg$get("Package"),
    '" in publications use:'
  )

  field_list <- list(
    "cff-version" = cff_version,
    message = msg,
    type = "software",
    title = get_desc_title(pkg),
    version = get_desc_version(pkg),
    authors = get_desc_authors(pkg, authors_roles = authors_roles),
    abstract = get_desc_abstract(pkg),
    repository = get_desc_repository(pkg),
    "repository-code" = get_desc_urls(pkg)$repo,
    url = get_desc_urls(pkg)$url,
    identifiers = get_desc_urls(pkg)$identifiers,
    "date-released" = get_desc_date_released(pkg),
    contact = get_desc_contacts(pkg),
    keywords = get_desc_keywords(pkg),
    license = unlist(get_desc_license(pkg)),
    commit = get_desc_sha(pkg),
    doi = get_desc_doi(pkg)
  )

  if (gh_keywords) {
    ghtopics <- get_gh_topics(field_list)
    field_list$keywords <- unique(c(field_list$keywords, ghtopics))
  }

  new_cff(field_list)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_read
cff_read_citation <- function(path, meta = NULL, ...) {
  file_exist_abort(path, abort = TRUE)

  if (!any(is.null(meta), inherits(meta, "packageDescription"))) {
    # nolint start
    # Object for cli only
    ex <- packageDescription("cffr")
    # nolint end

    cli::cli_alert_warning(
      paste0(
        "{.arg meta} should be {.val NULL} or {.obj_type_friendly {ex}}",
        " not {.obj_type_friendly {meta}}. Using {.arg meta = NULL}"
      )
    )
    meta <- NULL
  }

  new_meta <- clean_package_meta(meta)
  the_cit <- try(utils::readCitationFile(path, meta = new_meta), silent = TRUE)

  # If error then new try
  if (inherits(the_cit, "try-error")) {
    cli::cli_alert_warning(
      paste0(
        "It was not possible to read {.file {path}} with the {.arg meta} ",
        "provided. Trying with {.code packageDescription('base')}"
      )
    )
    new_meta <- packageDescription("base")
    the_cit <- try(
      utils::readCitationFile(path, meta = new_meta),
      silent = TRUE
    )
    # nocov start
    if (inherits(the_cit, "try-error")) {
      cli::cli_alert_danger(
        "Can't read {.file path}, returning {.val NULL}"
      )
      return(NULL)
    }
    # nocov end
  }
  tocff <- as_cff(the_cit)

  tocff
}

#' @export
#' @encoding UTF-8
#' @family bibtex
#' @rdname cff_read
cff_read_bib <- function(path, encoding = "UTF-8", ...) {
  file_exist_abort(path, abort = TRUE)

  # nocov start
  if (!requireNamespace("bibtex", quietly = TRUE)) {
    msg <- paste0(
      "{.pkg bibtex} package required for using this function: ",
      '{.run install.packages("bibtex")}'
    )
    cli::cli_abort(msg)
  }
  # nocov end

  # Read from tempfile
  read_bib <- bibtex::read.bib(file = path, encoding = encoding, ...)

  tocff <- as_cff(read_bib)
  tocff
}

# Internal safe ----
#' Internal version of cff_read_citation, safe
#' @noRd
cff_safe_read_citation <- function(desc_path, cit_path) {
  if (!file_exist_abort(cit_path) || !file_exist_abort(desc_path)) {
    return(NULL)
  }
  # Create meta
  meta <- desc_to_meta(desc_path)
  meta <- clean_package_meta(meta)

  the_cit <- try(utils::readCitationFile(cit_path, meta = meta), silent = TRUE)
  # Try
  if (inherits(the_cit, "try-error")) {
    return(NULL)
  }

  # Need to be named here
  tocff <- as_cff(the_cit)
  tocff
}

# See utils.R

test_that("Merge all DESCRIPTION files with CITATION_basic", {
  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")
  skip_on_cran()

  allfiles <- list.files(
    system.file("examples", package = "cffr"),
    pattern = "^DESC",
    full.names = TRUE
  )

  citpath <- system.file("examples/CITATION_basic", package = "cffr")
  for (i in seq_along(allfiles)) {
    desc_cff <- cff_read_description(allfiles[i], gh_keywords = FALSE)
    generate_cit <- cff_safe_read_citation(allfiles[i], citpath)
    merged <- merge_desc_cit(desc_cff, generate_cit)
    merged <- as_cff(merged)

    expect_snapshot(merged)

    expect_true(cff_validate(merged, verbose = FALSE))
  }
})

test_that("Check dependencies", {
  skip_on_cran()
  deps <- get_dependencies(system.file("DESCRIPTION", package = "cffr"))

  # Extract selected fields
  selected <- lapply(deps, function(x) {
    y <- x[names(x) %in% c("title", "url", "repository")]
    y
  })

  class(selected) <- "cff"
  expect_snapshot(print(selected))
})

test_that("Merge DESCRIPTION wrong url with CITATION_dx", {
  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")
  skip_on_cran()

  dd <- list.files(
    system.file("examples", package = "cffr"),
    pattern = "wrong",
    full.names = TRUE
  )

  citpath <- system.file("examples/CITATION_dx_doi", package = "cffr")
  desc_cff <- cff_read_description(dd, gh_keywords = FALSE)
  generate_cit <- cff_safe_read_citation(dd, citpath)
  merged <- merge_desc_cit(desc_cff, generate_cit)
  merged <- as_cff(merged)

  expect_true(cff_validate(merged, verbose = FALSE))
  expect_identical(
    paste0("https://dx.doi.org/", merged$doi),
    merged[["preferred-citation"]]$url
  )
})

test_that("as data frame complete", {
  path <- system.file("examples/CITATION_complete.cff", package = "cffr")

  the_cff <- cff_read(path)

  # To df
  the_df <- as.data.frame(the_cff)

  expect_s3_class(the_df, "data.frame")
  expect_equal(nrow(the_df), 1)

  # lengths
  allf <- unlist(the_cff, recursive = TRUE, use.names = FALSE)
  expect_identical(length(allf), ncol(the_df))

  expect_identical(ncol(the_df), length(unique(names(the_df))))

  # As vector
  df_as_v <- as.character(as.vector(the_df[1, ]))

  expect_identical(allf, df_as_v)
  expect_snapshot(names(the_df))
})

test_that("as data frame partial", {
  path <- system.file("examples/CITATION_complete.cff", package = "cffr")

  the_cff <- cff_read(path)

  # To df authors only
  subcff <- the_cff$authors[[1]]
  the_df <- as.data.frame(subcff)
  expect_s3_class(the_df, "data.frame")
  expect_equal(nrow(the_df), 1)
  # lengths
  allf <- unlist(subcff, recursive = TRUE, use.names = FALSE)
  expect_identical(length(allf), ncol(the_df))
  expect_identical(ncol(the_df), length(unique(names(the_df))))

  # To df references only
  subcff <- the_cff$references[[1]]
  the_df <- as.data.frame(subcff)
  expect_s3_class(the_df, "data.frame")
  expect_equal(nrow(the_df), 1)
  # lengths
  allf <- unlist(subcff, recursive = TRUE, use.names = FALSE)
  expect_identical(length(allf), ncol(the_df))
  expect_identical(ncol(the_df), length(unique(names(the_df))))
})

test_that("Convert a citation only", {
  path <- system.file("examples/DESCRIPTION_many_persons", package = "cffr")
  a_cit <- as_bibentry(cff_create(path))

  the_cff <- as_cff(a_cit)

  # To df
  the_df <- as.data.frame(the_cff)

  expect_s3_class(the_df, "data.frame")
  expect_equal(nrow(the_df), 1)

  # lengths
  allf <- unlist(the_cff, recursive = TRUE, use.names = FALSE)
  expect_identical(length(allf), ncol(the_df))

  expect_identical(ncol(the_df), length(unique(names(the_df))))

  # As vector
  df_as_v <- as.character(as.vector(the_df[1, ]))

  expect_identical(allf, df_as_v)
  expect_snapshot(names(the_df))
})


test_that("Convert authors only", {
  a_pers_lst <- as_cff_person(
    "A person and {A Entity inc.} and {One person} more"
  )

  the_cff <- a_pers_lst[[1]]

  # To df
  the_df <- as.data.frame(the_cff)

  expect_s3_class(the_df, "data.frame")
  expect_equal(nrow(the_df), 1)

  # lengths
  allf <- unlist(the_cff, recursive = TRUE, use.names = FALSE)
  expect_identical(length(allf), ncol(the_df))

  expect_identical(ncol(the_df), length(unique(names(the_df))))

  # As vector
  df_as_v <- as.character(as.vector(the_df[1, ]))

  expect_identical(allf, df_as_v)
  expect_snapshot(names(the_df))
})

test_that("Convert list of authors", {
  path <- system.file("examples/CITATION_complete.cff", package = "cffr")

  the_cff <- cff_read(path)$authors

  # To df
  the_df <- as.data.frame(the_cff)

  expect_s3_class(the_df, "data.frame")
  expect_equal(nrow(the_df), 1)

  # lengths
  allf <- unlist(the_cff, recursive = TRUE, use.names = FALSE)
  expect_identical(length(allf), ncol(the_df))

  expect_identical(ncol(the_df), length(unique(names(the_df))))

  # As vector
  df_as_v <- as.character(as.vector(the_df[1, ]))

  expect_identical(allf, df_as_v)
  expect_snapshot(names(the_df))
})


test_that("as.person method", {
  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")
  skip_on_cran()
  path <- system.file("examples/CITATION_complete.cff", package = "cffr")

  the_cff <- cff_read(path)

  getref <- the_cff$references[[1]]

  # Single entity
  pub <- as.person(getref$publisher)
  expect_s3_class(pub, "person")

  expect_snapshot(dput(pub))
  expect_snapshot(
    format(pub, include = c("given", "family", "email", "role", "comment"))
  )

  # Single person
  aut <- as.person(getref$authors[[1]])
  expect_s3_class(aut, "person")
  expect_snapshot(dput(aut))
  expect_snapshot(
    format(aut, include = c("given", "family", "email", "role", "comment"))
  )

  # List of authors
  aut2 <- as.person(getref$authors)

  expect_s3_class(aut2, "person")
  expect_s3_class(aut2[1], "person")
  expect_s3_class(aut2[2], "person")
  expect_snapshot(dput(aut2))

  # Malformed
  malf <- getref$authors
  malf[[1]] <- list(a = "list")
  expect_s3_class(malf, "cff_pers_lst")

  expect_snapshot(end <- as.person(malf))
  expect_s3_class(end, "person")
  expect_length(end, 1)

  # Duplicates
  aa <- getref$authors
  aa[[3]] <- aa[[1]]
  expect_s3_class(aa, "cff_pers_lst")

  expect_snapshot(aa2 <- as.person(aa))
  expect_s3_class(aa2, "person")
  expect_length(aa2, 2)
})

test_that("as.person method names and particles", {
  skip_on_cran()
  str <- "von Wicksteed, III, P. H. and {The translator factory}"

  cf <- as_cff_person(str)
  expect_snapshot(cf)

  pers_bib <- toBibtex(as.person(cf))
  again <- as_cff_person(pers_bib)
  expect_identical(cf, again)
})

test_that("Errors on other as.person methods", {
  path <- system.file("examples/CITATION_complete.cff", package = "cffr")
  the_cff <- cff_read(path)
  expect_s3_class(the_cff, "cff", exact = TRUE)
  expect_snapshot(as.person(the_cff), error = TRUE)

  # identifiers
  key <- the_cff$identifiers
  expect_s3_class(key, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_snapshot(as.person(key), error = TRUE)

  # preferred
  key <- the_cff$`preferred-citation`
  expect_s3_class(key, c("cff_ref", "cff"), exact = TRUE)
  expect_snapshot(as.person(key), error = TRUE)
})

test_that("head and tail", {
  a_cff <- cff()
  expect_snapshot(a_cff)
  expect_snapshot(head(a_cff, 2))
  expect_s3_class(head(a_cff, 2), "cff")

  expect_snapshot(tail(a_cff, 2))
  expect_s3_class(tail(a_cff, 2), "cff")
})

test_that("toBibtex", {
  skip_on_cran()

  # Create several alternatives
  descobj <- cff_read_description(system.file(
    "examples/DESCRIPTION_basic",
    package = "cffr"
  ))

  citobj <- cff_read_citation(system.file(
    "examples/CITATION_basic",
    package = "cffr"
  ))
  newbib <- cff_read_bib(system.file("examples/example.bib", package = "cffr"))

  full_cff <- merge_desc_cit(descobj, c(newbib, citobj))
  full_cff <- new_cff(full_cff)
  expect_true(cff_validate(full_cff, verbose = FALSE))

  # A. validate extractions
  expect_snapshot(toBibtex(full_cff))
  expect_snapshot(toBibtex(full_cff, what = "references"))
  expect_snapshot(toBibtex(full_cff, what = "all"))

  # single entries
  single <- toBibtex(full_cff$`preferred-citation`)
  expect_s3_class(single, "Bibtex")
  expect_equal(sum(names(single) == "title"), 1)

  # Several entries
  several <- toBibtex(full_cff$references)
  expect_s3_class(several, "Bibtex")
  expect_equal(sum(names(several) == "title"), 3)

  # One entry
  oneent <- toBibtex(full_cff$references[[1]])
  expect_s3_class(single, "Bibtex")
  expect_equal(sum(names(oneent) == "title"), 1)

  fromfile <- toBibtex(newbib)
  expect_s3_class(fromfile, "Bibtex")
  expect_equal(sum(names(fromfile) == "title"), 2)

  # From lines
  string <- "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925}"

  froml <- toBibtex(cff_read_bib_text(string))
  expect_equal(sum(names(froml) == "title"), 1)

  # Persons

  sev_auth <- as_cff_person(
    "{The Big Bopper} and Frank Sinatra and Dean Martin and Davis, Jr., Sammy"
  )

  comp <- c(
    person("The Big Bopper"),
    person("Frank", "Sinatra"),
    person("Dean", "Martin"),
    person("Sammy", "Davis, Jr.")
  )

  expect_length(sev_auth, 4)
  expect_s3_class(sev_auth, "cff_pers_lst")
  expect_identical(toBibtex(sev_auth), toBibtex(comp))

  # Single person
  single <- as_cff_person(person("A", "person", email = "a@b.d"))[[1]]
  expect_s3_class(single, "cff_pers")
  expect_identical(toBibtex(single), toBibtex(person("A", "person")))

  # Single entity
  single <- as_cff_person(person("{A and B co}", email = "a@b.d"))[[1]]
  expect_s3_class(single, "cff_pers")
  expect_identical(
    toBibtex(single),
    toBibtex(person("A and B co", email = "a@b.d"))
  )
})


test_that("as.list", {
  f <- system.file("examples/CITATION_complete.cff", package = "cffr")

  full_cff <- cff_read_cff_citation(f)

  # Capture dput and search
  dput_cff <- capture.output(dput(full_cff))

  ntot_class <- sum(grepl("\"cff\"", dput_cff))

  expect_gt(ntot_class, 30)

  # Unlist
  unl <- as.list(full_cff)

  # Capture dput and search
  dput_unl <- capture.output(dput(unl))

  ntot_class_unl <- sum(grepl("\"cff\"", unl))

  expect_identical(ntot_class_unl, 0L)

  # Reclass
  regen_cff <- as_cff(unl)

  expect_identical(full_cff, regen_cff)

  expect_true(cff_validate(regen_cff, verbose = FALSE))
})

test_that("Validate full CITATION.cff", {
  full <- system.file("examples/CITATION_complete.cff", package = "cffr")

  # From object
  fcff <- cff_read(full)
  expect_snapshot(ok <- cff_validate(fcff))
  expect_true(ok)
  expect_null(attr(ok, "errors"))
  expect_silent(cff_validate(full, verbose = FALSE))

  # From file
  expect_true(cff_validate(full))
})

test_that("Validate minimal CITATION.cff", {
  full <- system.file("examples/CITATION_skeleton.cff", package = "cffr")
  expect_true(cff_validate(full, verbose = FALSE))
})

test_that("Validate error CITATION.cff", {
  err <- system.file("examples/CITATION_error.cff", package = "cffr")
  # From cff
  ferr <- cff_read(err)
  expect_snapshot(tab <- cff_validate(ferr))
  expect_false(tab)
  # Extract attr table
  df <- attr(tab, "errors")
  expect_s3_class(df, "data.frame")

  snap_fields <- vapply(df, class, FUN.VALUE = character(1))
  expect_snapshot(df[, snap_fields == "character"])

  expect_silent(cff_validate(err, verbose = FALSE))
})

test_that("Validate cffr objects from installed packages", {
  cffr <- cff_create("jsonlite")
  expect_true(cff_validate(cffr, verbose = FALSE))
})

test_that("Validate error for invalid input", {
  nocff <- system.file("CITATION", package = "cffr")
  expect_error(cff_validate(nocff))

  nofile <- system.file("examples/no_file_here", package = "cffr")
  expect_error(cff_validate(nofile))
})

test_that("File that is not cff", {
  expect_error(
    cff_validate(system.file("examples/DESCRIPTION_basic", package = "cffr"))
  )
})

test_that("Test message on cff", {
  expect_snapshot(def <- cff("abcde"))

  expect_identical(def, cff())
  nocff <- system.file("examples/CITATION_skeleton.cff", package = "cffR")
  expect_snapshot(afile <- cff(nocff))

  expect_identical(afile, cff())
})

test_that("Walk trough full lifecycle", {
  complete <- system.file("examples/CITATION_complete.cff", package = "cffr")

  # Read
  read <- cff_read(complete)
  expect_s3_class(read, "cff")
  expect_true(cff_validate(read, verbose = FALSE))
  expect_snapshot(read)

  # Modify
  modify <- cff_create(read, keys = list(title = "A new title"))
  expect_snapshot(modify)
  expect_true(length(read) == length(modify))
  expect_true(length((setdiff(names(read), names(modify)))) == 0)
  expect_false(read$title == modify$title)

  # Write
  tmp <- tempfile(fileext = ".cff")
  cff_write(modify, outfile = tmp, validate = FALSE, verbose = FALSE)
  expect_silent(file_exist_abort(tmp))

  # Validate
  expect_true(cff_validate(tmp, verbose = FALSE))

  file.remove(tmp)
})

test_that("Recursive parsing", {
  complete <- system.file("examples/CITATION_complete.cff", package = "cffr")

  # Read
  expect_snapshot(read <- cff(complete))
  read <- cff_read(complete)

  # Test all levels
  expect_s3_class(read, "cff")
  expect_s3_class(read$authors, "cff")
  expect_s3_class(read$authors[[1]], "cff")
  expect_s3_class(read$references, "cff")
  expect_s3_class(read$references[[1]]$authors, "cff")
  expect_s3_class(read$references[[1]]$authors[[1]], "cff")
})


test_that("Fuzzy matching of keys on cff", {
  expect_snapshot(cff(
    tittle = "a",
    cff_version = "ar",
    version = "200",
    messange = "Fix my keys"
  ))

  expect_snapshot(
    cffobj <- cff(
      tittle = "a",
      cff_version = "1.2.0",
      version = "200",
      messange = "aa",
      anthor = list(list(
        "family-names" = "a",
        "given-names" = "b"
      ))
    )
  )

  expect_true(is_cff(cffobj))
  expect_true(cff_validate(cffobj, verbose = FALSE))
})

test_that("duplicated", {
  expect_snapshot(
    ss <- cff(
      tittle = "a",
      tittle = "ar",
      version = "200",
      messange = "Fix my keys"
    )
  )

  expect_s3_class(ss, "cff")
  expect_length(ss, 3)
})

test_that("unnamed", {
  expect_snapshot(
    ss <- cff(
      path = "a",
      "200",
      "Fix my keys"
    ),
    error = TRUE
  )

  expect_snapshot(s1 <- cff(path = NULL, title = "a", "b", version = 1))
  expect_snapshot(
    s2 <- cff(
      path = NULL,
      title = "a",
      "aa",
      "bb",
      "cc",
      "b",
      version = 1,
      "h",
      "j"
    )
  )

  expect_identical(s1, s2)
})

test_that("Test errors on cff_read", {
  expect_snapshot(cff_read(c("abcde", "b")), error = TRUE)
  expect_snapshot(cff_read("abcde"), error = TRUE)

  f <- system.file("schema/schema.json", package = "cffr")
  expect_error(cff_read(f), "Don't recognize the file type of")
})

test_that("cff_read citation.cff", {
  expect_snapshot(cff_read_cff_citation("a"), error = TRUE)

  f <- system.file("examples/CITATION_complete.cff", package = "cffr")
  f1 <- cff_read(f)
  expect_true(cff_validate(f1, verbose = FALSE))
  expect_s3_class(f1, "cff", exact = TRUE)

  # With the alias
  f2 <- cff_read_cff_citation(f)
  expect_identical(f1, f2)
})

test_that("cff_read DESCRIPTION", {
  expect_snapshot(cff_read_description("a"), error = TRUE)

  f <- system.file("examples/DESCRIPTION_no_URL", package = "cffr")

  f1 <- cff_read(f, gh_keywords = FALSE)
  expect_true(cff_validate(f1, verbose = FALSE))
  expect_s3_class(f1, "cff", exact = TRUE)

  # With the alias
  f2 <- cff_read_description(f, gh_keywords = FALSE)
  expect_identical(f1, f2)

  # Use other params
  f1_1 <- cff_read(
    f,
    gh_keywords = FALSE,
    cff_version = 3000,
    authors_roles = c("aut", "cre", "ctb")
  )

  expect_equal(f1_1$`cff-version`, "3000")

  expect_gt(length(f1_1$authors), length(f1$authors))

  f2_1 <- cff_read_description(
    f,
    gh_keywords = FALSE,
    cff_version = 3000,
    authors_roles = c("aut", "cre", "ctb")
  )

  expect_identical(f1_1, f2_1)

  skip_on_cran()
  # With gh keywords
  f <- system.file(
    "examples/DESCRIPTION_posit_package_manager",
    package = "cffr"
  )
  fno <- cff_read_description(f, gh_keywords = FALSE)
  f2 <- cff_read_description(f, gh_keywords = TRUE)

  # In some instances keywords are not retrieved
  skip_if(is.null(f2$keywords), "keywords not gathered")

  expect_false(is.null(f2$keywords))
  expect_gt(length(f2$keywords), length(fno$keywords))
})


test_that("cff_read bib", {
  skip_if_not_installed("bibtex", "0.5.0")

  expect_snapshot(cff_read_bib("a"), error = TRUE)

  f <- system.file("REFERENCES.bib", package = "cffr")

  f1 <- cff_read(f)
  expect_s3_class(f1, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_gt(length(f1), 1)

  # Specific
  f2 <- cff_read_bib(f)
  expect_identical(f1, f2)

  # With encodings

  f <- system.file("examples/example.bib", package = "cffr")
  f1_2 <- cff_read(f)
  expect_s3_class(f1_2, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_length(f1_2, 2)

  d <- f1_2[[2]]

  expect_snapshot(d)
  f2_2 <- cff_read_bib(f)
  expect_identical(f1_2, f2_2)
})

test_that("cff_read citation messages", {
  expect_snapshot(cff_read_citation("a"), error = TRUE)

  f <- system.file("examples/CITATION_basic", package = "cffr")
  my_meta <- packageDescription("testthat")

  expect_snapshot(s <- cff_read(f, meta = "aa"))
  expect_silent(s <- cff_read(f, meta = NULL))
  expect_silent(s <- cff_read(f, meta = my_meta))

  # Now try with a file that needs meta
  f <- system.file("examples/CITATION_auto", package = "cffr")

  expect_message(s <- cff_read(f), "Trying with")
  expect_s3_class(s, c("cff_ref_lst", "cff"), exact = TRUE)
})

test_that("cff_read CITATION_basic", {
  a_desc <- system.file("examples/DESCRIPTION_basic", package = "cffr")
  my_meta <- desc_to_meta(a_desc)

  path <- system.file("examples/CITATION_basic", package = "cffr")
  a_cff <- cff_read(path, my_meta)
  expect_s3_class(a_cff, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_equal(length(a_cff), 2)
})

test_that("cff_read CITATION with no encoding", {
  desc_path <- system.file("examples/DESCRIPTION_no_encoding", package = "cffr")
  cit_path <- system.file("examples/CITATION_basic", package = "cffr")

  my_meta <- desc_to_meta(desc_path)
  a_cff <- cff_read_citation(cit_path, my_meta)
  expect_s3_class(a_cff, c("cff_ref_lst", "cff"), exact = TRUE)
})

test_that("cff_read CITATION_auto", {
  # Needs an installed package
  desc_path <- system.file("examples/DESCRIPTION_rgeos", package = "cffr")
  cit_path <- system.file("examples/CITATION_auto", package = "cffr")
  my_meta <- desc_to_meta(desc_path)

  a_cff <- cff_read(cit_path, my_meta)

  expect_equal(length(a_cff), 3)
})

test_that("cff_read CITATION_rmarkdown", {
  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")
  cit_path <- system.file("examples/CITATION_rmarkdown", package = "cffr")

  my_meta <- desc_to_meta(desc_path)
  a_cff <- cff_read(cit_path, my_meta)

  expect_equal(length(a_cff), 3)
})

test_that("cff_read_safe CITATION_basic", {
  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")
  cit_path <- system.file("examples/CITATION_basic", package = "cffr")
  a_cff <- cff_safe_read_citation(desc_path, cit_path)

  expect_s3_class(a_cff, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_equal(length(a_cff), 2)
})

test_that("cff_read_safe CITATION with no encoding", {
  desc_path <- system.file("examples/DESCRIPTION_no_encoding", package = "cffr")
  cit_path <- system.file("examples/CITATION_basic", package = "cffr")

  a_cff <- cff_safe_read_citation(desc_path, cit_path)

  expect_s3_class(a_cff, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_equal(length(a_cff), 2)
})

test_that("cff_read_safe CITATION_auto", {
  # Needs an installed package
  desc_path <- system.file("examples/DESCRIPTION_rgeos", package = "cffr")
  cit_path <- system.file("examples/CITATION_auto", package = "cffr")
  a_cff <- cff_safe_read_citation(desc_path, cit_path)

  expect_s3_class(a_cff, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_equal(length(a_cff), 3)
})

test_that("cff_read_safe CITATION_rmarkdown", {
  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")
  cit_path <- system.file("examples/CITATION_rmarkdown", package = "cffr")

  a_cff <- cff_safe_read_citation(desc_path, cit_path)

  expect_s3_class(a_cff, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_equal(length(a_cff), 3)
})


test_that("cff_read_safe CITATION NULL", {
  desc_path <- system.file("x", package = "cffr")
  cit_path <- system.file("y", package = "cffr")

  expect_null(cff_safe_read_citation(
    desc_path,
    cit_path
  ))
})

test_that("Corrupt CITATION", {
  tmp <- tempfile("CITATION")
  writeLines("I am a bad CITATION", tmp)
  expect_message(
    expect_message(anull <- cff_read(tmp), "It was not possible to read"),
    "Can't"
  )
  expect_null(anull)

  # Internal
  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")
  expect_silent(anull <- cff_safe_read_citation(desc_path = desc_path, tmp))

  expect_null(anull)
})

test_that("Creating cff from packages encoded in latin1", {
  # Surveillance package
  desc_path <- system.file(
    "examples/DESCRIPTION_surveillance",
    package = "cffr"
  )
  cit_path <- system.file("examples/CITATION_surveillance", package = "cffr")

  expect_true(desc::desc(desc_path)$get("Encoding") == "latin1")

  # Parse citation
  bib <- cff_safe_read_citation(desc_path, cit_path)

  expect_true("UTF-8" %in% Encoding(unlist(bib)))
  expect_false("latin1" %in% Encoding(unlist(bib)))

  # Create cff
  cffobj <- cff_create(
    desc_path,
    keys = list(
      references = bib
    )
  )

  expect_s3_class(cffobj, "cff")
  expect_snapshot(cffobj)
  expect_true(cff_validate(cffobj, verbose = FALSE))
})

#' Internal functions for `as_cff.bibentry` method
#'
#' @noRd
as_cff_reference <- function(x) {
  # Need always to be unnamed bibentry
  bib <- unname(x)
  if (anyDuplicated(bib) > 0) {
    cli::cli_alert_info("Removing duplicate {.cls bibentry} objects.")
    bib <- unique(bib)
  }

  # Return always a list
  the_list <- lapply(bib, make_cff_reference)
  the_list
}

make_cff_reference <- function(bib) {
  # Get BibTeX entry ----
  cit_list <- get_bibtex_entry(bib)

  ## If no title (case of some Misc) then return null
  if (!("title" %in% names(cit_list))) {
    entry <- capture.output(print(bib, bibtex = FALSE))
    entry <- as.character(entry)

    cli::cli_alert_warning("Entry {.val {entry}} without title. Skipping")

    return(NULL)
  }

  # Get BibTeX fields ----
  field_list <- get_bibtex_fields(cit_list)
  # VGAM: title is a vector
  field_list$title <- clean_str(field_list$title)

  ## Handle collection types ----
  field_list <- add_bibtex_coltype(field_list)

  ## Add conference
  field_list <- add_conference(field_list)

  # Create BibTeX to CFF institution logic ----
  field_list <- get_bibtex_inst(field_list)

  # Coerce persons ----
  # Special case: authors
  # Some keys does not strictly require authors, so we create one for cff
  # https://github.com/citation-file-format/citation-file-format/blob/main/
  # (cont) schema-guide.md#how-to-deal-with-unknown-individual-authors

  if (is.null(field_list$authors)) {
    field_list$authors <- person(family = "anonymous")
  }

  ## authors ----
  get_all_authors <- as_cff_person(field_list$authors)
  field_list$authors <- unique(get_all_authors)

  ## other persons----
  get_other_persons <- get_bibtex_other_pers(field_list)

  # Keep order here, we would use it later
  init_ord <- names(field_list)

  cit_list <- c(
    field_list[!names(field_list) %in% names(get_other_persons)],
    get_other_persons
  )

  # Building blocks----

  # Fallback for year and month: use date-published ----
  cit_list <- fallback_dates(cit_list)

  ## doi----
  bb_doi <- get_bibtex_doi(cit_list)
  cit_list$doi <- bb_doi$doi

  ### identifiers ----
  if (!is.null(bb_doi$identifiers)) {
    cit_list$identifiers <- bb_doi$identifiers
  }

  ## url----
  bb_url <- get_bibtex_url(cit_list)
  cit_list$url <- bb_url$url

  ### final identifiers----
  # Identifies (additional dois and urls)
  if (!is.null(bb_url$identifiers)) {
    cit_list$identifiers <- append(
      cit_list$identifiers,
      bb_url$identifiers
    )
  }

  ## Add thesis type ----
  cit_list <- add_thesis(cit_list)

  ## Handle location ----
  cit_list <- add_address(cit_list)

  # Last step----

  # Initial order but starting with type, title, authors
  final_order <- unique(
    c("type", "title", "authors", init_ord, names(cit_list))
  )

  cit_list <- cit_list[final_order]

  # Remove non-valid names
  validnames <- cff_schema_definitions_refs()
  cit_list <- cit_list[names(cit_list) %in% validnames]

  cit_list <- drop_null(cit_list)

  cit_list
}

# nocov start

# Detect repositories on package load.
.onLoad <- function(libname, pkgname) {
  avail_on_init <- as.data.frame(
    available.packages(
      repos = detect_repos(),
    )
  )[, c("Package", "Repository")]

  assign("avail_on_init", avail_on_init, envir = parent.env(environment()))
}

# nocov end

#' Schema utils
#'
#' @name cff_schema
#' @description
#' Helper functions with the valid values of different fields, according to the
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("\n", "[Citation File Format schema version 1.2.0]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#'
#' - [cff_schema_keys()] provides the valid high-level keys of the Citation
#'   File Format.
#' - [cff_schema_keys_license()] provides the valid
#'   [SPDX license identifier(s)](https://spdx.org/licenses/) to be used on the
#'   `CITATION.cff` file.
#' - [cff_schema_definitions_person()] and [cff_schema_definitions_entity()]
#'   return the valid fields to include when defining a
#'   person or entity.
#' - [cff_schema_definitions_refs()] provides the valid
#'   keys to be used on the `preferred-citation` and `references` keys.
#'
#' @source
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("[Guide to Citation File Format schema version 1.2.0]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#'
#' @family schemas
#'
#' @export
#' @encoding UTF-8
#'
#' @return
#' A character vector with the names of valid keys for Citation File Format
#'   version 1.2.0.
#'
#' @param sorted Logical `TRUE/FALSE`. Should the keys be arranged
#'   alphabetically?
#'
#' @examples
#'
#' cff_schema_keys(sorted = TRUE)
cff_schema_keys <- function(sorted = FALSE) {
  if (sorted) {
    # Alphabetically
    schema_keys <- c(
      "abstract",
      "authors",
      "cff-version",
      "commit",
      "contact",
      "date-released",
      "doi",
      "identifiers",
      "keywords",
      "license",
      "license-url",
      "message",
      "preferred-citation",
      "references",
      "repository",
      "repository-artifact",
      "repository-code",
      "title",
      "type",
      "url",
      "version"
    )
  } else {
    # Arranged arbitrarily
    schema_keys <- c(
      "cff-version",
      "message",
      "type",
      "license",
      "title",
      "version",
      "doi",
      "identifiers",
      "abstract",
      "authors",
      "preferred-citation",
      "repository",
      "repository-artifact",
      "repository-code",
      "commit",
      "url",
      "date-released",
      "contact",
      "keywords",
      "references",
      "license-url"
    )
  }

  schema_keys
}

#' @rdname cff_schema
#' @export
#' @encoding UTF-8
#' @examples
#'
#' # Valid Licenses keys
#' head(cff_schema_keys_license(), 20)
cff_schema_keys_license <- function() {
  license <- read.csv(system.file(
    "extdata/SPDX_License_List.csv",
    package = "cffr"
  ))

  license <- license$Identifier
  license
}

#' @rdname cff_schema
#' @export
#' @encoding UTF-8
#' @examples
#'
#' cff_schema_definitions_person()
cff_schema_definitions_person <- function() {
  definitions_person <- c(
    "address",
    "affiliation",
    "alias",
    "city",
    "country",
    "email",
    "family-names",
    "fax",
    "given-names",
    "name-particle",
    "name-suffix",
    "orcid",
    "post-code",
    "region",
    "tel",
    "website"
  )

  definitions_person
}

#' @rdname cff_schema
#' @export
#' @encoding UTF-8
#' @examples
#'
#' cff_schema_definitions_entity()
cff_schema_definitions_entity <- function() {
  definitions_entity <- c(
    "address",
    "alias",
    "city",
    "country",
    "date-end",
    "date-start",
    "email",
    "fax",
    "location",
    "name",
    "orcid",
    "post-code",
    "region",
    "tel",
    "website"
  )
  definitions_entity
}


#' @rdname cff_schema
#' @export
#' @encoding UTF-8
#' @examples
#'
#' cff_schema_definitions_refs()
cff_schema_definitions_refs <- function() {
  definitions_reference <- c(
    "abbreviation",
    "abstract",
    "authors",
    "collection-doi",
    "collection-title",
    "collection-type",
    "commit",
    "conference",
    "contact",
    "copyright",
    "data-type",
    "database-provider",
    "database",
    "date-accessed",
    "date-downloaded",
    "date-published",
    "date-released",
    "department",
    "doi",
    "edition",
    "editors",
    "editors-series",
    "end",
    "entry",
    "filename",
    "format",
    "identifiers",
    "institution",
    "isbn",
    "issn",
    "issue",
    "issue-date",
    "issue-title",
    "journal",
    "keywords",
    "languages",
    "license",
    "license-url",
    "loc-end",
    "loc-start",
    "location",
    "medium",
    "month",
    "nihmsid",
    "notes",
    "number",
    "number-volumes",
    "pages",
    "patent-states",
    "pmcid",
    "publisher",
    "recipients",
    "repository",
    "repository-artifact",
    "repository-code",
    "scope",
    "section",
    "senders",
    "start",
    "status",
    "term",
    "thesis-type",
    "title",
    "translators",
    "type",
    "url",
    "version",
    "volume",
    "volume-title",
    "year",
    "year-original"
  )
  definitions_reference
}


# Helper lists and vectors ----

#' Vector other persons
#' @noRd
other_persons <- function() {
  pers_ent <- c(
    "contact",
    "editors",
    "editors-series",
    "recipients",
    "senders",
    "translators"
  )

  pers_ent <- sort(unique(c(
    pers_ent,
    other_persons_entity(),
    entity_person()
  )))

  pers_ent
}

#' Vector other persons to be coerced as entities
#' @noRd
other_persons_entity <- function() {
  entities <- c(
    "conference",
    "database-provider",
    "institution",
    "location",
    "publisher"
  )

  entities
}

#' This may be entities or persons
#' @noRd
entity_person <- function() {
  forced <- c(
    "editors",
    "editors-series"
  )
  forced
}

test_that("Check schema keys", {
  sorted <- cff_schema_keys(TRUE)
  unsorted <- cff_schema_keys(FALSE)

  expect_equal(length(setdiff(sorted, unsorted)), 0)

  expect_false(all(sorted == unsorted))
})

test_that("Keys snapshot", {
  expect_snapshot({
    cff_schema_keys(FALSE)
  })
})

test_that("License run", {
  expect_silent(
    cff_schema_keys_license()
  )
})

#' Encode UTF-8 text to LaTeX
#'
#' @description
#' Transform a UTF-8 string into LaTeX special characters.
#'
#' @return A string with the corresponding transformations.
#'
#' @param x A string, possibly encoded in UTF-8 encoding system.
#'
#' @family bibtex
#'
#' @seealso [tools::encoded_text_to_latex()]
#'
#' @importFrom tools encoded_text_to_latex
#'
#' @details
#'
#' This is a variation of [tools::encoded_text_to_latex()] with some
#' additional replacements for better compatibility.
#'
#' @keywords internal
#' @export
#' @encoding UTF-8
#'
#' @examples
#' # Full range of supported characters on R
#' library(tools)
#'
#' range <- 1:511
#'
#' ascii_table <- data.frame(
#'   dec = range,
#'   utf8 = intToUtf8(range, multiple = TRUE)
#' )
#'
#' # Add latex using base approach
#' ascii_table$latex_base <- encoded_text_to_latex(ascii_table$utf8,
#'   encoding = "UTF-8"
#' )
#'
#' # With cffr
#' ascii_table$latex_cffr <- encoded_utf_to_latex(ascii_table$utf8)
#'
#' ascii_table
encoded_utf_to_latex <- function(x) {
  encoded <- enc2utf8(x)
  out <- encoded_text_to_latex(encoded, encoding = "UTF-8")
  out <- gsub("(?<=\\\\)a ", " ", out, perl = TRUE, ignore.case = FALSE)
  out <- gsub(
    "(?<=\\\\)a(?=[[:punct:]])",
    "",
    out,
    perl = TRUE,
    ignore.case = FALSE
  )
  out <- gsub("(?<=\\\\)k ", " ", out, perl = TRUE, ignore.case = FALSE)
  out
}

#' Check if a string is an email
#' @param email A string to be evaluated.
#' @noRd
is_email <- function(email) {
  if (is.null(email)) {
    return(FALSE)
  }

  email <- trimws(as.character(email))

  # See CFF validation schema
  x <- grepl(
    "^[\\S]+@[\\S]+\\.[\\S]{2,}$",
    email,
    ignore.case = TRUE,
    perl = TRUE
  )
  x
}

#' Check if a string is a URL
#' @param url A URL to be evaluated.
#' @noRd
is_url <- function(url) {
  if (is.null(url)) {
    return(FALSE)
  }

  x <- grepl("^(https|http|ftp|sftp)://.+", url)
  x
}

#' Check if a string contains a substring
#'
#' @param x A string.
#' @param sub A substring to be evaluated.
#' @noRd
is_substring <- function(x, sub) {
  if (is.null(x)) {
    return(FALSE)
  }

  if (isTRUE(grep(sub, x) == 1)) {
    TRUE
  } else {
    FALSE
  }
}

#' Check if an object is a `cff` object
#' @param x An object to be evaluated.
#' @noRd
is_cff <- function(x) {
  inherits(x, "cff")
}

#' Check if an object is a cff file
#' @param x An object to be evaluated.
#' @noRd
is_cff_file <- function(x) {
  src <- detect_x_source(x)
  val <- src == "cff_citation"
  val
}

#' Check if a URL is from GitHub
#' @param x An object to be evaluated.
#' @noRd
is_github <- function(x) {
  res <- isTRUE(
    grep(
      "^http[a-z]://github.com/",
      x["repository-code"]
    ) ==
      1
  )

  res
}

#' Check if an object has names
#' @param x An object to be evaluated.
#' @noRd
is_named <- function(x) {
  !is.null(names(x))
}

test_that("Check is_email", {
  # Examples from https://www.nicebread.de/validating-email-adresses-in-r/

  # Valid addresses
  expect_true(
    all(
      is_email("felix@nicebread.de"),
      is_email("felix.123.honeyBunny@nicebread.lmu.de"),
      is_email("felix@nicebread.de  "),
      is_email("    felix@nicebread.de"),
      is_email("felix+batman@nicebread.de"),
      is_email("felix@nicebread.office")
    )
  )
  # invalid addresses
  expect_false(
    any(
      is_email("felix@nicebread"),
      is_email("felix@nicebread@de"),
      is_email("felixnicebread.de"),
      is_email("@felixnicebread"),
      is_email("a @felixnicebread.co"),
      is_email("a@fe lixnicebread.de"),
      is_email(NULL),
      is_email(NA)
    )
  )
})

test_that("Check is_url", {
  # Valid urls
  expect_true(
    all(
      is_url("https://github.com/dieghernan"),
      is_url("http://github.com/dieghernan"),
      is_url("ftp://github.com/dieghernan"),
      is_url("sftp://github.com/dieghernan")
    )
  )
  # invalid addresses
  expect_false(
    any(
      is_url(NULL),
      is_url(NA),
      is_url(""),
      is_url("https:/github.com/dieghernan"),
      is_url("http:/github.com/dieghernan"),
      is_url("ftp:/github.com/dieghernan"),
      is_url("sftp:/github.com/dieghernan"),
      is_url("www.github.com/dieghernan")
    )
  )
})


test_that("Check is_substring", {
  expect_true(
    all(
      is_substring("amanda", "a"),
      is_substring("amanda erele", "e")
    )
  )
  # invalid addresses
  expect_false(
    any(
      is_substring(NULL, "a"),
      is_substring("amanda", "e"),
      is_substring("amanda", "A")
    )
  )
})

test_that("Check cff", {
  a_pers <- as_cff_person("Barnes and Noble")
  sing <- a_pers[[1]]
  expect_s3_class(sing, "cff_pers")

  a_bib <- as_cff(c(
    bibentry("misc", title = "a"),
    bibentry("misc", title = "b")
  ))
  sing2 <- a_bib[[1]]
  expect_s3_class(sing2, "cff_ref")
  expect_true(
    all(
      is_cff(cff()),
      is_cff(a_bib),
      is_cff(a_pers),
      is_cff(sing),
      is_cff(sing2)
    )
  )
  # invalid
  expect_false(
    any(
      is_cff("amanda"),
      is_cff(list(a = 1))
    )
  )
})


test_that("Check is cff file", {
  expect_true(is_cff_file("CIt_aT.cff"))
  expect_false(is_cff_file("CIt_aT_cff"))
})

test_that("is named", {
  a_n <- c("a" = 1, b = "2")
  expect_true(is_named(a_n))
  expect_true(is_named(as.list(a_n)))

  expect_false(is_named(LETTERS))
  expect_false(is_named(as.list(LETTERS)))
})

test_that("is github", {
  x <- list("repository-code" = "https://github.com/")
  x2 <- x
  x2$`repository-code` <- "gogle.com"
  expect_true(is_github(x))
  expect_false(is_github(x2))
})

test_that("Errors and messages", {
  a_cff <- cff()
  a_list <- as.list(a_cff)
  expect_true(inherits(a_list, "list"))
  expect_false(is_cff(a_list))
  expect_snapshot(cff_modify(a_list, abstract = "An abstract"), error = TRUE)

  # Nothing provided
  expect_snapshot(xend <- cff_modify(a_cff))
  expect_identical(xend, a_cff)

  # But silent in cff create
  expect_silent(aa <- cff_create("testthat", keys = NULL))

  # Duplicate args
  expect_snapshot(dup <- cff_modify(a_cff, abstract = "a", abstract = "b"))
  expect_length(dup, length(a_cff) + 1)
  expect_identical(dup$abstract, "a")
  expect_snapshot(cff_modify(a_cff, "a", "b"), error = TRUE)
  expect_snapshot(mod <- cff_modify(a_cff, "a", abstract = "b"))
  expect_length(mod, length(a_cff) + 1)
  expect_identical(mod$abstract, "b")
})

test_that("Can reclass", {
  a_cff <- cff()

  mod <- cff_modify(
    a_cff,
    contact = list(list(
      name = "a contact",
      address = "here"
    ))
  )

  expect_true(cff_validate(mod, verbose = FALSE))
  expect_s3_class(mod$contact, c("cff_pers_lst", "cff"), exact = TRUE)
  expect_s3_class(mod$contact[1], c("cff_pers", "cff"), exact = TRUE)
})

#' Use a git pre-commit hook `r lifecycle::badge("experimental")`
#'
#' @description
#'
#' Install a
#'
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [pre-commit hook]",
#'            "(https://git-scm.com/book/en/v2/Customizing-Git-Git-Hooks",
#'            "#_committing_workflow_hooks)"))
#'
#' ```
#' that reminds you to update your `CITATION.cff` file. This is a wrapper around
#' [usethis::use_git_hook()].
#'
#' @name cff_git_hook
#'
#' @family git
#'
#' @export
#' @encoding UTF-8
#'
#' @return Invisible. This function is called for its side effects.
#'
#' @seealso
#'
#' - [usethis::use_git_hook()], which is the underlying function used by
#'   `cff_git_hook_install()`.
#'
#' - [usethis::use_git()] and related function of \CRANpkg{usethis} for using
#'   Git with **R** packages.
#'
#' @details
#'
#' This function installs a pre-commit hook using
#' [usethis::use_git_hook()].
#'
#' A pre-commit hook is a script that identifies simple issues before
#' submission to code review. This pre-commit hook warns you if any of the
#' following conditions are met:
#' - You included your `DESCRIPTION` or `inst/CITATION` file in a commit but
#'   did not include your `CITATION.cff`, and the `CITATION.cff` file is
#'   "older" than your `DESCRIPTION` or `inst/CITATION` file.
#' - You updated your `CITATION.cff` but did not include it in
#'   your commit.
#'
#' # A word of caution
#'
#' The pre-commit hook may prevent you from committing if you are not updating
#' your `CITATION.cff`. However, the detection mechanism is not perfect and may
#' be triggered even if you have attempted to update your `CITATION.cff` file.
#'
#' This typically occurs when you have updated your `DESCRIPTION` or
#' `inst/CITATION` files, but those changes do not affect your
#' `CITATION.cff` file (for example, you are adding new dependencies).
#'
#' In those cases, you can override the check by running
#' `git commit --no-verify` in the terminal.
#'
#' If you are using **RStudio**, you can also run this command from an **R**
#' script by selecting that line and sending it to the terminal using:
#'
#' - Windows & Linux: `Ctrl+Alt+Enter`.
#' - Mac: `Cmd+Option+Return`.
#'
#' # Removing the git pre-commit hook
#'
#' You can remove the pre-commit hook using `cff_git_hook_remove()`.
#'
#' @examples
#' \dontrun{
#' cff_git_hook_install()
#' }
#'
cff_git_hook_install <- function() {
  # nocov start
  bash_file <- system.file("bash/citation-cff-pre-commit.sh", package = "cffr")

  if (requireNamespace("usethis", quietly = TRUE)) {
    usethis::use_git_hook("pre-commit", readLines(con = bash_file))
  } else {
    cli::cli_alert_danger(
      paste0(
        "Please install {.pkg usethis} to use the pre-commit hook: ",
        '{.run install.packages("usethis")}'
      )
    )
  }
  invisible()
  # nocov end
}

#' @rdname cff_git_hook
#' @export
#' @encoding UTF-8
cff_git_hook_remove <- function() {
  # nocov start
  hookfile <- file.path(".git", "hooks", "pre-commit")

  if (file_exist_abort(hookfile)) {
    cli::cli_alert_info(
      "Removing git pre-commit hook (was on {.path {hookfile}})"
    )
    unlink(hookfile, force = TRUE)
  }

  invisible()
  # nocov end
}

protect_bib_braces <- function(x) {
  paste0("{", x, "}")
}

# Utils for as_bibentry ----
get_bib_howpublised <- function(x) {
  howpublished <- x$medium

  if (!is.null(howpublished)) {
    f <- toupper(substr(howpublished, 0, 1))
    rest <- substr(howpublished, 2, nchar(howpublished))
    howpublished <- paste0(c(f, rest), collapse = "")
  }

  clean_str(howpublished)
}

get_bib_note <- function(x) {
  note <- x$notes

  # unpublished needs a note
  if (all(is.null(note), tolower(x$type) == "unpublished")) {
    note <- "Extracted with cffr R package"
  }

  clean_str(note)
}

guess_bibtype <- function(x) {
  init_guess <- switch(tolower(x$type),
    "article" = "article",
    "book" = "book",
    "manual" = "manual",
    "unpublished" = "unpublished",
    "conference" = "inproceedings",
    "conference-paper" = "inproceedings",
    "proceedings" = "proceedings",
    "magazine-article" = "article",
    "newspaper-article" = "article",
    "pamphlet" = "booklet",
    "report" = "techreport",
    "thesis" = "mastersthesis",
    # We would need to guess
    "misc"
  )

  # Try guess Inbook ----
  # inbook is a book where chapter or pages are present
  has_chapter <- !is.null(clean_str(x$section))
  has_pages <- !is.null(
    clean_str(paste(unique(c(x$start, x$end)), collapse = "--"))
  )

  if (all(init_guess == "book", any(has_chapter, has_pages))) {
    init_guess <- "inbook"
    return(init_guess)
  }

  # Try guess Phdthesis  ----
  if (init_guess == "mastersthesis") {
    ttype <- clean_str(gsub("[[:punct:]]", "", x$`thesis-type`, perl = TRUE))
    # phd
    if (all(!is.null(ttype), grepl("phd", ttype, ignore.case = TRUE))) {
      return("phdthesis")
    }
  }

  # Try guess InCollection  ----
  # Hint: is misc with collection-title and publisher
  if (init_guess == "misc") {
    if (!is.null(clean_str(x$`collection-title`))) {
      return("incollection")
    }
  }

  init_guess
}

get_bib_address <- function(x) {
  # BibTeX 'address' is taken from the publisher (book, others) or the
  # conference (inproceedings).
  # Set logic: conference > institution > publisher
  if (!is.null(x$conference)) {
    addr_search <- x$conference
  } else if (!is.null(x$institution)) {
    addr_search <- x$institution
  } else {
    addr_search <- x$publisher
  }

  address <- clean_str(
    paste(
      c(
        addr_search$address,
        addr_search$city,
        addr_search$region,
        addr_search$country
      ),
      collapse = ", "
    )
  )

  # As a fallback, use also location
  if (is.null(address) && !is.null(x$location)) {
    address <- clean_str(x$location$name)
  }

  address
}

get_bib_booktitle <- function(x, bibtype) {
  # This map collection title.
  # If inproceedings, incollection to booktitle
  # rest of cases to series

  book_series <- list()
  tag_value <- clean_str(x[["collection-title"]])

  if (!bibtype %in% c("incollection", "inproceedings")) {
    book_series$series <- tag_value
  } else {
    # Only for incollections and inproceedings map booktitle
    book_series$booktitle <- tag_value

    # Fallback to conference name for inproceedings
    if (all(bibtype == "inproceedings", is.null(tag_value))) {
      book_series$booktitle <- clean_str(x$conference$name)
    }
  }
  book_series
}

get_bib_inst_org <- function(x, bibtype) {
  # For inproceedings, proceedings and manual this field
  # is organization
  # For thesis it should be school

  inst_org <- list()
  # Just name
  inst_name <- clean_str(x$institution$name)

  if (bibtype %in% c("inproceedings", "proceedings", "manual")) {
    inst_org$organization <- inst_name
  } else if (grepl("thesis", bibtype, ignore.case = TRUE)) {
    inst_org$school <- inst_name
  } else {
    inst_org$institution <- inst_name
  }

  # Fallback for techreport, search on affiliation first author

  if (bibtype == "techreport" && is.null(inst_org$institution)) {
    inst_org$institution <- clean_str(x$authors[[1]]$affiliation)
  }

  inst_org
}

make_bibkey <- function(tobibentry) {
  # Be kind and provided a bibentry key

  y <- tobibentry$year

  # Init etall
  etall <- NULL

  # Also Some entries don't have authors, but editors
  # Others may have none (misc, pamphlet)

  init_aut <- tobibentry$author

  # Try get editor in null
  if (is.null(init_aut)) {
    init_aut <- tobibentry$editor
  }

  # If none then get a key based in title
  if (any(is.null(init_aut), length(init_aut) == 0)) {
    r <- tolower(tobibentry$title)
    # Reduce lenght to 15 max
    r <- substr(r, 0, 15)
  } else {
    # First name/surname and et_all if additional authors
    # Bear in mind institutions has only given

    nauths <- length(init_aut)
    if (nauths > 1) {
      etall <- "_etall"
    }

    # Get info of first author
    unz <- unlist(init_aut[1])
    if ("family" %in% names(unz)) {
      r <- unz["family"]
    } else {
      r <- unz["given"]
    }

    r <- tolower(paste0(r, collapse = ""))
  }

  # Try hard to remove accents
  # First with iconv
  r <- iconv(r, from = "UTF-8", to = "ASCII//TRANSLIT", sub = "?")

  # Next to latex
  r <- encoded_utf_to_latex(r)

  # Finally keep only a-z letters for key
  r <- gsub("[^_a-z]", "", r)

  # Append etall and year
  key <- paste0(c(r, etall), collapse = "")
  key <- paste(c(key, y), collapse = ":")
  key
}

get_bib_month <- function(x) {
  m <- x$month

  # Fallback

  if (is.null(m) && !is.null(x$`date-published`)) {
    # Should be YYYY-MM-DD to be valid on cff, so
    m <- as.integer(format(as.Date(x$`date-published`), "%m"))
  }

  # Try to get 3 month string
  m_int <- suppressWarnings(as.integer(m))
  m_letters <- clean_str(tolower(month.abb[m_int]))

  if (!is.null(m_letters)) {
    month <- m_letters
  } else {
    month <- clean_str(m)
  }

  month
}

get_bib_year <- function(x) {
  year <- x$year

  # Fallback

  if (is.null(year) && !is.null(x$`date-released`)) {
    # Should be YYYY-MM-DD to be valid on cff, so
    year <- substr(x$`date-released`, 1, 4)
  }

  clean_str(year)
}

#' Create [`cff`] objects from direct inputs
#'
#' A class and utility methods for reading, creating and holding CFF
#' information. See [cff_class] to learn more about `cff` objects.
#'
#' @rdname cff
#' @name cff
#' @return
#'
#' A [`cff`] object. Under the hood, a `cff` object is a regular [`list`]
#' object with a special [`print`][print.cff()] method.
#'
#' @family core
#'
#' @param path `r lifecycle::badge("deprecated")` `path` is no longer supported,
#'   use [cff_read_cff_citation()] instead.
#' @param ... Named arguments to be used for creating a [`cff`] object. If no
#'   arguments are supplied (the default behavior), a minimal valid `cff`
#'   object is created.
#'
#' @details
#'
#' `cff()` converts `_` in the argument name to `-`. For example,
#' `cff_version = "1.2.0"` is converted to `cff-version = "1.2.0"`.
#'
#' Valid arguments are those specified on [cff_schema_keys()]:
#'
#' ```{r, echo=FALSE, results='asis'}
#' p <- paste0('  - `', cffr::cff_schema_keys(), '`', collapse = "\n")
#'
#' cat(p)
#'
#' ```
#' @export
#' @encoding UTF-8
#' @examples
#' # Blank cff
#' cff()
#'
#' # Use custom params
#' test <- cff(
#'   title = "Manipulating files",
#'   keywords = c("A", "new", "list", "of", "keywords"),
#'   authors = as_cff_person("New author")
#' )
#' test
#' \donttest{
#' # Would fail
#' cff_validate(test)
#' }
#'
#' # Modify with cff_create
#' new <- cff_create(test, keys = list(
#'   "cff_version" = "1.2.0",
#'   message = "A blank file"
#' ))
#' new
#'
#' # Would pass
#' cff_validate(new)
#'
#' @export
cff <- function(path, ...) {
  if (!missing(path)) {
    src <- detect_x_source(path)
    if (src == "cff_citation") {
      lifecycle::deprecate_warn("1.0.0", "cff(path)", "cff_read_cff_citation()")
      return(cff_read_cff_citation(path))
    } else {
      lifecycle::deprecate_warn(
        "1.0.0",
        "cff(path)",
        details = "Argument ignored."
      )
    }
  }

  cffobj <- list(...)
  if (length(cffobj) == 0) {
    # If nothing is provided use a minimal cff
    path <- system.file("examples/CITATION_skeleton.cff", package = "cffr")

    return(cff_read_cff_citation(path))
  }

  # Check names

  cffobj <- validate_extra_keys(cffobj)
  cffobj <- fuzzy_keys(cffobj)

  if (anyDuplicated(names(cffobj)) > 0) {
    cli::cli_alert_warning("Removing duplicated keys.")
    cffobj <- cffobj[!duplicated(names(cffobj))]
  }

  cffobj <- as_cff(cffobj)

  cffobj
}

test_that("Write basic", {
  desc_file <-
    system.file("examples/DESCRIPTION_basic", package = "cffr")
  desc_file <- cff_create(desc_file)
  expect_s3_class(desc_file, "cff")
  tmp <- file.path(tempdir(), "basic.cff")
  expect_message(cff_write(desc_file, outfile = tmp, validate = FALSE))
  expect_silent(cff_write(desc_file, outfile = tmp, verbose = FALSE))
  expect_true(file_exist_abort(tmp))

  # Validate from file
  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

test_that("Write to a non-existing folder", {
  desc_file <-
    system.file("examples/DESCRIPTION_basic", package = "cffr")
  desc_file <- cff_create(desc_file)
  expect_s3_class(desc_file, "cff")
  tmp <- file.path(tempdir(), "/test_new_folder/recursive/new_folder.cff")
  cff_write(
    desc_file,
    outfile = tmp,
    validate = FALSE,
    verbose = FALSE
  )

  expect_true(dir.exists(file.path(
    tempdir(),
    "test_new_folder",
    "recursive"
  )))
  expect_true(file_exist_abort(tmp))

  # Validate from file
  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

test_that("Write no encoding", {
  desc_file <-
    system.file("examples/DESCRIPTION_no_encoding", package = "cffr")
  desc_file <- cff_create(desc_file)
  expect_s3_class(desc_file, "cff")
  tmp <- file.path(tempdir(), "noencoding.cff")
  cff_write(
    desc_file,
    outfile = tmp,
    validate = FALSE,
    verbose = FALSE
  )

  expect_true(file_exist_abort(tmp))

  # Validate from file
  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

test_that("Add new keys", {
  desc_file <-
    system.file("examples/DESCRIPTION_basic", package = "cffr")
  desc_file <- cff_create(desc_file)
  expect_s3_class(desc_file, "cff")
  tmp <- file.path(tempdir(), "newkeys.cff")

  # Add additional keys
  newkeys <- list(
    message = "This overwrites fields",
    abstract = "New abstract",
    keywords = c("A", "new", "list", "of", "keywords"),
    authors = as_cff_person(
      person("Don", "Nadie", comment = c(website = "error"))
    ),
    "date-released" = "1900-01-01",
    "error" = "This is an error"
  )
  expect_message(
    s <- cff_write(
      desc_file,
      keys = newkeys,
      outfile = tmp,
      validate = FALSE,
      verbose = FALSE
    ),
    "error: No match"
  )
  expect_snapshot(s)

  expect_true(file_exist_abort(tmp))

  # Validate from file
  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

test_that("Append keys", {
  desc_file <-
    system.file("examples/DESCRIPTION_basic", package = "cffr")
  desc_file <- cff_create(desc_file)
  expect_snapshot(desc_file)

  expect_s3_class(desc_file, "cff")

  old_aut <- desc_file$authors

  # It should be a list
  new_aut <- append(
    old_aut,
    as_cff_person(
      person(
        "New",
        "author",
        comment = c(
          "error" = 123,
          website = "https://stackoverflow.com/",
          country = "IT"
        )
      )
    )
  )

  tmp <- file.path(tempdir(), "appendkeys.cff")
  s <- cff_write(
    desc_file,
    outfile = tmp,
    keys = list("authors" = new_aut),
    verbose = FALSE
  )

  expect_snapshot(s)

  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

test_that("Fix extension of the file", {
  cffobj <- cff()
  cffobj <- cff_modify(cffobj, authors = as_cff_person("Diego Pérez"))
  tmp <- file.path(tempdir(), "fix_extension")
  expect_silent(cff_write(cffobj, tmp, verbose = FALSE))

  expect_false(file_exist_abort(tmp))
  expect_true(file_exist_abort(paste0(tmp, ".cff")))
  expect_true(cff_validate(paste0(tmp, ".cff"), verbose = FALSE))
  expect_snapshot_file(paste0(tmp, ".cff"))

  file.remove(paste0(tmp, ".cff"))
  expect_false(file_exist_abort(paste0(tmp, ".cff")))
})

test_that("test encoding utf8", {
  cffobj <- cff()
  cffobj <- cff_modify(cffobj, authors = as_cff_person("Diego Pérez"))
  tmp <- file.path(tempdir(), "utf8.cff")
  expect_silent(cff_write(cffobj, tmp, verbose = FALSE))

  expect_true(file_exist_abort(tmp))
  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

test_that("test encoding others", {
  skip_on_os("mac")
  cffobj <- cff()
  cffobj <- cff_modify(cffobj, authors = as_cff_person("Diego Pérez"))
  tmp <- file.path(tempdir(), "asci_trans.cff")
  expect_silent(cff_write(
    cffobj,
    tmp,
    verbose = FALSE,
    encoding = "ASCII//TRANSLIT"
  ))

  expect_true(file_exist_abort(tmp))
  expect_true(cff_validate(tmp, verbose = FALSE))
  expect_snapshot_file(tmp)

  file.remove(tmp)
  expect_false(file_exist_abort(tmp))
})

# Utility functions used in the package.

#' Clean strings
#'
#' @param str A string to be cleaned.
#' @noRd
clean_str <- function(str) {
  # Collapse to a single string.
  str <- unlist(paste(str, collapse = " "))

  clean <- gsub("[\n\r]", " ", str)
  clean <- gsub("\\s+", " ", clean)
  clean <- gsub("\\{", "", clean)
  clean <- gsub("\\}", "", clean)
  clean <- gsub("^NA$", "", clean)
  clean <- gsub("\\s{2,}", " ", clean)
  clean <- trimws(clean)
  # Collapse to single char
  clean <- paste(clean, collapse = " ")

  if (clean == "") {
    return(NULL)
  }
  # Encoding
  enc <- Encoding(clean)

  if (enc != "UTF-8") {
    clean <- iconv(clean, to = "UTF-8")
  }

  clean
}

#' Drop nulls from list
#' @source https://github.com/cboettig/codemeta/blob/master/R/utils.R
#' @param x A list to be cleaned.
#' @noRd
drop_null <- function(x) {
  # Already been here
  if (inherits(x, "cff")) {
    return(x)
  }
  x[lengths(x) != 0]
}

#' Pretty printing of snapshots
#' @noRd
print_snapshot <- function(title = "----", obj) {
  cat("\n\n##", title, "\n\n")
  print(obj)
  cat("\n---")
}

#' Search for a package in available repositories.
#' @param name Name of the package.
#' @param avail Data frame with available packages. See
#' [utils::available.packages()].
#' @param repos Vector of repositories.
#' @noRd
search_on_repos <- function(
  name,
  avail = avail_on_init,
  repos = detect_repos()
) {
  get <- avail[name == avail$Package, "Repository"]

  get <- clean_str(get)

  if (is.null(get)) {
    return(NULL)
  }

  # Try to find in CRAN
  cran_repo <- clean_str(repos["CRAN"])

  if (length(grep(cran_repo, get)) == 1) {
    # Canonical url to CRAN

    repos <- paste0("https://CRAN.R-project.org/package=", name)
    return(repos)
  }

  repos <- gsub("src/contrib$", "", get)

  repos
}


#' Detect current repositories
#'
#' Detect the current repositories of the user. If not set, use CRAN.
#' @param repos Current repositories setup.
#' @noRd
detect_repos <- function(repos = getOption("repos")) {
  # Not use RSPM
  repos <- repos[names(repos) != "RSPM"]
  repos <- repos[!grepl("rspm", repos, fixed = TRUE)]
  repos <- repos[!grepl("posit", repos, fixed = TRUE)]
  repos <- repos[!grepl("rstudio", repos, fixed = TRUE)]

  # If not set use 0-Cloud
  if (!is_url(repos["CRAN"])) {
    repos["CRAN"] <- "https://cloud.r-project.org/"
  }

  repos
}

#' Function for fuzzy matching the names of the keys
#'
#' @description
#' Proposed by @sckott
#' @inheritParams cff_create
#' @noRd
fuzzy_keys <- function(keys) {
  nm <- names(keys)
  names(keys) <- gsub("_", "-", nm, fixed = TRUE)
  valid_keys <- unique(c(
    cff_schema_keys(),
    cff_schema_definitions_entity(),
    cff_schema_definitions_person(),
    cff_schema_definitions_refs()
  ))
  names <- names(keys)
  # Check valid keys as is
  is_valid_key <- names %in% valid_keys

  # If not all are valid try to fuzzy match
  if (isFALSE(all(is_valid_key))) {
    names_fuzzy <- names[!(is_valid_key)]

    keys_match <- lapply(
      names_fuzzy,
      agrep,
      valid_keys,
      ignore.case = TRUE,
      value = TRUE,
      fixed = FALSE
    )

    # Modify NULL correspondences
    keys_match <- unlist(lapply(
      keys_match,
      function(x) {
        if (length(x) == 0) {
          return("No match, removing.")
        }
        x[1]
      }
    ))

    # Message
    ll <- paste0("{.dt ", names_fuzzy, "}{.dl ", keys_match, "}")

    bullets <- rep("v", length(ll))
    bullets[keys_match == "No match, removing."] <- "x"
    names(ll) <- bullets
    cli::cli_alert_info(
      paste0("Found misspelled keys. Trying to map:")
    )

    cli::cli_bullets(ll)
    # Modify names
    names[!is_valid_key] <- keys_match
  }

  new_keys <- keys
  names(new_keys) <- names

  new_keys <- new_keys[names %in% valid_keys]

  new_keys
}

guess_cff_named_part <- function(x) {
  nms <- names(x)
  # Search for names
  is_person <- any(grepl("^name$|family|given|particle", nms))
  if (is_person) {
    return("cff_pers")
  }

  # VALID full cff file
  is_full <- any(grepl("cff-version|message", nms))
  if (is_full) {
    return("cff_full")
  }

  # Reference
  is_ref <- any(grepl("title|type", nms))
  if (is_ref) {
    return("cff_ref")
  }

  # Else
  "unclear"
}


guess_cff_part <- function(x) {
  named <- is_named(x)
  if (named) {
    return(guess_cff_named_part(x))
  }

  # Look to first element
  guess <- guess_cff_named_part(x[[1]])

  fin <- switch(guess,
    "cff_pers" = "cff_pers_lst",
    "cff_ref" = "cff_ref_lst",
    "unclear"
  )

  fin
}


detect_x_source <- function(x) {
  if (any(missing(x), is.null(x))) {
    return("indev")
  }

  if (is_cff(x)) {
    return("cff_obj")
  }

  x <- as.character(x)[1]
  instpack <- as.character(installed.packages()[, "Package"])

  if (x %in% instpack) {
    return("package")
  }

  if (grepl("\\.cff$", x, ignore.case = TRUE)) {
    return("cff_citation")
  }
  if (grepl("\\.bib$", x, ignore.case = TRUE)) {
    return("bib")
  }
  if (grepl("citat", x, ignore.case = TRUE)) {
    return("citation")
  }
  if (grepl("desc", x, ignore.case = TRUE)) {
    return("description")
  }

  "dontknow"
}

file_path_or_null <- function(x) {
  x_c <- clean_str(x)
  if (is.null(x_c)) {
    return(x)
  }
  if (file_exist_abort(x)) {
    return(x)
  }
  NULL
}

#' Coerce and clean data from DESCRIPTION to create metadata
#' @noRd
clean_package_meta <- function(meta) {
  if (!inherits(meta, "packageDescription")) {
    # Add encoding
    meta <- list()
    meta$Encoding <- "UTF-8"
    return(meta)
  }

  # Convert to a desc object

  # First write to a dcf file
  tmp <- tempfile("DESCRIPTION")
  meta_unl <- unclass(meta)
  write.dcf(meta_unl, tmp)
  pkg <- desc::desc(tmp)
  pkg$coerce_authors_at_r()
  # Extract package data
  meta <- pkg$get(desc::cran_valid_fields)

  # Clean missing and drop empty fields
  meta <- drop_null(lapply(meta, clean_str))

  # Check encoding
  if (!is.null(meta$Encoding)) {
    meta <- lapply(meta, iconv, from = meta$Encoding, to = "UTF-8")
  } else {
    meta$Encoding <- "UTF-8"
  }
  unlink(tmp, force = TRUE)
  meta
}


# Convert a DESCRIPTION object to meta object using desc package
desc_to_meta <- function(x) {
  src <- x
  my_meta <- desc::desc(src)
  my_meta$coerce_authors_at_r()

  # As list
  my_meta_l <- my_meta$get(desc::cran_valid_fields)
  my_meta_l <- as.list(my_meta_l)
  v_nas <- vapply(my_meta_l, is.na, logical(1))
  my_meta_l <- my_meta_l[!v_nas]

  meta_proto <- packageDescription("cffr")

  class(my_meta_l) <- class(meta_proto)
  attr(my_meta_l, "file") <- x

  my_meta_l
}

test_that("Try encoding conversion", {
  a <- iconv("España", to = "latin1")

  expect_true(Encoding(a) == "latin1")

  b <- clean_str(a)

  expect_false(Encoding(b) == Encoding(a))
  expect_true(Encoding(b) == "UTF-8")
})

test_that("Try cleaning string", {
  expect_identical(
    clean_str(c("a", "b")),
    c("a b")
  )

  expect_null(clean_str(NA))
  expect_null(clean_str(NULL))
  expect_null(clean_str(""))
  expect_null(clean_str(character(0)))
  expect_null(clean_str(integer(0)))
  expect_null(clean_str(logical(0)))
  expect_null(clean_str(list()))
})

test_that("Use right repo", {
  # Use some other repos
  repos <- c("https://ropensci.r-universe.dev", "https://cloud.r-project.org")

  final <- detect_repos(repos)
  expect_identical(final, c(CRAN = "https://cloud.r-project.org/"))

  # With posit package manager
  repos <- c(
    CRAN = "https://packagemanager.posit.co/cran/latest",
    RPSM = "https://packagemanager.rstudio.com/",
    ANOTHER = "https://packagemanager.rspm.com/"
  )
  final <- detect_repos(repos)
  expect_identical(final, c(CRAN = "https://cloud.r-project.org/"))
})

test_that("Error if file not exists", {
  expect_error(
    cff_create("DESCRIPTION_not_exists")
  )
})

test_that("Test installed packages", {
  skip_on_cran()
  expect_silent(cff_create("jsonlite"))
  expect_silent(cff_create("yaml"))
})

test_that("Test indev", {
  skip_on_cran()
  current_dir <- getwd()

  name <- paste0("mock-pack", runif(1) * 10)
  new_dir <- file.path(tempdir(), name)

  dir.create(new_dir, recursive = TRUE)

  expect_true(dir.exists(new_dir))

  setwd(new_dir)

  # Move files
  file.copy(
    system.file("examples/DESCRIPTION_basic", package = "cffr"),
    to = "DESCRIPTION"
  )

  a_cff <- cff_create()

  expect_true(cff_validate(a_cff, verbose = FALSE))
  # Revert to initial wd
  setwd(current_dir)

  expect_snapshot(a_cff)

  unlink(new_dir, recursive = TRUE, force = TRUE)
})

test_that("Test dependencies extraction", {
  yes <- cff_create("jsonlite")
  no <- cff_create("jsonlite", dependencies = FALSE)

  expect_true(length(yes$references) > length(no$references))
})

test_that("Test error formats on inputs", {
  df <- data.frame(x = 1, b = "c")
  expect_error(cff_create(df))
  l <- list(a = 1, b = 3)
  expect_error(cff_create(l))

  expect_error(cff_create("uanuanua"))
})

test_that("Validate all DESCRIPTION files", {
  allfiles <- list.files(
    system.file("examples", package = "cffr"),
    pattern = "^DESC",
    full.names = TRUE
  )

  for (i in seq_along(allfiles)) {
    cffobj <- cff_create(allfiles[i], gh_keywords = FALSE)
    # Check that no preferred citation is created
    expect_null(cffobj$`preferred-citation`)

    expect_true(cff_validate(cffobj, verbose = FALSE))
  }
})

test_that("No auto generate preferred citations", {
  rgeos <- system.file("examples/DESCRIPTION_rgeos", package = "cffr")

  expect_snapshot(cff_create(
    rgeos,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  ))

  basicdate <- system.file("examples/DESCRIPTION_basicdate", package = "cffr")
  expect_snapshot(cff_create(
    basicdate,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  ))
})

test_that("Fuzzy match on cff_create", {
  newobject <- cff_create(cff())
  newkeys <- list(
    "url" = "https://ropensci.org/",
    "version" = "0.0.1",
    "repository" = "https://github.com/user/repo",
    # If the field is already present, it would be overridden
    tittle = "Modifying a 'cff' object"
  )
  expect_message(
    modobject <- cff_create(newobject, keys = newkeys),
    "tittle: title"
  )
  expect_snapshot(print_snapshot(
    "Fuzzy match on cff_create",
    modobject
  ))
})

test_that("Test installed packages vs call to file", {
  skip_on_cran()
  call1 <- cff_create("jsonlite")
  call2 <- cff_create(system.file("DESCRIPTION", package = "jsonlite"))

  expect_identical(call1, call2)
})

# Additional authors ----

test_that("Default roles", {
  p <- system.file("examples/DESCRIPTION_no_URL", package = "cffr")

  cf <- cff_create(p, dependencies = FALSE)

  # Same as
  cf2 <- cff_create(p, authors_roles = c("aut", "cre"), dependencies = FALSE)
  expect_identical(cf, cf2)
})

test_that("Add new roles", {
  p <- system.file("examples/DESCRIPTION_no_URL", package = "cffr")

  cf <- cff_create(p, dependencies = FALSE)
  cf2 <- cff_create(
    p,
    authors_roles = c("aut", "cre", "ctb"),
    dependencies = FALSE
  )

  expect_gt(length(cf2$authors), length(cf$authors))
  expect_true(cff_validate(cf2, verbose = FALSE))
})

test_that("Default roles on write", {
  p <- system.file("examples/DESCRIPTION_no_URL", package = "cffr")

  cf <- cff_create(p, dependencies = FALSE)

  # Same as
  tmp <- tempfile(fileext = ".cff")
  cf2 <- cff_write(
    p,
    authors_roles = c("aut", "cre"),
    dependencies = FALSE,
    outfile = tmp,
    verbose = FALSE,
    validate = FALSE
  )

  expect_identical(cf, cf2)
})

test_that("Add new roles", {
  p <- system.file("examples/DESCRIPTION_no_URL", package = "cffr")

  cf <- cff_create(p, dependencies = FALSE)
  cf2 <- cff_create(
    p,
    authors_roles = c("aut", "cre", "ctb"),
    dependencies = FALSE
  )

  expect_gt(length(cf2$authors), length(cf$authors))
  expect_true(cff_validate(cf2, verbose = FALSE))
})

test_that("Add new roles on write", {
  p <- system.file("examples/DESCRIPTION_no_URL", package = "cffr")

  cf_def <- cff_create(p)
  cf <- cff_create(p, authors_roles = "ctb")

  expect_gt(length(cf$authors), length(cf_def$authors))

  # Same as
  tmp <- tempfile(fileext = ".cff")
  expect_message(
    cf2 <- cff_write(p, authors_roles = "ctb", outfile = tmp, validate = FALSE),
    "generated"
  )

  expect_identical(cf, cf2)
})

# Check DESCRIPTION ----
test_that("Coerce date", {
  desc_path <- system.file("examples/DESCRIPTION_rgeos", package = "cffr")

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_false(is.null(a_cff$`date-released`))

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})

test_that("Coerce date in another format", {
  desc_path <- system.file("examples/DESCRIPTION_basicdate", package = "cffr")

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_false(is.null(a_cff$`date-released`))

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})


test_that("No date coerced in DESCRIPTION without it", {
  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")

  a_cff <- cff_create(desc_path, keys = list(references = NULL))

  expect_true(is.null(a_cff$`date-released`))

  expect_s3_class(a_cff, "cff")
  expect_true(cff_validate(a_cff, verbose = FALSE))
})

test_that("Parsing many urls", {
  desc_path <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_length(a_cff$`repository-code`, 1)
  expect_length(a_cff$url, 1)
  expect_length(a_cff$identifiers, 3)
  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})


test_that("Parsing Gitlab", {
  desc_path <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")

  a_cff <- cff_create(desc_path, keys = list(references = NULL))

  expect_length(a_cff$`repository-code`, 1)
  expect_length(a_cff$url, 1)
  expect_length(a_cff$identifiers, 0)
  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})

test_that("Parsing many persons", {
  desc_path <- system.file(
    "examples/DESCRIPTION_many_persons",
    package = "cffr"
  )

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_length(a_cff$authors, 4)

  authors <- unlist(a_cff$authors)

  expect_length(grep("erro", authors, fixed = TRUE), 0)
  names <- unlist(lapply(a_cff$authors, names))

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})


test_that("Parsing wrong urls", {
  skip_on_cran()

  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")

  desc_path <- system.file("examples/DESCRIPTION_wrong_urls", package = "cffr")

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_null(a_cff$`repository-code`)
  expect_length(a_cff$url, 1)
  expect_length(a_cff$identifiers, 2)

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})


test_that("Parsing two maintainers", {
  desc_path <- system.file(
    "examples/DESCRIPTION_twomaintainers",
    package = "cffr"
  )

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_length(a_cff$authors, 3)
  expect_length(a_cff$contact, 2)

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})

test_that("Parsing r-universe", {
  desc_path <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_length(a_cff$repository, 1)

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})


test_that("Parsing Bioconductor", {
  desc_path <- system.file(
    "examples/DESCRIPTION_bioconductor",
    package = "cffr"
  )

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_length(a_cff$repository, 1)

  expect_s3_class(a_cff, "cff")
  expect_true(cff_validate(a_cff, verbose = FALSE))

  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")

  expect_snapshot(a_cff)
})

test_that("Parsing Posit Package Manager", {
  desc_path <- system.file(
    "examples/DESCRIPTION_posit_package_manager",
    package = "cffr"
  )

  a_cff <- cff_create(
    desc_path,
    gh_keywords = FALSE,
    keys = list(references = NULL)
  )

  expect_length(a_cff$repository, 1)
  expect_identical(
    a_cff$repository,
    "https://CRAN.R-project.org/package=resmush"
  )
  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})

test_that("Search package on CRAN", {
  basic_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")

  tmp <- tempfile("DESCRIPTION_basic")
  # Create a temporary file
  file.copy(basic_path, tmp)

  newfile <- desc::desc_set("Package", "ggplot2", file = tmp)

  a_cff <- cff_create(tmp, gh_keywords = FALSE)
  expect_length(a_cff$repository, 1)
  expect_equal(clean_str(newfile$get("Package")), "ggplot2")
  expect_equal(a_cff$repository, "https://CRAN.R-project.org/package=ggplot2")

  expect_s3_class(a_cff, "cff")
  expect_snapshot(a_cff)
  expect_true(cff_validate(a_cff, verbose = FALSE))
})


test_that("Search package on r-universe", {
  skip_on_cran()
  skip_if_offline()

  basic_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")

  tmp <- tempfile("DESCRIPTION_basic")
  # Create a temporary file
  file.copy(basic_path, tmp)

  # Get packages from my r-universe
  dhh <- "tidyterra"

  newpack <- desc::desc(tmp)

  oldtitle <- clean_str(newpack$get("Package"))

  newtitle <- desc::desc_set("Package", dhh, file = tmp)

  expect_false(oldtitle == clean_str(newtitle$get("Package")))

  # Configure to search on r-universe
  newrepos <- c(
    dieghernan = "https://dieghernan.r-universe.dev",
    CRAN = "https://cloud.r-project.org"
  )

  runiverse <- as.data.frame(available.packages(
    repos = newrepos
  ))

  expect_equal(
    search_on_repos(dhh, runiverse),
    "https://dieghernan.r-universe.dev/"
  )

  # Search now ggplot2, should be canonical url

  expect_equal(
    search_on_repos("ggplot2", runiverse, newrepos),
    "https://CRAN.R-project.org/package=ggplot2"
  )
})


test_that("Validate keywords", {
  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")

  tmp <- tempfile("DESCRIPTION_keyword")

  copy <- file.copy(desc_path, tmp)

  cffobj <- cff_create(tmp)
  expect_null(cffobj$keywords)

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # Add keywords
  silent <- desc::desc_set(
    "X-schema.org-keywords",
    "keyword1, keyword1, keyword3",
    file = tmp
  )
  cffobj2 <- cff_create(tmp)
  expect_length(cffobj2$keywords, 2)
  expect_equal(cffobj2$keywords, c("keyword1", "keyword3"))
  expect_true(cff_validate(cffobj2, verbose = FALSE))

  # Single keyword
  silent <- desc::desc_set(
    "X-schema.org-keywords",
    "keyword1, keyword1",
    file = tmp
  )
  cffobj3 <- cff_create(tmp)
  expect_length(cffobj3$keywords, 2)
  expect_equal(cffobj3$keywords, c("keyword1", "r-package"))
  expect_true(cff_validate(cffobj3, verbose = FALSE))

  # NULL case keyword
  silent <- desc::desc_set("X-schema.org-keywords", "r-package", file = tmp)
  cffobj4 <- cff_create(tmp)
  expect_null(cffobj4$keywords)
  expect_true(cff_validate(cffobj4, verbose = FALSE))
})


test_that("Coerce keywords from GH", {
  skip_on_cran()
  skip_if_offline()
  skip_if(
    nchar(Sys.getenv("GITHUB_TOKEN")) == 0,
    "No GITHUB_TOKEN environment variable found"
  )

  desc_path <- system.file("examples/DESCRIPTION_basic", package = "cffr")

  tmp <- tempfile("DESCRIPTION_keyword_gh")

  copy <- file.copy(desc_path, tmp)

  cffobj <- cff_create(tmp)
  expect_null(cffobj$keywords)

  expect_true(cff_validate(cffobj, verbose = FALSE))

  # A site with no topics
  silent <- desc::desc_set(
    "BugReports",
    "https://github.com/dieghernan/cfftest/issues",
    file = tmp
  )

  cffobjnokeys <- cff_create(tmp)
  expect_true(cff_validate(cffobjnokeys, verbose = FALSE))
  expect_null(cffobjnokeys$keywords)

  # Add keywords from url
  silent <- desc::desc_set(
    "URL",
    "https://github.com/ropensci/cffr",
    file = tmp
  )

  silent <- desc::desc_set(
    "BugReports",
    "https://github.com/ropensci/cffr/issues",
    file = tmp
  )

  cffobj1 <- cff_create(tmp)
  expect_true(cff_validate(cffobj1, verbose = FALSE))

  skip_if(is.null(cffobj1$keywords), "keywords not gathered")

  expect_false(is.null(cffobj1$keywords))

  # Concatenate keywords of both sources

  # Add keywords
  silent <- desc::desc_set("X-schema.org-keywords", "keyword1", file = tmp)

  cffobj2 <- cff_create(tmp)
  expect_true(cff_validate(cffobj2, verbose = FALSE))

  expect_true(length(cffobj2$keywords) > length(cffobj1$keywords))
})

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
print.cff <- function(x, ...) {
  cat(yaml::as.yaml(x))
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
c.cff <- function(..., recursive = FALSE) {
  args <- list(...)
  args <- lapply(args, unclass)
  rval <- do.call("c", args)

  rval <- as.list(rval)
  # Reclass
  as_cff(rval)
}

# Used for prettier printing when extracting elements.

# Based on the `person` method.
# https://github.com/wch/r-source/blob/trunk/src/library/utils/R/citation.R

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
`[.cff_ref_lst` <- function(x, i) {
  rval <- unclass(x)[i]
  class(rval) <- class(x[[i]])
  rval
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
`[.cff_pers_lst` <- function(x, i) {
  rval <- unclass(x)[i]
  class(rval) <- class(x[[i]])
  rval
}


# nolint start
#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.data.frame.cff <- function(x, row.names = NULL, optional = FALSE, ...) {
  # For better dispatching
  x <- as_cff(as.list(x))

  len <- length(x)
  key_len <- seq_len(len)
  ref_n <- names(x)

  df_l <- lapply(key_len, function(y) {
    el <- x[[y]]
    nm <- ref_n[y]
    nm <- gsub("-", "_", nm, fixed = TRUE)

    if (nm == "preferred_citation") {
      return(as.data.frame(el, prefix = nm))
    }

    if (any(inherits(el, "cff_pers"), inherits(el, "cff_pers_lst"))) {
      return(as.data.frame(el, prefix = nm))
    }

    if (inherits(el, "cff_ref_lst")) {
      return(as.data.frame(el, prefix = nm))
    }

    if (length(el) > 1) {
      ltot <- length(el)
      df <- as.data.frame(matrix(el, nrow = 1))
      nm2 <- paste0(nm, ".", sprintf("%02d", seq_len(ltot) - 1))

      names(df) <- nm2
      return(df)
    }

    df <- as.data.frame(x[y])
    names(df) <- nm
    df
  })

  the_df <- do.call(cbind, df_l)

  as.data.frame(the_df, row.names = row.names, optional = optional, ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.data.frame.cff_pers_lst <- function(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  prefix = "person"
) {
  # For better dispatching
  x <- as_cff(as.list(x))

  len <- length(x)
  key_len <- seq_len(len)

  df_l <- lapply(key_len, function(y) {
    prefix <- paste0(prefix, ".", sprintf("%02d", y - 1))
    el <- x[[y]]
    df <- as.data.frame(el, prefix = prefix)
    df
  })

  the_df <- do.call(cbind, df_l)

  as.data.frame(the_df, row.names = row.names, optional = optional, ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.data.frame.cff_pers <- function(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  prefix = NULL
) {
  # For better dispatching
  x <- as_cff(as.list(x))

  vals <- unlist(x)
  nm <- names(vals)
  nm <- gsub("-", "_", nm, fixed = TRUE)
  amat <- matrix(vals, nrow = 1, ncol = length(vals))
  m <- as.data.frame(amat)

  if (!is.null(clean_str(prefix))) {
    nm <- paste0(prefix, ".", nm)
  }

  names(m) <- nm
  m

  as.data.frame(m, row.names = row.names, optional = optional, ...)
}


#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.data.frame.cff_ref_lst <- function(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  prefix = "references"
) {
  # For better dispatching
  x <- as_cff(as.list(x))

  len <- length(x)
  key_len <- seq_len(len)

  df_l <- lapply(key_len, function(y) {
    prefix <- paste0(prefix, ".", sprintf("%02d", y - 1))
    el <- x[[y]]
    df <- as.data.frame(
      el,
      row.names = row.names,
      optional = optional,
      ...,
      prefix = prefix
    )
    df
  })

  the_df <- do.call(cbind, df_l)

  as.data.frame(the_df, row.names = row.names, optional = optional, ...)
}


#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.data.frame.cff_ref <- function(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  prefix = NULL
) {
  # For better dispatching
  # cff_ref is similar to cff, so we add only cff class
  x <- as_cff(as.list(x))
  class(x) <- "cff"

  the_df <- as.data.frame(x)

  if (!is.null(clean_str(prefix))) {
    names(the_df) <- paste0(prefix, ".", names(the_df))
  }

  the_df
}
# nolint end

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
head.cff <- function(x, n = 6L, ...) {
  as_cff(NextMethod())
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
tail.cff <- function(x, n = 6L, ...) {
  as_cff(NextMethod())
}


#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.list.cff <- function(x, ...) {
  xl <- rapply(
    x,
    function(x) {
      if (is.list(x) || length(x) > 1) {
        return(unclass(x))
      }
      unclass(x)
    },
    how = "list"
  )

  as.list(xl)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
toBibtex.cff <- function(
  object,
  ...,
  what = c("preferred", "references", "all")
) {
  toBibtex(as_bibentry(object, what = what), ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
toBibtex.cff_ref_lst <- function(object, ...) {
  toBibtex(as_bibentry(object), ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
toBibtex.cff_ref <- function(object, ...) {
  toBibtex(as_bibentry(object), ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
toBibtex.cff_pers_lst <- function(object, ...) {
  toBibtex(as.person(object), ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
toBibtex.cff_pers <- function(object, ...) {
  toBibtex(as.person(object), ...)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.person.cff_pers <- function(x) {
  # Enlist to dispatch to Next method
  x_l <- list(as.list(x))
  as.person(as_cff(x_l))
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.person.cff_pers_lst <- function(x) {
  pers <- lapply(x, make_r_person)

  # If not all extracted inform
  if (!all(lengths(pers) > 0)) {
    cli::cli_alert_info(
      "Can't create {.cls person} for some elements of {.arg x}."
    )
  }
  end <- do.call(c, pers)
  if (anyDuplicated(end) > 0) {
    cli::cli_alert_info("Removing duplicate {.cls person} objects.")
    end <- end[!duplicated(end)]
  }
  end
}


#  as.person methods not implemented ----

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.person.cff <- function(x) {
  cli::cli_abort(
    "({.pkg cffr}) {.fn as.person.cff} method not implemented yet."
  )
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.person.cff_ref <- function(x) {
  cli::cli_abort(
    "({.pkg cffr}) {.fn as.person.cff_ref} method not implemented yet."
  )
}

#' @export
#' @encoding UTF-8
#' @rdname cff_class
#' @usage NULL
as.person.cff_ref_lst <- function(x) {
  cli::cli_abort(
    "({.pkg cffr}) {.fn as.person.cff_ref_lst} method not implemented yet."
  )
}

#' Install a \CRANpkg{cffr} GitHub Action
#'
#' @description
#'
#' This function installs a [GitHub
#' Action](https://github.com/features/actions) on your repository. The action
#' updates your `CITATION.cff` when any of these events occur:
#' - You publish a new release of the package.
#' - Your `DESCRIPTION` or `inst/CITATION` file is modified.
#' - The action can be run manually.
#'
#' @param path Project root directory.
#' @param overwrite Logical. If the action already exists, should it be
#'   overwritten?
#'
#' @return Invisible, this function is called by its side effects.
#'
#' @details
#'
#' Triggers on your action can be modified, see
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [Events that trigger workflows]",
#'            "(https://docs.github.com/en/actions/learn-github-actions/",
#'            "events-that-trigger-workflows)."))
#'
#' ```
#'
#' @examples
#' \dontrun{
#' cff_gha_update()
#' }
#' @export
#' @encoding UTF-8
#'
#' @family git
cff_gha_update <- function(path = ".", overwrite = FALSE) {
  destdir <- file.path(path, ".github", "workflows")
  checkdir <- dir.exists(destdir)

  if (isFALSE(checkdir)) {
    cli::cli_alert_info("Creating directory {.path {destdir}}.")
    dir.create(destdir, recursive = TRUE, showWarnings = FALSE)
  }

  newfile <- file.path(destdir, "update-citation-cff.yaml")

  if (!file_exist_abort(newfile) || isTRUE(overwrite)) {
    cli::cli_alert_success("Installing {.file {newfile}}")

    file.copy(
      system.file("yaml/update-citation-cff.yaml", package = "cffr"),
      newfile,
      overwrite = TRUE
    )
  } else {
    cli::cli_alert_warning(
      paste0(
        "File {.file {newfile}} already installed. ",
        "Use {.arg overwrite = TRUE}  for overwrite"
      )
    )
  }

  if (file_exist_abort(file.path(path, ".Rbuildignore"))) {
    ignore <- readLines(file.path(path, ".Rbuildignore"))

    # If not already
    if (!("^\\.github$" %in% ignore)) {
      ignore <- c(ignore, "^\\.github$")
      ignore <- unique(ignore)
      cli::cli_alert_info("Adding {.val .github} to {.file .Rbuildignore}")
      writeLines(ignore, ".Rbuildignore")
    }
  }

  invisible()
}

test_that("Test in mock package", {
  skip_on_cran()
  current_dir <- getwd()

  name <- paste0("mock-pack", runif(1) * 10)
  new_dir <- file.path(tempdir(), name)

  dir.create(new_dir, recursive = TRUE)

  expect_true(dir.exists(new_dir))

  setwd(new_dir)

  # Move files
  file.copy(
    system.file("examples/DESCRIPTION_many_urls", package = "cffr"),
    to = "DESCRIPTION"
  )

  # Create Rbuildignore
  file.create(".Rbuildignore", showWarnings = FALSE)
  expect_true(file_exist_abort(".Rbuildignore"))

  # Add action
  expect_snapshot(cff_gha_update())

  expect_snapshot(cff_gha_update())
  expect_snapshot(cff_gha_update(overwrite = TRUE))

  expect_true(file_exist_abort(file.path(
    ".github",
    "workflows",
    "update-citation-cff.yaml"
  )))

  # Revert to initial wd
  setwd(current_dir)

  unlink(new_dir, recursive = TRUE, force = TRUE)
})

#' Error if it is not a `cff` file or object
#' @param x File to be evaluated.
#' @noRd
abort_if_not_cff <- function(x) {
  if (is_cff(x)) {
    return(invisible())
  }

  # x should be character at least
  if (!inherits(x, "character")) {
    cli::cli_abort(
      "{.var x} is an object of class {.cls {class(x)}}, not {.cls cff}."
    )
  }

  guess <- detect_x_source(x)

  if (guess != "cff_citation") {
    cli::cli_abort(
      "{.var x} is not a {.file *.cff} file."
    )
  }
}

#' Throw an error if the file does not exist.
#' @param x A file to be evaluated.
#' @param abort Logical. Should an error be thrown if file does not exist?
#' @noRd
file_exist_abort <- function(x, abort = FALSE) {
  res <- file.exists(x)

  if (all(abort, isFALSE(res))) {
    cli::cli_abort(
      "{.file {x}} doesn't exist. Check the {.file {dirname(x)}} directory"
    )
  }
  invisible(res)
}

match_cff_arg <- function(arg, valid, for_msg, call = environment()) {
  arg <- as.character(arg)[1]
  valid <- as.character(valid)

  if (!arg %in% valid) {
    cli::cli_abort(
      "{.arg {for_msg}} should be {.or  {.val {valid}}}, not {.val {arg}}.",
      call = call
    )
  }

  arg
}

write_lines_msg <- function(lines, file, verbose, append) {
  # Check that the directory exists, if not create
  dir <- dirname(path.expand(file))
  if (!dir.exists(dir)) {
    if (verbose) {
      cli::cli_alert_info("Creating directory {.path {dir}}")
    }
    dir.create(dir, recursive = TRUE)
  }

  # If exists creates a backup
  if (file_exist_abort(file)) {
    for (i in seq(1, 100)) {
      f <- paste0(file, ".bk", i)
      if (!file_exist_abort(f)) break
    }

    if (verbose) {
      cli::cli_alert_info(
        "Creating a backup of {.file {file}} in {.file {f}}"
      )
    }
    file.copy(file, f)
  }

  fh <- file(file, encoding = "UTF-8", open = ifelse(append, "a+", "w+"))
  on.exit(if (isOpen(fh)) close(fh))
  if (verbose) {
    cli::cli_alert_info("Writing {length(lines[lines != ''])} entr{?y/ies} ...")
  }

  writeLines(lines, fh)
  if (verbose) {
    cli::cli_alert_success("Results written to {.file {file}}")
  }
}

test_that("Check abort_if_not_cff", {
  obj <- cff()

  expect_silent(abort_if_not_cff(obj))

  file <- system.file("examples/CITATION_skeleton.cff", package = "cffr")

  expect_silent(abort_if_not_cff(file))

  # Error

  err <- "Some string"

  expect_snapshot(abort_if_not_cff(err), error = TRUE)

  # Other objects
  x <- data.frame(a = 1)
  expect_snapshot(abort_if_not_cff(x), error = TRUE)
})

test_that("Check file_exist_abort", {
  nonexits <- "aaaaaaaa"
  expect_silent(file_exist_abort(nonexits))
  expect_false(file_exist_abort(nonexits))

  # Signal an error
  expect_snapshot(file_exist_abort(nonexits, abort = TRUE), error = TRUE)
  # With dir
  non_with_path <- "./R/no/existe"
  expect_snapshot(file_exist_abort(non_with_path, abort = TRUE), error = TRUE)

  tmp <- tempfile()

  writeLines("a", tmp)
  expect_true(file_exist_abort(tmp))
  expect_true(file_exist_abort(tmp, abort = TRUE))

  unlink(tmp)
})

test_that("Check match_cff_arg", {
  x <- "aaaaaaaa"
  expect_snapshot(match_cff_arg("a", "b", "..."), error = TRUE)
  expect_snapshot(match_cff_arg("a", c("b", "c", "d"), "what"), error = TRUE)

  # Test call
  my_fun <- function(x = "a") {
    match_cff_arg("a", LETTERS[1:5], "aa", environment())
  }

  expect_snapshot(my_fun(), error = TRUE)
  expect_silent(a <- match_cff_arg(c("a", "b"), letters, "..."))
  expect_identical(a, "a")
})

#' Previous API: Create BibTeX entries from several sources
#'
#' @description
#' `r lifecycle::badge('deprecated')` Please use [as_bibentry()] instead.
#'
#' @rdname deprecated_cff_to_bib
#' @inheritParams as_bibentry
#' @export
#' @encoding UTF-8
#' @keywords internal
#' @family deprecated
#'
#' @return See [as_bibentry()].
#' @examples
#' \donttest{
#' # From a cff object
#' cff_object <- cff()
#'
#' cff_object
#'
#' # bibentry object
#' bib <- as_bibentry(cff_object)
#' }
cff_extract_to_bibtex <- function(
  x,
  what = c("preferred", "references", "all")
) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "0.5.0",
      "cff_extract_to_bibtex()",
      details = "Function renamed, use `as_bibentry()` instead."
    )
  }
  as_bibentry(x, what)
}

#' @rdname deprecated_cff_to_bib
#' @export
#' @encoding UTF-8
#' @keywords internal
cff_to_bibtex <- function(x, what = c("preferred", "references", "all")) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "cff_extract_to_bibtex()",
      details = "Function renamed, use `as_bibentry()` instead."
    )
  }
  as_bibentry(x, what)
}

#' Previous API: Create a [`cff`] object from BibTeX entries
#'
#' @description
#'
#' `r lifecycle::badge('deprecated')` Please use either [cff_read_bib()] or
#' [cff_read_bib_text()] instead.
#'
#' @rdname deprecated_cff_from_bib
#' @export
#' @encoding UTF-8
#' @keywords internal
#' @family deprecated
#'
#' @param x The source used to generate the
#'   [`cff`] object. Must be a `character` object indicating either:
#'   - The path to a BibTeX file.
#'   - A vector of characters with a complete BibTeX string. See **Examples**.
#' @param encoding Encoding to be assumed for `x`. See [readLines()].
#' @param ... Other arguments passed to [bibtex::read.bib()].
#'
#' @return
#'
#' See [cff_read_bib()] for reading `*.bib` files and [cff_read_bib_text()]
#' for reading a `character` object representing a BibTeX entry.
#'
#' @examples
#' if (requireNamespace("bibtex", quietly = TRUE)) {
#'   x <- c(
#'     "@book{einstein1921,
#'     title        = {Relativity: The Special and the General Theory},
#'     author       = {Einstein, Albert},
#'     year         = 1920,
#'     publisher    = {Henry Holt and Company},
#'     address      = {London, United Kingdom},
#'     isbn         = 9781587340925
#' }",
#'     "@misc{misc-full,
#'     title        = {Handing out random pamphlets in airports},
#'     author       = {Joe-Bob Missilany},
#'     year         = 1984,
#'     month        = oct,
#'     note         = {This is a full MISC entry},
#'     howpublished = {Handed out at O'Hare}
#' }"
#'   )
#'
#'   cff_read_bib_text(x)
#'
#'   # From a file
#'
#'   x2 <- system.file("examples/example.bib", package = "cffr")
#'   cff_read_bib(x2)
#' }
cff_from_bibtex <- function(x, encoding = "UTF-8", ...) {
  if (length(x) == 1 && file_exist_abort(x)) {
    if (requireNamespace("lifecycle", quietly = TRUE)) {
      lifecycle::deprecate_warn(
        "1.0.0",
        "cff_from_bibtex()",
        "cff_read_bib()"
      )
    }

    # Read bib file
    return(cff_read_bib(x, encoding = encoding, ...))
  }

  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "cff_from_bibtex()",
      "cff_read_bib_text()"
    )
  }
  cff_read_bib_text(x, encoding = encoding, ...)
}


#' Previous API: Write files
#'
#' @description
#'
#' `r lifecycle::badge('deprecated')` Please use [cff_write_bib()] or
#' [cff_write_citation()] instead.
#'
#' @rdname deprecated_write
#' @export
#' @encoding UTF-8
#' @keywords internal
#' @family deprecated
#' @inheritParams cff_write_bib
#'
#' @return Write a file.
#'
#' @seealso
#' - [cff_write_bib()] for writing `*.bib` files.
#' - [cff_write_citation()] for writing **R** `CITATION` files.
#'
#' @examples
#'
#' bib <- bibentry("Misc",
#'   title = "My title",
#'   author = "Fran Pérez"
#' )
#'
#' my_temp_bib <- tempfile(fileext = ".bib")
#'
#' cff_write_bib(bib, file = my_temp_bib)
#'
#' cat(readLines(my_temp_bib), sep = "\n")
#'
#' cff_write_bib(bib, file = my_temp_bib, ascii = TRUE, append = TRUE)
#'
#' cat(readLines(my_temp_bib), sep = "\n")
write_bib <- function(
  x,
  file = tempfile(),
  append = FALSE,
  verbose = TRUE,
  ascii = FALSE
) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "write_bib()",
      "cff_write_bib()"
    )
  }

  cff_write_bib(x, file, append, verbose, ascii)
}

#' @rdname deprecated_write
#' @export
#' @encoding UTF-8
write_citation <- function(
  x,
  file = "./inst/CITATION",
  append = FALSE,
  verbose = TRUE,
  ...
) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "write_citation()",
      "cff_write_citation()"
    )
  }
  cff_write_citation(x, file, append, verbose, ...)
}


#' Previous API: Parse a `person` to [`cff`]
#'
#' @description
#'
#' `r lifecycle::badge('deprecated')` Please use [as_cff_person()]
#'
#' @rdname deprecated_cff_person
#' @export
#' @encoding UTF-8
#' @keywords internal
#' @family deprecated
#' @param person It can be either:
#'   - A `person` or list of `person` object created with [utils::person()].
#'   - A `character` object or vector representing a person or persons.
#'
#' @return A person in format `cff`.
#'
#' @seealso [as_cff_person()]
#'
#' @examples
#' # Create a person object
#' a_person <- person(
#'   given = "First", family = "Author",
#'   role = c("aut", "cre"),
#'   email = "first.last@example.com", comment = c(
#'     ORCID = "0000-0001-8457-4658",
#'     affiliation = "An affiliation"
#'   )
#' )
#'
#' a_person
#'
#' cff_person <- as_cff_person(a_person)
#'
#' cff_person
#'
#' # Back to person object with S3 Method
#' as.person(cff_person)
#'
#' # Parse a string
#' a_str <- paste0(
#'   "Julio Iglesias <fake@email.com> ",
#'   "(<https://orcid.org/0000-0001-8457-4658>)"
#' )
#' as_cff_person(a_str)
#'
#' # Several persons
#' persons <- c(person("Clark", "Kent"), person("Lois", "Lane"))
#'
#' as_cff_person(persons)
#'
#' # Or you can use BibTeX style if you prefer
#'
#' x <- "Frank Sinatra and Dean Martin and Davis, Jr., Sammy and Joey Bishop"
#'
#' as_cff_person(x)
#'
#' as_cff_person("Herbert von Karajan")
cff_parse_person <- function(person) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "cff_parse_person()",
      "as_cff_person()"
    )
  }
  as_cff_person(person)
}

#' @rdname deprecated_cff_person
#' @export
#' @encoding UTF-8
#'
cff_parse_person_bibtex <- function(person) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "cff_parse_person_bibtex()",
      "as_cff_person()"
    )
  }
  as_cff_person(person)
}

#' Previous API: Parse a `bibentry` to `cff`
#'
#' @description
#'
#' `r lifecycle::badge('deprecated')` Please use [as_cff.bibentry()] method
#'
#' @rdname deprecated_cff_bibentry
#' @export
#' @encoding UTF-8
#' @keywords internal
#' @family deprecated
#'
#' @param bib A `bibentry` object.
#' @return A `bibentry` in format [`cff`].
#'
#' @seealso [as_cff.bibentry()]
#'
#' @examples
#'
#' bib <- citation("base")
#' bib
#'
#' # To cff
#' bib_to_cff <- as_cff(bib)
#' bib_to_cff
#'
cff_parse_citation <- function(bib) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "cff_parse_citation()",
      "as_cff.bibentry()"
    )
  }
  as_cff(bib)
}

test_that("cff_extract_to_bibtex", {
  a_cff <- cff_create("cffr")

  current <- as_bibentry(a_cff)
  expect_snapshot(old1 <- cff_extract_to_bibtex(a_cff))

  expect_identical(current, old1)
})

test_that("cff_to_bibtex", {
  a_cff <- cff_create("cffr")

  current <- as_bibentry(a_cff)
  expect_snapshot(old1 <- cff_to_bibtex(a_cff))

  expect_identical(current, old1)
})

test_that("cff_from_bibtex", {
  # From file
  x2 <- system.file("examples/example.bib", package = "cffr")
  expect_snapshot(ffile <- cff_from_bibtex(x2))
  expect_identical(ffile, cff_read_bib(x2))

  # From lines
  x <- "@book{einstein1921,
          title        = {Relativity: The Special and the General Theory},
          author       = {Einstein, Albert},
          year         = 1920,
          publisher    = {Henry Holt and Company},
          address      = {London, United Kingdom},
          isbn         = 9781587340925
      }"

  expect_snapshot(flines <- cff_from_bibtex(x))
  expect_identical(flines, cff_read_bib_text(x))
})

test_that("write_bib", {
  bib <- bibentry("Misc", title = "My title", author = "Fran Pérez")

  tmp <- tempfile(fileext = ".bib")
  expect_snapshot(write_bib(bib, tmp, verbose = FALSE))

  expect_snapshot(cat(readLines(tmp), sep = "\n"))
})

test_that("write_citation", {
  bib <- bibentry("Misc", title = "My title", author = "Fran Pérez")

  tmp <- tempfile("CIT_ATION")
  expect_snapshot(write_citation(bib, tmp, verbose = FALSE))

  expect_snapshot(cat(readLines(tmp), sep = "\n"))
})

test_that("cff_parse_person", {
  p <- person("A", "person")
  expect_snapshot(pend <- cff_parse_person(p))

  expect_identical(pend, as_cff_person(p))
})

test_that("cff_parse_person_bibtex", {
  p <- "{Elephant and Castle}"
  expect_snapshot(pend <- cff_parse_person_bibtex(p))

  expect_identical(pend, as_cff_person(p))
})


test_that("cff_parse_citation", {
  p <- citation("testthat")
  expect_snapshot(pend <- cff_parse_citation(p))

  expect_identical(pend, as_cff(p))
})

# Utils for as_cff_reference

#' Extract and map BibTeX entry
#' @noRd
get_bibtex_entry <- function(bib) {
  # Unclass and manage entry type
  # Extract type from BibTeX
  init_type <- attr(unclass(bib)[[1]], "bibtype")
  init_type <- clean_str(tolower(init_type))

  cit_list <- drop_null(unclass(bib)[[1]])

  # Add fields
  cit_list$bibtex_entry <- init_type

  # Manage type from BibTeX and convert to CFF
  # This overwrite the BibTeX type field. Not treated by this function
  cit_list$type <- switch(init_type,
    "article" = "article",
    "book" = "book",
    "booklet" = "pamphlet",
    "conference" = "conference-paper",
    "inbook" = "book",
    # "incollection" = ,
    "inproceedings" = "conference-paper",
    "manual" = "manual",
    "mastersthesis" = "thesis",
    "misc" = "generic",
    "phdthesis" = "thesis",
    "proceedings" = "proceedings",
    "techreport" = "report",
    "unpublished" = "unpublished",
    "generic"
  )

  # Check if it an inbook with booktitle (BibLaTeX style)
  if (all(init_type == "inbook", "booktitle" %in% names(cit_list))) {
    # Make it incollection
    cit_list$bibtex_entry <- "incollection"
    cit_list$type <- "generic"
  }

  cit_list
}

#' Adapt names from R citation()/BibTeX to cff format
#' @noRd
get_bibtex_fields <- function(cit_list) {
  # to lowercase
  names(cit_list) <- tolower(names(cit_list))
  nm <- names(cit_list)
  # Standard BibTeX fields:
  # address annote author booktitle chapter crossref edition editor
  # howpublished institution journal key month note number organization pages
  # publisher school series title type year

  # No mapping needed (direct mapping)
  # edition journal month publisher title volume year

  # Mapped:
  # author booktitle series chapter editor howpublished note number

  nm[nm == "author"] <- "authors"
  # Make collection title
  # booktitle takes precedence over series
  nm[nm == "booktitle"] <- "collection-title"
  if (!"collection-title" %in% nm) {
    nm[nm == "series"] <- "collection-title"
  }
  nm[nm == "chapter"] <- "section"
  nm[nm == "editor"] <- "editors"
  nm[nm == "howpublished"] <- "medium"
  nm[nm == "note"] <- "notes"
  nm[nm == "number"] <- "issue"
  nm[nm == "address"] <- "location"
  nm[nm == "pages"] <- "bibtex_pages" # This would be removed later

  # Get some fields from BibLaTeX
  nm[nm == "date"] <- "date-published"
  nm[nm == "file"] <- "filename"
  nm[nm == "issuetitle"] <- "issue-title"
  nm[nm == "translator"] <- "translators"
  nm[nm == "urldate"] <- "date-accessed"
  nm[nm == "pagetotal"] <- "pages"

  # Other BibLaTeX fields that does not require any mapping
  # abstract, doi, isbn, issn, url, version

  # Keywords may be duplicated, unify
  if ("keywords" %in% nm) {
    kwords <- unlist(cit_list["keywords" == nm])
    kwords <- clean_str(paste(kwords, collapse = ", "))
    kwords <- trimws(unique(unlist(strsplit(kwords, ",|;"))))
    cit_list$keywords <- unique(kwords)
  }

  # Not mapped:
  # annote crossref key organization series type
  #
  # Fields address, organization, series and type are treated on
  # main function
  # key is a special field, treated apart
  # Fields ignored: annote, crossref

  names(cit_list) <- nm

  # Additionally, need to delete keywords if length is less than 2,
  # errors on validation
  if (length(cit_list$keywords) < 2) {
    cit_list$keywords <- NULL
  }

  # Treat location ----

  loc <- cit_list$location

  if (!is.null(loc)) {
    cit_list$location <- loc
  }

  # Treat additional dates ----
  dpub <- clean_str(cit_list$`date-published`)
  cit_list$`date-published` <- clean_str(as.Date(dpub, optional = TRUE))

  datacc <- clean_str(cit_list$`date-accessed`)
  cit_list$`date-accessed` <- clean_str(as.Date(datacc, optional = TRUE))

  # Treat pages

  pages <- cit_list$bibtex_pages
  if (!is.null(pages)) {
    spl <- unlist(strsplit(pages, "--"))

    cit_list$start <- spl[1]

    if (length(spl) > 1) cit_list$end <- paste(spl[-1], collapse = "--")
  }

  cit_list
}

#' Modify mapping of some org. fields on BibTeX to CFF
#' @noRd
get_bibtex_inst <- function(field_list) {
  # Initial values
  bibtex_entry <- field_list$bibtex_entry
  to_replace <- switch(bibtex_entry,
    "mastersthesis" = "school",
    "phdthesis" = "school",
    "conference" = "organization",
    "inproceedings" = "organization",
    "manual" = "organization",
    "proceedings" = "organization",
    "institution"
  )

  if (to_replace == "institution") {
    return(field_list)
  }

  # Rest of cases remove bibtex institution and rename
  nms <- names(field_list)
  field_list <- field_list["institution" != nms]

  # Rename
  nms2 <- names(field_list)
  nms2[nms2 == to_replace] <- "institution"
  names(field_list) <- nms2

  field_list
}

add_conference <- function(field_list) {
  bibtex_entry <- field_list$bibtex_entry

  if (bibtex_entry %in% c("conference", "inproceedings", "proceedings")) {
    field_list$conference <- field_list$`collection-title`
  }
  field_list
}

#' Adapt cff keys to bibtex entries
#' @noRd
add_thesis <- function(cit_list) {
  bibtex_entry <- cit_list$bibtex_entry
  if (!bibtex_entry %in% c("phdthesis", "mastersthesis")) {
    return(cit_list)
  }

  cit_list$`thesis-type` <- switch(bibtex_entry,
    phdthesis = "PhD Thesis",
    "Master's Thesis"
  )

  cit_list
}

add_address <- function(cit_list) {
  loc <- cit_list$location$name
  # If available
  if (is.null(loc)) {
    return(cit_list)
  }

  # At this point is in location, see to move

  # Logic order.
  # 1. To conference
  # 2. To institution
  # 3. To publisher
  # Otherwise leave on location

  nms <- names(cit_list)
  has_conf <- "conference" %in% nms
  has_inst <- "institution" %in% nms
  has_publish <- "publisher" %in% nms

  if (!any(has_conf, has_inst, has_publish)) {
    return(cit_list)
  }

  if (has_conf) {
    cit_list$conference$address <- loc
    cit_list$location <- NULL
  } else if (has_inst) {
    cit_list$institution$address <- loc
    cit_list$location <- NULL
  } else {
    cit_list$publisher$address <- loc
    cit_list$location <- NULL
  }

  cit_list
}

add_bibtex_coltype <- function(field_list) {
  # Add collection-type if applicable and rearrange fields
  nms <- names(field_list)

  if (!"collection-title" %in% nms) {
    return(field_list)
  }

  # Made collection-type if we create collection-title
  bibtex_type <- field_list$bibtex_entry

  # Remove `in` at init: inbook, incollection affected
  coltype <- clean_str(gsub("^in", "", bibtex_type))
  field_list$`collection-type` <- coltype

  # Rearrange to make both collection keys together
  nm_first <- nms[seq(1, match("collection-title", nms))]

  nms_end <- unique(c(nm_first, "collection-type", nms))

  field_list <- field_list[nms_end]

  field_list
}

fallback_dates <- function(cit_list) {
  # Fallback for year and month: use date-published
  if (is.null(cit_list$month) && !is.null(cit_list$`date-published`)) {
    cit_list$month <- format(as.Date(cit_list$`date-published`), "%m")
  }

  if (is.null(cit_list$year) && !is.null(cit_list$`date-published`)) {
    cit_list$year <- format(as.Date(cit_list$`date-published`), "%Y")
  }

  ## month ----
  cit_list$month <- get_bibtex_month(cit_list)

  cit_list
}

#' Build the DOI field list.
#' @noRd
get_bibtex_doi <- function(cit_list) {
  dois <- unlist(cit_list[names(cit_list) == "doi"])

  # Check urls as well
  url_for_doi <- unlist(cit_list$url)
  if (all(!is.null(url_for_doi), grepl("doi.org", url_for_doi))) {
    dois <- c(dois, url_for_doi)
  }

  dois <- unlist(lapply(dois, function(x) {
    x <- gsub("^.*doi.org/", "", x)
    x <- clean_str(x)
  }))

  dois <- unique(as.character(dois))

  # The first doi goes to doi key
  doi <- unlist(dois[1])

  # The rest goes to identifies
  identifiers <- lapply(dois[-1], function(x) {
    list(
      type = "doi",
      value = clean_str(x)
    )
  })
  if (length(identifiers) == 0) {
    identifiers <- NULL
  }
  doi_list <- list(
    doi = clean_str(doi),
    identifiers = identifiers
  )
  doi_list
}

#' Build the month field.
#' @noRd
get_bibtex_month <- function(cit_list) {
  mnt <- clean_str(cit_list$month)

  if (is.null(mnt)) {
    return(NULL)
  }

  # If number
  if (grepl("^\\d+$", mnt)) {
    # Guess if a valid integer is provided and output
    mnt_num <- as.numeric(mnt)
    mnt_num <- mnt_num[mnt_num %in% seq(1, 12)]
    return(clean_str(mnt_num))
  }

  # else transform
  # Get month, everything in lowercase
  month <- clean_str(tolower(mnt))
  month <- substr(month, 1, 3)

  # Index on abbreviation
  low_month <- tolower(month.abb)
  res <- seq(1, 12)[month == low_month]
  clean_str(res[1])
}

#' Build the URL field list.
#' @noRd
get_bibtex_url <- function(cit_list) {
  ## Get url: see bug with cff_create("rgeos")
  if (is.character(cit_list$url)) {
    allurls <- as.character(cit_list[names(cit_list) == "url"])
    allurls <- unlist(strsplit(allurls, " |,|\\n"))
  } else {
    allurls <- cit_list$url
  }

  allurls <- allurls[is_url(allurls)]
  # The first url goes to url key

  url <- unlist(allurls[1])

  # The rest goes to identifies
  identifiers <- lapply(allurls[-1], function(x) {
    list(
      type = "url",
      value = clean_str(x)
    )
  })

  if (length(identifiers) == 0) {
    identifiers <- NULL
  }

  url_list <- list(
    url = clean_str(url),
    identifiers = identifiers
  )

  url_list
}

#' Build fields for additional people.
#' @noRd
get_bibtex_other_pers <- function(field_list) {
  others <- drop_null(field_list[other_persons()])

  # Select subsets
  all_pers <- other_persons()
  toent <- other_persons_entity()
  toent_pers <- entity_person()

  toauto_end <- all_pers[!all_pers %in% c(toent, toent_pers)]
  toent_end <- toent[!toent %in% toent_pers]

  # As persons or entities using bibtex
  toentity_pers <- others[names(others) %in% toent_pers]
  toentity_pers <- lapply(toentity_pers, function(x) {
    if (inherits(x, "person")) {
      x <- format(
        x,
        include = c("given", "family")
      )
    }

    bibtex <- paste(x, collapse = " and ")
    end <- as_cff_person(bibtex)

    end
  })

  # Rest

  rest <- others[!names(others) %in% toent_pers]

  # If any has several persons then paste and collapse
  rest <- lapply(others, function(x) {
    if (length(x) > 1) {
      and <- paste(
        format(
          x,
          include = c("given", "family")
        ),
        collapse = " and "
      )
      and
    } else if (inherits(x, "person")) {
      as_cff(x)
    } else {
      x
    }
  })

  # Entity
  toentity <- rest[names(rest) %in% toent_end]
  toentity <- lapply(toentity, function(x) {
    if (inherits(x, "cff")) {
      return(unclass(unlist(x, recursive = FALSE)))
    }
    list(name = clean_str(x))
  })

  toperson <- rest[names(rest) %in% toauto_end]
  toperson <- lapply(toperson, as_cff_person)

  # Bind and reorder
  other_list <- c(toentity, toperson, toentity_pers)
  other_list <- other_list[names(others)]

  other_list
}

test_that("fallback_dates", {
  xinit <- list(year = "2025")
  p <- fallback_dates(list(year = "2025"))
  expect_identical(xinit, p)

  p2 <- fallback_dates(list("date-published" = "2025-09-25"))
  expect_snapshot(as_cff(p2))
})

#' Coerce **R** objects to [`cff_pers_lst`] objects (`cff` persons)
#'
#' @description
#' `as_cff_person()` turns an existing list-like **R** object into a
#' [`cff_pers_lst`] object representing a list of `definitions.person` or
#' `definitions.entity`, as defined by the
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [Citation File Format schema]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#'
#' `as_cff_person` is an S3 generic, with methods for:
#' - `person`: Objects created with [person()].
#' - `character`: Strings with the definition of an author or multiple authors,
#'   using the standard BibTeX notation (see Markey, 2007) and others, such as
#'   the output of [format()] for person (see [`format.person()`][person()]).
#' - Default: Other inputs are first coerced with [as.character()].
#'
#' The inverse transformation (`cff_pers_lst` to `person`) can be done using
#' the methods [as.person.cff_pers()] and [as.person.cff_pers_lst()].
#'
#' @seealso
#' Examples in `vignette("cffr", "cffr")` and [utils::person()].
#'
#' Learn more about the `cff_pers_lst` and `cff_pers` classes in [cff_class].
#'
#' @export
#' @encoding UTF-8
#' @rdname as_cff_person
#' @name as_cff_person
#' @order 1
#'
#' @family s3method
#'
#' @param x Any **R** object.
#' @param ... Ignored by this method.
#'
#' @return
#' `as_cff_person()` returns an object of classes
#' [`cff_pers_lst, cff`][cff_pers_lst] according to the `definitions.person`
#' or `definitions.entity` specified in the
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [Citation File Format schema]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#' Each element of the `cff_pers_lst` object has classes
#' [`cff_pers, cff`][cff_pers].
#'
#' @details
#'
#' `as_cff_person()` recognizes whether the input should be converted using the
#' CFF reference for `definitions.person` or `definitions.entity`.
#'
#' `as_cff_person()` uses a custom algorithm that breaks a name as
#' explained in Section 11 of "Tame the BeaST" (Markey, 2007) (see also
#' Decoret, 2007):
#'
#'  - `First von Last`.
#'  - `von Last, First`.
#'  - `von Last, Jr, First`.
#'
#'  Mapping is performed as follows:
#'  - `First` is mapped to the CFF field `given-names`.
#'  - `von` is mapped to the CFF field `name-particle`.
#'  - `Last` is mapped to the CFF field `family-names`.
#'  - `Jr` is mapped to the CFF field `name-suffix`.
#'
#'  For entities, the entire `character` is mapped to `name`.
#'  It is recommended to "protect" entity names with `{}`:
#'
#' ```{r child = "man/chunks/person.Rmd"}
#' ```
#' `as_cff_person()` attempts to extract as much information as possible.
#' For `character` strings from [`format.person()`][utils::person()], the
#' email and ORCID are also extracted.
#'
#' @references
#' - Patashnik, Oren. "BIBTEXTING" February 1988.
#'   <https://osl.ugr.es/CTAN/biblio/bibtex/base/btxdoc.pdf>.
#'
#' - Markey, Nicolas. "Tame the BeaST"
#'   *The B to X of BibTeX, Version 1.4* (October 2007).
#'   <https://osl.ugr.es/CTAN/info/bibtex/tamethebeast/ttb_en.pdf>.
#'
#' - Decoret X (2007). "A summary of BibTex."
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("<https://maverick.inria.fr/~Xavier.Decoret/resources/xdkbibtex/",
#'            "bibtex_summary.html#names>"))
#'
#' ```
#'
#' See **Examples** for more information.
#'
#' @examples
#' # Create a person object
#' a_person <- person(
#'   given = "First", family = "Author",
#'   role = c("aut", "cre"),
#'   email = "first.last@example.com", comment = c(
#'     ORCID = "0000-0001-8457-4658",
#'     affiliation = "An affiliation"
#'   )
#' )
#'
#' a_person
#'
#' cff_person <- as_cff_person(a_person)
#'
#' # Class cff_pers_lst / cff
#' class(cff_person)
#'
#' # With each element with class cff_pers / cff
#' class(cff_person[[1]])
#'
#' # Print
#' cff_person
#'
#' # Back to person object with S3 Method
#' as.person(cff_person)
#'
#' # Coerce a string
#' a_str <- paste0(
#'   "Julio Iglesias <fake@email.com> ",
#'   "(city: Miami, region: California, country: US)"
#' )
#' as_cff_person(a_str)
#'
#' # Several persons
#' persons <- c(
#'   person("Clark", "Kent", comment = c(affiliation = "Daily Planet")),
#'   person("Lois", "Lane"), person("Oscorp Inc.")
#' )
#'
#' a_cff <- as_cff_person(persons)
#'
#' a_cff
#'
#' # Printed as Bibtex thanks to the method
#' toBibtex(a_cff)
#'
#' # Or as person object
#' as.person(a_cff)
#'
#' # Or you can use BibTeX style as input if you prefer
#'
#' x <- "Frank Sinatra and Dean Martin and Davis, Jr., Sammy and Joey Bishop"
#'
#' as_cff_person(x)
#'
#' as_cff_person("Herbert von Karajan")
#'
#' toBibtex(as_cff_person("Herbert von Karajan"))
as_cff_person <- function(x, ...) {
  UseMethod("as_cff_person")
}


#' @export
#' @encoding UTF-8
#' @rdname as_cff_person
#' @order 2
as_cff_person.default <- function(x, ...) {
  # Check if this is protected
  if (!inherits(x, "Bibtex")) {
    x <- clean_str(x)
  }
  if (is.null(x)) {
    return(NULL)
  }

  x_char <- unname(as.character(x))

  as_cff_person(x_char)
}

#' @export
#' @encoding UTF-8
#' @rdname as_cff_person
#' @order 3
as_cff_person.person <- function(x, ...) {
  the_obj <- lapply(x, create_person_from_r)

  if (!length(the_obj) > 0) {
    return(NULL)
  }
  the_obj <- as_cff(the_obj)
  the_obj
}

#' @export
#' @encoding UTF-8
#' @rdname as_cff_person
#' @order 4
as_cff_person.character <- function(x, ...) {
  # Maybe is protected ...
  if (any(grepl("{", x, fixed = TRUE))) {
    test_x <- x
  } else {
    test_x <- clean_str(x)
  }

  if (is.null(test_x)) {
    return(NULL)
  }

  # Need to split the character
  person_split <- split_txt_persons(x)
  the_obj <- lapply(person_split, create_person_from_txt)

  the_obj <- as_cff(the_obj)
  the_obj
}


create_person_from_r <- function(person) {
  person <- as.person(person)

  # Special case for Bioconductor

  if (is_substring(person$given, "Bioconductor")) {
    person <- person(
      given = paste(
        clean_str(person$given),
        clean_str(person$family)
      ),
      email = person$email,
      role = person$role,
      comment = person$comment
    )
  }

  # Special case for R Core Team
  if (
    all(
      is_substring(clean_str(person$given), "R Core"),
      is_substring(person$family, "Team")
    )
  ) {
    person <- person(
      given = paste(
        clean_str(person$given),
        clean_str(person$family)
      ),
      email = person$email,
      role = person$role,
      comment = person$comment
    )
  }

  # Guess if entity of person.
  is_entity <- is.null(person$family) || is.null(person$given)

  # Create a text version of the person, would be treated as bibtex

  if (is_entity) {
    as_bib_text <- paste(c(person$family, person$given), collapse = " ")
    # And protect it
    as_bib_text <- protect_bib_braces(as_bib_text)
  } else {
    # Use von Family, Junior, Given
    # Protect given
    giv <- paste0(person$given, collapse = " ")
    giv <- protect_bib_braces(giv)
    as_bib_text <- paste0(c(person$family, giv), collapse = ", ")
  }

  pers_cff <- create_person_from_txt(as_bib_text)
  if (getRversion() < "4.5.0") {
    comm_cff <- extract_person_comments(person) # nocov
  } else {
    comm_cff <- extract_person_comments45(person)
  }
  # Add comments
  pers_cff <- c(pers_cff, comm_cff)

  # Validate fields
  pers_cff <- validate_cff_person_fields(pers_cff)
  pers_cff
}

create_person_from_txt <- function(as_bib_text) {
  # Can extract comments as R
  # Locate comment with R format.person default pattern
  # 'first last [role] <e@mail> (comment)
  # but we protect this if inside brackets
  comments_pattern <- "<|>|\\(|\\)|\\[|\\]"

  protected <- gsub(
    paste0("(", comments_pattern, ")(?![^\\}]*(\\{|$))"),
    "0",
    as_bib_text,
    perl = TRUE
  )

  start_comment <- min(unlist(regexpr(comments_pattern, protected)))

  if (start_comment > 0) {
    # has comments
    person_only <- trimws(substr(as_bib_text, 1, start_comment - 1))
    comment_only <- trimws(substr(
      as_bib_text,
      start_comment,
      nchar(as_bib_text)
    ))

    # Fake a person object to extract comments
    fake_person <- paste0("{Fake} ", comment_only)
    if (getRversion() < "4.5.0") {
      comm_cff <- extract_person_comments(fake_person) # nocov
    } else {
      comm_cff <- extract_person_comments45(fake_person)
    }
  } else {
    # Does not
    person_only <- as_bib_text
    comm_cff <- list()
  }

  # Clean several spaces (issue with quint package)
  person_only <- gsub("\\s{2,}", " ", person_only)

  # Special case for Bioconductor
  if (is_substring(tolower(person_only), "bioconductor")) {
    person_only <- protect_bib_braces(person_only)
  }
  # Special case for R Core Team
  if (is_substring(tolower(person_only), "r core")) {
    person_only <- protect_bib_braces(person_only)
  }

  # Now extract structure for person_only string
  # It may be one of:
  # A. Given von Family
  # B. von Family, Given
  # C. von Family, Junior, Given

  # Protect commas on brackets to avoid error counting
  protected <- gsub(",(?![^\\}]*(\\{|$))", "@comma@", person_only, perl = TRUE)

  commas <- as.character(
    lengths(regmatches(protected, gregexpr(",", protected, fixed = TRUE)))
  )

  # Assign the corresponding fun
  bibtex_name_str <- switch(commas,
    # Case A
    "0" = bibtex_pers_first_von_last(person_only),
    # Case B
    "1" = bibtex_pers_von_last_first(person_only),
    # Case C
    "2" = bibtex_pers_von_last_first_jr(person_only),
    # Empty
    list(family = paste(person_only, collapse = " "))
  )

  # Clean
  bibtex_name_str <- lapply(bibtex_name_str, function(z) {
    if (is.null(clean_str(z))) {
      return(NULL)
    }

    cleaned <- gsub("\\{|\\}", "", z)
    clean_str(cleaned)
  })

  # Final person
  if (is.null(bibtex_name_str$given)) {
    ent <- c(bibtex_name_str$von, bibtex_name_str$family, bibtex_name_str$jr)
    ent <- clean_str(paste(ent, collapse = " "))
    pers_cff <- list(name = ent)
  } else {
    pers_cff <- list(
      "family-names" = bibtex_name_str$family,
      "given-names" = bibtex_name_str$given,
      "name-particle" = bibtex_name_str$von,
      "name-suffix" = bibtex_name_str$jr
    )
  }

  pers_cff <- pers_cff[!lengths(pers_cff) == 0]

  # Add comments
  pers_cff <- c(pers_cff, comm_cff)

  # Validate fields
  pers_cff <- validate_cff_person_fields(pers_cff)
  pers_cff
}

test_that("NULL gives NULL", {
  expect_null(as_cff_person(NULL))
  expect_null(as_cff_person(NA))
  expect_null(as_cff_person(person()))
  expect_null(as_cff_person(list()))
  expect_null(as_cff_person(""))
})

test_that("default", {
  p <- list("One", "person")
  expect_identical(as_cff_person(p), as_cff_person("One person"))

  expect_identical(as_cff_person(1), as_cff_person("1"))
})

test_that("Coerce one person", {
  p <- person("one", "person")
  expect_snapshot(as_cff_person(p))
})

test_that("Coerce several persons", {
  p <- c(
    person("one", "person"),
    person("another", "human"),
    person("and one", "more")
  )
  expect_snapshot(as_cff_person(p))
})

test_that("Coerce bibtex persons", {
  s <- "Wright, III, Frank Edwin"

  expect_snapshot(as_cff_person(s))

  s <- "A person and another and Another one"

  expect_snapshot(as_cff_person(s))

  # As vector
  s2 <- c("A person", "another", "Another one")
  expect_identical(as_cff_person(s), as_cff_person(s2))
})

test_that("Coerce bibtex persons with masks", {
  s <- "Elephant and Castle"

  expect_snapshot(as_cff_person(s))

  s <- "{Elephant and Castle}"

  expect_snapshot(as_cff_person(s))

  s <- "{Elephant and Castle} and this AND Ltd."

  expect_snapshot(as_cff_person(s))

  s <- "{Elephant and Castle} and {this AND Ltd.}"

  expect_snapshot(as_cff_person(s))
})


test_that("Get same results with both", {
  s3 <- as_cff_person(
    c(person(family = "Entity"), person("A", "person"))
  )

  b3 <- as_cff_person("Entity and A person")

  expect_identical(s3, b3)

  s4 <- as_cff_person(person(family = "A big fat company"))
  b4 <- as_cff_person("{A big fat company}")

  expect_identical(s4, b4)
})

test_that("R Core Team", {
  p <- as_cff_person(person("R Core", "Team"))

  expect_equal(p[[1]]$name, "R Core Team")

  p <- as_cff_person("R Core Team")

  expect_equal(p[[1]]$name, "R Core Team")
})

test_that("Bioconductor", {
  # Several tastes of Bioconductor
  bio <- person(
    "Bioconductor Package Maintainer",
    role = "cre",
    email = "maintainer@bioconductor.org"
  )
  p <- as_cff_person(bio)

  expect_equal(p[[1]]$name, "Bioconductor Package Maintainer")

  p <- as_cff_person("Bioconductor Package Maintainer")

  expect_equal(p[[1]]$name, "Bioconductor Package Maintainer")

  p <- as_cff_person(person("The Bioconductor", "Package Maintainer"))
  expect_equal(p[[1]]$name, "The Bioconductor Package Maintainer")
})

test_that("Several emails, select first", {
  pp <- person(
    given = "John",
    family = "Doe",
    role = c("aut", "cre", "trl"),
    email = c("first_mail@gmail.com", "second_mail@espol.edu.ec")
  )

  p <- as_cff_person(pp)

  expect_equal(p[[1]]$email, "first_mail@gmail.com")
})


test_that("Several emails, select second if first no valid", {
  pp <- person(
    given = "John",
    family = "Doe",
    role = c("aut", "cre", "trl"),
    email = c("first_mail_gmail.com", "second_mail@espol.edu.ec")
  )

  p <- as_cff_person(pp)

  expect_equal(p[[1]]$email, "second_mail@espol.edu.ec")
})


test_that("No valid emails", {
  pp <- person(
    given = "John",
    family = "Doe",
    role = c("aut", "cre", "trl"),
    email = c("first_mail_gmail.com", "second_mail__espol.edu.ec")
  )

  p <- as_cff_person(pp)

  expect_equal(
    p,
    as_cff_person(person(
      given = "John",
      family = "Doe"
    ))
  )

  pp2 <- person(
    given = "John",
    family = "Doe"
  )

  p2 <- as_cff_person(pp2)

  expect_equal(
    p,
    as_cff_person(person(
      given = "John",
      family = "Doe"
    ))
  )
})

test_that("Can extract comments from format", {
  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")

  pp <- person(
    given = "John",
    family = "Doe",
    email = "first_mail@gmail.com",
    comment = c(
      ORCID = "https://orcid.org/0000-0001-8457-4658",
      website = "https://www.google.com/",
      address = "Sweet Home Chicago"
    )
  )

  pp1 <- as_cff_person(pp)
  txt <- format(pp)
  pp2 <- as_cff_person(txt)
  expect_identical(pp1, pp2)

  expect_snapshot(pp2)
})

# Based on
# https://maverick.inria.fr/~Xavier.Decoret/resources/xdkbibtex/ >>
# bibtex_summary.html

test_that("Test first von last", {
  x <- "AA BB"

  res <- create_person_from_txt(x)
  expect_true(is.list(res))

  expect_snapshot(unlist(res))

  x <- "AA"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA bb"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "aa"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA bb CC"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA bb CC dd EE"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA {b}B cc dd"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA \\BB{b} cc dd"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA {bb} cc DD"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA bb {cc} DD"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "AA {bb} CC"
  expect_snapshot(unlist(create_person_from_txt(x)))
})

test_that("Testing with random names First von Last", {
  x <- "Jean de La Fontaine"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Diego {Hernandez Sanz}"

  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Juan Manuel Miramontes"

  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Juan Manuel {Miramontes Garcia}"

  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Juan Manuel van Halen"

  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Bosco {de la Cruz y Ochoa}"

  expect_snapshot(unlist(create_person_from_txt(x)))
})

test_that("Test von Last, First", {
  x <- "bb CC, AA"

  res <- create_person_from_txt(x)
  expect_true(is.list(res))

  expect_snapshot(unlist(res))

  x <- "bb CC, aa"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "bb CC dd EE, AA"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "bb, AA"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "BB,"
  expect_snapshot(unlist(create_person_from_txt(x)))
})

test_that("Test von Last, First with brackets, etc", {
  x <- "de Armas, Ana"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "{de Armas}, Ana"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "{de Armas, Aguero}, Ana"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "{de Armas, Aguero}, Ana Maria"
  expect_snapshot(unlist(create_person_from_txt(x)))
})

test_that("Test von Last, Jr,  First", {
  x <- "bb CC,XX, AA"

  res <- create_person_from_txt(x)
  expect_true(is.list(res))

  expect_snapshot(unlist(res))

  x <- "BB,, AA"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "BB, AA,"
  expect_snapshot(unlist(create_person_from_txt(x)))
})

test_that("Test von Last, Jr,  First with masking", {
  x <- "Davis, Jr, Sammy"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "{Davis, and}, {Jr, another}, Sammy"
  expect_snapshot(unlist(create_person_from_txt(x)))
})

test_that("Rest of cases", {
  x <- "David, and, Jr, another, Sammy"

  res <- create_person_from_txt(x)
  expect_true(is.list(res))
  expect_length(res, 1)

  expect_snapshot(unlist(res))
})

test_that("tames da beast", {
  # http://tug.ctan.org/info/bibtex/tamethebeast/ttb_en.pdf

  x <- "jean de la fontaine"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Jean de la fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Jean {de} la fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "jean {de} {la} fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Jean {de} {la} fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Jean De La Fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "jean De la Fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "Jean de La Fontaine "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "jean de la fontaine,"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "de la fontaine, Jean "
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "De La Fontaine, Jean"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "De la Fontaine, Jean"
  expect_snapshot(unlist(create_person_from_txt(x)))

  x <- "de La Fontaine, Jean"
  expect_snapshot(unlist(create_person_from_txt(x)))
})

list(
  rd_family_title = list(
    core = "Other core functions of \\CRANpkg{cffr}:",
    reading = "Other functions for reading external files:",
    writing = "Other functions for creating external files:",
    bibtex = "Other functions for working with BibTeX format:",
    schemas = "Other CFF schemas:",
    git = "Other Git/GitHub helpers provided by \\CRANpkg{cffr}:",
    deprecated = "Other deprecated functions:",
    s3method = "Coercing between \\strong{R} classes with \\strong{S3 Methods}:"
  )
)

#' Coerce lists, `person` and `bibentry` objects to [`cff`]
#'
#' @description
#' `as_cff()` turns an existing list-like **R** object into a so-called
#' [`cff`], a list with class `cff`, with the corresponding
#' [sub-class][cff_class] if applicable.
#'
#' `as_cff` is an S3 generic, with methods for:
#' - `person` objects as produced by [utils::person()].
#' - `bibentry` objects as produced by [utils::bibentry()].
#' - `Bibtex` objects as produced by [toBibtex()].
#' - Default: Other inputs are first coerced with [as.list()].
#'
#' @param x A `person`, `bibentry` or other object that could be coerced to a
#'   list.
#' @param ... Additional arguments to be passed on to other methods.
#'
#' @rdname as_cff
#'
#' @return
#'
#' - `as_cff.person()` returns an object with classes
#'   [`cff_pers_lst, cff`][cff_pers_lst].
#' - `as_cff.bibentry()` and `as_cff.Bibtex()` return an object with classes
#'   [`cff_ref_lst, cff`][cff_ref_lst].
#' - The remaining methods return an object of class `cff`. However, if
#'   `x` has a structure compatible with `definitions.person`,
#'   `definitions.entity` or `definitions.reference`, the object has the
#'   corresponding subclass.
#'
#' Learn more about the \CRANpkg{cffr} class system in [cff_class].
#'
#' @family s3method
#'
#' @details
#' For `as_cff.bibentry()` and `as_cff.Bibtex()`, see
#' `vignette("bibtex_cff", "cffr")` to understand how the mapping is performed.
#'
#' [as_cff_person()] is preferred over `as_cff.person()`, since it can handle
#' `character` inputs such as `"Davis, Jr., Sammy"`. For `person` objects both
#' functions are similar.
#'
#' @seealso
#' - [cff()]: Create a full `cff` object from scratch.
#' - [cff_modify()]: Modify a `cff` object.
#' - [cff_create()]: Create a `cff` object for an **R** package.
#' - [cff_read()]: Create a `cff` object from an external file.
#' - [as_cff_person()]: Recommended way to create persons in CFF format.
#'
#' Learn more about the \CRANpkg{cffr} class system in [cff_class].
#'
#' @export
#' @encoding UTF-8
#'
#' @examples
#'
#' # Convert a list to "cff" object
#' cffobj <- as_cff(list(
#'   "cff-version" = "1.2.0",
#'   title = "Manipulating files"
#' ))
#'
#' class(cffobj)
#'
#' # Nice display thanks to yaml package
#' cffobj
#'
#' # bibentry method
#' a_cit <- citation("cffr")[[1]]
#'
#' a_cit
#'
#' as_cff(a_cit)
#'
#' # Bibtex method
#' a_bib <- toBibtex(a_cit)
#'
#' a_bib
#'
#' as_cff(a_cit)
#'
as_cff <- function(x, ...) {
  UseMethod("as_cff")
}

#' @rdname as_cff
#' @export
#' @encoding UTF-8
as_cff.default <- function(x, ...) {
  as_cff(as.list(x), ...)
}


#' @rdname as_cff
#' @export
#' @encoding UTF-8
as_cff.list <- function(x, ...) {
  # Clean up empty values on top
  clean_up <- vapply(x, is.null, FUN.VALUE = logical(1))
  x_clean <- x[!clean_up]
  new_cff(x_clean)
}

#' @rdname as_cff
#' @export
#' @encoding UTF-8
as_cff.person <- function(x, ...) {
  as_cff_person(x)
}


#' @rdname as_cff
#' @export
#' @encoding UTF-8
as_cff.bibentry <- function(x, ...) {
  cff_ref <- as_cff_reference(x)
  clean_up <- vapply(cff_ref, is.null, FUN.VALUE = logical(1))
  if (all(clean_up)) {
    return(NULL)
  }

  cff_refs <- as_cff(cff_ref, ...)

  # Add clases
  cff_refs_class <- lapply(cff_refs, function(x) {
    class(x) <- c("cff_ref", "cff")
    x
  })

  class(cff_refs_class) <- c("cff_ref_lst", "cff")
  cff_refs_class
}

#' @rdname as_cff
#' @export
#' @encoding UTF-8
as_cff.Bibtex <- function(x, ...) {
  tmp <- tempfile(fileext = ".bib")
  writeLines(x, tmp)
  abib <- cff_read_bib(tmp)
  cff_refs <- as_cff(abib, ...)

  # Add clases
  cff_refs_class <- lapply(cff_refs, function(x) {
    class(x) <- c("cff_ref", "cff")
    x
  })

  class(cff_refs_class) <- c("cff_ref_lst", "cff")
  cff_refs_class
}

# nolint start
#' @export
#' @encoding UTF-8
#' @rdname as_cff
#' @usage NULL
as.cff <- function(x) {
  as_cff(x)
}
# nolint end

# Helper----

#' Recursively clean lists
#'
#' @noRd
rapply_drop_null <- function(x) {
  if (is.list(x) && length(x) > 0) {
    x <- drop_null(x)
    x <- lapply(x, rapply_drop_null)
    x
  } else {
    x
  }
}


rapply_class <- function(x) {
  if (is_named(x)) {
    x <- x[!duplicated(names(x))]
  }

  xend <- lapply(x, function(el) {
    xelement <- el
    guess <- guess_cff_part(xelement)

    if (guess %in% c("unclear", "cff_full")) {
      return(xelement)
    }

    if (guess == "cff_pers_lst") {
      xelement <- lapply(xelement, function(j) {
        j_in <- j
        class(j_in) <- c("cff_pers", "cff")
        j_in
      })
      class(xelement) <- c("cff_pers_lst", "cff")
    }

    # Languages: handle single value
    if (all(guess == "cff_ref", "languages" %in% names(xelement))) {
      if (length(xelement$languages) < 2) {
        xelement$languages <- list(unlist(
          xelement$languages,
          use.names = FALSE
        ))
      }
    }

    if (guess == "cff_ref_lst") {
      xelement <- lapply(xelement, function(j) {
        j_in <- rapply_class(j)
        class(j_in) <- c("cff_ref", "cff")
        # Languages: handle single value
        if ("languages" %in% names(j_in)) {
          if (length(j_in$languages) < 2) {
            j_in$languages <- list(unlist(
              j_in$languages,
              use.names = FALSE
            ))
          }
        }
        j_in
      })
      class(xelement) <- c("cff_ref_lst", "cff")
    }

    if (guess %in% c("cff_ref", "cff_pers")) {
      xin <- rapply_class(xelement)
      class(xin) <- c(guess, "cff")
      xelement <- xin
    }
    xelement
  })
  xend
}

# https://adv-r.hadley.nz/s3.html#s3-constructor
# Constructor
new_cff <- function(x) {
  # Clean all strings recursively

  x <- rapply(
    x,
    function(x) {
      if (is.list(x) || length(x) > 1) {
        return(x)
      }
      clean_str(x)
    },
    how = "list"
  )

  # Remove NULLs
  x <- drop_null(x)

  # Remove duplicated names if named
  if (is_named(x)) {
    x <- x[!duplicated(names(x))]
  }

  # Now apply drop null to nested lists
  x <- lapply(x, rapply_drop_null)

  # Reclass nested
  guess_x <- guess_cff_part(x)
  if (guess_x == "cff_ref_lst") {
    x2 <- lapply(x, function(j) {
      j2 <- rapply_class(j)
      class(j2) <- c("cff_ref", "cff")
      j2
    })
    class(x2) <- c(guess_x, "cff")
    return(x2)
  }

  xend <- rapply_class(x)

  final_class <- switch(guess_x,
    "cff_full" = "cff",
    "unclear" = "cff",
    c(guess_x, "cff")
  )

  if (!is.null(final_class)) {
    class(xend) <- final_class
  }

  xend
}

test_that("as.cff still works", {
  l <- list(
    "cff-version" = "1.2.0",
    title = "Manipulating files"
  )

  expect_silent(l1 <- as_cff(l))
  expect_silent(l2 <- as.cff(l))
  expect_s3_class(l1, "cff", exact = TRUE)

  expect_snapshot(l2)
})

test_that("as_cff.person", {
  pers <- person(
    "A",
    "person",
    email = "fake@gmail.com",
    comment = c(
      ORCID = "0000-0001-8457-4658",
      affiliation = "Real Madrid",
      website = "https://www.google.com/"
    )
  )

  aa <- as_cff_person(pers)
  expect_s3_class(aa, c("cff_pers_lst", "cff"), exact = TRUE)
  expect_s3_class(aa[[1]], c("cff_pers", "cff"), exact = TRUE)
  expect_identical(aa, as_cff_person(pers))
  rvers <- getRversion()
  skip_if(!grepl("^4.6", rvers), "Snapshot created with R 4.6.*")
  expect_snapshot(as_cff(pers))

  # Check a single person
  single <- as.list(aa)[[1]]
  expect_false(inherits(single, "cff"))
  single_cff <- as_cff(single)
  expect_s3_class(single_cff, c("cff_pers", "cff"), exact = TRUE)
  expect_snapshot(single_cff)
})

test_that("as_cff.bibentry, toBibtex", {
  b <- bibentry("Misc", title = "title", author = "Author", editor = "Editor")

  bbb <- as_cff(b)

  expect_s3_class(bbb, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_s3_class(bbb[[1]], c("cff_ref", "cff"), exact = TRUE)
  expect_snapshot(bbb)

  b_bib <- toBibtex(b)
  expect_s3_class(b_bib, "Bibtex")

  bbbb <- as_cff(b_bib)

  expect_identical(bbb, bbbb)

  # Check single
  b_single <- as.list(bbb)[[1]]
  expect_false(inherits(b_single, "cff"))
  b_single_cff <- as_cff(b_single)
  expect_s3_class(b_single_cff, c("cff_ref", "cff"), exact = TRUE)
  expect_snapshot(b_single_cff)

  # Check empty
  s <- bibentry("misc")
  expect_null(as_cff(s))
})


test_that("as_cff.default", {
  b <- c(a = 1)

  expect_identical(as_cff(b), as_cff(as.list(b)))

  expect_snapshot(as_cff(b))
})

test_that("Other convertes", {
  a <- cff()
  expect_s3_class(a, "cff")
  a <- as_cff(a)
  expect_true(is_cff(a))
  expect_s3_class(a, "cff")

  expect_snapshot(noadd <- cff(chocolate = "New York", version = 5))
  expect_true(is_cff(noadd))
  expect_false(is_cff(list(a = 1, b = 2)))
  expect_true(is_cff(as_cff(list(a = 1, b = 2))))
})


test_that("]] cff_ref", {
  b1 <- bibentry("Misc", title = "title", author = "Author", editor = "Editor")
  b2 <- bibentry("Manual", author = "Another", title = "another title")

  b_all <- c(b1, b2)

  expect_s3_class(b_all, "bibentry", exact = TRUE)
  bbb <- as_cff(b_all)

  expect_s3_class(bbb, c("cff_ref_lst", "cff"), exact = TRUE)
  expect_length(bbb, 2)

  b2_reg <- bbb[2]
  expect_length(b2_reg, 1)
  expect_s3_class(b2_reg, c("cff_ref", "cff"), exact = TRUE)
})

test_that("]] cff_pers", {
  b1 <- person("One", "person")
  b2 <- person("ntity")

  b_all <- c(b1, b2)

  expect_s3_class(b_all, "person", exact = TRUE)
  bbb <- as_cff(b_all)

  expect_s3_class(bbb, c("cff_pers_lst", "cff"), exact = TRUE)
  expect_length(bbb, 2)

  b2_reg <- bbb[2]
  expect_length(b2_reg, 1)
  expect_s3_class(b2_reg, c("cff_pers", "cff"), exact = TRUE)
})

# Check full classes with recursion
test_that("Reading full cff", {
  full <- system.file("examples/CITATION_complete.cff", package = "cffr")
  cff_complete <- cff_read(full)
  expect_true(cff_validate(cff_complete, verbose = FALSE))

  nm <- names(cff_complete)

  class_v <- vapply(
    nm,
    function(x) {
      clean_str(paste0(class(cff_complete[[x]]), collapse = "|"))
    },
    character(1)
  )

  df <- data.frame(class = sort(class_v[class_v != "character"]))

  expect_snapshot(df)

  # Assess preferred
  pref <- cff_complete$`preferred-citation`
  nm2 <- names(pref)

  class_v2 <- vapply(
    nm2,
    function(x) {
      clean_str(paste0(class(pref[[x]]), collapse = "|"))
    },
    character(1)
  )

  df2 <- data.frame(class = sort(class_v2[class_v2 != "character"]))

  expect_snapshot(df2)

  # Relist
  asl <- as.list(cff_complete)

  expect_false(inherits(asl, "cff"))
  expect_identical(cff_complete, as_cff(asl))
})
test_that("Reading languages", {
  # full cff has two languages in preferred-citation
  full <- system.file("examples/CITATION_complete.cff", package = "cffr")
  cff_complete <- cff_read(full)
  expect_true(cff_validate(cff_complete, verbose = FALSE))

  expect_identical(
    cff_complete[["preferred-citation"]]$languages,
    c("aaa", "zu")
  )

  # Several languages
  lang_file <- system.file("examples/CITATION_lang.cff", package = "cffr")

  cff_lang <- cff_read(lang_file)
  expect_true(cff_validate(cff_lang, verbose = FALSE))

  expect_identical(
    cff_lang[["preferred-citation"]]$languages,
    list("en")
  )

  expect_identical(
    cff_lang[["references"]][[1]]$languages,
    list("en")
  )

  expect_identical(
    cff_lang[["references"]][[2]]$languages,
    c("en", "es", "fr")
  )
})

test_that("Language round-trip", {
  lang_file <- system.file("examples/CITATION_lang.cff", package = "cffr")

  cff_lang <- cff_read(lang_file)
  expect_true(cff_validate(cff_lang, verbose = FALSE))

  expect_identical(
    cff_lang[["preferred-citation"]]$languages,
    list("en")
  )

  expect_identical(
    cff_lang[["references"]][[1]]$languages,
    list("en")
  )

  expect_identical(
    cff_lang[["references"]][[2]]$languages,
    c("en", "es", "fr")
  )

  # Write
  tmp <- tempfile(fileext = ".cff")
  cff_write(cff_lang, tmp)

  # And re-check
  cff_lang <- cff_read(tmp)
  expect_true(cff_validate(cff_lang, verbose = FALSE))

  expect_identical(
    cff_lang[["preferred-citation"]]$languages,
    list("en")
  )

  expect_identical(
    cff_lang[["references"]][[1]]$languages,
    list("en")
  )

  expect_identical(
    cff_lang[["references"]][[2]]$languages,
    c("en", "es", "fr")
  )

  unlink(tmp)
})

bibtex_pers_von_last_first_jr <- function(x) {
  # Protect commas on brackets to avoid error on splitting
  protected <- gsub(",(?![^\\}]*(\\{|$))", "@comma@", x, perl = TRUE)

  parts_comma <- trimws(unlist(strsplit(protected, ",")))

  # Unprotect
  parts_comma <- gsub("@comma@", ",", parts_comma, fixed = TRUE)

  if (length(parts_comma) == 3) {
    given <- parts_comma[3]
  } else {
    # Case jean de la fontaine, junior,
    given <- NULL
  }

  # Junior part
  jr <- parts_comma[2]

  # Now it is the same than in bibtex_person_von_last_first

  # Now identify the von part

  # Protect spaces between braces before splitting
  protected <- gsub(
    "\\s(?![^\\}]*(\\{|$))",
    "@blank@",
    parts_comma[1],
    perl = TRUE
  )

  parts <- unlist(strsplit(protected, " "))

  # Unprotect
  parts <- gsub("@blank@", " ", parts, fixed = TRUE)

  # Start building
  # Assess casing

  is_upper <- vapply(parts, FUN.VALUE = logical(1), function(y) {
    case <- substr(y, 1, 1)
    if (case == toupper(case)) {
      TRUE
    } else {
      FALSE
    }
  })

  # Family should be always provided
  family <- parts[length(parts)]

  upper <- is_upper[setdiff(names(is_upper), family)]

  # Get more family parts
  invert <- rev(upper)
  # Then need to get sequentally the family
  for (i in seq_along(invert)) {
    if (!invert[i]) {
      break
    }
    family <- c(names(invert[i]), family)
  }

  # The remaining is von
  von <- setdiff(names(is_upper), family)

  if (length(von) == 0) {
    von <- NULL
  }

  # Compose final list to be pased to person()

  # Last clean up
  end_list <- list(
    given = given,
    von = von,
    family = family,
    jr = jr
  )

  end_list
}

bibtex_pers_von_last_first <- function(x) {
  # Protect commas on brackets to avoid error on splitting
  protected <- gsub(",(?![^\\}]*(\\{|$))", "@comma@", x, perl = TRUE)

  parts_comma <- trimws(unlist(strsplit(protected, ",")))

  # Unprotect
  parts_comma <- gsub("@comma@", ",", parts_comma, fixed = TRUE)

  if (length(parts_comma) == 2) {
    given <- parts_comma[2]
  } else {
    # Case jean de la fontaine,
    given <- NULL
  }

  # Now identify the von part

  # Protect spaces between braces before splitting
  protected <- gsub(
    "\\s(?![^\\}]*(\\{|$))",
    "@blank@",
    parts_comma[1],
    perl = TRUE
  )

  parts <- unlist(strsplit(protected, " "))

  # Unprotect
  parts <- gsub("@blank@", " ", parts, fixed = TRUE)

  # Start building
  # Assess casing

  is_upper <- vapply(parts, FUN.VALUE = logical(1), function(y) {
    case <- substr(y, 1, 1)
    if (case == toupper(case)) {
      TRUE
    } else {
      FALSE
    }
  })

  # Family should be always provided
  family <- parts[length(parts)]

  upper <- is_upper[setdiff(names(is_upper), family)]

  # Get more family parts
  invert <- rev(upper)
  # Then need to get sequentally the family
  for (i in seq_along(invert)) {
    if (!invert[i]) {
      break
    }
    family <- c(names(invert[i]), family)
  }

  # The remaining is von
  von <- setdiff(names(is_upper), family)

  if (length(von) == 0) {
    von <- NULL
  }

  # Compose final list to be pased to person()

  # Last clean up
  end_list <- list(
    given = given,
    von = von,
    family = family
  )

  end_list
}

bibtex_pers_first_von_last <- function(x) {
  # Based on Tame the BeaST
  # http://tug.ctan.org/info/bibtex/tamethebeast/ttb_en.pdf
  # Testing
  # string                  -> "First"        "von"         "Last"
  # jean de la fontaine     -> ""             "jean de la"  "fontaine"
  # Jean de la fontaine     -> "Jean"         "de la"       "fontaine"
  # Jean {de} la fontaine   -> "Jean de"      "la"          "fontaine"
  # jean {de} {la} fontaine -> ""             "jean"        "de la fontaine"
  # Jean {de} {la} fontaine -> "Jean de la"   ""            "fontaine"
  # Jean De La Fontaine     -> "Jean De La"   ""            "Fontaine"
  # jean De la Fontaine     -> ""             "jean De la"  "Fontaine"
  # Jean de La Fontaine     -> "Jean"         "de"          "La Fontaine"

  # Protect spaces between braces before splitting
  x <- gsub("\\s(?![^\\}]*(\\{|$))", "@blank@", x, perl = TRUE)

  # Collapse blanks
  x <- gsub("\\s+", " ", x)

  parts <- trimws(unlist(strsplit(x, " ")))

  # Unprotect
  parts <- gsub("@blank@", " ", parts, fixed = TRUE)

  # Start building
  # Assess casing

  is_upper <- vapply(parts, FUN.VALUE = logical(1), function(y) {
    case <- substr(y, 1, 1)
    if (case == toupper(case)) {
      TRUE
    } else {
      FALSE
    }
  })

  # Family should be always provided
  family <- parts[length(parts)]

  # Get if there is a mix of casings on the remaining parts
  upper <- is_upper[setdiff(names(is_upper), family)]
  mix <- (length(unique(upper)) == 1)

  if (!all(mix, upper)) {
    invert <- rev(upper)
    # Then need to get sequentally the family
    for (i in seq_along(invert)) {
      if (!invert[i]) {
        break
      }
      family <- c(names(invert[i]), family)
    }
  }

  # Clean again
  upper <- is_upper[setdiff(names(is_upper), family)]

  # von part
  # Detect casing index on remaining parts
  von_assess <- as.integer(which(!upper))

  # von
  # if no lenght then no von
  if (length(von_assess) == 0) {
    von <- NULL
  } else {
    von <- names(upper[seq(min(von_assess), max(von_assess))])
  }

  # get remaining part, it should be First
  given <- names(upper[setdiff(names(upper), von)])

  # Compose final list to be pased to person()

  # Last clean up
  end_list <- list(
    given = given,
    von = von,
    family = family
  )

  end_list
}

validate_cff_person_fields <- function(person_cff) {
  # Entity of person

  # Guess entity or person
  is_entity <- as.character("name" %in% names(person_cff))

  # Keep only valid tags - Would depend on entity or person
  definition <- switch(is_entity,
    "TRUE" = cff_schema_definitions_entity(),
    cff_schema_definitions_person()
  )

  person_cff <- person_cff[names(person_cff) %in% definition]

  # Duplicates removed
  person_cff <- person_cff[!duplicated(names(person_cff))]

  person_cff
}

split_txt_persons <- function(person) {
  person <- trimws(person)
  person <- paste0(person, collapse = " and ")

  # Remove role on [] as it comes from print.person by default
  # We don't use it here
  person <- gsub("\\[[^()]*\\]", "", person)

  # Protect 'and' on brackets {}
  # Lower
  protected <- gsub("(and)(?![^\\}]*(\\{|$))", "@nd@", person, perl = TRUE)

  # upper
  protected <- gsub("AND(?![^\\}]*(\\{|$))", "@ND@", protected, perl = TRUE)

  # Do the same for 'and' in comments "()" as provided by print.person
  # Lower
  protected <- gsub("(and)(?![^\\)]*(\\(|$))", "@nd@", protected, perl = TRUE)

  # upper
  protected <- gsub("AND(?![^\\)]*(\\(|$))", "@ND@", protected, perl = TRUE)

  # Do the same for 'and' in "<>". These are email, should never happen
  # Lower
  protected <- gsub("(and)(?![^>]*(<|$))", "@nd@", protected, perl = TRUE)

  # upper
  protected <- gsub("AND(?![^>]*(<|$))", "@ND@", protected, perl = TRUE)

  auths <- unlist(strsplit(protected, " and | AND "))

  # Unprotec
  auths_un <- gsub("@nd@", "and", auths, fixed = TRUE)
  auths_un <- gsub("@ND@", "AND", auths_un, fixed = TRUE)

  auths_un
}

extract_person_comments <- function(person) {
  # nocov start
  # Not used in R >= 4.5.0
  # Ensure person type
  person <- as.person(person)

  # Extract from comments
  comm_cff <- as.list(person$comment)
  names(comm_cff) <- tolower(names(comm_cff))
  nms_com <- names(comm_cff)
  comment_as_text <- tolower(clean_str(comm_cff))

  # Special case when coerced from text, only can extract orcid and web
  if (
    all(
      any(length(nms_com) == 0, is.na(nms_com)),
      length(comment_as_text) > 0
    )
  ) {
    split_comments <- unlist(strsplit(comment_as_text, ",| |<|>"))

    # Guess that seems to be a web
    url_comment <- split_comments[is_url(split_comments)]

    # guess orcid
    orcid <- url_comment[grepl("orcid.org/", url_comment)]

    # Case for R > 4.4.1
    if ("orcid" %in% names(comm_cff)) {
      orcid <- clean_str(comm_cff$orcid)
    }

    # Get the first non-orcid url
    web <- url_comment[!grepl("orcid.org/", url_comment)][1]

    # Reset comment list
    comm_cff <- list()

    comm_cff$orcid <- clean_str(orcid)
    comm_cff$website <- clean_str(web)
  }

  # Add url to orcid if not present
  # Get leading invalid urls

  if (!is.null(comm_cff$orcid)) {
    orcid <- gsub("^orcid.org/", "", comm_cff$orcid)
    orcid <- gsub("^https://orcid.org/", "", orcid)
    orcid <- gsub("^http://orcid.org/", "", orcid)

    comm_cff$orcid <- paste0("https://orcid.org/", orcid)
  }

  # Add website
  web <- comm_cff$website

  if (!is.null(web)) {
    comm_cff$website <- clean_str(web[is_url(web)])
  }

  # Add also email
  # Check if several mails (MomTrunc 6.0)
  look_emails <- c(unlist(person$email), comm_cff$email)
  valid_emails <- unlist(lapply(look_emails, is_email))
  email <- look_emails[valid_emails][1]

  # Final list
  fin_list <- c(
    list(email = NULL),
    comm_cff["email" != names(comm_cff)]
  )
  fin_list$email <- clean_str(email)

  fin_list
  # nocov end
}

extract_person_comments45 <- function(person) {
  # Detect comments
  # Last is )?

  if (inherits(person, "person")) {
    person <- format(person)
  }
  last <- substr(person, nchar(person), nchar(person))
  # Has another )?
  start_comment <- min(regexpr("(", person, fixed = TRUE))

  has_comment <- all(last == ")", start_comment > 5)

  # If does not have comment convert to person
  if (!has_comment) {
    p <- as.person(person)
    comm_cff <- list(fake = 123)
  } else {
    # Split and convert
    the_person <- substr(person, 1, start_comment - 1)
    p <- as.person(the_person)

    # Comments
    # Can't get commas in the comment :(
    comm <- strsplit(substr(person, start_comment + 1, nchar(person) - 1), ",")
    comm <- unlist(comm)
    comm_list <- lapply(comm, function(x) {
      detect_split <- min(regexpr(":", x, fixed = TRUE))
      nm <- substr(x, 1, detect_split - 1)
      val <- substr(x, detect_split + 1, nchar(x))
      val <- clean_str(val)
      val <- gsub("<|>", "", val)

      nm <- clean_str(nm)
      names(val) <- nm
      val
    })

    comm_cff <- as.list(unlist(comm_list))
  }

  names(comm_cff) <- tolower(names(comm_cff))
  # Delete non-named comments
  nms_com <- names(comm_cff)
  comm_cff <- comm_cff[nchar(nms_com) > 1]

  if (!is.null(comm_cff$orcid)) {
    orcid <- gsub("^orcid.org/", "", comm_cff$orcid)
    orcid <- gsub("^https://orcid.org/", "", orcid)
    orcid <- gsub("^http://orcid.org/", "", orcid)

    comm_cff$orcid <- paste0("https://orcid.org/", orcid)
  }

  # Add website
  web <- comm_cff$website

  if (!is.null(web)) {
    comm_cff$website <- clean_str(web[is_url(web)])
  }

  # Fallback: ROR if website not present
  ror <- comm_cff$ror
  if (all(!is.null(ror), is.null(comm_cff$website))) {
    comm_cff$website <- clean_str(ror[is_url(ror)])
  }

  # Add also email
  # Check if several mails (MomTrunc 6.0)
  look_emails <- c(unlist(p$email), comm_cff$email)
  valid_emails <- unlist(lapply(look_emails, is_email))
  email <- look_emails[valid_emails][1]

  # Final list
  fin_list <- c(
    list(email = NULL),
    comm_cff["email" != names(comm_cff)]
  )
  fin_list$email <- clean_str(email)
  fin_list
}

#' Modify a [`cff`] object
#'
#' Add new keys or modify existing ones in a [`cff`] object.
#'
#' @param x A [`cff`] object.
#' @param ... Named arguments used to modify `x`. See also the `...`
#'   argument in [cff()].
#'
#' @details
#'
#' Keys provided in `...` override the corresponding key in `x`.
#'
#' @return
#'
#' A [`cff`] object.
#'
#' @family core
#' @export
#' @encoding UTF-8
#' @seealso
#' This function is a wrapper of [utils::modifyList()].
#'
#' See [cff()] for creating [`cff`] objects from scratch.
#'
#' @details
#'
#' You can add additional keys not detected by [cff_create()] using
#' the `keys` argument. A list of valid keys can be retrieved with
#' [cff_schema_keys()]. For details, refer to
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("\n", "[Guide to Citation File Format schema version 1.2.0]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#' for additional details.
#'
#' @examples
#' x <- cff()
#' x
#'
#' cff_validate(x)
#'
#' x_mod <- cff_modify(x,
#'   contact = as_cff_person("A contact"),
#'   message = "This overwrites fields",
#'   title = "New Title",
#'   abstract = "New abstract",
#'   doi = "10.21105/joss.03900"
#' )
#'
#' x_mod
#'
#' cff_validate(x_mod)
#'
cff_modify <- function(x, ...) {
  if (!inherits(x, "cff")) {
    cli::cli_abort(
      "{.arg x} should be a {.cls cff} object, not {.cls {class(x)}}."
    )
  }
  new_keys <- list(...)
  if (length(new_keys) == 0) {
    cli::cli_alert_info("Args {.arg ...} empty. Returning {.arg x}.")
    return(x)
  }

  modify_cff(x, new_keys, "...")
}

modify_cff <- function(x, keys, argname = "...") {
  # Don't throw message here, these cases are coming from cff_create
  if (all(argname == "keys", length(keys) == 0)) {
    return(x)
  }

  new_keys <- validate_extra_keys(keys, argname)
  new_keys <- fuzzy_keys(new_keys)
  if (anyDuplicated(names(new_keys)) > 0) {
    cli::cli_alert_warning("Removing duplicated keys.")
    new_keys <- new_keys[!duplicated(names(new_keys))]
  }

  init_ord <- names(x)

  xmod <- x[setdiff(names(x), names(new_keys))]
  xend <- modifyList(xmod, new_keys, keep.null = FALSE)

  # Name order
  sorted_nm <- unique(c(init_ord, names(xend)))

  # Relist and add classes
  xend <- as.list(xend[sorted_nm])
  as_cff(xend)
}


# Check names
validate_extra_keys <- function(cffobj, argname = "...") {
  has_names <- names(cffobj)
  if (is.null(has_names)) {
    cli::cli_abort(
      "Elements in {.arg {argname}} should be named."
    )
  }

  if (any(has_names == "")) {
    # nolint start
    # For printing only
    index <- as.character(which(has_names %in% ""))
    # nolint end

    cli::cli_alert_warning(
      "Found {length(index)} not-named argument{?s} in position{?s} {index}."
    )
    cli::cli_alert_info("Removing unnamed arguments")
    cffobj <- cffobj[has_names != ""]
  }
  cffobj
}

#' Write a `CITATION.cff` file
#'
#' @description
#'
#' **This is the core function of the package and likely to be the only one
#' you would need when developing a package**.
#'
#' This function writes out a `CITATION.cff` file for a given package. This
#' function is basically a wrapper around [cff_create()] to both create the
#' [`cff`] object and write it out to a YAML-formatted file in
#' one command.
#'
#' @family writing
#'
#' @param outfile The name and path of the `CITATION.cff` to be created.
#'
#' @param r_citation Logical `TRUE/FALSE`. When `TRUE`, the **R** package
#'   citation (for example, `inst/CITATION`) is created or updated.
#'   **No backup copy is created**. For more control, use
#'   [cff_write_citation()].
#'
#' @param verbose Logical `TRUE/FALSE`. When `TRUE`, the function displays
#'   informative messages.
#'
#' @param validate Logical `TRUE/FALSE`. Should the new file be validated
#'   using `cff_validate()`?
#'
#' @param encoding The name of the encoding to be assumed. Default is `"UTF-8"`,
#'   but it can be any other value as accepted by [iconv()], such as
#'   `"ASCII//TRANSLIT"`.
#'
#' @inheritParams cff_create
#' @inheritParams cff_validate
#'
#' @export
#' @encoding UTF-8
#'
#' @return A `CITATION.cff` file and an (invisible) `cff` object.
#'
#' @seealso
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("[Guide to Citation File Format schema version 1.2.0]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#' This function unifies the workflow [cff_create()] + [cff_validate()] +
#' write a file.
#'
#' @examples
#' \donttest{
#' tmpfile <- tempfile(fileext = ".cff")
#' cff_obj <- cff_write("jsonlite", outfile = tmpfile)
#'
#' cff_obj
#'
#' # Force clean-up
#' file.remove(tmpfile)
#' }
#' @details
#'
#' For details of `authors_roles` see [cff_create()].
#'
#' When creating and writing a `CITATION.cff` for the first time, the function
#' adds the pattern `"^CITATION\.cff$"` to your `.Rbuildignore` file.
#'
cff_write <- function(
  x,
  outfile = "CITATION.cff",
  keys = list(),
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  r_citation = FALSE,
  dependencies = TRUE,
  validate = TRUE,
  verbose = TRUE,
  authors_roles = c("aut", "cre"),
  encoding = "UTF-8"
) {
  # # On missing use NULL
  if (missing(x)) {
    x <- getwd()
  }

  # Issue 86 Need to remove first "inst/CITATION"
  fpath <- "./inst/CITATION"
  if (r_citation) {
    unlink(fpath)
  }

  citat <- cff_create(
    x,
    keys = keys,
    cff_version = cff_version,
    gh_keywords = gh_keywords,
    dependencies = dependencies,
    authors_roles = authors_roles
  )

  # Fix string if it is not cff
  if (!is_substring(outfile, ".cff$")) {
    outfile <- paste0(outfile, ".cff")
  }

  # Check if dir exist and if not create
  outdir <- dirname(outfile)

  if (!dir.exists(outdir)) {
    dir.create(outdir, showWarnings = FALSE, recursive = TRUE)
  }

  # See https://github.com/r-universe-org/help/issues/382
  # Write CITATION with comment
  com <- c(
    "# --------------------------------------------",
    "# CITATION file created with {cffr} R package",
    "# See also: https://docs.ropensci.org/cffr/",
    "# --------------------------------------------",
    " "
  )

  # MAX effort to get right the encoding
  com <- enc2utf8(com)
  out_yaml <- enc2utf8(capture.output(print(citat)))
  full_text <- enc2utf8(c(com, out_yaml, ""))
  fh <- file(outfile, encoding = encoding)
  on.exit(if (isOpen(fh)) close(fh))
  writeLines(full_text, fh)

  if (verbose) {
    cli::cli_alert_success("{.file {outfile}} generated")
  }

  # Add CITATION.cff to .Rbuildignore
  if (!is_cff(x) && x == getwd() && file_exist_abort(".Rbuildignore")) {
    ignore <- readLines(".Rbuildignore")

    # If not already
    if (!("^CITATION\\.cff$" %in% ignore)) {
      ignore <- c(ignore, "^CITATION\\.cff$")
      ignore <- unique(ignore)

      if (verbose) {
        cli::cli_alert_info("Adding {.val {outfile}} to {.file .Rbuildignore}")
      }
      writeLines(ignore, ".Rbuildignore")
    }
  }

  if (validate) {
    cff_validate(outfile, verbose)
  }

  # Issue #79
  auto_r_citation(
    r_citation = r_citation,
    outfile = outfile,
    verbose = verbose
  )

  invisible(citat)
}


auto_r_citation <- function(
  r_citation = TRUE,
  outfile = "CITATION.cff",
  verbose = TRUE
) {
  # Do nothing
  if (isFALSE(r_citation)) {
    return(invisible(NULL))
  }

  # Else
  if (verbose) {
    cli::cat_rule("Updating inst/CITATION file", col = "cyan", line = 2)
  }
  cffobj <- cff_read(outfile)
  fpath <- "./inst/CITATION"

  cff_write_citation(cffobj, file = fpath, verbose = verbose)
}

# Functions to convert fields in the DESCRIPTION file.

#' Mapped to Description
#' @noRd
get_desc_abstract <- function(pkg) {
  abstract <- pkg$get("Description")

  abstract <- clean_str(abstract)
  abstract <- unname(abstract)

  # Convert doi to url
  abstract <- gsub("<doi:", "<https://doi.org/", abstract, fixed = TRUE)

  abstract
}

#' Map to people with the roles "aut" and "cre".
#' Uncertain: this approach may need review.
#' On CRAN, only the first "aut" is used.
#' @noRd
get_desc_authors <- function(pkg, authors_roles = c("aut", "cre")) {
  # This extracts all the persons
  persons <- as.person(pkg$get_authors())

  authors <- persons[vapply(
    persons,
    function(x, r = authors_roles) {
      any(x$role %in% r)
    },
    logical(1)
  )]

  get_all_authors <- as_cff_person(authors)
  get_all_authors <- unique(get_all_authors)

  get_all_authors
}

#' Mapped to Maintainer
#' @noRd
get_desc_contacts <- function(pkg) {
  persons <- as.person(pkg$get_authors())

  # Extract creators only
  contact <- persons[vapply(
    persons,
    function(x) {
      "cre" %in% x$role
    },
    logical(1)
  )]

  get_all_contacts <- as_cff_person(contact)
  get_all_contacts <- unique(get_all_contacts)
  get_all_contacts
}

#' Mapped to Date, Date/Publication or Packaged
#' @noRd
get_desc_date_released <- function(pkg) {
  # See https://cran.r-project.org/doc/manuals/R-exts.html#The-DESCRIPTION-file
  date1 <- pkg$get("Date")
  # This is for CRAN/BioConductor packages
  date2 <- pkg$get("Date/Publication")
  # R-universe
  date3 <- pkg$get("Packaged")
  # Work with vector
  alldates <- unname(c(date1, date2, date3))
  clean_dates <- lapply(alldates, function(x) {
    if (is.na(x) || is.null(x)) {
      return(NULL)
    }
    if (!is.character(x)) {
      return(NULL)
    }
    substr(x, 1, 10)
  })

  clean_dates <- unlist(clean_dates)[1]

  # Validate with format YYYY-MM-DD
  date <- tryCatch(
    as.character(as.Date(clean_dates, format = "%Y-%m-%d")),
    error = function(cond) {
      NULL # nocov
    }
  )

  date <- clean_str(date)
  date
}

#' Mapped to X-schema.org-keywords, as codemeta/codemetar
#' @noRd
get_desc_keywords <- function(pkg) {
  kword <- pkg$get("X-schema.org-keywords")

  kword <- clean_str(kword)
  kword <- unname(kword)

  if (is.null(kword)) {
    return(kword)
  }

  kword <- unlist(strsplit(kword, ", "))
  kword <- unlist(strsplit(unique(kword), ","))

  # Hack: The validator doesn't seem to recognize when keyword is
  # unique. I add a new keyword r-package

  if (length(kword) == 1) {
    kword <- unique(c(kword, "r-package"))
  }

  # If still is 1 return NULL
  if (length(kword) == 1) {
    return(NULL)
  }

  kword
}

#' Mapped to License
#' @noRd
get_desc_license <- function(pkg) {
  licenses <- pkg$get_field("License")

  # The schema only accepts two LiCENSES max

  licenses <- unlist(strsplit(licenses, "\\| "))[1:2]

  # Clean up and split
  split <- unlist(strsplit(licenses, " \\+ |\\+"))

  # Clean leading and trailing blanks
  split <- unique(trimws(split))

  licenses_df <- data.frame(LICENSE = split)

  # Read mapping

  # Merge
  licenses_df <- merge(licenses_df, cffr::cran_to_spdx)

  # Clean results
  licenses_list <- lapply(licenses_df$SPDX, clean_str)
  licenses_list <- drop_null(licenses_list)

  license_char <- unlist(licenses_list)

  license_char
}

#' Try to get Repository
#' @noRd
get_desc_repository <- function(pkg) {
  name <- pkg$get("Package")
  repo <- clean_str(pkg$get("Repository"))

  # Repo is url
  if (is_url(repo)) {
    return(repo)
  }

  # Check if Bioconductor
  # biocViews is required in Bioconductor packages
  # http://contributions.bioconductor.org/description.html#biocviews
  if (!is.null(clean_str(pkg$get("biocViews")))) {
    return("https://bioconductor.org/")
  }

  # Repo is CRAN
  # Canonic url to CRAN
  if (is_substring(repo, "^CRAN$")) {
    return(
      paste0("https://CRAN.R-project.org/package=", name)
    )
  }

  # If not, search on installed repos

  repourl <- search_on_repos(name)

  repourl
}

#' Mapped to Package & Title
#' @noRd
get_desc_title <- function(pkg) {
  title <- paste0(
    pkg$get("Package"),
    ": ",
    pkg$get("Title")
  )

  title <- clean_str(title)
  title
}

#' Mapped to `URL` and `BugReports`.
#' Additional URLs as identifiers.
#' @noRd
get_desc_urls <- function(pkg) {
  url <- pkg$get_urls()

  # Get issue url
  issues <- tryCatch(pkg$get_field("BugReports")[1], error = function(cond) {
    pkg$get_urls()
  })

  # Clean if GitLab
  issues <- gsub("/-/issues$", "", issues)
  # Clean if GitHub and codeberg.org
  issues <- gsub("/issues$", "", issues)

  # Join issues and urls
  allurls <- unique(c(issues, url))
  allurls <- allurls[is_url(allurls)]

  # If no urls then return as null
  if (length(allurls) == 0) {
    url_list <- list(url = NULL)
    return(url_list)
  }
  # Try to find an url of the repo
  domains <- paste0(
    c(
      "github.com",
      "www.github.com",
      "gitlab.com",
      "r-forge.r-project.org",
      "bitbucket.org",
      "codeberg.org"
    ),
    collapse = "|"
  )

  # Extract repo url
  repo_line <- grep(domains, allurls, ignore.case = TRUE)[1]

  repository_code <- clean_str(allurls[repo_line][1])

  if (!is.na(repo_line)) {
    remaining <- allurls[-repo_line]
  } else {
    remaining <- allurls
  }

  # The second url is considered for url arbitrarily
  if (isTRUE(length(remaining) > 0)) {
    url_end <- remaining[1]
    remaining <- remaining[-1]
  } else {
    url_end <- repository_code
  }
  url_end <- clean_str(url_end)

  # If there are more, move them to identifiers

  if (isTRUE(length(remaining) > 0)) {
    identifiers <- lapply(remaining, function(x) {
      list(
        type = "url",
        value = clean_str(x)
      )
    })
  } else {
    identifiers <- NULL
  }

  url_list <- list(
    repo = clean_str(repository_code),
    url = clean_str(url_end),
    identifiers = identifiers
  )
  url_list
}

#' Mapped to Version
#' @noRd
get_desc_version <- function(pkg) {
  version <- pkg$get("Version")

  version <- clean_str(version)
  version <- unname(version)

  version
}

#' Extract topics as keywords for GH hosted packages
#' @noRd
get_gh_topics <- function(x) {
  # Only for GitHub repos
  if (!is_github(x)) {
    return(NULL)
  }

  # Get topics from repo
  api_url <- paste0(
    "https://api.github.com/repos",
    "/",
    gsub(
      "^http[a-z]://github.com/",
      "",
      x["repository-code"]
    )
  )

  tmpfile <- tempfile(fileext = ".json")

  # See if GH_TOKEN is set on Renv
  # I have an issue on testing, I reach
  # fast the GH api limit (no auth)
  # Need to auth to increase limit
  # Try to get an stored token
  token <- c(Sys.getenv(c("GITHUB_TOKEN", "GITHUB_PAT")))
  token <- token[!token %in% c(NA, NULL, "")][1]

  ghtoken <- paste("token", token)

  # nocov start

  # Try with GHTOKEN
  res <- tryCatch(
    download.file(
      api_url,
      tmpfile,
      quiet = TRUE,
      headers = c(Authorization = ghtoken),
      mode = "wb"
    ),
    warning = function(e) {
      TRUE
    },
    error = function(e) {
      TRUE
    }
  )

  # If it fails try with normal call
  if (isTRUE(res)) {
    # Regular call
    res <- tryCatch(
      download.file(api_url, tmpfile, quiet = TRUE, mode = "wb"),
      warning = function(e) {
        TRUE
      },
      error = function(e) {
        TRUE
      }
    )
  }

  # nocov end
  if (isTRUE(res)) {
    return(NULL)
  }

  remotetopics <- lapply(jsonlite::read_json(tmpfile)$topics, clean_str)
  remotetopics <- unique(unlist(remotetopics))

  # If no topics NULL
  if (is.null(remotetopics)) {
    return(NULL)
  }

  remotetopics
}

get_desc_sha <- function(pkg) {
  sha <- pkg$get("RemoteSha")

  sha <- clean_str(sha)
  sha <- unname(sha)

  sha
}

get_desc_doi <- function(pkg) {
  pkg <- pkg$get("Package")
  if (is.null(search_on_repos(pkg))) {
    return(NULL)
  }

  doi <- paste0("10.32614/CRAN.package.", pkg)
  clean_str(doi)
}

#' Merge the information of a coerced description with a coerced citation
#' @noRd
merge_desc_cit <- function(cffobj, citobj) {
  # If no citobj then return null

  if (is.null(citobj)) {
    return(cffobj)
  }
  # Play with cran DOI
  cran_doi <- NULL
  has_cran_doi <- FALSE
  if (all(!is.null(cffobj$doi), grepl("CRAN", cffobj$doi, fixed = TRUE))) {
    has_cran_doi <- TRUE
    cran_doi <- cffobj$doi
    # First the citation object
    cffobj$doi <- NULL
  }

  # Add doi from citation if missing
  if (is.null(cffobj$doi)) {
    cffobj$doi <- clean_str(citobj[[1]]$doi)
  }

  citobj <- as.list(citobj)
  cffobjend <- c(
    cffobj,
    "preferred-citation" = citobj[1],
    references = list(citobj[-1])
  )

  # Merge identifiers
  merged_ids <- c(citobj[[1]]$identifiers, cffobjend$identifiers)

  if (has_cran_doi) {
    cranid <- as_cff(list(
      list(type = "doi", value = clean_str(cran_doi))
    ))
    merged_ids <- c(cranid, merged_ids)
  }
  cffobjend$identifiers <- merged_ids

  # Reorder
  cffobjfinal <- c(
    cffobjend[!names(cffobjend) %in% c("identifiers", "references")],
    cffobjend["identifiers"],
    cffobjend["references"]
  )

  cffobjfinal <- drop_null(cffobjfinal)

  # Reclass everything
  cffobjfinal <- as_cff(as.list(cffobjfinal))

  cffobjfinal
}

#' Enhance authors info from preferred-citation using metadata from DESCRIPTION
#' @noRd
enhance_pref_authors <- function(cffobjend) {
  # Create index of authors extracted from DESCRIPTION (First cff level)
  auth_desc <- cffobjend$authors
  key_aut_desc <- lapply(auth_desc, function(x) {
    l <- list(x["family-names"], x["given-names"], x["name"])
    l <- unlist(drop_null(l))
    tolower(paste0(l, collapse = ""))
  })
  names(auth_desc) <- unlist(key_aut_desc)

  # Create index of authors from preferred-citation
  auth_pref <- cffobjend$`preferred-citation`$authors
  key_aut_cit <- lapply(auth_pref, function(x) {
    l <- list(x["family-names"], x["given-names"], x["name"])
    l <- unlist(drop_null(l))
    tolower(paste0(l, collapse = ""))
  })
  names(auth_pref) <- unlist(key_aut_cit)

  # Add missing keys to authors
  enhancedauth <- lapply(names(auth_pref), function(x) {
    newdata <- auth_desc[x]
    olddata <- auth_pref[x]

    # New fields only
    oldkeys <- names(olddata[[1]])
    newkeys <- names(newdata[[1]])
    fieldstoadd <- newdata[[1]][!newkeys %in% oldkeys]

    newinfo <- c(olddata[[1]], fieldstoadd)

    newinfo
  })

  enhancedauth
}

get_dependencies <- function(
  desc_path,
  instpack = as.character(
    installed.packages()[, "Package"]
  )
) {
  # nocov start
  if (!is.character(desc_path)) {
    return(NULL)
  }
  if (!file_exist_abort(desc_path)) {
    return(NULL)
  }
  # nocov end

  getdeps <- desc::desc(desc_path)

  deps <- getdeps$get_deps()

  # Adapt version

  deps$version_clean <- gsub("*", "", deps$version, fixed = TRUE)

  # Save copy for later
  origdeps <- deps

  # Dedupe rows
  deps <- unique(deps[, c("package", "version_clean")])

  # Get dependency type and add to scope
  scope <- vapply(
    deps$package,
    function(x) {
      y <- origdeps[origdeps$package == x, "type"]

      y[1]
    },
    FUN.VALUE = character(1)
  )
  deps$scope <- scope

  av_deps <- deps[deps$package %in% c("R", instpack), ]

  # Get references from DESCRIPTION of dependencies
  cff_deps <- lapply(seq_len(nrow(av_deps)), function(y) {
    n <- av_deps[y, ]

    if (n$package == "R") {
      mod <- as_cff(citation())[[1]]
      mod$year <- format(Sys.Date(), "%Y")
    } else {
      mod <- try(
        as_cff(citation(n$package, auto = TRUE)[1])[[1]],
        silent = TRUE
      )

      if (inherits(mod, "try-error")) {
        return(NULL) # nocov
      }

      # Simplified version of the cff obj
      # Avoid cluttering the output
      mod$abstract <- mod$title
      mod$title <- n$package
      # If on CRAN add CRAN DOI
      # Base packs
      base_pkgs <- rownames(installed.packages(priority = "base"))

      cran_doi <- all(
        n$package %in% avail_on_init$Package,
        !n$package %in% base_pkgs
      )
      if (cran_doi) {
        mod$doi <- paste0("10.32614/CRAN.package.", n$package)
      }
    }

    mod$type <- "software"
    mod$version <- ifelse(is.na(n$version_clean), NULL, paste(n$version_clean))
    # Get url and repo from package DESCRIPTION
    # urls from citation() vary due to auto = TRUE
    dfile <- system.file("DESCRIPTION", package = n$package)

    if (file_exist_abort(dfile)) {
      pkg <- desc::desc(dfile)
      mod$url <- get_desc_urls(pkg)$url
      mod$repository <- get_desc_repository(pkg)
    }

    mod <- drop_null(mod)

    # Get year
    date_rel <- mod[["date-released"]]

    if (is.null(date_rel)) {
      year <- format(Sys.Date(), "%Y")
    } else {
      year <- format(as.Date(date_rel), "%Y")
    }

    mod$year <- year
    mod$notes <- clean_str(n$scope)

    # Re-arrange
    mod <- c(
      mod[c("type", "title", "abstract", "notes", "url", "repository")],
      mod[
        !names(mod) %in%
          c(
            "type",
            "title",
            "abstract",
            "notes",
            "url",
            "repository"
          )
      ]
    )

    mod <- as_cff(mod)
  })

  cff_deps <- drop_null(cff_deps)

  cff_deps <- unique(cff_deps)

  class(cff_deps) <- "cff"

  cff_deps
}

#' Export **R** objects to different file types
#'
#' @description
#'
#' Export **R** objects representing citations to specific file formats:
#' - [cff_write_bib()] creates a `.bib` file.
#' - [cff_write_citation()] creates an **R** citation file as described in
#'   Section 1.9 of *Writing R Extensions* (R Core Team 2023).
#'
#' @param x A [`bibentry`] or a [`cff`] object.
#' @param file Name of the file to be created. If `NULL`, the lines are
#'   displayed instead.
#' @param append Logical. Should entries be appended to an existing file?
#' @param verbose Logical. Display informative messages.
#' @param ascii Logical. Should entries be written using ASCII characters only?
#' @inheritDotParams as_bibentry.cff what
#' @inheritDotParams as_bibentry.cff_ref
#' @inheritDotParams as_bibentry.cff_ref_lst
#'
#' @references
#'
#' - R Core Team (2023). _Writing R Extensions_.
#'   <https://cran.r-project.org/doc/manuals/r-release/R-exts.html>
#'
#' @export
#' @encoding UTF-8
#' @rdname cff_write_misc
#' @family bibtex
#' @family writing
#'
#' @return
#' Writes the corresponding file specified on the `file` argument.
#'
#' @details
#'
#' When `x` is a `cff` object, it is converted to BibTeX using
#' [toBibtex.cff()].
#'
#' For security reasons, if the file already exists, the function creates
#' a backup copy in the same directory.
#'
#' @seealso
#' `vignette("bibtex_cff", "cffr")`, [knitr::write_bib()] and the
#' following packages:
#' - \CRANpkg{bibtex}.
#' - \CRANpkg{RefManageR}
#' - \CRANpkg{rbibutils}
#'
#' @examples
#'
#' bib <- bibentry("Misc",
#'   title = "My title",
#'   author = "Fran Pérez"
#' )
#'
#' my_temp_bib <- tempfile(fileext = ".bib")
#'
#' cff_write_bib(bib, file = my_temp_bib)
#'
#' cat(readLines(my_temp_bib), sep = "\n")
#'
#' cff_write_bib(bib, file = my_temp_bib, ascii = TRUE, append = TRUE)
#'
#' cat(readLines(my_temp_bib), sep = "\n")
cff_write_bib <- function(
  x,
  file = tempfile(fileext = ".bib"),
  append = FALSE,
  verbose = TRUE,
  ascii = FALSE,
  ...
) {
  if (inherits(x, "cff")) {
    x <- as_bibentry(x, ...)
  }

  if (!inherits(x, "bibentry")) {
    cli::cli_abort(
      paste0(
        "{.arg x} should be a {.cls bibentry} object, not a ",
        "{.cls {class(x)}} object."
      )
    )
  }

  btex <- toBibtex(x)

  if (ascii) {
    # Base encoding as per file()
    btex <- encoded_utf_to_latex(btex)
    class(btex) <- "Bibtex"
  } else {
    btex <- enc2utf8(btex)
  }

  if (detect_x_source(file) != "bib") {
    file <- paste0(file, ".bib")
  }
  write_lines_msg(btex, file, verbose, append)
  invisible(NULL)
}

#' @export
#' @encoding UTF-8
#' @rdname cff_write_misc
#' @name cff_write_citation
#' @examples
#'
#' # Create a CITATION file
#'
#' # Use a system file
#' f <- system.file("examples/preferred-citation-book.cff", package = "cffr")
#' a_cff <- cff_read(f)
#'
#' out <- file.path(tempdir(), "CITATION")
#' cff_write_citation(a_cff, file = out)
#'
#' # Check by reading, use meta object
#' meta <- packageDescription("cffr")
#' meta$Encoding <- "UTF-8"
#'
#' utils::readCitationFile(out, meta)
cff_write_citation <- function(
  x,
  file = tempfile("CITATION_"),
  append = FALSE,
  verbose = TRUE,
  ...
) {
  if (inherits(x, "cff")) {
    x <- as_bibentry(x, ...)
  }

  if (!inherits(x, "bibentry")) {
    cli::cli_abort(
      paste0(
        "{.arg x} should be a {.cls bibentry} object, not a ",
        "{.cls {class(x)}} object."
      )
    )
  }

  bentr <- format(x, style = "R")
  bentr <- c("", bentr)
  write_lines_msg(bentr, file, verbose, append)
  invisible(NULL)
}

test_that("Errors", {
  expect_snapshot(cff_write_bib(1:4), error = TRUE)
})

test_that("Write", {
  bib <- bibentry("Misc", title = "My title", author = "Fran Pérez")

  file <- file.path(tempdir(), "noext")
  expect_message(cff_write_bib(bib, file, verbose = TRUE))

  # Fix extensions
  file <- paste0(file, ".bib")
  expect_true(file_exist_abort(file))

  expect_snapshot_file(file)

  # Check backup
  expect_false(file_exist_abort(paste0(file, ".bk1")))

  # Check now backup exists
  cff_write_bib(bib, file, append = TRUE)
  expect_true(file_exist_abort(paste0(file, ".bk1")))

  file.remove(file)
  file.remove(paste0(file, ".bk1"))
})

test_that("Write ASCII", {
  bib <- bibentry("Misc", title = "My title", author = "Fran Pérez")

  file <- file.path(tempdir(), "ascii.bib")
  expect_silent(cff_write_bib(bib, file, verbose = FALSE, ascii = TRUE))

  # Fix extensions
  expect_snapshot_file(file)
  file.remove(file)
})

test_that("Test append", {
  bib <- bibentry("Misc", title = "My title", author = "Fran Herrero")
  cf <- system.file("examples/DESCRIPTION_basic", package = "cffr")

  a_cff <- cff_read(cf)

  file <- file.path(tempdir(), "append.bib")
  expect_silent(cff_write_bib(bib, file, verbose = FALSE, append = FALSE))

  # Initial lines
  lines1 <- readLines(file)

  # Append
  bib2 <- bibentry(
    "Misc",
    key = "key2",
    title = "Another title",
    author = "Ian Henderson"
  )

  cff_write_bib(bib2, file, verbose = FALSE, append = TRUE)
  cff_write_bib(a_cff, file, verbose = FALSE, append = TRUE)
  expect_snapshot_file(file)

  lines2 <- readLines(file)

  # First lines identical, more lines on append
  expect_true(all(lines1 == lines2[seq_along(lines1)]))

  expect_gt(length(lines2), length(lines1))

  # Overwrite
  cff_write_bib(bib2, file, verbose = FALSE, append = FALSE)
  lines3 <- readLines(file)

  expect_false(all(lines1 == lines3[seq_along(lines1)]))
  expect_lt(length(lines3), length(lines2))
})


test_that("Test dir creation", {
  bib <- bibentry("Misc", title = "My title", author = "Fran Herrero")

  file <- file.path(tempdir(), "idontexist", "append.bib")

  dir <- dirname(file)

  expect_false(dir.exists(dir))
  expect_silent(cff_write_bib(bib, file, verbose = FALSE))

  expect_true(dir.exists(dir))
  expect_true(file_exist_abort(file))

  unlink(dir, recursive = TRUE, force = TRUE)

  # With messages
  file <- file.path(tempdir(), "nowiamverbose", "append.bib")
  dir <- dirname(file)
  expect_false(dir.exists(dir))
  expect_message(cff_write_bib(bib, file, verbose = TRUE), "Creating directory")

  expect_true(dir.exists(dir))
  expect_true(file_exist_abort(file))

  unlink(dir, recursive = TRUE, force = TRUE)
})

test_that("Errors citation", {
  expect_snapshot(cff_write_citation(1:4), error = TRUE)
})

test_that("Write CITATION", {
  f <- system.file("examples", package = "cffr")
  thepath <- list.files(f, pattern = "DESCRIPTION_basicdate", full.names = TRUE)
  f1 <- cff_read(thepath)

  bib <- bibentry("Misc", title = "My title", author = "Fran Pérez")

  file <- file.path(tempdir(), "CITAT_ION")
  expect_message(cff_write_citation(bib, file, verbose = TRUE))

  # Check backup
  expect_false(file_exist_abort(paste0(file, ".bk1")))

  # Check now backup exists and use cff
  expect_silent(cff_write_citation(
    f1,
    file,
    verbose = FALSE,
    what = "all",
    append = TRUE
  ))

  expect_snapshot_file(file)

  file.remove(file)
  file.remove(paste0(file, ".bk1"))
})

## -----------------------------------------------------------------------------
#| include: false
library(cffr)


## -----------------------------------------------------------------------------
#| label: setup
#| eval: false
# library(cffr)
# 
# cff_write()
# 
# # You are done!


## -----------------------------------------------------------------------------
#| label: newfields
newobject <- cff()

newobject


## -----------------------------------------------------------------------------
#| label: validkeys
cff_schema_keys()


## -----------------------------------------------------------------------------
#| label: modify
modobject <- cff_modify(
  newobject,
  url = "https://ropensci.org/",
  version = "0.0.1",
  repository = "https://github.com/ropensci/cffr",
  # If the field is already present, it is overridden
  title = "Modifying a 'cff' object"
)

modobject

# Validate against the schema

cff_validate(modobject)


## -----------------------------------------------------------------------------
#| label: includeauthor
# Valid person keys

cff_schema_definitions_person()

# Create the person

chiquito <- person(
  "Gregorio",
  "Sánchez Fernández",
  email = "fake@email2.com",
  comment = c(
    alias = "Chiquito de la Calzada",
    city = "Malaga",
    country = "ES",
    ORCID = "0000-0000-0000-0001"
  )
)

chiquito

# To cff
chiquito_cff <- as_cff_person(chiquito)
chiquito_cff

# Append to previous authors

newauthors <- c(modobject$authors, chiquito_cff)
newauthors

newauthorobject <- cff_modify(modobject, authors = newauthors)

newauthorobject

cff_validate(newauthorobject)


## -----------------------------------------------------------------------------
#| label: parsingcits
# Valid reference keys

cff_schema_definitions_refs()

# Auto coercion from another R package
base_r <- citation("base")

bib <- bibentry(
  "Book",
  title = "This is a book",
  author = "Lisa Lee",
  year = 1980,
  publisher = "McGraw Hill",
  volume = 2
)

refs <- c(base_r, bib)

refs

# Now to cff

refs_cff <- as_cff(refs)

refs_cff


## -----------------------------------------------------------------------------
#| label: references
finalobject <- cff_modify(newauthorobject, references = refs_cff)

finalobject

cff_validate(finalobject)


## -----------------------------------------------------------------------------
#| label: write
# For example
tmp <- tempfile(fileext = ".cff")

see_res <- cff_write(finalobject, outfile = tmp)

cat(readLines(tmp), sep = "\n")


## -----------------------------------------------------------------------------
#| label: read
reading <- cff_read(tmp)

reading


## -----------------------------------------------------------------------------
allkeys <- list(
  "url" = "https://ropensci.org/",
  "version" = "0.0.1",
  "repository" = "https://github.com/ropensci/cffr",
  # If the field is already present, it is overridden
  title = "Modifying a 'cff' object",
  authors = newauthors,
  references = refs_cff
)

tmp2 <- tempfile(fileext = ".cff")

res <- cff_write(cff(), outfile = tmp2, keys = allkeys)

res


## -----------------------------------------------------------------------------
#| include: false
# Clean temps
unlink(tmp)
unlink(tmp2)

#' Mapping between `License` fields and SPDX
#'
#' A dataset containing the mapping between the `License` strings observed
#' on CRAN packages and its (approximate) match on the
#' [SPDX License List](https://spdx.org/licenses/).
#'
#' @family datasets
#' @docType data
#' @encoding UTF-8
#' @name cran_to_spdx
#' @rdname cran_to_spdx
#'
#' @format
#' A data frame with `r nrow(cran_to_spdx)` rows and 2 variables:
#' \describe{
#'   \item{`LICENSE`}{A valid `License` string on CRAN.}
#'   \item{`SPDX`}{A valid SPDX License Identifier.}
#' }
#'
#' @seealso *Writing R Extensions*,
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [Licensing section]",
#'            "(https://cran.r-project.org/doc/manuals/",
#'            "r-release/R-exts.html#Licensing)."))
#'
#' ```
#'
#' @source <https://spdx.org/licenses/>
#'
#' @examples
#'
#' data("cran_to_spdx")
#'
#' head(cran_to_spdx, 20)
NULL

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

# Import namespace directives
#' @importFrom utils download.file as.person read.csv person capture.output
#' @importFrom utils installed.packages packageDescription citation
#' @importFrom utils available.packages modifyList toBibtex bibentry head tail
NULL

#' Create a [`cff`] object from several sources
#'
#' @description
#'
#' Create a full and possibly valid [`cff`] object from a given source. This
#' object can be written to a `*.cff` file with [cff_write()],
#' see **Examples**.
#'
#' Most of the heavy lifting of \CRANpkg{cffr} is done by this function.
#'
#' @return A [`cff`] object.
#'
#' @family core
#'
#' @export
#' @encoding UTF-8
#'
#' @param x The source used to generate
#'   the [`cff`] object. It can be:
#'   - A missing value, which retrieves the `DESCRIPTION` file from your
#'     in-development **R** package.
#'   - An existing [`cff`] object.
#'   - The name of an installed package (`"jsonlite"`).
#'   - Path to a `DESCRIPTION` file (`"./DESCRIPTION"`).
#'
#' @param keys
#'   List of additional keys to add to the [`cff`] object. See
#'   [cff_modify()].
#' @param cff_version The Citation File Format schema version that the
#'   `CITATION.cff` file adheres to for providing the citation metadata.
#' @param gh_keywords Logical `TRUE/FALSE`. If the package is hosted on
#'   GitHub, would you like to add the repo topics as keywords?
#' @param dependencies Logical `TRUE/FALSE`. Should the dependencies of your
#'   package be added to the `references` CFF key?
#' @param authors_roles Roles to be considered as authors of the package when
#'   generating the `CITATION.cff` file. See **Details**.
#'
#' @seealso
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("[Guide to Citation File Format schema version 1.2.0]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#'
#' - [cff_modify()] as the recommended way to modify a `cff` object.
#' - [cff_write()] for creating a CFF file.
#' - `vignette("cffr", "cffr")` shows an introduction on how manipulate
#'   [`cff`] objects.
#' - `vignette("crosswalk", package = "cffr")` provides details on how the
#'  metadata of a package is mapped to produce a `cff` object.
#'
#' @details
#'
#' If `x` is a path to a `DESCRIPTION` file or if `inst/CITATION` is not present
#' on your package, \CRANpkg{cffr} would auto-generate a `preferred-citation`
#' key using the information provided on that file.
#'
#' By default, only persons whose role in the `DESCRIPTION` file of the package
#' is author (`"aut"`) or maintainer (`"cre"`) are considered to be authors
#' of the package. The default setting can be controlled via the `authors_roles`
#' argument. See **Details** on [person()] to get additional insights
#' on person roles.
#'
#' @examples
#' \donttest{
#' # Installed package
#' cff_create("jsonlite")
#'
#' # Demo file
#' demo_file <- system.file("examples/DESCRIPTION_basic", package = "cffr")
#' cff_create(demo_file)
#'
#' # Add additional keys
#'
#' newkeys <- list(
#'   message = "This overwrites fields",
#'   abstract = "New abstract",
#'   keywords = c("A", "new", "list", "of", "keywords"),
#'   authors = as_cff_person("New author")
#' )
#'
#' cff_create(demo_file, keys = newkeys)
#'
#' # Update a field on a list, for example authors or contacts.
#' # We are adding a new contact here
#'
#' old <- cff_create(demo_file)
#'
#' new_contact <- append(
#'   old$contact,
#'   as_cff_person(person(
#'     given = "I am",
#'     family = "New Contact"
#'   ))
#' )
#'
#' cff_create(demo_file, keys = list("contact" = new_contact))
#' }
cff_create <- function(
  x,
  keys = list(),
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  dependencies = TRUE,
  authors_roles = c("aut", "cre")
) {
  # Guess source
  # On missing add getwd()
  if (missing(x)) {
    hint_source <- "indev"
    x <- getwd()
  } else if (identical(getwd(), x)) {
    # This case is coming from cff_write
    hint_source <- "indev"
  } else {
    hint_source <- detect_x_source(x)
  }

  # Abort in non-valid sources
  valid_sources <- c("indev", "cff_obj", "package", "description")
  if (!hint_source %in% valid_sources) {
    # Abort, prepare message
    msg_hint <- switch(hint_source,
      "dontknow" = paste0(
        "If it is a package ",
        "you may need to install it with ",
        "{.fn install.packages}."
      ),
      "bib" = "Maybe try with {.fn cff_read}."
    )

    cli::cli_abort(
      paste0("{.arg x} not valid. ", msg_hint)
    )
  }

  # Build cff and return paths if any
  result_paths <- build_cff_and_paths(
    x,
    cff_version,
    gh_keywords,
    dependencies,
    authors_roles,
    hint_source
  )

  desc_path <- result_paths[["desc_path"]]
  cffobjend <- result_paths[["cffobjend"]]

  # Add software dependencies
  if (dependencies) {
    instpack <- as.character(installed.packages()[, "Package"])
    deps <- get_dependencies(desc_path, instpack)

    cffobjend$references <- unique(c(cffobjend$references, deps))
  }

  # Additional keys, using internals of cff_modify
  cffobjend <- modify_cff(cffobjend, keys, "keys")

  # Order
  cffobjend <- cffobjend[cff_schema_keys()]

  # Enhance authors info
  if (!is.null(cffobjend$`preferred-citation`)) {
    cffobjend$`preferred-citation`$authors <- enhance_pref_authors(cffobjend)
  }
  cffobjend <- as_cff(cffobjend)
  cffobjend
}

build_cff_and_paths <- function(
  x,
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  dependencies = TRUE,
  authors_roles = c("aut", "cre"),
  hint_source
) {
  collect_list <- list(
    desc_path = NULL,
    cffobjend = NULL
  )

  # "indev", "cff_obj", "package", "description"

  # Already cff, return it
  if (is_cff(x)) {
    # It is already an object
    cffobj <- as_cff(as.list(x))
    cffobj["cff-version"] <- cff_version

    collect_list$cffobjend <- cffobj
    return(collect_list)
  }

  # Get info from DESCRIPTION
  desc_path <- switch(hint_source,
    "indev" = file.path(getwd(), "DESCRIPTION"),
    "description" = x,
    "package" = system.file("DESCRIPTION", package = x)
  )

  if (is.null(file_path_or_null(desc_path))) {
    cli::cli_abort("No {.file DESCRIPTION} file found with {.arg x}.")
  }

  cffobj <- cff_read_description(
    desc_path,
    cff_version,
    gh_keywords = gh_keywords,
    authors_roles = authors_roles
  )

  # Just for description case
  try_get_citation <- function(x) {
    cit1 <- file.path(dirname(x), "inst/CITATION")
    cit2 <- file.path(dirname(x), "CITATION")

    c(file_path_or_null(cit1), file_path_or_null(cit2))[1]
  }

  cit_path <- switch(hint_source,
    "indev" = file.path(getwd(), "inst/CITATION"),
    "description" = try_get_citation(x),
    "package" = system.file("CITATION", package = x)
  )

  cit_path <- file_path_or_null(cit_path[1])

  if (!is.null(cit_path)) {
    citobj <- cff_safe_read_citation(desc_path, cit_path)
    citobj <- unique(citobj)
    # Merge DESCRIPTION and CITATION
    cffobj <- merge_desc_cit(cffobj, citobj)
  }

  collect_list$desc_path <- desc_path
  collect_list$cffobjend <- cffobj

  collect_list
}

test_that("Parse Licenses on CRAN (Top 10)", {
  cran_packs <- read.csv(system.file(
    "extdata/cran_licenses.csv",
    package = "cffr"
  ))
  cran_packs <- cran_packs[order(cran_packs$n, decreasing = TRUE), ]

  sel <- as.character(cran_packs[1:10, "LICENSE"])

  # Tempfile DESCRIPTION that we use for modifications
  tmp <- tempfile("DESCRIPTION")
  file.copy(
    system.file("examples/DESCRIPTION_basic", package = "cffr"),
    tmp,
    overwrite = TRUE
  )
  for (i in seq_along(sel)) {
    p <- desc::desc_set("License", sel[i], file = tmp)
    expect_silent(r <- cff_validate(cff_create(tmp), verbose = FALSE))
    expect_true(r)
  }
})

#' Validate a `CITATION.cff` file or a [`cff`] object
#'
#' @description
#' Validate a `CITATION.cff` file or a [`cff`] object using the corresponding
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0(" [validation schema]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema.json)."))
#'
#' ```
#' @export
#' @encoding UTF-8
#'
#' @family core
#'
#' @seealso
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("[Guide to Citation File Format schema version 1.2.0]",
#'            "(https://github.com/citation-file-format/",
#'            "citation-file-format/blob/main/schema-guide.md)."))
#'
#' ```
#'
#' @return
#'
#' A message indicating the result of the validation and an invisible value
#' `TRUE/FALSE`. On error, the result has an attribute `"errors"`
#' containing the error summary (see **Examples** and [attr()]).
#'
#' @param x This is expected to be either a full `cff` object created
#'   with [cff_create()] or the path to a `CITATION.cff` file to be validated.
#'   In the case of a `*.cff` file it would read with [cff_read_cff_citation()].
#' @inheritParams cff_write
#'
#' @seealso
#' [jsonvalidate::json_validate()], which is the function that performs the
#' validation.
#'
#' @examples
#' \donttest{
#' # Full .cff example
#' cff_validate(system.file("examples/CITATION_complete.cff", package = "cffr"))
#'
#' # Validate a cffr object
#' cffr <- cff_create("jsonlite")
#' class(cffr)
#' cff_validate(cffr)
#' }
#'
#' # .cff with errors
#' err_f <- system.file("examples/CITATION_error.cff", package = "cffr")
#' # Can manipulate the errors as data frame
#' res <- try(cff_validate(err_f))
#'
#' isTRUE(res)
#' isFALSE(res)
#'
#' # Detailed results
#' df_errors <- attr(res, "errors")
#'
#' df_errors[, c("field", "message")]
#'
#' # If a CITATION file is supplied (note that it is not a .cff file), it
#' # throws an error
#' try(cff_validate(system.file("CITATION", package = "cffr")))
cff_validate <- function(x = "CITATION.cff", verbose = TRUE) {
  # If the input is a cff object, create the object
  if (!is_cff(x)) {
    # Check
    abort_if_not_cff(x)
    is_a <- paste0("{.file ", x, "}")
    # nolint end
    x <- cff_read_cff_citation(x)
  } else {
    is_a <- "This {.cls cff}"
  }

  # Convert to list
  citfile <- as.list(x)

  # This prevents errors with jsonvalidate
  citfile <- rapply(citfile, function(x) as.character(x), how = "replace")

  # Convert to json
  cit_temp <- jsonlite::toJSON(citfile, pretty = TRUE, auto_unbox = TRUE)

  # Use local copy of the validator schema
  schema_local <- system.file("schema/schema.json", package = "cffr")

  # Validate
  result <- validate_schema(cit_temp, schema_local)

  if (!result) {
    get_errors <- attr(result, "errors")
    field <- get_errors$instancePath
    field[field == ""] <- "/"
    field <- paste0("cff", field)
    get_errors$field <- field
    msg <- get_errors$message
    # Escape braces
    msg <- gsub("{", "{{", msg, fixed = TRUE)
    msg <- gsub("}", "}}", msg, fixed = TRUE)

    if (verbose) {
      cli::cat_rule("Validating cff", col = "cyan", line = 2)
      ll <- paste0(
        "* {.dt {.strong ",
        get_errors$field,
        "}}{.dl ",
        msg,
        "}\n",
        collapse = ""
      )
      cli::cli_alert_danger(
        paste0("Oops! ", is_a, " has the following errors:\n", ll)
      )
    }

    # Prepare output
    attr(result, "errors") <- get_errors[, unique(c(
      "field",
      "message",
      names(get_errors)
    ))]
    return(invisible(result))
  }

  if (verbose) {
    cli::cat_rule("Validating cff", col = "cyan", line = 2)
    cli::cli_alert_success(
      paste0("Congratulations! ", is_a, " is valid")
    )
  }
  invisible(result)
}

# Validate schema
#' @noRd
validate_schema <- function(cit_temp, schema_temp) {
  x <- suppressMessages(
    jsonvalidate::json_validate(
      cit_temp,
      schema_temp,
      verbose = TRUE,
      engine = "ajv"
    )
  )

  x
}

#' The `cff` class
#' @encoding UTF-8
#'
#' @rdname cff_class
#' @name cff_class
#' @aliases cff_ref cff_ref_lst cff_pers cff_pers_lst
#'
#' @keywords internal
#'
#' @family s3method
#'
#' @description
#'
#' ```{r child = "man/chunks/cffclass.Rmd"}
#' ```
#'
#' @references
#'
#' - Wickham H (2019). "S3." In _Advanced R_, 2nd edition.
#'   Chapman and Hall/CRC. \doi{10.1201/9781351201315},
#'   <https://adv-r.hadley.nz/s3.html>.
NULL

#!/bin/bash
# Adapted from https://github.com/lorenzwalthert/precommit/blob/main/inst/bin/readme-rmd-rendered
MSG="use 'git commit --no-verify' to override this check"

# No CITATION.cff
if [[ ! -f "CITATION.cff" ]]; then
  exit 0
fi

# Check commit

CITATION=($(git diff --cached --name-only | grep -Ei '^CITATION\.cff$'))
DESC=($(git diff --cached --name-only | grep -Ei '^DESCRIPTION$'))
CIT=($(git diff --cached --name-only | grep -Ei '^inst\/CITATION$'))

# If DESCRIPTION staged but not CITATION.cff
if [[ ! -z "$DESC" && -z "$CITATION" ]]; then
  if [[ DESCRIPTION -nt CITATION.cff ]]; then
    echo -e "CITATION.cff may be outdated; please use cff_write()\n$MSG"
    exit 1
  elif [[ CITATION.cff -nt DESCRIPTION ]]; then
    echo -e "Remember to stage both CITATION.cff and DESCRIPTION\n$MSG"
    exit 1
  fi
fi

# If inst/CITATION staged but not CITATION.cff
if [[ ! -z "$CIT" && -z "$CITATION" ]]; then
  if [[ inst/CITATION -nt CITATION.cff ]]; then
    echo -e "CITATION.cff may be outdated; please use cff_write()\n$MSG"
    exit 1
  elif [[ CITATION.cff -nt inst/CITATION ]]; then
    echo -e "Remember to stage both CITATION.cff and inst/CITATION\n$MSG"
    exit 1
  fi
fi

---
title: "From R to CFF"
subtitle: "Crosswalk"
description: >-
  A comprehensive description of the internal mappings performed by 
  the cffr package.
author: Diego Hernangómez
bibliography: REFERENCES.bib
link-citations: yes
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{From R to CFF}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(cffr)
```

The goal of this vignette is to provide an explicit map between the metadata
fields used by **cffr** and each one of the valid keys of the [Citation File
Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#valid-keys).

## Summary {#summary}

We summarize here the fields that **cffr** can coerce and the original source of
information for each one of them. The details on each key are presented on the
next section of the document. The assessment of fields are based on the [Guide
to Citation File Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#valid-keys)
[@druskat_citation_2021].

```{r summary , echo=FALSE}
keys <- cff_schema_keys(sorted = TRUE)
origin <- vector(length = length(keys))
origin[keys == "cff-version"] <- "parameter on function"
origin[keys == "type"] <- "Fixed value: 'software'"
origin[keys == "identifiers"] <- "DESCRIPTION/CITATION files"
origin[keys == "references"] <- "DESCRIPTION/CITATION files"

origin[keys %in% c(
  "message",
  "title",
  "version",
  "authors",
  "abstract",
  "repository",
  "repository-code",
  "url",
  "date-released",
  "contact",
  "keywords",
  "license"
)] <- "DESCRIPTION file"

origin[keys %in% c(
  "doi",
  "preferred-citation"
)] <- "CITATION file"


origin[origin == FALSE] <- "Ignored by cffr"

df <- data.frame(
  key = paste0("<a href='#", keys, "'>", keys, "</a>"),
  source = origin
)


knitr::kable(df, escape = FALSE)
```

## Details

### abstract

This key is extracted from the `"Description"` field of the `DESCRIPTION` file.

<details>

<summary><strong>Example</strong></summary>

```{r abstract}
library(cffr)

# Create cffr for yaml

cff_obj <- cff_create("rmarkdown")

# Get DESCRIPTION of rmarkdown to check

pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cat(cff_obj$abstract)

cat(pkg$get("Description"))
```

</details>

[Back to summary](#summary).

### authors

This key is coerced from the `"Authors"` or `"Authors@R"` field of the
`DESCRIPTION` file. By default persons with the role `"aut"` or `"cre"` are
considered, however this can be modified via the `authors_roles` parameter.

<details>

<summary><strong>Example</strong></summary>

```{r authors}
# An example DESCRIPTION
path <- system.file("examples/DESCRIPTION_many_persons", package = "cffr")
pkg <- desc::desc(path)

# See persons listed
pkg$get_authors()


# Default behaviour, use authors and creators (maintainers)
cff_obj <- cff_create(path)
cff_obj$authors


# Use now Copyright holders and maintainers
cff_obj_alt <- cff_create(path, authors_roles = c("cre", "cph"))
cff_obj_alt$authors
```

</details>

[Back to summary](#summary).

### cff-version

This key can be set via the parameters of the `cff_create()`/`cff_write()`
functions:

<details>

<summary><strong>Example</strong></summary>

```{r cffversion}
cff_objv110 <- cff_create("jsonlite", cff_version = "v1.1.0")

cat(cff_objv110$`cff-version`)
```

</details>

[Back to summary](#summary).

### commit

This key is not extracted from the metadata of the package. See the description
on the [Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#commit).

> -   **description**: The commit hash or revision number of the software
>     version.
>
> -   **usage**:
>
>     ``` yaml
>     commit: 1ff847d81f29c45a3a1a5ce73d38e45c2f319bba
>
>     commit: "Revision: 8612"
>     ```

[Back to summary](#summary).

### contact

This key is coerced from the `"Authors"` or `"Authors@R"` field of the
`DESCRIPTION` file. Only persons with the role `"cre"` (i.e, the maintainer(s))
are considered.

<details>

<summary><strong>Example</strong></summary>

```{r contact}
cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cff_obj$contact

pkg$get_author()
```

</details>

[Back to summary](#summary).

### date-released

This key is extracted from the `DESCRIPTION` file following this logic:

-   `"Date"` field or,
-   If not present, from `"Date/Publication"`. This is present on packages built
    on **CRAN** and **Bioconductor**. or,
-   If not present, from `"Packaged"`, that is present on packages built by the
    [r-universe](https://r-universe.dev/search/).

<details>

<summary><strong>Example</strong></summary>

```{r date-released}
# From an installed package

cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))


cat(pkg$get("Date/Publication"))


cat(cff_obj$`date-released`)



# A DESCRIPTION file without a Date
nodate <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp <- tempfile("DESCRIPTION")

# Create a temporary file
file.copy(nodate, tmp)


pkgnodate <- desc::desc(tmp)
cffnodate <- cff_create(tmp)

# Won't appear
cat(cffnodate$`date-released`)

pkgnodate

# Adding a Date

desc::desc_set("Date", "1999-01-01", file = tmp)

cat(cff_create(tmp)$`date-released`)
```

</details>

[Back to summary](#summary).

### doi {#doi}

This key is coerced from the `"doi"` field of the
[preferred-citation](#preferred-citation) object.

<details>

<summary><strong>Example</strong></summary>

```{r doi}
cff_doi <- cff_create("cffr")

cat(cff_doi$doi)

cat(cff_doi$`preferred-citation`$doi)
```

</details>

[Back to summary](#summary).

### identifiers

This key includes all the possible identifiers of the package:

-   From the `DESCRIPTION` field, it includes all the urls not included in
    [url](#url) or [repository-code](#repository-code).

-   From the `CITATION` file, it includes all the dois not included in
    [doi](#doi) and the identifiers (if any) not included in the `"identifiers"`
    key of [preferred-citation](#preferred-citation).

<details>

<summary><strong>Example</strong></summary>

```{r identifiers}
file <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

pkg <- desc::desc(file)

cat(pkg$get_urls())

cat(cff_create(file)$url)

cat(cff_create(file)$`repository-code`)

cff_create(file)$identifiers
```

</details>

[Back to summary](#summary).

### keywords

This key is extracted from the `DESCRIPTION` file. The keywords should appear in
the `DESCRIPTION` as:

```         
...
X-schema.org-keywords: keyword1, keyword2, keyword3
```

<details>

<summary><strong>Example</strong></summary>

```{r keyword}
# A DESCRIPTION file without keywords
nokeywords <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp2 <- tempfile("DESCRIPTION")

# Create a temporary file
file.copy(nokeywords, tmp2)


pkgnokeywords <- desc::desc(tmp2)
cffnokeywords <- cff_create(tmp2)

# Won't appear
cat(cffnokeywords$keywords)

pkgnokeywords

# Adding Keywords

desc::desc_set("X-schema.org-keywords", "keyword1, keyword2, keyword3",
  file = tmp2
)

cat(cff_create(tmp2)$keywords)
```

</details>

Additionally, if the source code of the package is hosted on GitHub, **cffr**
can retrieve the topics of your repo via the [GitHub
API](https://docs.github.com/en/rest) and include those topics as keywords. This
option is controlled via the `gh_keywords` parameter:

<details>

<summary><strong>Example</strong></summary>

```{r ghkeyword}
# Get cff object from jsonvalidate

jsonval <- cff_create("jsonvalidate")

# Keywords are retrieved from the GitHub repo

jsonval

# Check keywords
jsonval$keywords

# The repo
jsonval$`repository-code`
```

</details>

[Back to summary](#summary).

### license

This key is extracted from the `"License"` field of the `DESCRIPTION` file.

<details>

<summary><strong>Example</strong></summary>

```{r license}
cff_obj <- cff_create("yaml")

cat(cff_obj$license)

pkg <- desc::desc(file.path(find.package("yaml"), "DESCRIPTION"))

cat(pkg$get("License"))
```

</details>

[Back to summary](#summary).

### license-url

This key is not extracted from the metadata of the package. See the description
on the [Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#license-url).

> -   **description**: The URL of the license text under which the software or
>     dataset is licensed (only for non-standard licenses not included in the
>     [SPDX License
>     List](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#definitionslicense-enum)).
>
> -   **usage**:
>
>     ``` yaml
>     license-url: "https://obscure-licenses.com?id=1234"
>     ```

[Back to summary](#summary).

### message

This key is extracted from the `DESCRIPTION` field, specifically as:

```{r eval=FALSE}
msg <- paste0(
  'To cite package "',
  "NAME_OF_THE_PACKAGE",
  '" in publications use:'
)
```

<details>

<summary><strong>Example</strong></summary>

```{r message}
cat(cff_create("jsonlite")$message)
```

</details>

[Back to summary](#summary).

### preferred-citation {#preferred-citation}

This key is extracted from the `CITATION` file. If several references are
provided, it would select the first citation as the `"preferred-citation"` and
the rest of them as [references](#references).

<details>

<summary><strong>Example</strong></summary>

```{r preferred-citation}
cffobj <- cff_create("rmarkdown")

cffobj$`preferred-citation`

citation("rmarkdown")[1]
```

</details>

[Back to summary](#summary).

### references {#references}

This key is extracted from the `CITATION` file if several references are
provided. The first citation is considered as the
[preferred-citation](#preferred-citation) and the rest of them as
`"references"`. It also extracts the package dependencies and adds those to this
fields using `citation(auto = TRUE)` on each dependency.

<details>

<summary><strong>Example</strong></summary>

```{r references}
cffobj <- cff_create("rmarkdown")

cffobj$references

citation("rmarkdown")[-1]
```

</details>

[Back to summary](#summary).

### repository

This key is extracted from the `"Repository"` field of the `DESCRIPTION` file.
Usually, this field is auto-populated when a package is hosted on a repo (like
**CRAN** or the [r-universe](https://r-universe.dev/)). For packages without
this field on the `DESCRIPTION` (that is the typical case for an in-development
package), **cffr** would try to search the package on any of the default
repositories specified on `options("repos")`.

In the case of [Bioconductor](https://bioconductor.org/) packages, those are
identified if a
["biocViews"](https://contributions.bioconductor.org/description.html#biocviews)
is present on the `DESCRIPTION` file.

If **cffr** detects that the package is available on **CRAN**, it would return
the canonical url form of the package (i.e.
<https://CRAN.R-project.org/package=jsonlite>).

<details>

<summary><strong>Example</strong></summary>

```{r repository}
# Installed package

inst <- cff_create("jsonlite")

cat(inst$repository)

# Demo file downloaded from the r-universe

runiv <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")
runiv_cff <- cff_create(runiv)

cat(runiv_cff$repository)

desc::desc(runiv)$get("Repository")

# For in development package

norepo <- system.file("examples/DESCRIPTION_basic", package = "cffr")

# No repo
norepo_cff <- cff_create(norepo)

cat(norepo_cff[["repository"]])

# Change the name to a known package on CRAN: ggplot2

tmp <- tempfile("DESCRIPTION")
file.copy(norepo, tmp)


# Change name
desc::desc_set("Package", "ggplot2", file = tmp)

cat(cff_create(tmp)[["repository"]])
```

</details>

[Back to summary](#summary).

### repository-artifact

This key is not extracted from the metadata of the package. See the description
on the [Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#repository-artifact).

> -   **description**: The URL of the work in a build artifact/binary repository
>     (when the work is software).
>
> -   **usage**:
>
>     ``` yaml
>     repository-artifact: "https://search.maven.org/artifact/org.corpus-tools/cff-maven-plugin/0.4.0/maven-plugin"
>     ```

[Back to summary](#summary).

### repository-code {#repository-code}

This key is extracted from the `"BugReports"` or `"URL"` fields on the
`DESCRIPTION` file. **cffr** tries to identify the url of the source on the
following repositories:

-   [GitHub](https://github.com/).
-   [GitLab](https://about.gitlab.com/).
-   [R-Forge](https://r-forge.r-project.org/).
-   [Bitbucket](https://bitbucket.org/).

<details>

<summary><strong>Example</strong></summary>

```{r repository-code}
# Installed package on GitHub

cff_create("jsonlite")$`repository-code`



# GitLab

gitlab <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")

cat(cff_create(gitlab)$`repository-code`)


# Check

desc::desc(gitlab)
```

</details>

[Back to summary](#summary).

### title

This key is extracted from the `"Description"` field of the `DESCRIPTION` file.

```{r eval=FALSE}
title <- paste0(
  "NAME_OF_THE_PACKAGE",
  ": ",
  "TITLE_OF_THE_PACKAGE"
)
```

<details>

<summary><strong>Example</strong></summary>

```{r title}
# Installed package

cat(cff_create("testthat")$title)
```

</details>

[Back to summary](#summary).

### type

Fixed value equal to `"software"`. The other possible value is `"dataset"`. See
the description on the [Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#type).

[Back to summary](#summary).

### url {#url}

This key is extracted from the `"BugReports"` or `"URL"` fields on the
`DESCRIPTION` file. It corresponds to the first url that is different to
[repository-code](#repository-code).

<details>

<summary><strong>Example</strong></summary>

```{r url}
# Many urls
manyurls <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

cat(cff_create(manyurls)$url)

# Check

desc::desc(manyurls)
```

</details>

[Back to summary](#summary).

### version

This key is extracted from the `"Version"` field on the `DESCRIPTION` file.

```{r version}
# Should be (>= 3.0.0)
cat(cff_create("testthat")$version)
```

[Back to summary](#summary).

## References

---
title: "From R to CFF"
subtitle: "Crosswalk"
description: >
  A comprehensive description of the internal mappings performed by **cffr**.
author: Diego Hernangómez
bibliography: REFERENCES.bib
link-citations: true
toc: true
vignette: >
  %\VignetteIndexEntry{From R to CFF}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
tbl-cap-location: bottom
knitr:
  opts_chunk:
    collapse: true
    comment: "#>"
    code-fold: true
    code-summary: "**Example**"
---

```{r}
#| include: false
library(cffr)
```

This vignette provides an explicit map between the metadata sources used by
**cffr** and each valid key in the [Citation File Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#valid-keys).

## Summary {#summary}

This section summarizes the keys that **cffr** can populate and the metadata
source for each one. Details for each key are presented in the next section. The
assessment is based on the [Guide to Citation File Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#valid-keys)
[@druskat_citation_2021].

```{r}
#| label: tbl-summary
#| echo: false
#| tbl-cap: "cffr and R packages: Conversion table"
keys <- cff_schema_keys(sorted = TRUE)
origin <- vector(length = length(keys))
origin[keys == "cff-version"] <- "function argument"
origin[keys == "type"] <- "Fixed value: 'software'"
origin[keys == "identifiers"] <- "DESCRIPTION and inst/CITATION"
origin[keys == "references"] <- "DESCRIPTION and inst/CITATION"

origin[
  keys %in%
    c(
      "message",
      "title",
      "version",
      "authors",
      "abstract",
      "repository",
      "repository-code",
      "url",
      "date-released",
      "contact",
      "keywords",
      "license",
      "commit"
    )
] <- "DESCRIPTION file"

origin[
  keys %in%
    c(
      "doi",
      "preferred-citation"
    )
] <- "inst/CITATION"

origin[origin == "FALSE"] <- "Ignored by cffr"

df <- data.frame(
  key = paste0("<a href='#", keys, "'>", keys, "</a>"),
  source = origin
)

knitr::kable(df, escape = FALSE)
```

## Details

### abstract

This key is extracted from the `"Description"` field of the `DESCRIPTION` file.

```{r}
#| label: abstract
library(cffr)

# Create a `cff` object for YAML.

cff_obj <- cff_create("rmarkdown")

# Get the DESCRIPTION of **rmarkdown** to check.

pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cat(cff_obj$abstract)

cat(pkg$get("Description"))
```

Back to @tbl-summary.

### authors

This key is coerced from the `"Authors"` or `"Authors@R"` field of the
`DESCRIPTION` file. By default, persons with the role `"aut"` or `"cre"` are
considered, but this can be modified with the `authors_roles` argument.

```{r}
#| label: authors
# An example DESCRIPTION.
path <- system.file("examples/DESCRIPTION_many_persons", package = "cffr")
pkg <- desc::desc(path)

# See the listed persons.
pkg$get_authors()

# Default behavior: use authors and creators (maintainers).
cff_obj <- cff_create(path)
cff_obj$authors

# Use copyright holders and maintainers.
cff_obj_alt <- cff_create(path, authors_roles = c("cre", "cph"))
cff_obj_alt$authors
```

Back to @tbl-summary.

### cff-version

Set this key with the `cff_version` argument of `cff_create()` or `cff_write()`:

```{r}
#| label: cffversion
cff_objv110 <- cff_create("jsonlite", cff_version = "v1.1.0")

cat(cff_objv110$`cff-version`)
```

Back to @tbl-summary.

### commit

This key is extracted from the `"RemoteSha"` field of the `DESCRIPTION` file.
This field is present in packages installed from
[**r-universe**](https://r-universe.dev/search) or with package managers such as
**remotes** and **pak**.

```{r}
#| label: commit
# An example DESCRIPTION.
path <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")
pkg <- desc::desc(path)

# See `RemoteSha`.
pkg$get("RemoteSha")

cff_read(path)
```

Back to @tbl-summary.

### contact

This key is coerced from the `"Authors"` or `"Authors@R"` field of the
`DESCRIPTION` file. Only persons with the role `"cre"` (that is, maintainers)
are considered.

```{r}
#| label: contact
cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cff_obj$contact

pkg$get_author()
```

Back to @tbl-summary.

### date-released

This key is extracted from the `DESCRIPTION` file using this logic:

- `"Date"`, when present.
- Otherwise, `"Date/Publication"`. This field is present in packages built on
  **CRAN** and **Bioconductor**.
- Otherwise, `"Packaged"`, which is present in packages built by
  [**r-universe**](https://r-universe.dev/search).

```{r}
#| label: date-released
# Use an installed package.

cff_obj <- cff_create("rmarkdown")
pkg <- desc::desc(file.path(find.package("rmarkdown"), "DESCRIPTION"))

cat(pkg$get("Date/Publication"))

cat(cff_obj$`date-released`)

# A DESCRIPTION file without a `Date`.
nodate <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp <- tempfile("DESCRIPTION")

# Create a temporary file.
file.copy(nodate, tmp)

pkgnodate <- desc::desc(tmp)
cffnodate <- cff_create(tmp)

# No value is returned.
cat(cffnodate$`date-released`)

pkgnodate

# Add a `Date`.

desc::desc_set("Date", "1999-01-01", file = tmp)

cat(cff_create(tmp)$`date-released`)
```

Back to @tbl-summary.

### doi {#doi}

This key is coerced from the `"doi"` field of the
[preferred-citation](#preferred-citation) object. If not present and the package
is on **CRAN**, it is populated with the DOI provided by **CRAN** (for example,
<https://doi.org/10.32614/CRAN.package.cffr>).

```{r}
#| label: doi
cff_doi <- cff_create("cffr")

cat(cff_doi$doi)

cat(cff_doi$`preferred-citation`$doi)
```

Back to @tbl-summary.

### identifiers

This key contains the available package identifiers:

- From `DESCRIPTION` metadata, it includes all the URLs not included in
  [url](#url) or [repository-code](#repository-code).
- From the `CITATION` file, it includes all the DOIs not included in [doi](#doi)
  and the identifiers (if any) not included in the `"identifiers"` key of
  [preferred-citation](#preferred-citation).
- If the package is on **CRAN** and it has a `CITATION` file providing a DOI,
  the DOI provided by **CRAN** is added as well.

```{r}
#| label: identifiers
file <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

pkg <- desc::desc(file)

cat(pkg$get_urls())

cat(cff_create(file)$url)

cat(cff_create(file)$`repository-code`)

cff_create(file)$identifiers
```

Back to @tbl-summary.

### keywords

This key is extracted from the `DESCRIPTION` file. The keywords should appear in
the `DESCRIPTION` as:

```         
...
X-schema.org-keywords: keyword1, keyword2, keyword3
```

```{r}
#| label: keyword
# A DESCRIPTION file without keywords
nokeywords <- system.file("examples/DESCRIPTION_basic", package = "cffr")
tmp2 <- tempfile("DESCRIPTION")

# Create a temporary file.
file.copy(nokeywords, tmp2)

pkgnokeywords <- desc::desc(tmp2)
cffnokeywords <- cff_create(tmp2)

# No value is returned.
cat(cffnokeywords$keywords)

pkgnokeywords

# Add keywords.

desc::desc_set(
  "X-schema.org-keywords",
  "keyword1, keyword2, keyword3",
  file = tmp2
)

cat(cff_create(tmp2)$keywords)
```

If the package source code is hosted on **GitHub**, **cffr** can also retrieve
repository topics with the [**GitHub** API](https://docs.github.com/en/rest) and
include those topics as keywords. This option is controlled with the
`gh_keywords` argument:

```{r}
#| label: ghkeyword
# Get a `cff` object from **jsonvalidate**.

jsonval <- cff_create("jsonvalidate")

# Keywords are retrieved from the **GitHub** repository.

jsonval

# Check keywords.
jsonval$keywords

# The repository.
jsonval$`repository-code`
```

Back to @tbl-summary.

### license

This key is extracted from the `"License"` field of the `DESCRIPTION` file.

```{r}
#| label: license
cff_obj <- cff_create("yaml")

cat(cff_obj$license)

pkg <- desc::desc(file.path(find.package("yaml"), "DESCRIPTION"))

cat(pkg$get("License"))
```

Back to @tbl-summary.

### license-url

This key is not extracted from package metadata. See its description in the
[Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#license-url).

> - **description**: The URL of the license text under which the software or
>   dataset is licensed (only for non-standard licenses not included in the
>   [SPDX License
>   List](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#definitionslicense-enum)).
>
> - **usage**:
>
>   ``` yaml
>   license-url: "https://obscure-licenses.com?id=1234"
>   ```

Back to @tbl-summary.

### message

This key is generated from the package metadata with this template:

```{r}
#| eval: false
#| code-fold: false
msg <- paste0(
  'To cite package "',
  "NAME_OF_THE_PACKAGE",
  '" in publications use:'
)
```

```{r}
#| label: message
cat(cff_create("jsonlite")$message)
```

Back to @tbl-summary.

### preferred-citation {#preferred-citation}

This key is extracted from `inst/CITATION`. If multiple references are provided,
it selects the first citation as `"preferred-citation"` and uses the remaining
citations as [references](#references).

```{r}
#| label: preferred-citation
cffobj <- cff_create("rmarkdown")

cffobj$`preferred-citation`

citation("rmarkdown")[1]
```

Back to @tbl-summary.

### references {#references}

This key is extracted from `inst/CITATION` if multiple references are provided.
The first citation becomes the [preferred citation](#preferred-citation) and the
remaining citations become `"references"`. **cffr** can also extract package
dependencies and add them to this field by calling `citation(auto = TRUE)` for
each dependency.

```{r}
#| label: references
cffobj <- cff_create("rmarkdown")

cffobj$references

citation("rmarkdown")[-1]
```

Back to @tbl-summary.

### repository

This key is extracted from the `"Repository"` field of the `DESCRIPTION` file.
This field is usually populated when a package is hosted in a repository such as
**CRAN** or [**r-universe**](https://r-universe.dev/search). For packages without
this field in the `DESCRIPTION` (which is the typical case for an in-development
package), **cffr** tries to find the package in any of the default repositories
specified in `options("repos")`.

[**Bioconductor**](https://bioconductor.org/) packages are identified when the
[`"biocViews"`](https://contributions.bioconductor.org/description.html#biocviews)
field is present in the `DESCRIPTION` file.

If **cffr** detects that the package is available on **CRAN**, it returns the
canonical URL form of the package (that is,
<https://CRAN.R-project.org/package=jsonlite>).

```{r}
#| label: repository
# Use an installed package.

inst <- cff_create("jsonlite")

cat(inst$repository)

# Demo file downloaded from the r-universe.

runiv <- system.file("examples/DESCRIPTION_r_universe", package = "cffr")
runiv_cff <- cff_create(runiv)

cat(runiv_cff$repository)

desc::desc(runiv)$get("Repository")

# For an in-development package.

norepo <- system.file("examples/DESCRIPTION_basic", package = "cffr")

# No repository is available.
norepo_cff <- cff_create(norepo)

cat(norepo_cff[["repository"]])

# Change the name to a known package on CRAN: ggplot2.

tmp <- tempfile("DESCRIPTION")
file.copy(norepo, tmp)

# Change the package name.
desc::desc_set("Package", "ggplot2", file = tmp)

cat(cff_create(tmp)[["repository"]])
```

Back to @tbl-summary.

### repository-artifact

This key is not extracted from package metadata. See its description in the
[Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#repository-artifact).

> - **description**: The URL of the work in a build artifact/binary repository
>   (when the work is software).
>
> - **usage**:
>
>   ``` yaml
>   repository-artifact: "https://search.maven.org/artifact/org.corpus-tools/cff-maven-plugin/0.4.0/maven-plugin"
>   ```

Back to @tbl-summary.

### repository-code {#repository-code}

This key is extracted from the `"BugReports"` or `"URL"` fields in the
`DESCRIPTION` file. **cffr** tries to identify the URL of the source on the
following repositories:

- [**GitHub**](https://github.com/).
- [**GitLab**](https://about.gitlab.com/).
- [**R-Forge**](https://r-forge.r-project.org/).
- [**Bitbucket**](https://bitbucket.org/).
- [**Codeberg**](https://codeberg.org/).

```{r}
#| label: repository-code
# Installed package on GitHub.

cat(cff_create("jsonlite")$`repository-code`)

# GitLab

gitlab <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")

cat(cff_create(gitlab)$`repository-code`)

# Codeberg
codeberg <- system.file("examples/DESCRIPTION_codeberg", package = "cffr")

cat(cff_create(codeberg)$`repository-code`)
```

Back to @tbl-summary.

### title

This key combines the `"Package"` and `"Title"` fields of the `DESCRIPTION`
file.

```{r}
#| eval: false
#| code-fold: false
title <- paste0("NAME_OF_THE_PACKAGE", ": ", "TITLE_OF_THE_PACKAGE")
```

```{r}
#| label: title
# Use an installed package.

cat(cff_create("testthat")$title)
```

Back to @tbl-summary.

### type

This key has the fixed value `"software"`. The other valid value is `"dataset"`.
See the [Guide to CFF schema
v1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#type).

Back to @tbl-summary.

### url {#url}

This key is extracted from the `"BugReports"` or `"URL"` fields in the
`DESCRIPTION` file. It corresponds to the first URL that is different from
[repository-code](#repository-code).

```{r}
#| label: url
# Many URLs.
manyurls <- system.file("examples/DESCRIPTION_many_urls", package = "cffr")

cat(cff_create(manyurls)$url)

# Check the source metadata.

desc::desc(manyurls)
```

Back to @tbl-summary.

### version

This key is extracted from the `"Version"` field in the `DESCRIPTION` file.

```{r}
#| label: version
#| code-fold: false
# Should be (>= 3.0.0).
cat(cff_create("testthat")$version)
```

Back to @tbl-summary.

## References

---
title: "BibTeX and CFF"
subtitle: A potential crosswalk
bibliography: REFERENCES.bib
author: Diego Hernangómez
description: >-
  This article presents a crosswalk between BibTeX and Citation File Format, as 
  it is performed by the cffr package.
abstract: >-
  This article introduces a crosswalk between **BibTeX** and the Citation File 
  Format (CFF) [@druskat_citation_2021], as implemented by the **cffr** package 
  [@hernangomez2021]. The crosswalk aims to facilitate seamless translation 
  between these two reference formats. Specifically, it proposes various 
  crosswalk models tailored to different **BibTeX** entry types 
  [@patashnik1988]. Additionally, the article includes practical examples using 
  real **BibTeX** entries and offers tips for developers interested in 
  implementing this crosswalk across various programming languages.
link-citations: yes
documentclass: article
editor_options:
  markdown:
    wrap: 80
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{BibTeX and CFF}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = ""
)

# Load the table of tables

p2file <- system.file("extdata/crosswalk_tables.csv", package = "cffr")

table_master <- read.csv(p2file)
```

## Disclaimer

*This article was reviewed and updated in 2024, along with the release of
**cffr** v1.0.0.*

## Citation

Please cite this article using this **BibTeX** entry:

``` bib
@article{hernangomez2022,
    title        = {{BibTeX} and {CFF}, a potential crosswalk},
    author       = {Diego Hernangómez},
    year         = 2022,
    journal      = {The {cffr} package},
    volume       = {Vignettes},
    doi          = {10.21105/joss.03900},
    url          = {https://docs.ropensci.org/cffr/articles/bibtex_cff.html}
}
```

## BibTeX and R

[**BibTeX**](https://en.wikipedia.org/wiki/BibTeX) is a well-known format for
storing references created by [Oren
Patashnik](https://en.wikipedia.org/wiki/Oren_Patashnik "Oren Patashnik") and
[Leslie Lamport](https://en.wikipedia.org/wiki/Leslie_Lamport "Leslie Lamport")
back in 1985. **BibTeX** can be reused by other software, such as
[LaTeX](https://en.wikipedia.org/wiki/LaTeX), to add references to scholarly
works. An example structure of a **BibTeX** entry would be:

``` bibtex
@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925
}
```

In this case, the entry (identified as `einstein1921`) refers to a book. This
entry can then be used in a document to include references to it. In **R**
[@R_2021], we can replicate this structure using the `bibentry()` and
`toBibtex()` functions:

```{r bibentry, comment="#>"}
entry <- bibentry("book",
  key = "einstein1921",
  title = "Relativity: The Special and the General Theory",
  author = person("A.", "Einstein"),
  year = 1920,
  publisher = "Henry Holt and Company",
  address = "London, United Kingdom",
  isbn = 9781587340925,
)

toBibtex(entry)
```

The final results of the entry as a text string would be coerced as[^1]:

[^1]: By default **R** Pandoc would generate the cite on the Chicago author-date
    format [@rmarkdowncookbook2020]

```{r echo=FALSE, results='asis'}
entry
```

Additionally, the **cffr** package [@hernangomez2021] incorporates the following
capabilities that can be used to read and transform **BibTeX** format into
different formats:

-   `cff_read_bib()` reads \*.bib files.
-   `cff_read_bib_text()` can read **BibTeX** entries that are already stored in
    a variable.
-   A S3 method `toBibtex.cff()` that converts from `cff` objects to Bibtex
    objects (see `utils::tpBibtex()`).

```{r cffbibread, comment="#>"}
string <- "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925}"

# To cff
library(cffr)
cff_format <- cff_read_bib_text(string)

cff_format

# To BibTeX with S3 method
toBibtex(cff_format)
```

## BibTeX Definitions

@patashnik1988 provides a comprehensive explanation of the **BibTeX** formats.
Let's distinguish between **Entries** and **Fields**.

### Entries {#entries}

Each entry type defines a different type of work. The 14 entry types defined in
**BibTeX** are:

1.  **\@article**: An article from a journal or magazine.
2.  **\@book**: A book with an explicit publisher.
3.  **\@booklet**: A work that is printed and bound, but without a named
    publisher or sponsoring institution.
4.  **\@conference**: Equivalent to **\@inproceedings**, included for
    [Scribe](https://en.wikipedia.org/wiki/Scribe_(markup_language))
    compatibility.
5.  **\@inbook**: A part of a book, which may be a chapter (or section) and/or a
    range of pages.
6.  **\@incollection**: A part of a book having its own title.
7.  **\@inproceedings**: An article in conference proceedings.
8.  **\@manual**: Technical documentation.
9.  **\@mastersthesis**: A Master's thesis.
10. **\@misc**: Use this type when nothing else fits.
11. **\@phdthesis**: A PhD thesis.
12. **\@proceedings**: The proceedings of a conference.
13. **\@techreport**: A report published by a school or other institution,
    usually numbered within a series.
14. **\@unpublished**: A document having an author and title, but not formally
    published.

Other implementations similar to **BibTeX**, such as **BibLaTeX**
[@biblatexpack], expand the definitions of entries to include other types such
as online resources, software, or datasets. In **BibTeX**, these entries should
be reclassified as **\@misc**.

In **R**, the `bibentry()` base function does not implement **\@conference**.
However, we can use **\@inproceedings**, instead given that the definition is
identical.

### Fields {#fields}

Similar to the **Entries**, @patashnik1988 also provides definitions for each of
the possible standard **BibTeX fields**. While an entry must include certain
required fields, it can also include additional fields that might be ignored in
the raw implementation of **BibTeX**. Let's explore some of these fields:

1.  **address**: Typically represents the address of the **publisher** or
    another **institution**. In the case of **\@conference**,
    **\@inproceedings** and **\@proceedings** this field indicates where the
    conference was held.
2.  **annote**: An annotation. Although not used by standard bibliography
    styles, it may be relevant for producing annotated bibliographies.
3.  **author**: Contains the name(s) of the author(s), following the format
    described in the LaTeX book [@lamport86latex].
4.  **booktitle**: Refers to the title of a book, part of which is being cited.
    For **\@book** entries, use the **title** field instead.
5.  **chapter**: Indicates a chapter (or section) number.
6.  **crossref**: Refers to the database key of the entry being
    cross-referenced.
7.  **edition**: Specifies the edition of a **\@book**, e.g., `"Second"`. The
    ordinal should have the first letter capitalized; standard styles convert to
    lowercase when necessary.
8.  **editor**: Contains the name(s) of editor(s), following the conventions in
    the LaTeX book [@lamport86latex]. If there is also an **author** field, the
    **editor** field specifies the editor of the book or collection where the
    reference appears.
9.  **howpublished**: Describes how something unusual has been published. The
    first word should be capitalized.
10. **institution**: Represents the sponsoring institution of a technical
    report.
11. **journal**: Refers to the name of a journal.
12. **key**: Used for alphabetizing, cross-referencing, and creating a label
    when **author** information is missing.
13. **month**: Indicates the month in which the work was published or, for an
    unpublished work, when it was written. Use the standard three-letter
    abbreviation (e.g., `jan`, `feb`, `mar`) as described in **Appendix B.1.3**
    of the LaTeX book [@lamport86latex].
14. **note**: Provides any additional information that can assist the reader.
    The first word should be capitalized.
15. **number**: Represents the number of a journal, magazine, technical report,
    or a work in a series. An issue of a journal or magazine is usually
    identified by its **volume** and number. Organizations issuing technical
    reports often assign them numbers, and sometimes books are given numbers
    within a named series.
16. **organization**: Refers to the organization that sponsors a
    **\@conference** or publishes a manual.
17. **pages**: Specifies one or more page numbers or a range of numbers (e.g.,
    `42--111` or `7,41,73--97` or `43+`).
18. **publisher**: Indicates the publisher's name.
19. **school**: Provides the name of the school where a thesis was written.
20. **series**: Specifies the name of a series or set of books. When citing an
    entire book, the **title** field gives its title, and an optional **series**
    field provides the name of a series or multi-volume set in which the book is
    published.
21. **title**: Represents the work's title.
22. **type**: Describes the type of a technical report (e.g., "Research Note").
23. **volume**: Refers to the volume of a journal or multi-volume book.
24. **year**: Indicates the year of publication or, for an unpublished work, the
    year it was written. Generally, it should consist of four numerals (e.g.,
    `1984`).

As in the case of the **Entries**, other implementations such as **BibLaTeX**
recognize additional fields.

In **BibTeX**, there exists a strict relationship between **Entries** and
**Fields**. Depending on the type of entry, certain fields are required, while
others are optional or even ignored.

The following table summarizes the relationship between **Entries** and
**Fields** in **BibTeX**. Required fields are flagged as **x**, and optional
fields are flagged as **o**. For more detailed information, refer to
@patashnik1988.

```{r entry_fields1, echo=FALSE}
df_table <- table_master[table_master$table == "entry_fields", -1]

nms <- c(
  "**field**", "**\\@article**", "**\\@book**", "**\\@booklet**",
  "**\\@inbook**", "**\\@incollection**", "**\\@conference, \\@inproceedings**",
  "**\\@manual**", "**\\@mastersthesis, phdthesis**", "**\\@misc**",
  "**\\@proceedings**", "**\\@techreport**", "**\\@unpublished**"
)


df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
t1 <- df_table[, c(1:7)]
nm1 <- nms[1:7]

knitr::kable(t1,
  col.names = nm1, row.names = NA, align = c("l", rep("c", 6)),
  caption = "**BibTeX**, required fields by entry"
)
```

```{r entry_fields2, echo=FALSE}
t2 <- df_table[, c(1, 8:13)]
nm2 <- nms[c(1, 8:13)]
knitr::kable(t2,
  col.names = nm2, row.names = NA, align = c("l", rep("c", 6)),
  caption = "(cont) **BibTeX**, required fields by entry"
)
```

It can be seen that only a subset of fields is necessary for any entry. For
instance, **title**, **year**, and **author** are either required or optional in
almost every entry, whereas **crossref**, **annote**, or **key** are never
mandatory.

## Citation File Format

[Citation File Format (CFF](https://citation-file-format.github.io/))
[@druskat_citation_2021] consists of plain text files containing both human- and
machine-readable citation information for software and datasets. Within
[CFF]{.underline}, there are two important keys: `preferred-citation` and
`references`, which play a crucial role in citing and referring to related
works:

-   `preferred-citation`: Refers to another work that should be cited instead of
    the software or dataset itself.
-   `references`: Includes reference(s) to other creative works related to the
    software or dataset. Similar to a list of references in a scholarly paper,
    these references may encompass papers describing the abstract concepts of
    the software or algorithms implemented in the software version.

These two keys are expected to be `definition.reference objects`, as defined in
the [Guide to Citation File Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#preferred-citation),
and they may contain the following keys:

```{r refkeys, echo=FALSE, message=FALSE, warning=FALSE, results='asis'}
library(cffr)

# Fill with whites
init <- paste0("[", cff_schema_definitions_refs(), "]{.underline}")

l <- c(init, rep("", 4))


refkeys <- matrix(l, ncol = 5, byrow = TRUE)

knitr::kable(refkeys,
  row.names = NA,
  caption = "Valid keys on [CFF]{underline} `definition-reference` objects"
)
```

These keys are equivalent to the **BibTeX** [fields](#fields), with the
exception of the key [type]{.underline}. In [CFF]{.underline}, this key defines
the type of work[^2], making it analogous to the **BibTeX** [entries](#0).

[^2]: See a complete list of possible values of [CFF]{.underline}
    [type]{.underline} in [Appendix B](#appendix_cff_type).

## Proposed Crosswalk

The **cffr** package [@hernangomez2021] provides utilities for converting
**BibTeX** entries (via the **R** base function `bibentry()`) to
[CFF]{.underline} files and vice versa. This section describes how the
conversion between both formats has been implemented. The crosswalk is partially
based on @Haines_Ruby_CFF_Library_2021[^3].

[^3]: Note that this software performs only the conversion from
    [CFF]{.underline} to **BibTeX**, however **cffr** can perform the conversion
    in both directions.

In the following two sections, I present an overview of the proposed mapping
between the **Entries** and **Fields** of **BibTeX** and the [CFF]{.underline}
keys. After this initial mapping, I propose further transformations to enhance
compatibility between both systems using different [Entry Models](#entrymodels).

For better clarity, when a field is in **bold** (e.g., **\@book, edition**), it
corresponds to **BibTeX**, and when the field is [underlined]{.underline} (e.g.,
[book, edition]{.underline}), it corresponds to [CFF]{.underline}.

### Entry/Type Crosswalk

For converting general **BibTeX** entries to [CFF]{.underline}
[types]{.underline}, the following crosswalk is proposed:

```{r entry_bib2cff, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "entry_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("**BibTeX** Entry", "[**CFF key: type**]{.underline}", "Notes"),
  row.names = NA,
  caption = "Entry/Type crosswalk: From **BibTeX** to [CFF]{.underline}"
)
```

The previous crosswalk has the following specifications:

-   **\@book**, **\@inbook**, and **\@incollection** are closely related in
    **BibTeX**[^4]. While **\@inbook** and **\@incollection** both reference
    parts of a **\@book**, the former is used for citing sections, chapters,
    pages, or other specific parts, whereas the latter is used for citing parts
    with a specific title. Since [CFF]{.underline} allows keys `type:book` and
    `collection-type: book`, we may utilize a combination of these fields to tag
    each entry type in [CFF]{.underline} accordingly.
-   **\@mastersthesis** and **\@phdthesis** would be tagged using a combination
    of `type:thesis` and `thesis-type`.

[^4]: Note that **BibLaTeX** [@biblatexpack] handles **\@inbook** differently,
    see [Appendix A](#appendix_inbook).

Additionally, considering that [CFF]{.underline} allows for a wide range of
values[^5] for the [type]{.underline} field, the following conversion would be
applied from [CFF]{.underline} to **BibTeX**:

[^5]: See [Appendix B](#appendix_cff_type) for all possible values. Information
    extracted from @druskat2019.

```{r entry_cff2bib, echo=FALSE,results='asis'}
df_table <- table_master[table_master$table == "entry_cff2bib", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("[**CFF key: type**]{.underline}", "**BibTeX** Entry", "Notes"),
  caption = "Entry/Type crosswalk: From [CFF]{.underline} to **BibTeX**"
)
```

### Fields/Key Crosswalk

There is a significant similarity between the definitions and names of certain
**BibTeX** fields and [CFF]{.underline} keys. While the equivalence is
straightforward in some cases, there are instances where certain keys need to be
processed depending on the **entry** type.

```{r fields_bib2cff, echo=FALSE,results='asis'}
df_table <- table_master[table_master$table == "fields_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("**BibTeX Field**", "[CFF key]{.underline}", "Notes"),
  caption = "**BibTeX** - [CFF]{.underline} Field/Key crosswalk"
)
```

We provide more detail on some of the mappings presented in the table above:

-   Some fields are not mapped because there is no clear equivalence with
    [CFF]{.underline} keys (such as **annote**, **crossref**, and **key**).
    Regarding the **type** field, the [CFF]{.underline} key [type]{.underline}
    corresponds to the identifier of the work (similar to an entry in
    **BibTeX**), therefore, **BibTeX type** won't be mapped. These fields are
    always optional in **BibTeX**.

-   For the **address** field, its intended use in **BibTeX** varies depending
    on the entry type (e.g., for **\@inproceedings**, it denotes the **address**
    of the **conference**, while for **\@mastersthesis/\@phdthesis**, is the
    **address** of the **school**, etc.). Mapping between **BibTeX** and
    [CFF]{.underline} becomes more complex when related to institutions,
    resulting in varying final mappings in [CFF]{.underline}. When converting
    from [CFF]{.underline} to **BibTeX**, we propose to follow the same
    entry-based logic, using the key [location]{.underline} as a fallback value
    when converting to **address**.

-   In relation with this complexity mentioned above, **institution,
    organization** and **school** would be mapped to [institution]{.underline}.

-   **series** would be mapped to [collection-title]{.underline} only on those
    entries that does not requires **booktitle**. In practice, this means that
    collection-title would correspond to **booktitle** for **incollection** and
    **inproceedings**, and in the rest of cases it would correspond to series. A
    consequence of this is that series information would be lost for
    incollection and inproceedings, but on those cases is an optional field.

-   Regarding **series**, it would be mapped to [collection-title]{.underline}
    only for those entries that do not require **booktitle**. In practice, this
    means that [collection-title]{.underline} would correspond to **booktitle**
    for **\@incollection** and **\@inproceedings**, while in other cases it
    would correspond to **series**. As a consequence, **series** information
    would be lost for **\@incollection** and **\@inproceedings**, but in those
    cases, it is an optional field.

-   When converting from [CFF]{.underline} to **BibTeX**, we propose to use
    [date-published]{.underline} as a fallback for extracting **month** and
    **year** fields.

-   When **pages** is provided as a range separated by `--`, i.e, **pages =
    {3--5}** would be coerced as [start: 3]{.underline}, [end: 5]{.underline} in
    [CFF]{.underline}.

#### BibLaTeX

Additionally, there are other [CFF]{.underline} keys that correspond to
**BibLaTeX** fields. We propose to include these fields in the crosswalk[^6],
even though they are not part of the core **BibTeX** fields definition.

[^6]: See @biblatexcheatsheet for a preview of the accepted **BibLaTeX** fields.

```{r fields_biblatex2cff, echo=FALSE,results='asis'}
df_table <- table_master[table_master$table == "fields_biblatex2cff", c(2:3)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f2 <- gsub("link_to_entry_models", "#entrymodels", df_table$f2)
row.names(df_table) <- NULL

knitr::kable(df_table,
  col.names = c("**BibLaTeX Field**", "[CFF key]{.underline}"),
  caption = "**BibLaTeX** - [CFF]{.underline} Field/Key crosswalk"
)
```

## Entry Models {#entrymodels}

This section presents the specific mapping proposed for each of the **BibTeX**
entries, providing further information on how each field is treated. Examples
are adapted from the [xampl.bib](https://tug.org/texmf-docs/bibtex/xampl.bib)
file provided with the **bibtex** package [@patashnik].

### \@article {#article}

The crosswalk of **\@article** does not require any special treatment.

```{r model_article, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_article", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@article** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@article{article-full,
    title        = {The Gnats and Gnus Document Preparation System},
    author       = {Leslie A. Aamport},
    year         = 1986,
    month        = jul,
    journal      = {{G-Animal's} Journal},
    volume       = 41,
    number       = 7,
    pages        = {73+},
    note         = {This is a full ARTICLE entry}
}
```

[CFF entry]{.underline}

```{r echo=FALSE}
bib <- "@article{article-full,
    title        = {The Gnats and Gnus Document Preparation System},
    author       = {Leslie A. Aamport},
    year         = 1986,
    month        = jul,
    journal      = {{G-Animal's} Journal},
    volume       = 41,
    number       = 7,
    pages        = {73+},
    note         = {This is a full ARTICLE entry}}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE}
toBibtex(cff_read_bib_text(bib))
```

### \@book / \@inbook {#book-inbook}

In terms of the fields required in BibTeX, the primary difference between
**\@book** and **\@inbook** is that **\@inbook** requires a **chapter** or
**page** field, while **\@book** does not even allow these fields as optional.
Therefore, we propose that an **\@inbook** entry in [CFF]{.underline} be treated
as a **\@book** with the following supplementary fields:

1.  [section]{.underline}: To denote the specific **chapter** within the book.
2.  [start-end]{.underline}: To indicate the range of **pages** covered by the
    section.

Additionally, note that in [CFF]{.underline}, the **series** field corresponds
to [collection-title]{.underline}, and the **address** field represents the
[publisher]{.underline}'s [address]{.underline}. By last, the key
[collection-type]{.underline} would be populated with [book-series]{.underline}.

```{r model_book, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_book", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@book / \\@inbook** Model"
)
```

There are notable differences in how **BibTeX** and **BibLaTeX** handle the
**\@inbook** entry (further discussed in the [Appendix A](#appendix_inbook)). We
propose to treat a **BibLaTeX \@inbook** as a **BibTeX \@incollection.**

**Examples: \@book**

**BibTeX entry**

``` bibtex
@book{book-full,
    title        = {Seminumerical Algorithms},
    author       = {Donald E. Knuth},
    year         = 1981,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 2,
    note         = {This is a full BOOK entry},
    edition      = {Second}
}
```

[CFF entry]{.underline}

```{r echo=FALSE}
bib <- "@book{book-full,
    title        = {Seminumerical Algorithms},
    author       = {Donald E. Knuth},
    year         = 1981,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 2,
    note         = {This is a full BOOK entry},
    edition      = {Second}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r,  echo=FALSE}
toBibtex(cff_read_bib_text(bib))
```

**Examples: \@inbook**

**BibTeX entry**

``` bibtex
@inbook{inbook-full,
    title        = {Fundamental Algorithms},
    author       = {Donald E. Knuth},
    year         = 1973,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 1,
    pages        = {10--119},
    note         = {This is a full INBOOK entry},
    edition      = {Second},
    type         = {Section},
    chapter      = {1.2}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@inbook{inbook-full,
    title        = {Fundamental Algorithms},
    author       = {Donald E. Knuth},
    year         = 1973,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 1,
    pages        = {10--119},
    note         = {This is a full INBOOK entry},
    edition      = {Second},
    type         = {Section},
    chapter      = {1.2}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@booklet {#booklet}

In **\@booklet** **address** is mapped to [location]{.underline}.

```{r model_booklet, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_booklet", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@booklet** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@booklet{booklet-full,
    title        = {The Programming of Computer Art},
    author       = {Jill C. Knvth},
    date         = {1988-03-14},
    month        = feb,
    address      = {Stanford, California},
    note         = {This is a full BOOKLET entry},
    howpublished = {Vernier Art Center}
}
```

[CFF entry]{.underline}

```{r echo=FALSE, }
bib <- "@booklet{booklet-full,
    title        = {The Programming of Computer Art},
    author       = {Jill C. Knvth},
    date         = {1988-03-14},
    month        = feb,
    address      = {Stanford, California},
    note         = {This is a full BOOKLET entry},
    howpublished = {Vernier Art Center}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE, }
toBibtex(cff_read_bib_text(bib))
```

### \@conference / \@inproceedings {#conf_inproc}

Note that in this case, **organization** is mapped to [institution]{.underline}.
Additionally, **series** would be ignored as there is not clear mapping on
[CFF]{.underline} for this field.

```{r model_inproceedings, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_inproceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@conference / \\@inproceedings** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@inproceedings{inproceedings-full,
    title        = {On Notions of Information Transfer in {VLSI} Circuits},
    author       = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
    year         = 1983,
    month        = mar,
    booktitle    = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    pages        = {133--139},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@inproceedings{inproceedings-full,
    title        = {On Notions of Information Transfer in {VLSI} Circuits},
    author       = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
    year         = 1983,
    month        = mar,
    booktitle    = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    pages        = {133--139},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@incollection {#incol}

As **booktitle** is a required field, we propose to map that field to
[collection-title]{.underline} and the [type]{.underline} to
[generic]{.underline}. Therefore, an **\@incollection** is a [type:
generic]{.underline} with a [collection-title]{.underline} key.

Additionally, **series** and **type** would be ignored as there is not clear
mapping on [CFF]{.underline} for this field.

```{r model_incollection, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_incollection", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@incollection** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@incollection{incollection-full,
    title        = {Semigroups of Recurrences},
    author       = {Daniel D. Lincoll},
    year         = 1977,
    month        = sep,
    booktitle    = {High Speed Computer and Algorithm Organization},
    publisher    = {Academic Press},
    address      = {New York},
    series       = {Fast Computers},
    number       = 23,
    pages        = {179--183},
    note         = {This is a full INCOLLECTION entry},
    editor       = {David J. Lipcoll and D. H. Lawrie and A. H. Sameh},
    chapter      = 3,
    type         = {Part},
    edition      = {Third}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@incollection{incollection-full,
    title        = {Semigroups of Recurrences},
    author       = {Daniel D. Lincoll},
    year         = 1977,
    month        = sep,
    booktitle    = {High Speed Computer and Algorithm Organization},
    publisher    = {Academic Press},
    address      = {New York},
    series       = {Fast Computers},
    number       = 23,
    pages        = {179--183},
    note         = {This is a full INCOLLECTION entry},
    editor       = {David J. Lipcoll and D. H. Lawrie and A. H. Sameh},
    chapter      = 3,
    type         = {Part},
    edition      = {Third}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@manual

As in the case of [**\@conference** / **\@inproceedings**](#conf_inproc),
**organization** is mapped to [institution]{.underline}.

```{r model_manual, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_manual", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@manual** Model"
)
```

**Examples**

**BibTeX entry**

Note that **month** can't be coerce to a single integer in the range `1--12` as
required on CFF, so it is ignored to avoid validation errors.

``` bibtex
@manual{manual-full,
  title        = {The Definitive Computer Manual},
    author       = {Larry Manmaker},
    year         = 1986,
    month        = {apr-may},
    address      = {Silicon Valley},
    note         = {This is a full MANUAL entry},
    organization = {Chips-R-Us},
    edition      = {Silver}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@manual{manual-full,
  title        = {The Definitive Computer Manual},
    author       = {Larry Manmaker},
    year         = 1986,
    month        = {apr-may},
    address      = {Silicon Valley},
    note         = {This is a full MANUAL entry},
    organization = {Chips-R-Us},
    edition      = {Silver}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@mastersthesis / \@phdthesis

In terms of field required on BibTeX, it is identical for both
**\@mastersthesis** and **\@phdthesis.**

We propose here to identify each type of thesis using the key
[thesis-type]{.underline} So if [thesis-type]{.underline} contains a [regex
pattern](https://regex101.com/r/mBWfbs/1) `(?i)(phd)` it would be recognized as
**\@phdthesis**.

Additionally, **school** would be mapped to [institution]{.underline}.

```{r model_thesis, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_thesis", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@mastersthesis / \\@phdthesis** Model"
)
```

**Examples: \@mastersthesis**

**BibTeX entry**

``` bibtex
@mastersthesis{mastersthesis-full,
    title        = {Mastering Thesis Writing},
    author       = {Edouard Masterly},
    year         = 1988,
    month        = jun,
    address      = {English Department},
    note         = {This is a full MASTERSTHESIS entry},
    school       = {Stanford University},
    type         = {Master's project}
}
```

[CFF entry]{.underline}

```{r echo=FALSE}
bib <- "@mastersthesis{mastersthesis-full,
    title        = {Mastering Thesis Writing},
    author       = {Edouard Masterly},
    year         = 1988,
    month        = jun,
    address      = {English Department},
    note         = {This is a full MASTERSTHESIS entry},
    school       = {Stanford University},
    type         = {Master's project}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r,  echo=FALSE}
toBibtex(cff_read_bib_text(bib))
```

**Examples: \@phdthesis**

**BibTeX entry**

``` bibtex
@phdthesis{phdthesis-full,
    title        = {Fighting Fire with Fire: Festooning {F}rench Phrases},
    author       = {F. Phidias Phony-Baloney},
    year         = 1988,
    month        = jun,
    address      = {Department of French},
    note         = {This is a full PHDTHESIS entry},
    school       = {Fanstord University},
    type         = {{PhD} Dissertation}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@phdthesis{phdthesis-full,
    title        = {Fighting Fire with Fire: Festooning {F}rench Phrases},
    author       = {F. Phidias Phony-Baloney},
    year         = 1988,
    month        = jun,
    address      = {Department of French},
    note         = {This is a full PHDTHESIS entry},
    school       = {Fanstord University},
    type         = {{PhD} Dissertation}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@misc

The crosswalk of **\@misc** does not require any special treatment. This
**entry** does not require any **field**.

Note also that it is mapped to [type: generic]{.underline} as
[**\@incollection**](#incol), but in this case **booktitle** is not even an
option, so the proposed definition should cover both **\@misc** and
**\@incollection** without problems.

```{r model_misc, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_misc", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@misc** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@proceedings

The proposed model is consistent with [**\@conference** /
**\@inproceedings**](#conf_inproc). Note that **\@proceedings** does not
prescribe a **author** field. On this cases, as [authors]{.underline} is
required on [CFF]{.underline}, we would use *anonymous*[^7] when converting to
[CFF]{.underline} and omit it on the conversion from [CFF]{.underline} to
**BibTeX**.

[^7]: As proposed on [*How to deal with unknown individual
    authors?*](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#how-to-deal-with-unknown-individual-authors),
    **(Guide to Citation File Format schema version 1.2.0)**

```{r model_proceedings, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_proceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@proceedings** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@proceedings{proceedings-full,
    title        = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    year         = 1983,
    month        = mar,
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    note         = {This is a full PROCEEDINGS entry},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}
```

[*CFF entry*]{.underline}

```{r echo=FALSE,}
bib <- "@proceedings{proceedings-full,
    title        = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    year         = 1983,
    month        = mar,
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    note         = {This is a full PROCEEDINGS entry},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@techreport

The crosswalk of **\@techreport** does not require any special treatment.

```{r model_techreport, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_techreport", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@techreport** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@techreport{techreport-full,
    title        = {A Sorting Algorithm},
    author       = {Tom Terrific},
    year         = 1988,
    month        = oct,
    address      = {Computer Science Department, Fanstord, California},
    number       = 7,
    note         = {This is a full TECHREPORT entry},
    institution  = {Fanstord University},
    type         = {Wishful Research Result}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@techreport{techreport-full,
    title        = {A Sorting Algorithm},
    author       = {Tom Terrific},
    year         = 1988,
    month        = oct,
    address      = {Computer Science Department, Fanstord, California},
    number       = 7,
    note         = {This is a full TECHREPORT entry},
    institution  = {Fanstord University},
    type         = {Wishful Research Result}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

### \@unpublished

The crosswalk of **\@unpublished** does not require any special treatment.

```{r model_unpublished, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "model_unpublished", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("**BibTeX**", "[CFF]{.underline}", "Notes"),
  caption = "**\\@unpublished** Model"
)
```

**Examples**

**BibTeX entry**

``` bibtex
@unpublished{unpublished-minimal,
    title        = {Lower Bounds for Wishful Research Results},
    author       = {Ulrich Underwood and Ned Net and Paul Pot},
    note         = {Talk at Fanstord University (this is a minimal UNPUBLISHED entry)}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@unpublished{unpublished-minimal,
    title        = {Lower Bounds for Wishful Research Results},
    author       = {Ulrich Underwood and Ned Net and Paul Pot},
    note         = {Talk at Fanstord University (this is a minimal UNPUBLISHED entry)}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

## Appendix A: **\@inbook** in BibTeX and BibLaTeX {#appendix_inbook}

The definition of **\@inbook** and **\@incollection** in **BibTeX**
[@patashnik1988] is as follows:

> -   **\@inbook**: A part of a book, which may be a chapter (or section) and/or
>     a range of pages. Required fields: author or editor, title, chapter and/or
>     pages, publisher, year (...)
>
> -   **\@incollection**: A part of a book having its own title. Required
>     fields: author, title, booktitle, publisher, year (...)

Whereas **BibLaTeX** [@biblatexpack] specifies:

> -   **\@inbook:** A part of a book which forms a self-contained unit with its
>     own title. Note that the [profile]{.underline} of this entry type is
>     [different from standard BibTeX]{.underline}, see § 2.3.1. Required
>     fields: author, title, booktitle, year/date (...).

When considering required fields, an important difference is **booktitle**
requirement in **BibLaTeX**. Notably, **BibTeX \@incollection** requires also
this field. Moreover, both **BibTeX \@incollection** and **BibLaTeX \@inbook**
emphasize its reference to *"a part of a book (...) with its own title"*.

In this document, the proposed crosswalk ensures full compatibility with
**BibTeX**. Hence, we propose to consider a **BibLaTeX \@inbook** entry as
equivalent to a **BibTeX \@incollection**, given the congruence in their
definitions and field requirements.

**Examples**

**BibTeX entry**

``` bibtex
@inbook{inbook-biblatex,
    author       = {Yihui Xie and Christophe Dervieux and Emily Riederer},
    title        = {Bibliographies and citations},
    booktitle    = {{R} Markdown Cookbook},
    date         = {2023-12-30},
    publisher    = {Chapman and Hall/CRC},
    address      = {Boca Raton, Florida},
    series       = {The {R} Series},
    isbn         = 9780367563837,
    url          = {https://bookdown.org/yihui/rmarkdown-cookbook},
    chapter      = {4.5}
}
```

[CFF entry]{.underline}

```{r echo=FALSE,}
bib <- "@inbook{inbook-biblatex,
	author       = {Yihui Xie and Christophe Dervieux and Emily Riederer},
	title        = {Bibliographies and citations},
	booktitle    = {{R} Markdown Cookbook},
	date         = {2023-12-30},
	publisher    = {Chapman and Hall/CRC},
	address      = {Boca Raton, Florida},
	series       = {The {R} Series},
	isbn         = 9780367563837,
	url          = {https://bookdown.org/yihui/rmarkdown-cookbook},
	chapter      = {4.5}
}"

cff_read_bib_text(bib)
```

From [CFF]{.underline} to **BibTeX**

```{r echo=FALSE,}
toBibtex(cff_read_bib_text(bib))
```

## Appendix B: [CFF key:type]{.underline} values {#appendix_cff_type}

From @druskat2019 Table 4: Complete list of [CFF]{.underline} reference types.

```{r cff_types, echo=FALSE, results='asis'}
df_table <- table_master[table_master$table == "cff_types", c(2:3)]
df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
knitr::kable(df_table,
  col.names = c("Reference type string", "Description"),
  row.names = NA,
  caption = "Complete list of [CFF]{.underline} reference types."
)
```

## References

---
title: "BibTeX and CFF"
subtitle: A deterministic crosswalk between two non-equivalent citation schemas
bibliography: REFERENCES.bib
author:
  - name: Diego Hernangómez
    orcid: 0000-0001-8457-4658
    affiliations:
      - Independent Researcher
tbl-cap-location: bottom
description: >
  This article presents a crosswalk between BibTeX and the Citation File Format,
  as implemented by the cffr package.
abstract: >
  This article introduces a crosswalk between BibTeX and the Citation File
  Format (CFF) [@druskat_citation_2021], as implemented by the **cffr** package
  [@hernangomez2021]. Because the two formats differ in structure and
  expressiveness, the mapping is not bijective. Nevertheless, it provides a
  deterministic and reproducible strategy for practical interoperability across
  citation workflows.
link-citations: true
documentclass: article
toc: true
toc-depth: 3
vignette: >
  %\VignetteIndexEntry{BibTeX and CFF}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk:
    collapse: true
    comment: "#>"
---

```{r}
#| include: false
library(cffr)
# Load the table of tables

p2file <- system.file("extdata/crosswalk_tables.csv", package = "cffr")

table_master <- read.csv(p2file)
```

::: callout-important
Generative AI tools were used to assist in producing some of the material in
this article.
:::

## Citation

Please cite this article using this BibTeX entry:

``` bib
@article{hernangomez2022,
    title        = {{BibTeX} and {CFF}, A deterministic crosswalk between two non-equivalent citation schemas},
    author       = {Diego Hernangómez},
    year         = 2022,
    journal      = {The {cffr} package},
    volume       = {Vignettes},
    doi          = {10.21105/joss.03900},
    url          = {https://docs.ropensci.org/cffr/articles/bibtex-cff.html}
}
```

## BibTeX and R

[BibTeX](https://en.wikipedia.org/wiki/BibTeX) is a widely used format for
storing bibliographic references, originally designed in 1985 for
document‑centric workflows. It represents citations as relatively flat records
with loosely constrained fields, relying on convention rather than an explicit
schema.

The Citation File Format (CFF) [@druskat_citation_2021] provides a structured
and extensible alternative for representing citation metadata, particularly for
software and research outputs. It supports explicit typing, nested objects, and
richer semantics, including contributor roles and identifiers.

In modern research workflows, both formats frequently coexist. BibTeX remains
dominant in academic publishing pipelines, while CFF is increasingly adopted by
infrastructure platforms such as GitHub and Zenodo.

The **cffr** package [@hernangomez2021] provides a deterministic mapping
(crosswalk) between BibTeX and CFF, enabling practical interoperability. Due to
fundamental differences in their data models, this mapping is not bijective and
may involve structural transformations, heuristic parsing, and controlled
information loss.

## Conceptual differences between BibTeX and CFF

At a conceptual level, BibTeX and CFF embody different design philosophies.

BibTeX represents bibliographic information as flat records with loosely defined
fields, optimized for citation rendering in documents. Semantics are largely
implicit and contextual, depending on entry types and bibliography styles.

CFF, in contrast, defines a structured schema with explicit typing, nested
objects, and support for richer metadata. It is designed not only for producing
citations, but also for machine‑actionable interoperability across platforms.

These conceptual differences explain why a direct one‑to‑one correspondence
between BibTeX and CFF is not possible. Any crosswalk must therefore rely on
explicit design decisions that balance fidelity, usability, and schema
constraints.

## BibTeX Definitions

@patashnik1988 provides the canonical description of the BibTeX format. Two key
concepts are central: **entries** and **fields**.

### Entries {#entries}

The original BibTeX specification defines fourteen canonical entry types, each
corresponding to a class of cited work:

1.  **\@article**: An article from a journal or magazine.
2.  **\@book**: A book with an explicit publisher.
3.  **\@booklet**: A printed and bound work without a named publisher.
4.  **\@conference**: Equivalent to **\@inproceedings**, included for
    [Scribe](https://en.wikipedia.org/wiki/Scribe_(markup_language))
    compatibility.
5.  **\@inbook**: A part of a book, such as a chapter or page range.
6.  **\@incollection**: A part of a book with its own title.
7.  **\@inproceedings**: An article in conference proceedings.
8.  **\@manual**: Technical documentation.
9.  **\@mastersthesis**: A Master’s thesis.
10. **\@misc**: A fallback type when no other entry fits.
11. **\@phdthesis**: A PhD thesis.
12. **\@proceedings**: The proceedings of a conference.
13. **\@techreport**: A numbered technical report.
14. **\@unpublished**: A work not formally published.

Other implementations, notably BibLaTeX [@biblatexpack], extend this set with
additional entry types such as online resources, software, and datasets. In
standard BibTeX, such entries must typically be represented as **\@misc**.

In **R** [@R_2021], the base function `bibentry()` does not implement
**\@conference**. Instead, **\@inproceedings** is used, as both share the same
conceptual definition.

### Fields {#fields}

BibTeX entries consist of fields whose relevance and requirement depend on the
entry type. Some fields are mandatory, others optional, and some are ignored by
standard bibliography styles but may still carry useful information.

The following table summarizes the relationship between BibTeX **entries** and
their **required or optional fields**, following @patashnik1988. Required fields
are marked with **x**, and optional fields with **o**.

```{r}
#| label: tbl-entry_fields1
#| echo: false
#| tbl-cap: BibTeX entries
#| tbl-subcap:
#|   - "BibTeX: required fields by entry"
#|   - "BibTeX: required fields by entry (cont.)"
#|
df_table <- table_master[table_master$table == "entry_fields", -1]

nms <- c(
  "field",
  "\\@article",
  "\\@book",
  "\\@booklet",
  "\\@inbook",
  "\\@incollection",
  "\\@conference, \\@inproceedings",
  "\\@manual",
  "\\@mastersthesis, phdthesis",
  "\\@misc",
  "\\@proceedings",
  "\\@techreport",
  "\\@unpublished"
)

df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
t1 <- df_table[, c(1:7)]
nm1 <- nms[1:7]

knitr::kable(
  t1,
  col.names = nm1,
  row.names = NA,
  align = c("l", rep("c", 6))
)

t2 <- df_table[, c(1, 8:13)]
nm2 <- nms[c(1, 8:13)]
knitr::kable(
  t2,
  col.names = nm2,
  row.names = NA,
  align = c("l", rep("c", 6))
)
```

Only a subset of fields is required for any given entry. Fields such as
**title**, **author**, and **year** appear across most entry types, whereas
others are optional or never mandatory. This strict coupling between entry types
and fields is a defining feature of BibTeX and a key source of friction when
interoperating with schema‑driven formats such as CFF.

## Citation File Format

Citation File Format (CFF) consists of plain‑text, YAML‑based files that encode
human‑ and machine‑readable citation metadata for software and datasets.

Two keys play a central role in CFF reference modeling:

- `preferred-citation`: Identifies a work that should be cited instead of the
  software or dataset itself.
- `references`: Lists related creative works, analogous to references in a
  scholarly article.

Both keys expect `definition-reference` objects, as defined in the [CFF schema
guide](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#preferred-citation).
These objects support explicit typing, metadata nesting, and structured
identifiers, in contrast to BibTeX’s flat field model.

The following table summarizes the valid keys for CFF `definition-reference`:

```{r}
#| label: tbl-refkeys
#| echo: false
#| message: false
#| warning: false
#| results: asis
#| tbl-cap: "Valid keys in CFF `definition-reference` objects"
library(cffr)

# Fill with whites
init <- paste0("`", cff_schema_definitions_refs(), "`")

l <- c(init, rep("", 4))


refkeys <- matrix(l, ncol = 5, byrow = TRUE)

knitr::kable(
  refkeys,
  row.names = NA
)
```

Conceptually, most of these keys correspond to BibTeX [**fields**](#fields),
with one key playing a special role: `type`. In CFF, `type` explicitly
identifies the kind of referenced work, making it conceptually analogous to a
BibTeX [**entry**](#entries) type rather than a **field**[^1].

[^1]: See a complete list of possible values of CFF `type` in [Appendix
    B](#appendix_cff_type)

## Mapping strategy

The mapping implemented in **cffr** follows a three‑step pipeline:

1.  Parse BibTeX entries into an intermediate representation.
2.  Apply deterministic mapping rules to transform entry types and fields.
3.  Serialize the result into CFF (and vice versa).

Whenever possible, mappings are deterministic. However, some fields require
heuristic parsing, particularly when normalizing names, dates, or page ranges.
As a result, semantic round‑trip reversibility is not guaranteed, even when the
conversion process itself is reproducible.

```{r}
#| label: cffbibread
#| comment: '#>'
string <- "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925}"

# To cff
library(cffr)
cff_format <- cff_read_bib_text(string)

cff_format

# To BibTeX with S3 method
toBibtex(cff_format)
```

## Mapping tables

The following tables summarize the mapping rules implemented in **cffr**. They
are intended as implementation documentation rather than as a normative
specification.

Mapping types include:

- **direct**: preserved without modification.
- **transform**: renamed or structurally reorganized.
- **split**: one field mapped to multiple fields.
- **heuristic**: requires parsing or inference.
- **unsupported**: omitted due to lack of correspondence.
- **enrichment**: exists only in CFF.

These mappings are not bijective and may introduce normalization effects.

::: callout-note
For clarity throughout this document, **bold** formatting (e.g., **\@book**,
**edition**) is used for BibTeX entries and fields, whereas inline code
formatting (e.g., `book`, `edition`) is used for CFF keys.
:::

::: {#tbl-mapentry}
| BibTeX entry | CFF key `type` | Mapping type | Notes |
|:-------------------|:-------------------|:-------------------|:-------------------|
| **\@article** | `type: article` | direct | \- |
| **\@book** | `type: book` | direct | \- |
| **\@inbook** | `type: chapter` | transform | Normalized to CFF taxonomy |
| **\@incollection** | `type: chapter` | transform | Same as **\@inbook** |
| **\@manual** | `type: manual` | direct | \- |
| **\@misc** | `type: generic` | heuristic | `generic` is used as a fallback when no more specific CFF type applies. |
| **\@phdthesis** | `type: thesis` | transform | The specific thesis subtype (e.g., master’s vs PhD) may be lost when mapping from BibTeX to CFF |
| **\@techreport** | `type: report` | transform | \- |
| **other** | `type: generic` | heuristic | Default fallback |

: BibTeX entries and CFF key `type` mapping
:::

::: {#tbl-mapcore}
| BibTeX field  | CFF key            | Mapping type | Notes                               |
|:-------------------|:-------------------|:-------------------|:--------------------|
| **title**     | `title`            | direct       | \-                                  |
| **author**    | `authors`          | transform    | Parsed into structured name objects |
| **editor**    | `editors`          | transform    | Same parsing logic as authors       |
| **year**      | `year`             | direct       | \-                                  |
| **month**     | `month`            | heuristic    | Format normalization required       |
| **journal**   | `journal`          | direct       | \-                                  |
| **booktitle** | `collection-title` | transform    | Container title                     |
| **volume**    | `volume`           | direct       | \-                                  |
| **number**    | `issue`            | transform    | Naming normalization                |
| **pages**     | `start`/`end`      | split        | Parsed from page range              |
| **doi**       | `doi`              | direct       | \-                                  |
| **url**       | `url`              | direct       | \-                                  |

: Core fields
:::

::: {#tbl-mapstruct}
| BibTeX field    | CFF key              | Mapping type | Notes                          |
|:-------------------|:-------------------|:-------------------|:--------------------|
| **publisher**   | `publisher.name`     | transform    | Converted to structured object |
| **address**     | `publisher.location` | transform    | Combined with `publisher`      |
| **institution** | `institution`        | direct       | Used mainly in reports/thesis  |
| **school**      | `institution`        | transform    | Thesis normalization           |

: Structural fields
:::

::: {#tbl-mapparsed}
| BibTeX field | CFF key       | Mapping type | Notes                                 |
|:-------------|:--------------|:-------------|:--------------------------------------|
| **pages**    | `start`/`end` | heuristic    | Requires parsing, may fail            |
| **author**   | `authors`     | heuristic    | Name splitting is not always reliable |
| **month**    | `month`       | heuristic    | Multiple formats possible             |
| **note**     | `notes`       | heuristic    | Free text                             |

: Parsed fields
:::

::: {#tbl-mapnobib}
| CFF key              | BibTeX equivalent | Mapping type | Notes                |
|:---------------------|:------------------|:-------------|:---------------------|
| `preferred-citation` | \-                | enrichment   | CFF-specific concept |
| `references`         | \-                | enrichment   | Not representable    |
| `type: software`     | \-                | unsupported  | No BibTeX equivalent |
| `identifiers`        | partial           | transform    | Only DOI/URL mapped  |

: CFF keys with no BibTeX equivalence
:::

## Proposed Crosswalk

The **cffr** package provides utilities for mapping BibTeX entries (via
`bibentry()`) to CFF and back. This section describes how the crosswalk is
implemented, partially informed by @Haines_Ruby_CFF_Library_2021[^2].

[^2]: Note that this software performs only the mapping from CFF to BibTeX,
    however **cffr** can perform the mapping in both directions.

The crosswalk is primarily defined for BibTeX to CFF conversion, reverse mapping
is a best-effort approximation and may be lossy.

After presenting the general mapping between BibTeX **entries** and **fields**
and CFF keys, the next section introduces [**Entry Models**](#entrymodels) that
refine these rules for specific BibTeX entry types.

The mapping is structured according to the transformation semantics defined
below.

## Transformation Semantics

The crosswalk applies different transformation strategies:

- **Direct mapping**: one-to-one correspondence between a BibTeX field and a CFF
  key.
- **Structural transformation**: a single field is expanded into multiple keys,
  such as **pages** to `start` and `end`.
- **Heuristic mapping**: equivalence is inferred based on entry type or context.
- **Lossy mapping**: information is discarded due to lack of an equivalent
  representation.

Unless stated otherwise, mappings are lossy in at least one direction.

All mappings and examples in the following sections are instances of these
transformation classes.

### Entry/Key `type` Crosswalk

For mapping general BibTeX entries to CFF key `type`, the following equivalence
is proposed:

```{r}
#| label: tbl-entry_bib2cff
#| echo: false
#| results: asis
#| tbl-cap: "Entry/Type crosswalk: From BibTeX to CFF"
df_table <- table_master[table_master$table == "entry_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("BibTeX Entry", "CFF key: `type`", "Notes"),
  row.names = NA
)
```

The previous mapping has the following specifications:

- **\@book**, **\@inbook**, and **\@incollection** are closely related in
  BibTeX[^3]. While **\@inbook** and **\@incollection** both reference parts of
  a **\@book**, the former is used for citing sections, chapters, pages, or
  other specific parts, whereas the latter is used for citing parts with a
  specific title. Since CFF allows keys `type: book` and
  `collection-type: book`, we may utilize a combination of these fields to tag
  each entry type in CFF accordingly.
- **\@mastersthesis** and **\@phdthesis** are tagged using a combination of
  `type: thesis` and `thesis-type` keys.

[^3]: Note that BibLaTeX [@biblatexpack] handles **\@inbook** differently, see
    [Appendix A](#appendix_inbook).

Additionally, considering that CFF allows for a wide range of values[^4] for the
key `type`, the following mapping is applied from CFF to BibTeX:

[^4]: See [Appendix B](#appendix_cff_type) for all possible values. Information
    extracted from @druskat2019.

The reverse mapping prioritizes BibTeX compatibility and normalization over
exact round-trip preservation.

```{r}
#| label: tbl-entry_cff2bib
#| echo: false
#| results: asis
#| tbl-cap: "Entry/Key `type` crosswalk: From CFF to BibTeX"
df_table <- table_master[table_master$table == "entry_cff2bib", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("CFF key `type`", "BibTeX Entry", "Notes")
)
```

### Fields/Key Crosswalk

There is a significant similarity between the definitions and names of certain
BibTeX fields and CFF keys. While the equivalence is straightforward in some
cases, there are instances where certain keys need to be processed depending on
the **entry** type.

```{r}
#| label: tbl-fields_bib2cff
#| echo: false
#| results: asis
#| tbl-cap: "BibTeX - CFF key crosswalk"
df_table <- table_master[table_master$table == "fields_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("BibTeX **Field**", "CFF key", "Notes")
)
```

We provide more detail on some of the mappings presented in the table above:

- Some fields are not mapped because there is no clear equivalence with CFF keys
  (such as **annote**, **crossref**, and **key**). Regarding the **type** field,
  the CFF key `type` corresponds to the identifier of the work (similar to an
  entry in BibTeX), therefore, BibTeX **type** won't be mapped. These fields are
  always optional in BibTeX.
- For the **address** field, its intended use in BibTeX varies depending on the
  entry type (e.g., for **\@inproceedings**, it denotes the **address** of the
  **conference**, while for **\@mastersthesis/\@phdthesis**, it is the
  **address** of the **school**, etc.). Mapping between BibTeX and CFF becomes
  more complex when institutional metadata is involved. This results in varying
  final mappings in CFF. When mapping from CFF to BibTeX, we propose to follow
  the same entry-based logic, using the key `location` as a fallback value when
  mapping to **address**.
- In relation with this complexity mentioned above, **institution,
  organization** and **school** are mapped to `institution`.
- **series** is mapped to `collection-title` only on those entries that do not
  require **booktitle**. In practice, this means that `collection-title`
  corresponds to **booktitle** (for **\@incollection** and **\@inproceedings**),
  and in the other cases it corresponds to **series**. As a consequence,
  **series** information is lost for **\@incollection** and **\@inproceedings**,
  but in those cases it is an optional field.
- When mapping from CFF to BibTeX, we propose to use `date-published` as a
  fallback for extracting **month** and **year** fields.
- When **pages** is provided as a range separated by `--`, i.e. **pages =
  {3--5}**, it is coerced as `start: 3`, `end: 5` in CFF.

#### BibLaTeX

Additionally, there are other CFF keys that correspond to BibLaTeX fields. We
propose to include these fields in the crosswalk[^5], even though they are not
part of the core BibTeX fields definition.

[^5]: See @biblatexcheatsheet for a preview of the accepted BibLaTeX fields.

```{r}
#| label: tbl-fields_biblatex2cff
#| echo: false
#| results: asis
#| tbl-cap: "BibLaTeX - CFF Field/Key crosswalk"
df_table <- table_master[table_master$table == "fields_biblatex2cff", c(2:3)]
df_table[is.na(df_table)] <- ""
# fix links
df_table$f2 <- gsub("link_to_entry_models", "#entrymodels", df_table$f2)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("**BibLaTeX Field**", "CFF key")
)
```

## Limitations

The proposed crosswalk is subject to several structural limitations arising from
differences between BibTeX and CFF schemas:

- **Lossy mappings**: Some BibTeX fields (e.g., **crossref**, **annote**,
  **key**) have no equivalent in CFF and are omitted during conversion.
- **Ambiguous semantics**: Certain fields (e.g., **address**, **series**) have
  context-dependent meanings in BibTeX, requiring heuristic mapping to CFF keys
  such as `institution`, `location`, or `collection-title`.
- **Type collapsing**: Multiple BibTeX entry types (e.g., **\@misc**,
  **\@incollection**) are mapped to a single CFF type (`generic`), which acts as
  a fallback when no more specific type is available.
- **Structural transformations**: Some fields require transformation rather than
  direct mapping (e.g., **pages** → `start`/`end`), altering the representation
  of the original data.
- **Non-reversibility**: Due to the above factors, round-trip conversion (BibTeX
  → CFF → BibTeX) is not guaranteed to preserve the original structure or
  semantics.

These limitations reflect fundamental differences between the two formats rather
than implementation-specific constraints.

## Design decisions

The mapping implemented in **cffr** is guided by the following principles:

- Prefer deterministic mappings whenever possible
- Use heuristics only when unavoidable
- Normalize metadata to align with CFF conventions
- Prioritize interoperability over exact round‑trip fidelity

## Entry Models {#entrymodels}

This section documents entry‑specific mapping behavior, expanding the general
crosswalk into concrete and testable models. Examples are adapted from the
[xampl.bib](https://tug.org/texmf-docs/bibtex/xampl.bib) distributed with BibTeX
[@patashnik].

### \@article {#article}

The crosswalk of **\@article** does not require any special treatment.

```{r}
#| label: tbl-model_article
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_article", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@article** Model"
)
```

**Round-trip**

```{r}
bib <- "@article{article-full,
    title        = {The Gnats and Gnus Document Preparation System},
    author       = {Leslie A. Aamport},
    year         = 1986,
    month        = jul,
    journal      = {{G-Animal's} Journal},
    volume       = 41,
    number       = 7,
    pages        = {73+},
    note         = {This is a full ARTICLE entry}}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@book / \@inbook {#book-inbook}

In terms of the fields required in BibTeX, the primary difference between
**\@book** and **\@inbook** is that **\@inbook** requires a **chapter** or
**page** field, while **\@book** does not even allow these fields as optional.
Therefore, we propose that an **\@inbook** entry in CFF be treated as a
**\@book** with the following supplementary fields:

1.  `section`: To denote the specific **chapter** within the book.
2.  `start`/`end`: To indicate the range of **pages** covered by the section.

Additionally, note that in CFF, the **series** field corresponds to
`collection-title`, and the **address** field represents the `publisher`'s
`address`. Finally, the key `collection-type` is populated with `book-series`.

```{r}
#| label: tbl-model_book
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_book", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@book / \\@inbook** Model"
)
```

There are notable differences in how BibTeX and **BibLaTeX** handle the
**\@inbook** entry (further discussed in the [Appendix A](#appendix_inbook)). We
propose to treat a BibLaTeX **\@inbook** as a BibTeX **\@incollection.**

**Round-trip: \@book**

```{r}
bib <- "@book{book-full,
    title        = {Seminumerical Algorithms},
    author       = {Donald E. Knuth},
    year         = 1981,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 2,
    note         = {This is a full BOOK entry},
    edition      = {Second}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

**Round-trip: \@inbook**

```{r}
bib <- "@inbook{inbook-full,
    title        = {Fundamental Algorithms},
    author       = {Donald E. Knuth},
    year         = 1973,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 1,
    pages        = {10--119},
    note         = {This is a full INBOOK entry},
    edition      = {Second},
    type         = {Section},
    chapter      = {1.2}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@booklet {#booklet}

In **\@booklet** **address** is mapped to `location`.

```{r}
#| label: tbl-model_booklet
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_booklet", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@booklet** Model"
)
```

**Round-trip**

```{r }
bib <- "@booklet{booklet-full,
    title        = {The Programming of Computer Art},
    author       = {Jill C. Knvth},
    date         = {1988-03-14},
    month        = feb,
    address      = {Stanford, California},
    note         = {This is a full BOOKLET entry},
    howpublished = {Vernier Art Center}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@conference / \@inproceedings {#conf_inproc}

Note that in this case, **organization** is mapped to `institution`.
Additionally, **series** is ignored because there is no clear mapping in CFF for
this field.

```{r}
#| label: tbl-model_inproceedings
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_inproceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@conference / \\@inproceedings** Model"
)
```

**Round-trip**

```{r}
bib <- "@inproceedings{inproceedings-full,
    title        = {On Notions of Information Transfer in {VLSI} Circuits},
    author       = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
    year         = 1983,
    month        = mar,
    booktitle    = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    pages        = {133--139},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@incollection {#incol}

As **booktitle** is a required field, we propose to map that field to
`collection-title` and the `type` to `generic`. Therefore, an **\@incollection**
is a `type: generic` in CFF with a `collection-title` key. The `generic` type is
used as a fallback when no semantically equivalent CFF type exists.

Additionally, **series** and **type** are ignored because there is no clear
mapping in CFF for this field.

```{r}
#| label: tbl-model_incollection
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_incollection", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@incollection** Model"
)
```

**Round-trip**

```{r}
bib <- "@incollection{incollection-full,
    title        = {Semigroups of Recurrences},
    author       = {Daniel D. Lincoll},
    year         = 1977,
    month        = sep,
    booktitle    = {High Speed Computer and Algorithm Organization},
    publisher    = {Academic Press},
    address      = {New York},
    series       = {Fast Computers},
    number       = 23,
    pages        = {179--183},
    note         = {This is a full INCOLLECTION entry},
    editor       = {David J. Lipcoll and D. H. Lawrie and A. H. Sameh},
    chapter      = 3,
    type         = {Part},
    edition      = {Third}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@manual

As in the case of [**\@conference** / **\@inproceedings**](#conf_inproc),
**organization** is mapped to `institution`.

```{r}
#| label: tbl-model_manual
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_manual", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@manual** Model"
)
```

**Round-trip**

Note that **month** cannot be coerced to a single integer in the range `1--12`
as required in CFF, so it is ignored to avoid validation errors.

```{r}
bib <- "@manual{manual-full,
  title        = {The Definitive Computer Manual},
    author       = {Larry Manmaker},
    year         = 1986,
    month        = {apr-may},
    address      = {Silicon Valley},
    note         = {This is a full MANUAL entry},
    organization = {Chips-R-Us},
    edition      = {Silver}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@mastersthesis / \@phdthesis

In terms of fields required by BibTeX, it is identical for both
**\@mastersthesis** and **\@phdthesis.**

We propose here to identify each type of thesis using the key `thesis-type`. If
`thesis-type` contains a [regex pattern](https://regex101.com/r/mBWfbs/1)
`(?i)(phd)`, it is recognized as **\@phdthesis**.

Additionally, **school** is mapped to `institution`.

```{r}
#| label: tbl-model_thesis
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_thesis", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@mastersthesis / \\@phdthesis** Model"
)
```

**Round-trip: \@mastersthesis**

```{r}
bib <- "@mastersthesis{mastersthesis-full,
    title        = {Mastering Thesis Writing},
    author       = {Edouard Masterly},
    year         = 1988,
    month        = jun,
    address      = {English Department},
    note         = {This is a full MASTERSTHESIS entry},
    school       = {Stanford University},
    type         = {Master's project}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

**Round-trip: \@phdthesis**

```{r}
bib <- "@phdthesis{phdthesis-full,
    title        = {Fighting Fire with Fire: Festooning {F}rench Phrases},
    author       = {F. Phidias Phony-Baloney},
    year         = 1988,
    month        = jun,
    address      = {Department of French},
    note         = {This is a full PHDTHESIS entry},
    school       = {Fanstord University},
    type         = {{PhD} Dissertation}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@misc

The crosswalk of **\@misc** does not require any special treatment. This
**entry** does not require any **field**.

Note also that it is mapped to `type: generic` as [**\@incollection**](#incol),
but in this case **booktitle** is not even an option, so the proposed definition
should cover both **\@misc** and **\@incollection** without problems.

```{r}
#| label: tbl-model_misc
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_misc", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@misc** Model"
)
```

**Round-trip**

```{r}
bib <- "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@proceedings

The proposed model is consistent with [**\@conference** /
**\@inproceedings**](#conf_inproc). Note that **\@proceedings** does not
prescribe an **author** field. In these cases, as `authors` is required in CFF,
we use *anonymous*[^6] when mapping to CFF and omit it when mapping from CFF to
BibTeX.

[^6]: As proposed on [*How to deal with unknown individual
    authors?*](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#how-to-deal-with-unknown-individual-authors),
    **(Guide to Citation File Format schema version 1.2.0)**

```{r}
#| label: tbl-model_proceedings
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_proceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@proceedings** Model"
)
```

**Round-trip**

```{r}
bib <- "@proceedings{proceedings-full,
    title        = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    year         = 1983,
    month        = mar,
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    note         = {This is a full PROCEEDINGS entry},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@techreport

The crosswalk of **\@techreport** does not require any special treatment.

```{r}
#| label: tbl-model_techreport
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_techreport", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@techreport** Model"
)
```

**Round-trip**

```{r}
bib <- "@techreport{techreport-full,
    title        = {A Sorting Algorithm},
    author       = {Tom Terrific},
    year         = 1988,
    month        = oct,
    address      = {Computer Science Department, Fanstord, California},
    number       = 7,
    note         = {This is a full TECHREPORT entry},
    institution  = {Fanstord University},
    type         = {Wishful Research Result}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@unpublished

The crosswalk of **\@unpublished** does not require any special treatment.

```{r}
#| label: tbl-model_unpublished
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_unpublished", c(2:4)]
df_table[is.na(df_table)] <- ""

# fix links
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@unpublished** Model"
)
```

**Round-trip**

```{r}
bib <- "@unpublished{unpublished-minimal,
    title        = {Lower Bounds for Wishful Research Results},
    author       = {Ulrich Underwood and Ned Net and Paul Pot},
    note         = {Talk at Fanstord University (this is a minimal UNPUBLISHED entry)}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

## Conclusion

This article presents a practical and reproducible crosswalk between BibTeX and
CFF.

Although the formats are not fully equivalent, the deterministic mapping
implemented in **cffr** enables consistent transformations across heterogeneous
citation ecosystems. By making design decisions explicit and documenting
limitations, this work supports interoperable citation workflows bridging legacy
bibliographic systems and modern software‑centric practices.

## Appendix A: **\@inbook** in BibTeX and BibLaTeX {#appendix_inbook .appendix}

The definition of **\@inbook** and **\@incollection** in BibTeX [@patashnik1988]
is as follows:

> - **\@inbook**: A part of a book, which may be a chapter (or section) and/or a
>   range of pages. Required fields: author or editor, title, chapter and/or
>   pages, publisher, year (...)
>
> - **\@incollection**: A part of a book having its own title. Required fields:
>   author, title, booktitle, publisher, year (...)

Whereas BibLaTeX [@biblatexpack] specifies:

> - **\@inbook:** A part of a book which forms a self-contained unit with its
>   own title. Note that the [profile]{.underline} of this entry type is
>   [different from standard BibTeX]{.underline}, see § 2.3.1. Required fields:
>   author, title, booktitle, year/date (...).

When considering required fields, an important difference is the **booktitle**
requirement in BibLaTeX. Notably, BibTeX **\@incollection** requires also this
field. Moreover, both BibTeX **\@incollection** and BibLaTeX **\@inbook**
emphasize its reference to *"a part of a book (...) with its own title"*.

In this document, the proposed crosswalk ensures full compatibility with BibTeX.
Hence, we propose to consider a BibLaTeX **\@inbook** entry as equivalent to a
BibTeX **\@incollection**, given the congruence in their definitions and field
requirements.

**Round-trip**

```{r}
bib <- "@inbook{inbook-biblatex,
	author       = {Yihui Xie and Christophe Dervieux and Emily Riederer},
	title        = {Bibliographies and citations},
	booktitle    = {{R} Markdown Cookbook},
	date         = {2023-12-30},
	publisher    = {Chapman and Hall/CRC},
	address      = {Boca Raton, Florida},
	series       = {The {R} Series},
	isbn         = 9780367563837,
	url          = {https://yihui.org/rmarkdown-cookbook/},
	chapter      = {4.5}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

## Appendix B: CFF key: `type` values {#appendix_cff_type .appendix}

From @druskat2019, Table 4: Complete list of CFF reference types for key `type`.
Only a subset of these types is actively used in the proposed crosswalk.

```{r}
#| label: tbl-cff_types
#| echo: false
#| results: asis
#| tbl-cap: "Complete list of CFF reference types"
df_table <- table_master[table_master$table == "cff_types", c(2:3)]
df_table[is.na(df_table)] <- ""

row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("Reference type string", "Description"),
  row.names = NA
)
```

---
title: "Manipulating citations with cffr"
description: Learn how to modify `cff` objects.
toc: true
bibliography: REFERENCES.bib
link-citations: true
vignette: >
  %\VignetteIndexEntry{Manipulating citations with cffr}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk:
    collapse: true
    comment: "#>"
    warning: false
    message: true
---

```{r}
#| include: false
library(cffr)
```

**cffr** is designed for **R** package developers. Its main goal is to create a
`CITATION.cff` file using metadata from the following files:

- Your `DESCRIPTION` file.
- Your `inst/CITATION` file, when available.

## What is a `CITATION.cff` file?

A `CITATION.cff` file follows the [Citation File Format
(CFF)](https://citation-file-format.github.io/) [@druskat_citation_2021]
(v1.2.0). It contains human- and machine-readable citation metadata for software
and datasets. Developers can include it in a repository to explain how to cite
their software.

This format is increasingly used in the software citation ecosystem.
[**GitHub**](https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/customizing-your-repository/about-citation-files),
[**Zenodo**](https://citation-file-format.github.io/#/supported-by-zenodo-) and
[**Zotero**](https://citation-file-format.github.io/#/supported-by-zotero-) support
this citation format [@druskat_stephan_making_2021].

**GitHub** support is of special interest:

<figure>

![GitHub citation support announcement](tweet-1.png){.quarto-figure
.quarto-figure-center fig-align="center" width="400"}

```{=html}
<figcaption class="blockquote-footer">Nat Friedman (@natfriedman) July 27, 2021</figcaption>
</figure>
```

See [**GitHub**'s guide to CITATION
files](https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/customizing-your-repository/about-citation-files)
for more information.

## Creating a `CITATION.cff` file for your R package

Creating a `CITATION.cff` file with **cffr** is straightforward. You only need
to run `cff_write()`:

```{r}
#| label: setup
#| eval: false
library(cffr)

cff_write()

# Done.
```

Under the hood, `cff_write()` performs these tasks:

- Extracts the metadata using `cff_create()`.
- Optionally modifies it with `cff_modify()`.
- Writes a `CITATION.cff` file.
- Validates the result using `cff_validate()`.

You now have a complete `CITATION.cff` file for your **R** package.

## Modifying your `CITATION.cff` file

You can customize a `cff` object using `cff_modify()`, the package's coercion
methods and the `keys` argument.

For demonstration, we create a `cff` object using `cff()` and then add or modify
its contents.

### Adding new keys

```{r}
#| label: newfields
newobject <- cff()

newobject
```

You can display the valid keys from the [Citation File Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md)
with `cff_schema_keys()`:

```{r}
#| label: validkeys
cff_schema_keys()
```

In this case, we add `url`, `version` and `repository`. We also overwrite the
`title` key by passing these arguments to `cff_modify()`:

```{r}
#| label: modify
modobject <- cff_modify(
  newobject,
  url = "https://ropensci.org/",
  version = "0.0.1",
  repository = "https://github.com/ropensci/cffr",
  # If the key is already present, it is overridden.
  title = "Modifying a 'cff' object"
)

modobject

# Validate against the schema.

cff_validate(modobject)
```

### Persons and references

`as_cff_person()` converts `person` objects and `as_cff()` converts `bibentry`
objects to metadata that follows the [Citation File Format
schema](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md).

Following the previous example, we first add a new author. To do that, we need
to extract the current package authors and append the converted person:

```{r}
#| label: includeauthor
# Valid person keys.

cff_schema_definitions_person()

# Create the person.

chiquito <- person(
  "Gregorio",
  "Sánchez Fernández",
  email = "fake@email2.com",
  comment = c(
    alias = "Chiquito de la Calzada",
    city = "Malaga",
    country = "ES",
    ORCID = "0000-0000-0000-0001"
  )
)

chiquito

# Convert to `cff`.
chiquito_cff <- as_cff_person(chiquito)
chiquito_cff

# Append to previous authors.

newauthors <- c(modobject$authors, chiquito_cff)
newauthors

newauthorobject <- cff_modify(modobject, authors = newauthors)

newauthorobject

cff_validate(newauthorobject)
```

Next, we add `references` to the `cff` object. This example adds two references,
one created with `bibentry()` and another with `citation()`:

```{r}
#| label: parsingcits
# Valid reference keys.

cff_schema_definitions_refs()

# Automatic coercion from another **R** package.
base_r <- citation("base")

bib <- bibentry(
  "Book",
  title = "This is a book",
  author = "Lisa Lee",
  year = 1980,
  publisher = "McGraw Hill",
  volume = 2
)

refs <- c(base_r, bib)

refs

# Convert to `cff`.

refs_cff <- as_cff(refs)

refs_cff
```

As with the `person` example, we modify the `cff` object:

```{r}
#| label: references
finalobject <- cff_modify(newauthorobject, references = refs_cff)

finalobject

cff_validate(finalobject)
```

### Write the modified `CITATION.cff` file

The result can be written with `cff_write()`:

```{r}
#| label: write
# Create a temporary output file.
tmp <- tempfile(fileext = ".cff")

see_res <- cff_write(finalobject, outfile = tmp)

cat(readLines(tmp), sep = "\n")
```

Finally, we can read the created `CITATION.cff` file using `cff_read()`:

```{r}
#| label: read
reading <- cff_read(tmp)

reading
```

Note that `cff_write()` also has the `keys` argument, so the workflow can be
simplified to:

```{r}
allkeys <- list(
  "url" = "https://ropensci.org/",
  "version" = "0.0.1",
  "repository" = "https://github.com/ropensci/cffr",
  # If the key is already present, it is overridden.
  title = "Modifying a 'cff' object",
  authors = newauthors,
  references = refs_cff
)

tmp2 <- tempfile(fileext = ".cff")

res <- cff_write(cff(), outfile = tmp2, keys = allkeys)

res
```

```{r}
#| include: false
# Clean temporary files.
unlink(tmp)
unlink(tmp2)
```

## References

---
title: "Manipulating Citations with cffr"
description: >
  Learn how to modify `cff` objects.
output: 
  rmarkdown::html_vignette:
    toc: true
bibliography: REFERENCES.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{Manipulating Citations with cffr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = TRUE
)

library(cffr)
```

**cffr** is a tool whose target audience are **R** package developers. The main
goal of **cffr** is to create a `CITATION.cff` file using the metadata
information of the following files:

-   Your `DESCRIPTION` file.
-   If available, the citation information located in `inst/CITATION`.

## What is a `CITATION.cff` file?

[Citation File Format (CFF](https://citation-file-format.github.io/))
[@druskat_citation_2021] (v1.2.0) are plain text files with human- and
machine-readable citation information for software (and data sets). Code
developers can include them in their repositories to let others know how to
correctly cite their software.

This format is becoming popular within the software citation ecosystem. Recently
[GitHub](https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/customizing-your-repository/about-citation-files),
[Zenodo](https://citation-file-format.github.io/#/supported-by-zenodo-) and
[Zotero](https://citation-file-format.github.io/#/supported-by-zotero-) have
included full support of this citation format [@druskat_stephan_making_2021].

GitHub support is of special interest:

```{r echo=FALSE, out.width="400", fig.align='center', fig.alt="GitHub-link"}
knitr::include_graphics("tweet-1.png")
```

*--- Nat Friedman (\@natfriedman) July 27, 2021*

See [Customize your repository/About CITATION
files](https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/customizing-your-repository/about-citation-files)
for more info.

## Creating a `CITATION.cff` file for my R package

With **cffr** creating a `CITATION.cff` file is quite straightforward. You just
need to run `cff_write()`:

```{r setup, eval=FALSE}
library(cffr)

cff_write()

# You are done!
```

Under the hood, `cff_write()` performs the following tasks:

-   It extracts the metadata using `cff_create()`.
-   Optionally modifies it with `cff_modify()`.
-   Writes a `CITATION.cff` file.
-   Validates the result using `cff_validate()`.

Congratulations! Now you have a full `CITATION.cff` file for your **R** package.

## Modifying your `CITATION.cff` file

You can easily customize the `cff` object (a custom class of **cffr**) using the
coercion system provided in the package, as well as making use of the `keys`
parameter.

We would create a `cff` object using `cff()` (for example purposes only) and we
would add or modify contents of it.

### Adding new fields

```{r newfields}
newobject <- cff()

newobject
```

The valid keys of the [Citation File Format schema version
1.2.0](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md)
can be displayed with `cff_schema_keys()`:

```{r validkeys}
cff_schema_keys()
```

In this case, we are going to add `url`, `version` and `repository`. We would
also overwrite the `title` key. We just need to add those parameters to
`cff_modify()`:

```{r modify}
modobject <- cff_modify(newobject,
  url = "https://ropensci.org/",
  version = "0.0.1",
  repository = "https://github.com/ropensci/cffr",
  # If the field is already present, it would be overridden
  title = "Modifying a 'cff' object"
)

modobject

# Validate against the schema

cff_validate(modobject)
```

### Persons and references

**cffr** provides two functions that convert `person` and `bibentry` objects
(see `?person` and `?bibentry`) according to the [Citation File Format
schema](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md).

Following the previous example, we are going to add a new author first. For
doing that, we need first to extract the current author of the package and
append the coerced person:

```{r includeauthor}
# Valid person keys

cff_schema_definitions_person()

# Create the person

chiquito <- person("Gregorio",
  "Sánchez Fernández",
  email = "fake@email2.com",
  comment = c(
    alias = "Chiquito de la Calzada",
    city = "Malaga",
    country = "ES",
    ORCID = "0000-0000-0000-0001"
  )
)

chiquito

# To cff
chiquito_cff <- as_cff_person(chiquito)
chiquito_cff


# Append to previous authors

newauthors <- c(modobject$authors, chiquito_cff)
newauthors

newauthorobject <- cff_modify(modobject, authors = newauthors)

newauthorobject

cff_validate(newauthorobject)
```

Now, we may want to add `references` to our data. On the following example, we
would add two references, one created with `bibentry()` and another with
`citation()`:

```{r parsingcits}
# Valid reference keys

cff_schema_definitions_refs()

# Auto coercion from another R package
base_r <- citation("base")

bib <- bibentry("Book",
  title = "This is a book",
  author = "Lisa Lee",
  year = 1980,
  publisher = "McGraw Hill",
  volume = 2
)

refs <- c(base_r, bib)

refs

# Now to cff

refs_cff <- as_cff(refs)

refs_cff
```

Now the process is similar to the example with `person`: we just modify our
`cff` object:

```{r references}
finalobject <- cff_modify(newauthorobject, references = refs_cff)

finalobject

cff_validate(finalobject)
```

### Create your modified `CITATION.cff` file

The results can be written with `cff_write()`:

```{r write}
# For example
tmp <- tempfile(fileext = ".cff")

see_res <- cff_write(finalobject, outfile = tmp)

cat(readLines(tmp), sep = "\n")
```

And finally we can read our created `CITATION.cff` file using `cff_read()`:

```{r read}
reading <- cff_read(tmp)

reading
```

Note that `cff_write()` also has the `keys` param, so the workflow can be
simplified as:

```{r}
allkeys <- list(
  "url" = "https://ropensci.org/",
  "version" = "0.0.1",
  "repository" = "https://github.com/ropensci/cffr",
  # If the field is already present, it would be overridden
  title = "Modifying a 'cff' object",
  authors = newauthors,
  references = refs_cff
)

tmp2 <- tempfile(fileext = ".cff")

res <- cff_write(cff(), outfile = tmp2, keys = allkeys)

res
```

```{r include=FALSE}
# Clean temps
unlink(tmp)
unlink(tmp2)
```

## References

---
title: "BibTeX and CFF"
subtitle: A deterministic crosswalk between two non-equivalent citation schemas
bibliography: REFERENCES.bib
author:
  - name: Diego Hernangómez
    orcid: 0000-0001-8457-4658
    affiliations:
      - Independent Researcher
tbl-cap-location: bottom
description: >
  This article presents a crosswalk between BibTeX and the Citation File Format,
  as implemented by the **cffr** package.
abstract: >
  This article introduces a crosswalk between BibTeX and the Citation File
  Format (CFF) [@druskat_citation_2021], as implemented by the **cffr** package
  [@hernangomez2021]. Because the two formats differ in structure and
  expressiveness, the mapping is not bijective. Nevertheless, it provides a
  deterministic and reproducible strategy for practical interoperability across
  citation workflows.
link-citations: true
documentclass: article
toc: true
toc-depth: 3
vignette: >
  %\VignetteIndexEntry{BibTeX and CFF}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk:
    collapse: true
    comment: "#>"
---

```{r}
#| include: false
# jarl-ignore-file fixed_regex: Not production code.

library(cffr)
# Load the table of tables.

p2file <- system.file("extdata/crosswalk_tables.csv", package = "cffr")

table_master <- read.csv(p2file)
```

::: callout-important
Generative AI tools were used to assist with parts of this article.
:::

## Citation

Cite this article with this BibTeX entry:

``` bib
@article{hernangomez2022,
    title        = {{BibTeX} and {CFF}, A deterministic crosswalk between two non-equivalent citation schemas},
    author       = {Diego Hernangómez},
    year         = 2022,
    journal      = {The {cffr} package},
    volume       = {Vignettes},
    doi          = {10.21105/joss.03900},
    url          = {https://docs.ropensci.org/cffr/articles/bibtex-cff.html}
}
```

## **BibTeX** and **R**

[**BibTeX**](https://en.wikipedia.org/wiki/BibTeX) is a widely used format for
storing bibliographic references, originally designed in 1985 for
document-centric workflows. It represents citations as relatively flat records
with loosely constrained fields, relying on convention rather than an explicit
schema.

The Citation File Format (CFF) [@druskat_citation_2021] provides a structured
and extensible alternative for representing citation metadata, particularly for
software and research outputs. It supports explicit typing, nested objects and
richer semantics, including contributor roles and identifiers.

In modern research workflows, both formats frequently coexist. **BibTeX**
remains dominant in academic publishing pipelines, while CFF is increasingly
adopted by infrastructure platforms such as **GitHub** and **Zenodo**.

**cffr** [@hernangomez2021] provides a deterministic mapping (crosswalk) between
**BibTeX** and CFF, enabling practical interoperability. Due to fundamental
differences in their data models, this mapping is not bijective and may require
structural transformations, heuristic parsing and controlled information loss.

## Conceptual differences between **BibTeX** and CFF

At a conceptual level, **BibTeX** and CFF embody different design philosophies.

**BibTeX** represents bibliographic information as flat records with loosely
defined fields, optimized for citation rendering in documents. Semantics are largely
implicit and contextual, depending on entry types and bibliography styles.

CFF, in contrast, defines a structured schema with explicit typing, nested
objects and richer metadata support. It is designed for producing citations and
for machine-actionable interoperability across platforms.

These conceptual differences explain why a direct one-to-one correspondence
between **BibTeX** and CFF is not possible. Any crosswalk must rely on explicit
design decisions that balance fidelity, usability and schema constraints.

## **BibTeX** definitions

@patashnik1988 provides the canonical description of the **BibTeX** format. Two
key concepts are central: **entries** and **fields**.

### Entries {#entries}

The original **BibTeX** specification defines fourteen canonical entry types,
each corresponding to a class of cited work:

1.  **\@article**: An article from a journal or magazine.
2.  **\@book**: A book with an explicit publisher.
3.  **\@booklet**: A printed and bound work without a named publisher.
4.  **\@conference**: Equivalent to **\@inproceedings**, included for
    [Scribe](https://en.wikipedia.org/wiki/Scribe_(markup_language))
    compatibility.
5.  **\@inbook**: A part of a book, such as a chapter or page range.
6.  **\@incollection**: A part of a book with its own title.
7.  **\@inproceedings**: An article in conference proceedings.
8.  **\@manual**: Technical documentation.
9.  **\@mastersthesis**: A Master’s thesis.
10. **\@misc**: A fallback type when no other entry fits.
11. **\@phdthesis**: A PhD thesis.
12. **\@proceedings**: The proceedings of a conference.
13. **\@techreport**: A numbered technical report.
14. **\@unpublished**: A work not formally published.

Other implementations, notably **BibLaTeX** [@biblatexpack], extend this set
with additional entry types such as online resources, software and datasets. In
standard **BibTeX**, such entries must typically be represented as **\@misc**.

In **R** [@R_2021], the base function `bibentry()` does not implement
**\@conference**. Instead, **\@inproceedings** is used, as both share the same
conceptual definition.

### Fields {#fields}

**BibTeX** entries consist of fields whose relevance and requirements depend on
the entry type. Some fields are mandatory, some are optional and others are
ignored by standard bibliography styles while still carrying useful information.

The following table summarizes the relationship between **BibTeX** **entries**
and their **required or optional fields**, following @patashnik1988. Required
fields are marked with **x** and optional fields with **o**.

```{r}
#| label: tbl-entry_fields1
#| echo: false
#| tbl-cap: BibTeX entries
#| tbl-subcap:
#|   - "BibTeX: required fields by entry"
#|   - "BibTeX: required fields by entry (cont.)"
#|
df_table <- table_master[table_master$table == "entry_fields", -1]

nms <- c(
  "field",
  "\\@article",
  "\\@book",
  "\\@booklet",
  "\\@inbook",
  "\\@incollection",
  "\\@conference, \\@inproceedings",
  "\\@manual",
  "\\@mastersthesis, phdthesis",
  "\\@misc",
  "\\@proceedings",
  "\\@techreport",
  "\\@unpublished"
)

df_table[is.na(df_table)] <- ""
row.names(df_table) <- NULL
t1 <- df_table[, c(1:7)]
nm1 <- nms[1:7]

knitr::kable(
  t1,
  col.names = nm1,
  row.names = NA,
  align = c("l", rep("c", 6))
)

t2 <- df_table[, c(1, 8:13)]
nm2 <- nms[c(1, 8:13)]
knitr::kable(
  t2,
  col.names = nm2,
  row.names = NA,
  align = c("l", rep("c", 6))
)
```

Only a subset of fields is required for any given entry. Fields such as
**title**, **author** and **year** appear across most entry types, whereas
others are optional or never mandatory. This strict coupling between entry types
and fields is a defining feature of **BibTeX** and a key source of friction when
interoperating with schema-driven formats such as CFF.

## Citation File Format

Citation File Format (CFF) consists of plain-text, YAML-based files that encode
human- and machine-readable citation metadata for software and datasets.

Two keys play a central role in CFF reference modeling:

- `preferred-citation`: Identifies a work that should be cited instead of the
  software or dataset itself.
- `references`: Lists related creative works, analogous to references in a
  scholarly article.

Both keys expect `definitions.reference` objects, as defined in the [CFF schema
guide](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#preferred-citation).
These objects support explicit typing, nested metadata and structured
identifiers, unlike **BibTeX**'s flat field model.

The following table summarizes the valid keys for CFF `definitions.reference`
objects:

```{r}
#| label: tbl-refkeys
#| echo: false
#| message: false
#| warning: false
#| results: asis
#| tbl-cap: "Valid keys in CFF `definitions.reference` objects"
library(cffr)

# Fill with blanks.
init <- paste0("`", cff_schema_definitions_refs(), "`")

l <- c(init, rep("", 4))

refkeys <- matrix(l, ncol = 5, byrow = TRUE)

knitr::kable(
  refkeys,
  row.names = NA
)
```

Conceptually, most of these keys correspond to **BibTeX** [**fields**](#fields),
with one key playing a special role: `type`. In CFF, `type` explicitly
identifies the kind of referenced work, making it conceptually analogous to a
**BibTeX** [**entry**](#entries) type rather than a **field**[^1].

## Mapping strategy

The mapping implemented in **cffr** follows a three-step pipeline:

1.  Parse **BibTeX** entries into an intermediate representation.
2.  Apply deterministic mapping rules to transform entry types and fields.
3.  Serialize the result into CFF (and vice versa).

Whenever possible, mappings are deterministic. However, some fields require
heuristic parsing, especially when normalizing names, dates or page ranges. As a
result, semantic round-trip reversibility is not guaranteed, even when the
conversion process itself is reproducible.

```{r}
#| label: cffbibread
#| comment: '#>'
string <- "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, A.},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925}"

# Convert to `cff`.
library(cffr)
cff_format <- cff_read_bib_text(string)

cff_format

# Convert to BibTeX with the S3 method.
toBibtex(cff_format)
```

## Mapping tables

The following tables summarize the mapping rules implemented in **cffr**. They
are intended as implementation documentation rather than as a normative
specification.

Mapping types include:

- **direct**: preserved without modification.
- **transform**: renamed or structurally reorganized.
- **split**: one field mapped to multiple fields.
- **heuristic**: requires parsing or inference.
- **unsupported**: omitted due to lack of correspondence.
- **enrichment**: exists only in CFF.

These mappings are not bijective and may introduce normalization effects.

::: callout-note
For clarity throughout this document, **bold** formatting (for example,
**\@book** and **edition**) is used for **BibTeX** entries and fields, whereas
inline code formatting (for example, `book` and `edition`) is used for CFF keys.
:::

::: {#tbl-mapentry}
| BibTeX entry | CFF key `type` | Mapping type | Notes |
|:---|:---|:---|:---|
| **\@article** | `type: article` | direct | \- |
| **\@book** | `type: book` | direct | \- |
| **\@inbook** | `type: chapter` | transform | Normalized to CFF taxonomy |
| **\@incollection** | `type: chapter` | transform | Same as **\@inbook** |
| **\@manual** | `type: manual` | direct | \- |
| **\@misc** | `type: generic` | heuristic | `generic` is used as a fallback when no more specific CFF type applies. |
| **\@phdthesis** | `type: thesis` | transform | The specific thesis subtype (e.g., master's vs PhD) may be lost when mapping from BibTeX to CFF. |
| **\@techreport** | `type: report` | transform | \- |
| **other** | `type: generic` | heuristic | Default fallback |

: BibTeX entries and CFF key `type` mapping
:::

::: {#tbl-mapcore}
| BibTeX field  | CFF key            | Mapping type | Notes                               |
|:--------------|:-------------------|:-------------|:------------------------------------|
| **title**     | `title`            | direct       | \-                                  |
| **author**    | `authors`          | transform    | Parsed into structured name objects |
| **editor**    | `editors`          | transform    | Same parsing logic as authors       |
| **year**      | `year`             | direct       | \-                                  |
| **month**     | `month`            | heuristic    | Format normalization required       |
| **journal**   | `journal`          | direct       | \-                                  |
| **booktitle** | `collection-title` | transform    | Container title                     |
| **volume**    | `volume`           | direct       | \-                                  |
| **number**    | `issue`            | transform    | Naming normalization                |
| **pages**     | `start`/`end`      | split        | Parsed from page range              |
| **doi**       | `doi`              | direct       | \-                                  |
| **url**       | `url`              | direct       | \-                                  |

: Core fields
:::

::: {#tbl-mapstruct}
| BibTeX field    | CFF key              | Mapping type | Notes                          |
|:----------------|:---------------------|:-------------|:-------------------------------|
| **publisher**   | `publisher.name`     | transform    | Converted to structured object |
| **address**     | `publisher.location` | transform    | Combined with `publisher`      |
| **institution** | `institution`        | direct       | Used mainly in reports/thesis  |
| **school**      | `institution`        | transform    | Thesis normalization           |

: Structural fields
:::

::: {#tbl-mapparsed}
| BibTeX field | CFF key       | Mapping type | Notes                                 |
|:-------------|:--------------|:-------------|:--------------------------------------|
| **pages**    | `start`/`end` | heuristic    | Requires parsing, may fail            |
| **author**   | `authors`     | heuristic    | Name splitting is not always reliable |
| **month**    | `month`       | heuristic    | Multiple formats possible             |
| **note**     | `notes`       | heuristic    | Free text                             |

: Parsed fields
:::

::: {#tbl-mapnobib}
| CFF key              | BibTeX equivalent | Mapping type | Notes                |
|:---------------------|:------------------|:-------------|:---------------------|
| `preferred-citation` | \-                | enrichment   | CFF-specific concept |
| `references`         | \-                | enrichment   | Not representable    |
| `type: software`     | \-                | unsupported  | No BibTeX equivalent |
| `identifiers`        | partial           | transform    | Only DOI/URL mapped  |

: CFF keys with no BibTeX equivalence
:::

## Proposed crosswalk

**cffr** provides utilities for mapping **BibTeX** entries (with `bibentry()`)
to CFF and back. This section describes how the crosswalk is implemented,
partially informed by the **Ruby** implementation in
@Haines_Ruby_CFF_Library_2021[^2].

The crosswalk is primarily defined for **BibTeX** to CFF conversion. Reverse
mapping is a best-effort approximation and may be lossy.

After presenting the general mapping between **BibTeX** **entries** and
**fields** and CFF keys, the next section introduces
[**entry models**](#entrymodels) that refine these rules for specific
**BibTeX** entry types.

The mapping is structured according to the transformation semantics defined
below.

## Transformation semantics

The crosswalk applies different transformation strategies:

- **Direct mapping**: one-to-one correspondence between a **BibTeX** field and a
  CFF key.
- **Structural transformation**: a single field is expanded into multiple keys,
  such as **pages** to `start` and `end`.
- **Heuristic mapping**: equivalence is inferred based on entry type or context.
- **Lossy mapping**: information is discarded due to lack of an equivalent
  representation.

Unless stated otherwise, mappings are lossy in at least one direction.

All mappings and examples in the following sections are instances of these
transformation classes.

### Entry/key `type` crosswalk

To map general **BibTeX** entries to CFF key `type`, the following equivalence
is proposed:

```{r}
#| label: tbl-entry_bib2cff
#| echo: false
#| results: asis
#| tbl-cap: "Entry/Type crosswalk: From BibTeX to CFF"
df_table <- table_master[table_master$table == "entry_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("BibTeX entry", "CFF key: `type`", "Notes"),
  row.names = NA
)
```

The previous mapping has the following specifications:

- **\@book**, **\@inbook** and **\@incollection** are closely related in
  **BibTeX**[^3]. While **\@inbook** and **\@incollection** both reference parts
  of a **\@book**, the former is used for citing sections, chapters, pages or other
  specific parts, whereas the latter is used for citing parts with a specific
  title. Since CFF allows keys `type: book` and `collection-type: book`, we can
  use a combination of these fields to tag each entry type in CFF accordingly.
- **\@mastersthesis** and **\@phdthesis** are tagged using a combination of
  `type: thesis` and `thesis-type` keys.

Because CFF allows a wide range of values[^4] for the key `type`, the following
mapping is applied from CFF to **BibTeX**:

The reverse mapping prioritizes **BibTeX** compatibility and normalization over
exact round-trip preservation.

```{r}
#| label: tbl-entry_cff2bib
#| echo: false
#| results: asis
#| tbl-cap: "Entry/Key `type` crosswalk: From CFF to BibTeX"
df_table <- table_master[table_master$table == "entry_cff2bib", c(2:4)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("CFF key `type`", "BibTeX entry", "Notes")
)
```

### Fields/key crosswalk

Some **BibTeX** fields and CFF keys have similar names and definitions. The
equivalence is straightforward in some cases, but some keys need to be processed
depending on the **entry** type.

```{r}
#| label: tbl-fields_bib2cff
#| echo: false
#| results: asis
#| tbl-cap: "BibTeX - CFF key crosswalk"
df_table <- table_master[table_master$table == "fields_bib2cff", c(2:4)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("BibTeX **Field**", "CFF key", "Notes")
)
```

We provide more detail on some of the mappings presented in the table above:

- Some fields are not mapped because they have no clear equivalent CFF keys
  (such as **annote**, **crossref** and **key**). Regarding the **type** field,
  the CFF key `type` corresponds to the identifier of the work, similar to an
  entry in **BibTeX**. Therefore, **BibTeX** **type** is not mapped. These
  fields are always optional in **BibTeX**.
- For the **address** field, its intended use in **BibTeX** varies depending on
  the entry type. For **\@inproceedings**, it denotes the **address** of the
  **conference**, while for **\@mastersthesis/\@phdthesis**, it is the
  **address** of the **school**. Mapping between **BibTeX** and CFF becomes more
  complex when institutional metadata is involved. This results in varying final
  mappings in CFF. When mapping from CFF to **BibTeX**, we propose to follow the
  same entry-based logic, using the key `location` as a fallback value when
  mapping to **address**.
- Because of the complexity mentioned above, **institution, organization** and
  **school** are mapped to `institution`.
- **series** is mapped to `collection-title` only on those entries that do not
  require **booktitle**. In practice, this means that `collection-title`
  corresponds to **booktitle** (for **\@incollection** and **\@inproceedings**),
  and in the other cases it corresponds to **series**. As a consequence,
  **series** information is lost for **\@incollection** and **\@inproceedings**,
  but in those cases it is an optional field.
- When mapping from CFF to **BibTeX**, we propose to use `date-published` as a
  fallback for extracting **month** and **year** fields.
- When **pages** is provided as a range separated by `--`, that is, **pages =
  {3--5}**, it is coerced as `start: 3`, `end: 5` in CFF.

#### **BibLaTeX**

Other CFF keys correspond to **BibLaTeX** fields. We include these fields in the
crosswalk[^5], even though they are not part of the core **BibTeX** field
definition.

```{r}
#| label: tbl-fields_biblatex2cff
#| echo: false
#| results: asis
#| tbl-cap: "BibLaTeX - CFF field/key crosswalk"
df_table <- table_master[table_master$table == "fields_biblatex2cff", c(2:3)]
df_table[is.na(df_table)] <- ""
# Fix links.
df_table$f2 <- gsub("link_to_entry_models", "#entrymodels", df_table$f2)
row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("**BibLaTeX Field**", "CFF key")
)
```

## Limitations

The proposed crosswalk is subject to multiple structural limitations arising
from differences between **BibTeX** and CFF schemas:

- **Lossy mappings**: Some **BibTeX** fields, including **crossref**,
  **annote** and **key**, have no equivalent in CFF and are omitted during
  conversion.
- **Ambiguous semantics**: Certain fields such as **address** and **series**
  have context-dependent meanings in **BibTeX**, requiring heuristic mapping to
  CFF keys such as `institution`, `location` or `collection-title`.
- **Type collapsing**: Multiple **BibTeX** entry types, including **\@misc** and
  **\@incollection**, are mapped to a single CFF type (`generic`), which acts as
  a fallback when no more specific type is available.
- **Structural transformations**: Some fields require transformation rather than
  direct mapping, such as **pages** → `start`/`end`, altering the representation
  of the original data.
- **Non-reversibility**: Because of these factors, round-trip conversion
  (**BibTeX** → CFF → **BibTeX**) is not guaranteed to preserve the original
  structure or semantics.

These limitations reflect fundamental differences between the two formats rather
than implementation-specific constraints.

## Design decisions

The mapping implemented in **cffr** is guided by the following principles:

- Prefer deterministic mappings whenever possible.
- Use heuristics only when unavoidable.
- Normalize metadata to align with CFF conventions.
- Prioritize interoperability over exact round-trip fidelity.

## Entry models {#entrymodels}

This section documents entry-specific mapping behavior, expanding the general
crosswalk into concrete and testable models. Examples are adapted from the
[xampl.bib](https://tug.org/texmf-docs/bibtex/xampl.bib) distributed with
**BibTeX** [@patashnik].

### \@article {#article}

The crosswalk for **\@article** uses the default mapping rules.

```{r}
#| label: tbl-model_article
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_article", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@article** model"
)
```

**Round-trip**

```{r}
bib <- "@article{article-full,
    title        = {The Gnats and Gnus Document Preparation System},
    author       = {Leslie A. Aamport},
    year         = 1986,
    month        = jul,
    journal      = {{G-Animal's} Journal},
    volume       = 41,
    number       = 7,
    pages        = {73+},
    note         = {This is a full ARTICLE entry}}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@book / \@inbook {#book-inbook}

In terms of fields required in **BibTeX**, the primary difference between
**\@book** and **\@inbook** is that **\@inbook** requires a **chapter** or
**page** field, while **\@book** does not allow these fields as optional.
Therefore, we propose
that an **\@inbook** entry in CFF be treated as a **\@book** with the following
supplementary fields:

1.  `section`: To denote the specific **chapter** within the book.
2.  `start`/`end`: To indicate the range of **pages** covered by the section.

In CFF, the **series** field corresponds to `collection-title` and the
**address** field represents the `publisher`'s `address`. Finally, the key
`collection-type` is populated with `book-series`.

```{r}
#| label: tbl-model_book
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_book", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@book / \\@inbook** model"
)
```

There are notable differences in how **BibTeX** and **BibLaTeX** handle the
**\@inbook** entry (discussed further in [Appendix A](#appendix_inbook)). We
propose treating a **BibLaTeX** **\@inbook** as a **BibTeX** **\@incollection**.

**Round-trip: \@book**

```{r}
bib <- "@book{book-full,
    title        = {Seminumerical Algorithms},
    author       = {Donald E. Knuth},
    year         = 1981,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 2,
    note         = {This is a full BOOK entry},
    edition      = {Second}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

**Round-trip: \@inbook**

```{r}
bib <- "@inbook{inbook-full,
    title        = {Fundamental Algorithms},
    author       = {Donald E. Knuth},
    year         = 1973,
    month        = 10,
    publisher    = {Addison-Wesley},
    address      = {Reading, Massachusetts},
    series       = {The Art of Computer Programming},
    volume       = 1,
    pages        = {10--119},
    note         = {This is a full INBOOK entry},
    edition      = {Second},
    type         = {Section},
    chapter      = {1.2}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@booklet {#booklet}

In **\@booklet**, **address** is mapped to `location`.

```{r}
#| label: tbl-model_booklet
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_booklet", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@booklet** model"
)
```

**Round-trip**

```{r }
bib <- "@booklet{booklet-full,
    title        = {The Programming of Computer Art},
    author       = {Jill C. Knvth},
    date         = {1988-03-14},
    month        = feb,
    address      = {Stanford, California},
    note         = {This is a full BOOKLET entry},
    howpublished = {Vernier Art Center}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@conference / \@inproceedings {#conf_inproc}

In this case, **organization** is mapped to `institution`. **series** is ignored
because there is no clear mapping in CFF for this field.

```{r}
#| label: tbl-model_inproceedings
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_inproceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@conference / \\@inproceedings** model"
)
```

**Round-trip**

```{r}
bib <- "@inproceedings{inproceedings-full,
    title        = {On Notions of Information Transfer in {VLSI} Circuits},
    author       = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
    year         = 1983,
    month        = mar,
    booktitle    = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    pages        = {133--139},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@incollection {#incol}

Because **booktitle** is a required field, we propose to map that field to
`collection-title` and the `type` to `generic`. Therefore, an **\@incollection**
is a `type: generic` in CFF with a `collection-title` key. The `generic` type is
used as a fallback when no semantically equivalent CFF type exists.

Additionally, **series** and **type** are ignored because there is no clear
mapping in CFF for these fields.

```{r}
#| label: tbl-model_incollection
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_incollection", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@incollection** model"
)
```

**Round-trip**

```{r}
bib <- "@incollection{incollection-full,
    title        = {Semigroups of Recurrences},
    author       = {Daniel D. Lincoll},
    year         = 1977,
    month        = sep,
    booktitle    = {High Speed Computer and Algorithm Organization},
    publisher    = {Academic Press},
    address      = {New York},
    series       = {Fast Computers},
    number       = 23,
    pages        = {179--183},
    note         = {This is a full INCOLLECTION entry},
    editor       = {David J. Lipcoll and D. H. Lawrie and A. H. Sameh},
    chapter      = 3,
    type         = {Part},
    edition      = {Third}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@manual

As with [**\@conference** / **\@inproceedings**](#conf_inproc), **organization**
is mapped to `institution`.

```{r}
#| label: tbl-model_manual
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_manual", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@manual** model"
)
```

**Round-trip**

Note that **month** cannot be coerced to a single integer in the range `1--12`
as required in CFF, so it is ignored to avoid validation errors.

```{r}
bib <- "@manual{manual-full,
  title        = {The Definitive Computer Manual},
    author       = {Larry Manmaker},
    year         = 1986,
    month        = {apr-may},
    address      = {Silicon Valley},
    note         = {This is a full MANUAL entry},
    organization = {Chips-R-Us},
    edition      = {Silver}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@mastersthesis / \@phdthesis

The **BibTeX** field requirements are identical for **\@mastersthesis** and
**\@phdthesis**.

We propose here to identify each type of thesis using the key `thesis-type`. If
`thesis-type` contains a [regex pattern](https://regex101.com/r/mBWfbs/1)
`(?i)(phd)`, it is recognized as **\@phdthesis**.

Additionally, **school** is mapped to `institution`.

```{r}
#| label: tbl-model_thesis
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_thesis", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@mastersthesis / \\@phdthesis** model"
)
```

**Round-trip: \@mastersthesis**

```{r}
bib <- "@mastersthesis{mastersthesis-full,
    title        = {Mastering Thesis Writing},
    author       = {Edouard Masterly},
    year         = 1988,
    month        = jun,
    address      = {English Department},
    note         = {This is a full MASTERSTHESIS entry},
    school       = {Stanford University},
    type         = {Master's project}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

**Round-trip: \@phdthesis**

```{r}
bib <- "@phdthesis{phdthesis-full,
    title        = {Fighting Fire with Fire: Festooning {F}rench Phrases},
    author       = {F. Phidias Phony-Baloney},
    year         = 1988,
    month        = jun,
    address      = {Department of French},
    note         = {This is a full PHDTHESIS entry},
    school       = {Fanstord University},
    type         = {{PhD} Dissertation}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@misc

The crosswalk for **\@misc** uses the default mapping rules. This **entry** does
not require any **field**.

Note also that it is mapped to `type: generic`, as with
[**\@incollection**](#incol). In this case, **booktitle** is not an option, so
the proposed definition should cover both **\@misc** and **\@incollection**.

```{r}
#| label: tbl-model_misc
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_misc", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@misc** model"
)
```

**Round-trip**

```{r}
bib <- "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@proceedings

The proposed model is consistent with [**\@conference** /
**\@inproceedings**](#conf_inproc). Note that **\@proceedings** does not
prescribe an **author** field. In these cases, because `authors` is required in
CFF, we use *anonymous*[^6] when mapping to CFF and omit it when mapping from
CFF to **BibTeX**.

```{r}
#| label: tbl-model_proceedings
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_proceedings", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@proceedings** model"
)
```

**Round-trip**

```{r}
bib <- "@proceedings{proceedings-full,
    title        = {Proc. Fifteenth Annual ACM Symposium on the Theory of Computing},
    year         = 1983,
    month        = mar,
    publisher    = {Academic Press},
    address      = {Boston},
    series       = {All ACM Conferences},
    number       = 17,
    note         = {This is a full PROCEEDINGS entry},
    editor       = {Wizard V. Oz and Mihalis Yannakakis},
    organization = {The OX Association for Computing Machinery}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@techreport

The crosswalk for **\@techreport** uses the default mapping rules.

```{r}
#| label: tbl-model_techreport
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_techreport", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@techreport** model"
)
```

**Round-trip**

```{r}
bib <- "@techreport{techreport-full,
    title        = {A Sorting Algorithm},
    author       = {Tom Terrific},
    year         = 1988,
    month        = oct,
    address      = {Computer Science Department, Fanstord, California},
    number       = 7,
    note         = {This is a full TECHREPORT entry},
    institution  = {Fanstord University},
    type         = {Wishful Research Result}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

### \@unpublished

The crosswalk for **\@unpublished** uses the default mapping rules.

```{r}
#| label: tbl-model_unpublished
#| echo: false
#| results: asis
df_table <- table_master[table_master$table == "model_unpublished", c(2:4)]
df_table[is.na(df_table)] <- ""

# Fix links.
df_table$f3 <- gsub("link_to_entry_models", "#entrymodels", df_table$f3)
df_table$f3 <- gsub("link_to_article", "#article", df_table$f3)
df_table$f3 <- gsub("link_to_booklet", "#booklet", df_table$f3)
df_table$f3 <- gsub("link_to_book", "#book-inbook", df_table$f3)

row.names(df_table) <- NULL
knitr::kable(
  df_table,
  col.names = c("BibTeX", "CFF", "Notes"),
  caption = "**\\@unpublished** model"
)
```

**Round-trip**

```{r}
bib <- "@unpublished{unpublished-minimal,
    title        = {Lower Bounds for Wishful Research Results},
    author       = {Ulrich Underwood and Ned Net and Paul Pot},
    note         = {Talk at Fanstord University (this is a minimal UNPUBLISHED entry)}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

## Conclusion

This article presents a practical and reproducible crosswalk between **BibTeX**
and CFF.

Although the formats are not fully equivalent, the deterministic mapping
implemented in **cffr** enables consistent transformations across heterogeneous
citation ecosystems. By making design decisions explicit and documenting
limitations, this work supports interoperable citation workflows bridging legacy
bibliographic systems and modern software-centric practices.

## Appendix A: **\@inbook** in **BibTeX** and **BibLaTeX** {#appendix_inbook .appendix}

The definition of **\@inbook** and **\@incollection** in **BibTeX**
[@patashnik1988] is as follows:

> - **\@inbook**: A part of a book, which may be a chapter (or section) and/or a
>   range of pages. Required fields: author or editor, title, chapter and/or
>   pages, publisher, year (...)
>
> - **\@incollection**: A part of a book having its own title. Required fields:
>   author, title, booktitle, publisher, year (...)

Whereas **BibLaTeX** [@biblatexpack] specifies:

> - **\@inbook:** A part of a book which forms a self-contained unit with its
>   own title. Note that the [profile]{.underline} of this entry type is
>   [different from standard **BibTeX**]{.underline}, see § 2.3.1. Required
>   fields: author, title, booktitle, year/date (...).

When considering required fields, an important difference is the **booktitle**
requirement in **BibLaTeX**. Notably, **BibTeX** **\@incollection** also
requires this field. Moreover, both **BibTeX** **\@incollection** and
**BibLaTeX** **\@inbook**
emphasize its reference to *"a part of a book (...) with its own title"*.

In this article, the proposed crosswalk ensures full compatibility with
**BibTeX**. Therefore, we treat a **BibLaTeX** **\@inbook** entry as equivalent
to a **BibTeX** **\@incollection**, given the close alignment in their
definitions and field requirements.

**Round-trip**

```{r}
bib <- "@inbook{inbook-biblatex,
	author       = {Yihui Xie and Christophe Dervieux and Emily Riederer},
	title        = {Bibliographies and citations},
	booktitle    = {{R} Markdown Cookbook},
	date         = {2023-12-30},
	publisher    = {Chapman and Hall/CRC},
	address      = {Boca Raton, Florida},
	series       = {The {R} Series},
	isbn         = 9780367563837,
	url          = {https://yihui.org/rmarkdown-cookbook/},
	chapter      = {4.5}
}"

cff_read_bib_text(bib)

toBibtex(cff_read_bib_text(bib))
```

## Appendix B: CFF key: `type` values {#appendix_cff_type .appendix}

From @druskat2019, Table 4: complete list of CFF reference types for key `type`.
Only a subset of these types is actively used in the proposed crosswalk.

```{r}
#| label: tbl-cff_types
#| echo: false
#| results: asis
#| tbl-cap: "Complete list of CFF reference types"
df_table <- table_master[table_master$table == "cff_types", c(2:3)]
df_table[is.na(df_table)] <- ""

row.names(df_table) <- NULL

knitr::kable(
  df_table,
  col.names = c("Reference type string", "Description"),
  row.names = NA
)
```

## References

[^1]: See a complete list of possible values of CFF `type` in [Appendix
    B](#appendix_cff_type).

[^2]: This software performs only the mapping from CFF to **BibTeX**, but
    **cffr** can perform the mapping in both directions.

[^3]: Note that **BibLaTeX** [@biblatexpack] handles **\@inbook** differently,
    see [Appendix A](#appendix_inbook).

[^4]: See [Appendix B](#appendix_cff_type) for all possible values. Information
    extracted from @druskat2019.

[^5]: See @biblatexcheatsheet for a preview of the accepted **BibLaTeX** fields.

[^6]: As proposed on [*How to deal with unknown individual
    authors?*](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md#how-to-deal-with-unknown-individual-authors),
    (Guide to Citation File Format schema version 1.2.0).

---
title: "cffr: Generate Citation File Format Metadata for R Packages"
subtitle: "JOSS paper"
description: >
  Paper published on The Journal of Open Source Software.
tags:
  - R
  - cff
  - citation
  - credit
  - metadata
author:
  -  Diego Hernangómez^[Independent Researcher, https://orcid.org/0000-0001-8457-4658]
date: "`r as.Date('2021-11-08')`"
output: rmarkdown::html_vignette
bibliography: REFERENCES.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{cffr: Generate Citation File Format Metadata for R Packages}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

[![DOI](https://joss.theoj.org/papers/10.21105/joss.03900/status.svg)](https://doi.org/10.21105/joss.03900)

## Summary

The Citation File Format project [@druskat_citation_2021] defines a standardized
format for providing software or datasets citation metadata in plaintext files
that are easy to read by both humans and machines.

This metadata format is being adopted by GitHub as the primary format for its
built-in citation support [@github_about_citation]. Other leading archives for
scientific software, including Zenodo and Zotero [@druskat_stephan_making_2021],
have included as well support for CITATION.cff files in their GitHub
integration.

The cffr package provides utilities to generate and validate these CITATION.cff
files automatically for R [@R_2021] packages by parsing the DESCRIPTION file and
the native R citation file. The package also includes utilities and examples for
parsing components as persons and additional citations, as well as several
vignettes which illustrate both the basic usage of the package as well as some
more technical details about the metadata extraction process.

## Statement of need

Citation of research software on research project is often omitted [@salmon2021]
. Among many reasons why software is not cited, one is the lack of a clear
citation information from package developers.

Some of the main reasons for citing software used on research are:

1.  **Reproducibility**: Software and their versions are important information
    to include in any research project. It helps peers to understand and
    reproduce effectively the results of any work. Including versions is also
    crucial as a way of recording the context of your manuscript when software
    changes.
2.  **Developer Credit:** On the context of Free and Open Source Software
    (FOSS), many of the software developers themselves are also researches.
    Receive credit for software development shouldn't be different from the
    credit received on other formats, as books or articles.

CITATION.cff files provides a clear citation rules for software. The format is
easily readable by humans and also can be parsed by appropriate software. The
adoption of GitHub of this format sends a strong message that research software
is something worthy of citation, and therefore deserves credit.

The cffr package allow R software developers to create CITATION.cff files from
the metadata already included on the package. Additionally, the package also
include validation tools via the jsonvalidate package [@jsonvalidate2021], that
allow developers to assess the validity of the file created using the latest CFF
schema.json.

## Acknowledgements

I would like to thank [Carl
Boettiger](https://ropensci.org/author/carl-boettiger/), [Maëlle
Salmon](https://ropensci.org/author/ma%C3%ABlle-salmon/) and the [rest of
contributors](https://github.com/ropensci/codemetar/graphs/contributors) of the
codemetar package [@codemetar2021]. This package was the primary inspiration for
developing cffr and shares a common goal of increasing awareness on the efforts
of software developers.

I would like also to thank [João Martins](https://zambujo.github.io/) and [Scott
Chamberlain](https://ropensci.org/author/scott-chamberlain/) for thorough
reviews, that helps improving the package and the documentation as well as
[Emily Riederer](https://emilyriederer.netlify.app/) for handling the [review
process](https://github.com/ropensci/software-review/issues/463).

## Citation

Hernangómez, D., (2021). cffr: Generate Citation File Format Metadata for R
Packages. Journal of Open Source Software, 6(67), 3900,
<https://doi.org/10.21105/joss.03900>

``` bibtex
@article{hernangomez2021,
  doi = {10.21105/joss.03900},
  url = {https://doi.org/10.21105/joss.03900},
  year = {2021},
  publisher = {The Open Journal},
  volume = {6},
  number = {67},
  pages = {3900},
  author = {Diego Hernangómez},
  title = {cffr: Generate Citation File Format Metadata for R Packages},
  journal = {Journal of Open Source Software}
}
```

## References

---
title: "cffr: Generate Citation File Format Metadata for R Packages"
subtitle: "JOSS paper"
description: Paper published on The Journal of Open Source Software.
keywords:
  - R
  - cff
  - citation
  - credit
  - metadata
author:
  - name: Diego Hernangómez
    orcid: 0000-0001-8457-4658
    affiliations:
      - Independent Researcher
citation:
  type: article-journal
  container-title: Journal of Open Source Software
  volume: 6
  issue: 67
  page: 3900
  available-date: 2021
  doi: 10.21105/joss.03900
  url: https://doi.org/10.21105/joss.03900
date: 2021-11-08
doi: 10.21105/joss.03900
bibliography: REFERENCES.bib
link-citations: true
vignette: >
  %\VignetteIndexEntry{cffr: Generate Citation File Format Metadata for R Packages}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

[![](https://joss.theoj.org/papers/10.21105/joss.03900/status.svg)](https://doi.org/10.21105/joss.03900)

## Summary

The Citation File Format project [@druskat_citation_2021] defines a standardized
format for providing software or datasets citation metadata in plaintext files
that are easy to read by both humans and machines.

This metadata format is being adopted by GitHub as the primary format for its
built-in citation support [@github_about_citation]. Other leading archives for
scientific software, including Zenodo and Zotero [@druskat_stephan_making_2021],
have also included support for CITATION.cff files in their GitHub integration.

The cffr package provides utilities to generate and validate these CITATION.cff
files automatically for R [@R_2021] packages by parsing the DESCRIPTION file and
the native R citation file. The package also includes utilities and examples for
parsing components such as persons and additional citations, as well as several
vignettes that illustrate both the basic usage of the package and more technical
details about the metadata extraction process.

## Statement of need

Citation of research software in research projects is often omitted [@salmon2021].
One reason software is not cited is the lack of clear citation information from
package developers.

Some of the main reasons for citing software used on research are:

1.  **Reproducibility**: Software and its versions are important information
    to include in any research project. They help peers understand and
    reproduce the results of a study effectively. Including versions is also
    crucial for recording the context of a manuscript when software changes.
2.  **Developer Credit:** In the context of Free and Open Source Software
    (FOSS), many of the software developers themselves are also researchers.
    Receiving credit for software development should be no different from the
    credit received in other formats, such as books or articles.

CITATION.cff files provide clear citation rules for software. The format is
easy to read by humans and can also be parsed by appropriate software. GitHub's
adoption of this format sends a strong message that research software is worthy
of citation and therefore deserves credit.

The cffr package allows R software developers to create CITATION.cff files from
metadata already included in the package. It also includes validation tools via
the jsonvalidate package [@jsonvalidate2021], which allow developers to assess
the validity of the file created using the latest CFF schema.json.

## Acknowledgements

I would like to thank [Carl
Boettiger](https://ropensci.org/author/carl-boettiger/), [Maëlle
Salmon](https://ropensci.org/author/ma%C3%ABlle-salmon/) and the [rest of the
contributors](https://github.com/ropensci/codemetar/graphs/contributors) of the
codemetar package [@codemetar2021]. This package was the primary inspiration for
developing cffr and shares a common goal of increasing awareness of the efforts
of software developers.

I would also like to thank [João Martins](https://zambujo.github.io/) and [Scott
Chamberlain](https://ropensci.org/author/scott-chamberlain/) for thorough
reviews that helped improve the package and documentation, and [Emily
Riederer](https://emilyriederer.netlify.app/) for handling the [review
process](https://github.com/ropensci/software-review/issues/463).

## Citation

Hernangómez, D., (2021). cffr: Generate Citation File Format Metadata for R
Packages. Journal of Open Source Software, 6(67), 3900,
<https://doi.org/10.21105/joss.03900>

``` bibtex
@article{hernangomez2021,
  doi = {10.21105/joss.03900},
  url = {https://doi.org/10.21105/joss.03900},
  year = {2021},
  publisher = {The Open Journal},
  volume = {6},
  number = {67},
  pages = {3900},
  author = {Diego Hernangómez},
  title = {cffr: Generate Citation File Format Metadata for R Packages},
  journal = {Journal of Open Source Software}
}
```

## References

```{r}
#| include: false
library(cffr)
```

```{r}
#| comment: ''
# Don't do
entity <- "Elephant and Castle"
as_cff_person(entity)

# Do
entity_protect <- "{Elephant and Castle}"
as_cff_person(entity_protect)
```

[**cffr**](https://cran.r-project.org/package=cffr) implements an S3 object with
[`class`] `cff`, which is used to represent the information of a `CITATION.cff`
file in **R**.

Under the hood, a `cff` object is simply a named [`list`] to which we added
additional methods, most notably [`print`] and [as_cff()].

```{r}
#| include: false
library(cffr)
```

```{r}
a_named_list <- list(
  first = "I", second = "am", third = "a", fourth = "list",
  fifth = "with", sixth = "names", "none" = NULL
)

# As cff
a_cff_object <- as_cff(a_named_list)

class(a_cff_object)

a_cff_object

is.list(a_cff_object)
```

[as_cff()] not only converts a `list` to `cff` but also removes items (known as
`keys` in CFF terminology) that are `NULL` or `NA`.

### Sub-classes

**cffr** implements two special sub-classes of `cff`, with the aim of
representing two special types of objects defined in the [CFF
Schema](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md):

-   `definition.person` and `definition.entity`: CFF definitions for sub-lists
    representing persons or entities. In **cffr**, the sub-class `cff_pers_lst`
    has been implemented to collect an array of `definition.person/entity`,
    where each individual person or entity has a sub-class `cff_pers`.

-   Similarly, `definition.reference` is the CFF definition for collecting
    references to related works and other articles of software used in the
    development of the main work of the CFF file. This is implemented in
    **cffr** as a sub-class named `cff_ref_lst` for arrays of
    `definition.reference`, where each element has a sub-class named `cff_ref`.

These two sub-classes do not provide full valid `cff` objects, but adapt
information to the schema required by CFF:

```{r}
# Using the utils::person() function

## Array
two_persons <- as_cff_person(
  c(
    person("A", "person", comment = c(ORCID = "0000-0001-8457-4658")),
    person("An entity", email = "fake@gmail.com")
  )
)

two_persons

class(two_persons)

# Single element

two_persons[[1]]

class(two_persons[[1]])

# Array of references

cit <- c(citation(), citation("yaml"))

ref_list <- as_cff(cit)

ref_list

class(ref_list)

# Single element

ref_list[[1]]

class(ref_list[[1]])
```

## Valid `cff` objects

Creating a `cff` object does not ensure its validity according to the [CFF
Schema](https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md):

```{r}
class(a_cff_object)

cff_validate(a_cff_object)
```

[cff_validate()] gives minimal messages about what's wrong with our `cff` and
(invisibly) returns the result of the validation (`TRUE/FALSE`).

We can use [cff_modify()] to add more keys:

```{r}
cff_valid <- cff_modify(a_cff_object,
  authors = as_cff_person("{James and James}"),
  cff_version = "1.2.0",
  message = "Hi there",
  title = "My title"
)


# Remove invalid keys
cff_valid <- as_cff(cff_valid[names(cff_valid) %in% cff_schema_keys()])

cff_valid

cff_validate(cff_valid)
```

## Base methods provided by **cffr**

**cffr** version 1.0.0 provides additional S3 methods for common coercion
functions of the **base** and **utils** packages.

### [as.data.frame()]

```{r}
minimal_cff <- cff()

minimal_cff

as_df <- as.data.frame(minimal_cff)

class(as_df)

t(as_df)
```

### [c()]

```{r}
new_keys <- c("date-released" = "2020-01-31", abstract = "Minimal example")

c(minimal_cff, new_keys)
```

### [as.list()]

```{r}
as.list(minimal_cff)
```

### [as.person()]

Only for `cff_pers_lst` and `cff_pers` objects:

```{r}
as.person(two_persons)
```

### [toBibtex()]

```{r}
# For cff
toBibtex(minimal_cff)

# cff_ref, cff_ref_lst
toBibtex(cit)

# cff_pers, cff_pers_lst
toBibtex(two_persons)
```

# Test errors on cff_read

    Code
      cff_read(c("abcde", "b"))
    Condition
      Error in `cff_read()`:
      ! Use a single value, `path` has length 2

---

    Code
      cff_read("abcde")
    Condition
      Error in `file_exist_abort()`:
      ! 'abcde' doesn't exist. Check the '.' directory

# cff_read citation.cff

    Code
      cff_read_cff_citation("a")
    Condition
      Error in `file_exist_abort()`:
      ! 'a' doesn't exist. Check the '.' directory

# cff_read DESCRIPTION

    Code
      cff_read_description("a")
    Condition
      Error in `file_exist_abort()`:
      ! 'a' doesn't exist. Check the '.' directory

# cff_read bib

    Code
      cff_read_bib("a")
    Condition
      Error in `file_exist_abort()`:
      ! 'a' doesn't exist. Check the '.' directory

---

    Code
      d
    Output
      type: article
      title: 'cffr: Generate Citation File Format Metadata for R Packages'
      authors:
      - family-names: Hernangómez
        given-names: Diego
      year: '2021'
      journal: Journal of Open Source Software
      publisher:
        name: The Open Journal
      volume: '6'
      issue: '67'
      doi: 10.21105/joss.03900
      url: https://doi.org/10.21105/joss.03900
      copyright: All rights reserved
      notes: 'Publisher: The Open Journal'
      start: '3900'

# cff_read citation messages

    Code
      cff_read_citation("a")
    Condition
      Error in `file_exist_abort()`:
      ! 'a' doesn't exist. Check the '.' directory

---

    Code
      s <- cff_read(f, meta = "aa")
    Message
      ! `meta` should be "NULL" or a <packageDescription> object not a string. Using `meta = NULL`

# Creating cff from packages encoded in latin1

    Code
      cffobj
    Output
      cff-version: 1.2.0
      message: 'To cite package "surveillance" in publications use:'
      type: software
      license: GPL-2.0-only
      title: 'surveillance: Temporal and Spatio-Temporal Modeling and Monitoring of Epidemic
        Phenomena'
      version: 1.19.1
      doi: 10.32614/CRAN.package.surveillance
      abstract: Statistical methods for the modeling and monitoring of time series of counts,
        proportions and categorical data, as well as for the modeling of continuous-time
        point processes of epidemic phenomena. The monitoring methods focus on aberration
        detection in count data time series from public health surveillance of communicable
        diseases, but applications could just as well originate from environmetrics, reliability
        engineering, econometrics, or social sciences. The package implements many typical
        outbreak detection procedures such as the (improved) Farrington algorithm, or the
        negative binomial GLR-CUSUM method of Höhle and Paul (2008) <https://doi.org/10.1016/j.csda.2008.02.015>.
        A novel CUSUM approach combining logistic and multinomial logistic modeling is also
        included. The package contains several real-world data sets, the ability to simulate
        outbreak data, and to visualize the results of the monitoring in a temporal, spatial
        or spatio-temporal fashion. A recent overview of the available monitoring procedures
        is given by Salmon et al. (2016) <https://doi.org/10.18637/jss.v070.i10>. For the
        retrospective analysis of epidemic spread, the package provides three endemic-epidemic
        modeling frameworks with tools for visualization, likelihood inference, and simulation.
        hhh4() estimates models for (multivariate) count time series following Paul and
        Held (2011) <https://doi.org/10.1002/sim.4177> and Meyer and Held (2014) <https://doi.org/10.1214/14-AOAS743>.
        twinSIR() models the susceptible-infectious-recovered (SIR) event history of a fixed
        population, e.g, epidemics across farms or networks, as a multivariate point process
        as proposed by Höhle (2009) <https://doi.org/10.1002/bimj.200900050>. twinstim()
        estimates self-exciting point process models for a spatio-temporal point pattern
        of infective events, e.g., time-stamped geo-referenced surveillance data, as proposed
        by Meyer et al. (2012) <https://doi.org/10.1111/j.1541-0420.2011.01684.x>. A recent
        overview of the implemented space-time modeling frameworks for epidemic phenomena
        is given by Meyer et al. (2017) <https://doi.org/10.18637/jss.v077.i11>.
      authors:
      - family-names: Höhle
        given-names: Michael
        email: hoehle@math.su.se
        orcid: https://orcid.org/0000-0002-0423-6702
      - family-names: Meyer
        given-names: Sebastian
        email: seb.meyer@fau.de
        orcid: https://orcid.org/0000-0002-1791-9449
      - family-names: Paul
        given-names: Michaela
      repository: https://CRAN.R-project.org/package=surveillance
      repository-code: https://surveillance.R-Forge.R-project.org/
      url: https://surveillance.R-Forge.R-project.org/
      date-released: '2021-03-30'
      contact:
      - family-names: Meyer
        given-names: Sebastian
        email: seb.meyer@fau.de
        orcid: https://orcid.org/0000-0002-1791-9449
      references:
      - type: article
        title: 'Monitoring Count Time Series in R: Aberration Detection in Public Health
          Surveillance'
        authors:
        - family-names: Salmon
          given-names: Maëlle
        - family-names: Schumacher
          given-names: Dirk
        - family-names: Höhle
          given-names: Michael
        journal: Journal of Statistical Software
        year: '2016'
        volume: '70'
        issue: '10'
        doi: 10.18637/jss.v070.i10
        start: '1'
        end: '35'
      - type: article
        title: Spatio-Temporal Analysis of Epidemic Phenomena Using the R Package surveillance
        authors:
        - family-names: Meyer
          given-names: Sebastian
        - family-names: Held
          given-names: Leonhard
        - family-names: Höhle
          given-names: Michael
        journal: Journal of Statistical Software
        year: '2017'
        volume: '77'
        issue: '11'
        doi: 10.18637/jss.v077.i11
        start: '1'
        end: '55'

# Test message on cff

    Code
      def <- cff("abcde")
    Condition
      Warning:
      The `path` argument of `cff()` is deprecated as of cffr 1.0.0.
      i Argument ignored.

---

    Code
      afile <- cff(nocff)
    Condition
      Warning:
      The `path` argument of `cff()` is deprecated as of cffr 1.0.0.
      i Argument ignored.

# Walk trough full lifecycle

    Code
      read
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it as below.
      abstract: This is an awesome piece of research software!
      authors:
      - family-names: Real Person
        given-names: One Truly
        name-particle: van der
        name-suffix: IV
        alias: Citey
        affiliation: Excellent University, Niceplace, Arcadia
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
      - name: Entity Project Team Conference entity
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
        date-start: '2017-01-01'
        date-end: '2017-01-31'
        location: The team garage
      commit: 156a04c74a8a79d40c5d705cddf9d36735feab4d
      contact:
      - family-names: Real Person
        given-names: One Truly
        name-particle: van der
        name-suffix: IV
        alias: Citey
        affiliation: Excellent University, Niceplace, Arcadia
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
      - name: Entity Project Team Conference entity
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
        date-start: '2017-01-01'
        date-end: '2017-01-31'
        location: The team garage
      date-released: '2017-12-11'
      doi: 10.5281/zenodo.1003150
      identifiers:
      - type: doi
        value: 10.5281/zenodo.1003150
      - type: swh
        value: swh:1:rel:99f6850374dc6597af01bd0ee1d3fc0699301b9f
      - type: url
        value: https://example.com
      - type: other
        value: other-schema://abcd.1234.efgh.5678
      keywords:
      - One
      - Two
      - Three
      - '4'
      license: CC-BY-SA-4.0
      license-url: https://spdx.org/licenses/CC-BY-SA-4.0.html#licenseText
      repository: https://www.example.com/foo/?bar=baz&inga=42&quux
      repository-code: http://foo.com/blah_(wikipedia)_blah#cite-1
      repository-artifact: https://files.pythonhosted.org/packages/0a/84/10507b69a07768bc16981184b4d147a0fc84b71fbf35c03bafc8dcced8e1/cffconvert-1.3.3.tar.gz
      title: Citation File Format 1.0.0
      type: software
      url: http://userid:password@example.com:8080/
      version: 1.0.0
      preferred-citation:
        type: book
        title: Book Title
        abbreviation: Abbr
        abstract: Description of the book.
        collection-doi: 10.5281/zenodo.1003150
        collection-title: Collection Title
        collection-type: Collection Type
        commit: 156a04c74a8a79d40c5d705cddf9d36735feab4d
        copyright: 2017 Stephan Druskat
        data-type: Data Type
        database: Database
        date-accessed: '2017-10-31'
        date-downloaded: '2017-10-31'
        date-released: '2017-10-31'
        date-published: '2017-10-31'
        department: Department
        doi: 10.5281/zenodo.1003150
        edition: 2nd edition
        end: '456'
        entry: Chapter 9
        filename: book.zip
        format: Printed book
        identifiers:
        - type: doi
          value: 10.5281/zenodo.1003150
        - type: swh
          value: swh:1:rel:99f6850374dc6597af01bd0ee1d3fc0699301b9f
        - type: url
          value: https://example.com
        - type: other
          value: other-schema://abcd.1234.efgh.5678
        isbn: 978-1-89183-044-0
        issn: 1234-543X
        issue: '123'
        issue-date: December
        issue-title: Special Issue on Software Citation
        journal: PeerJ
        keywords:
        - Software
        - Citation
        languages:
        - aaa
        - zu
        license: Apache-2.0
        license-url: https://spdx.org/licenses/Apache-2.0.html#licenseText
        loc-start: '14'
        loc-end: '54'
        medium: hardcover book
        month: '3'
        nihmsid: Don't know what format a NIHMSID is in
        notes: A field for general notes about the reference, usable in other formats such
          as BibTeX.
        number: A general-purpose field for accession numbers, cf. the specifications for
          examples.
        number-volumes: '7'
        pages: '765'
        patent-states:
        - Germany
        - ROI
        - 'but also for example US states, such as:'
        - IL
        - RI
        pmcid: PMC1234567
        repository: http://code.google.com/events/#&product=browser
        repository-code: http://142.42.1.1:8080/
        repository-artifact: https://files.pythonhosted.org/packages/0a/84/10507b69a07768bc16981184b4d147a0fc84b71fbf35c03bafc8dcced8e1/cffconvert-1.3.3.tar.gz
        scope: Cite this book if you want to reference the general concepts implemented
          in Citation File Format 1.0.0.
        section: Chapter 2 - "Reference keys"
        status: advance-online
        start: '123'
        thesis-type: Doctoral dissertation
        url: http://j.mp
        version: 0.0.1423-BETA
        volume: '2'
        volume-title: Advances in Software Citation
        year: '2017'
        year-original: '2012'
        conference:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        authors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        contact:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        database-provider:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors-series:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        institution:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        location:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        publisher:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        recipients:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        senders:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        translators:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
      references:
      - type: book
        title: Book Title
        abbreviation: Abbr
        abstract: Description of the book.
        collection-doi: 10.5281/zenodo.1003150
        collection-title: Collection Title
        collection-type: Collection Type
        commit: 156a04c74a8a79d40c5d705cddf9d36735feab4d
        copyright: 2017 Stephan Druskat
        data-type: Data Type
        database: Database
        date-accessed: '2017-10-31'
        date-downloaded: '2017-10-31'
        date-released: '2017-10-31'
        date-published: '2017-10-31'
        department: Department
        doi: 10.5281/zenodo.1003150
        edition: 2nd edition
        end: '123'
        entry: Chapter 9
        filename: book.zip
        format: Printed book
        identifiers:
        - type: doi
          value: 10.5281/zenodo.1003150
        - type: swh
          value: swh:1:rel:99f6850374dc6597af01bd0ee1d3fc0699301b9f
        - type: url
          value: https://example.com
        - type: other
          value: other-schema://abcd.1234.efgh.5678
        isbn: 978-1-89183-044-0
        issn: 1234-543X
        issue: '123'
        issue-date: December
        issue-title: Special Issue on Software Citation
        journal: PeerJ
        keywords:
        - Software
        - Citation
        languages:
        - aaa
        - zu
        license: Apache-2.0
        license-url: https://spdx.org/licenses/Apache-2.0.html#licenseText
        loc-start: '14'
        loc-end: '54'
        medium: hardcover book
        month: '3'
        nihmsid: Don't know what format a NIHMSID is in
        notes: A field for general notes about the reference, usable in other formats such
          as BibTeX.
        number: A general-purpose field for accession numbers, cf. the specifications for
          examples.
        number-volumes: '7'
        pages: '765'
        patent-states:
        - Germany
        - ROI
        - 'but also for example US states, such as:'
        - IL
        - RI
        pmcid: PMC1234567
        repository: http://code.google.com/events/#&product=browser
        repository-code: http://142.42.1.1:8080/
        repository-artifact: https://files.pythonhosted.org/packages/0a/84/10507b69a07768bc16981184b4d147a0fc84b71fbf35c03bafc8dcced8e1/cffconvert-1.3.3.tar.gz
        scope: Cite this book if you want to reference the general concepts implemented
          in Citation File Format 1.0.0.
        section: Chapter 2 - "Reference keys"
        status: advance-online
        start: '123'
        thesis-type: Doctoral dissertation
        url: http://j.mp
        version: 0.0.1423-BETA
        volume: '2'
        volume-title: Advances in Software Citation
        year: '2017'
        year-original: '2012'
        conference:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        authors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        contact:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        database-provider:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors-series:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        institution:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        location:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        publisher:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        recipients:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        senders:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        translators:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage

---

    Code
      modify
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it as below.
      type: software
      license: CC-BY-SA-4.0
      title: A new title
      version: 1.0.0
      doi: 10.5281/zenodo.1003150
      identifiers:
      - type: doi
        value: 10.5281/zenodo.1003150
      - type: swh
        value: swh:1:rel:99f6850374dc6597af01bd0ee1d3fc0699301b9f
      - type: url
        value: https://example.com
      - type: other
        value: other-schema://abcd.1234.efgh.5678
      abstract: This is an awesome piece of research software!
      authors:
      - family-names: Real Person
        given-names: One Truly
        name-particle: van der
        name-suffix: IV
        alias: Citey
        affiliation: Excellent University, Niceplace, Arcadia
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
      - name: Entity Project Team Conference entity
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
        date-start: '2017-01-01'
        date-end: '2017-01-31'
        location: The team garage
      preferred-citation:
        type: book
        title: Book Title
        abbreviation: Abbr
        abstract: Description of the book.
        collection-doi: 10.5281/zenodo.1003150
        collection-title: Collection Title
        collection-type: Collection Type
        commit: 156a04c74a8a79d40c5d705cddf9d36735feab4d
        copyright: 2017 Stephan Druskat
        data-type: Data Type
        database: Database
        date-accessed: '2017-10-31'
        date-downloaded: '2017-10-31'
        date-released: '2017-10-31'
        date-published: '2017-10-31'
        department: Department
        doi: 10.5281/zenodo.1003150
        edition: 2nd edition
        end: '456'
        entry: Chapter 9
        filename: book.zip
        format: Printed book
        identifiers:
        - type: doi
          value: 10.5281/zenodo.1003150
        - type: swh
          value: swh:1:rel:99f6850374dc6597af01bd0ee1d3fc0699301b9f
        - type: url
          value: https://example.com
        - type: other
          value: other-schema://abcd.1234.efgh.5678
        isbn: 978-1-89183-044-0
        issn: 1234-543X
        issue: '123'
        issue-date: December
        issue-title: Special Issue on Software Citation
        journal: PeerJ
        keywords:
        - Software
        - Citation
        languages:
        - aaa
        - zu
        license: Apache-2.0
        license-url: https://spdx.org/licenses/Apache-2.0.html#licenseText
        loc-start: '14'
        loc-end: '54'
        medium: hardcover book
        month: '3'
        nihmsid: Don't know what format a NIHMSID is in
        notes: A field for general notes about the reference, usable in other formats such
          as BibTeX.
        number: A general-purpose field for accession numbers, cf. the specifications for
          examples.
        number-volumes: '7'
        pages: '765'
        patent-states:
        - Germany
        - ROI
        - 'but also for example US states, such as:'
        - IL
        - RI
        pmcid: PMC1234567
        repository: http://code.google.com/events/#&product=browser
        repository-code: http://142.42.1.1:8080/
        repository-artifact: https://files.pythonhosted.org/packages/0a/84/10507b69a07768bc16981184b4d147a0fc84b71fbf35c03bafc8dcced8e1/cffconvert-1.3.3.tar.gz
        scope: Cite this book if you want to reference the general concepts implemented
          in Citation File Format 1.0.0.
        section: Chapter 2 - "Reference keys"
        status: advance-online
        start: '123'
        thesis-type: Doctoral dissertation
        url: http://j.mp
        version: 0.0.1423-BETA
        volume: '2'
        volume-title: Advances in Software Citation
        year: '2017'
        year-original: '2012'
        conference:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        authors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        contact:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        database-provider:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors-series:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        institution:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        location:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        publisher:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        recipients:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        senders:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        translators:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
      repository: https://www.example.com/foo/?bar=baz&inga=42&quux
      repository-artifact: https://files.pythonhosted.org/packages/0a/84/10507b69a07768bc16981184b4d147a0fc84b71fbf35c03bafc8dcced8e1/cffconvert-1.3.3.tar.gz
      repository-code: http://foo.com/blah_(wikipedia)_blah#cite-1
      commit: 156a04c74a8a79d40c5d705cddf9d36735feab4d
      url: http://userid:password@example.com:8080/
      date-released: '2017-12-11'
      contact:
      - family-names: Real Person
        given-names: One Truly
        name-particle: van der
        name-suffix: IV
        alias: Citey
        affiliation: Excellent University, Niceplace, Arcadia
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
      - name: Entity Project Team Conference entity
        address: 22 Acacia Avenue
        city: Citationburgh
        region: Renfrewshire
        post-code: C13 7X7
        country: GB
        orcid: https://orcid.org/0000-0001-2345-6789
        email: project@entity.com
        tel: +44(0)141-323 4567
        fax: +44(0)141-323 45678
        website: https://www.entity-project-team.io
        date-start: '2017-01-01'
        date-end: '2017-01-31'
        location: The team garage
      keywords:
      - One
      - Two
      - Three
      - '4'
      references:
      - type: book
        title: Book Title
        abbreviation: Abbr
        abstract: Description of the book.
        collection-doi: 10.5281/zenodo.1003150
        collection-title: Collection Title
        collection-type: Collection Type
        commit: 156a04c74a8a79d40c5d705cddf9d36735feab4d
        copyright: 2017 Stephan Druskat
        data-type: Data Type
        database: Database
        date-accessed: '2017-10-31'
        date-downloaded: '2017-10-31'
        date-released: '2017-10-31'
        date-published: '2017-10-31'
        department: Department
        doi: 10.5281/zenodo.1003150
        edition: 2nd edition
        end: '123'
        entry: Chapter 9
        filename: book.zip
        format: Printed book
        identifiers:
        - type: doi
          value: 10.5281/zenodo.1003150
        - type: swh
          value: swh:1:rel:99f6850374dc6597af01bd0ee1d3fc0699301b9f
        - type: url
          value: https://example.com
        - type: other
          value: other-schema://abcd.1234.efgh.5678
        isbn: 978-1-89183-044-0
        issn: 1234-543X
        issue: '123'
        issue-date: December
        issue-title: Special Issue on Software Citation
        journal: PeerJ
        keywords:
        - Software
        - Citation
        languages:
        - aaa
        - zu
        license: Apache-2.0
        license-url: https://spdx.org/licenses/Apache-2.0.html#licenseText
        loc-start: '14'
        loc-end: '54'
        medium: hardcover book
        month: '3'
        nihmsid: Don't know what format a NIHMSID is in
        notes: A field for general notes about the reference, usable in other formats such
          as BibTeX.
        number: A general-purpose field for accession numbers, cf. the specifications for
          examples.
        number-volumes: '7'
        pages: '765'
        patent-states:
        - Germany
        - ROI
        - 'but also for example US states, such as:'
        - IL
        - RI
        pmcid: PMC1234567
        repository: http://code.google.com/events/#&product=browser
        repository-code: http://142.42.1.1:8080/
        repository-artifact: https://files.pythonhosted.org/packages/0a/84/10507b69a07768bc16981184b4d147a0fc84b71fbf35c03bafc8dcced8e1/cffconvert-1.3.3.tar.gz
        scope: Cite this book if you want to reference the general concepts implemented
          in Citation File Format 1.0.0.
        section: Chapter 2 - "Reference keys"
        status: advance-online
        start: '123'
        thesis-type: Doctoral dissertation
        url: http://j.mp
        version: 0.0.1423-BETA
        volume: '2'
        volume-title: Advances in Software Citation
        year: '2017'
        year-original: '2012'
        conference:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        authors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        contact:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        database-provider:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        editors-series:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        institution:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        location:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        publisher:
          name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        recipients:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        senders:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
        translators:
        - family-names: Real Person
          given-names: One Truly
          name-particle: van der
          name-suffix: IV
          alias: Citey
          affiliation: Excellent University, Niceplace, Arcadia
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
        - name: Entity Project Team Conference entity
          address: 22 Acacia Avenue
          city: Citationburgh
          region: Renfrewshire
          post-code: C13 7X7
          country: GB
          orcid: https://orcid.org/0000-0001-2345-6789
          email: project@entity.com
          tel: +44(0)141-323 4567
          fax: +44(0)141-323 45678
          website: https://www.entity-project-team.io
          date-start: '2017-01-01'
          date-end: '2017-01-31'
          location: The team garage
      license-url: https://spdx.org/licenses/CC-BY-SA-4.0.html#licenseText

# Recursive parsing

    Code
      read <- cff(complete)
    Condition
      Warning:
      The `path` argument of `cff()` is deprecated as of cffr 1.0.0.
      i Please use `cff_read_cff_citation()` instead.

# Fuzzy matching of keys on cff

    Code
      cff(tittle = "a", cff_version = "ar", version = "200", messange = "Fix my keys")
    Message
      i Found misspelled keys. Trying to map:
      v tittle: title
      v messange: message
    Output
      title: a
      cff-version: ar
      version: '200'
      message: Fix my keys

---

    Code
      cffobj <- cff(tittle = "a", cff_version = "1.2.0", version = "200", messange = "aa",
        anthor = list(list(`family-names` = "a", `given-names` = "b")))
    Message
      i Found misspelled keys. Trying to map:
      v tittle: title
      v messange: message
      v anthor: authors

# duplicated

    Code
      ss <- cff(tittle = "a", tittle = "ar", version = "200", messange = "Fix my keys")
    Message
      i Found misspelled keys. Trying to map:
      v tittle: title
      v tittle: title
      v messange: message
      ! Removing duplicated keys.

# unnamed

    Code
      ss <- cff(path = "a", "200", "Fix my keys")
    Condition
      Warning:
      The `path` argument of `cff()` is deprecated as of cffr 1.0.0.
      i Argument ignored.
      Error in `validate_extra_keys()`:
      ! Elements in `...` should be named.

---

    Code
      s1 <- cff(path = NULL, title = "a", "b", version = 1)
    Condition
      Warning:
      The `path` argument of `cff()` is deprecated as of cffr 1.0.0.
      i Argument ignored.
    Message
      ! Found 1 not-named argument in position 2.
      i Removing unnamed arguments

---

    Code
      s2 <- cff(path = NULL, title = "a", "aa", "bb", "cc", "b", version = 1, "h",
        "j")
    Condition
      Warning:
      The `path` argument of `cff()` is deprecated as of cffr 1.0.0.
      i Argument ignored.
    Message
      ! Found 6 not-named arguments in positions 2, 3, 4, 5, 7, and 8.
      i Removing unnamed arguments

# Errors

    Code
      cff_write_bib(1:4)
    Condition
      Error in `cff_write_bib()`:
      ! `x` should be a <bibentry> object, not a <integer> object.

# Errors citation

    Code
      cff_write_citation(1:4)
    Condition
      Error in `cff_write_citation()`:
      ! `x` should be a <bibentry> object, not a <integer> object.

# Test in mock package

    Code
      auto_cit1
    Output
      Basic M (????). "manyurls: A lot of urls."
      <https://test.github.io/package/>.
      
      A BibTeX entry for LaTeX users is
      
        @Misc{basic,
          title = {manyurls: A lot of urls},
          author = {Marc Basic},
          url = {https://test.github.io/package/},
          abstract = {This package has many urls. Specifically, 1 Bug Reports and 6 URLs. Expected is to have 1 repository-code, 1 url and 3 URLs, since there is 1 duplicate and 1 invalid url.},
          version = {0.1.6},
        }

---

    Code
      auto_cit3
    Output
      Basic M (????). "manyurls: A lot of urls."
      <https://test.github.io/package/>.
      
      A BibTeX entry for LaTeX users is
      
        @Misc{basic,
          title = {manyurls: A lot of urls},
          author = {Marc Basic},
          url = {https://test.github.io/package/},
          abstract = {This package has many urls. Specifically, 1 Bug Reports and 6 URLs. Expected is to have 1 repository-code, 1 url and 3 URLs, since there is 1 duplicate and 1 invalid url.},
          version = {1.0.0},
        }

---

    Code
      cffobj
    Output
      cff-version: 1.2.0
      message: 'To cite package "manyurls" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'manyurls: A lot of urls'
      version: 1.0.0
      doi: 10.1111/2041-210X.12469
      identifiers:
      - type: url
        value: https://r-forge.r-project.org/projects/test/
      - type: url
        value: http://google.ru
      - type: url
        value: https://gitlab.com/r-packages/behaviorchange
      abstract: This package has many urls. Specifically, 1 Bug Reports and 6 URLs. Expected
        is to have 1 repository-code, 1 url and 3 URLs, since there is 1 duplicate and 1
        invalid url.
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      repository-code: https://github.com/test/package
      url: https://test.github.io/package/
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      toBibtex(cit)
    Output
      @Article{,
        title = {{RNeXML}: {A} Package for Reading and Writing Richly Annotated Phylogenetic, Character, and Trait Data in {R}},
        journal = {Methods in Ecology and Evolution},
        author = {Carl Boettiger and Scott Chamberlain and Rutger Vos and Hilmar Lapp},
        year = {2016},
        volume = {7},
        pages = {352--357},
        doi = {10.1111/2041-210X.12469},
      }
      
      @Book{,
        author = {Hadley Wickham},
        title = {ggplot2: Elegant Graphics for Data Analysis},
        publisher = {Springer-Verlag New York},
        year = {2016},
        isbn = {978-3-319-24277-4},
        url = {https://ggplot2.tidyverse.org},
      }

---

    Code
      toBibtex(a_bib)
    Output
      @Misc{basic,
        title = {manyurls: A lot of urls},
        author = {Marc Basic},
        url = {https://test.github.io/package/},
        abstract = {This package has many urls. Specifically, 1 Bug Reports and 6 URLs. Expected is to have 1 repository-code, 1 url and 3 URLs, since there is 1 duplicate and 1 invalid url.},
        version = {1.0.0},
      }

# Keys snapshot

    Code
      cff_schema_keys(FALSE)
    Output
       [1] "cff-version"         "message"             "type"               
       [4] "license"             "title"               "version"            
       [7] "doi"                 "identifiers"         "abstract"           
      [10] "authors"             "preferred-citation"  "repository"         
      [13] "repository-artifact" "repository-code"     "commit"             
      [16] "url"                 "date-released"       "contact"            
      [19] "keywords"            "references"          "license-url"        

# Article

    Code
      bib_cff
    Output
      - type: article
        title: Literate Programming
        authors:
        - name: R Core Team
        journal: The Computer Journal
        year: '1984'
        volume: '27'
        issue: '2'
        month: '1'
        notes: Example modified for testing purposes
        start: '97'
        end: '111'

# Book

    Code
      bib_cff
    Output
      - type: book
        title: The LaTeX Companion
        authors:
        - family-names: Mittelbach
          given-names: Frank
        - family-names: Gossens
          given-names: Michel
        - family-names: Braams
          given-names: Johannes
        - family-names: Carlisle
          given-names: David
        - family-names: Rowley
          given-names: Chris
        editors:
        - name: Barnes and Noble
        publisher:
          name: Addison-Wesley Professional
          address: Santa Monica
        year: '2004'
        volume: '3'
        issue: '7'
        collection-title: The LateX Books
        collection-type: book
        edition: Fourth
        month: '8'
        notes: Example modified for testing purposes
        keywords:
        - Two
        - keyword

# Booklet

    Code
      bib_cff
    Output
      - type: pamphlet
        title: Java Booklet
        authors:
        - family-names: Mustermann
          given-names: Max
        medium: Internet
        location:
          name: Stuttgart
        month: '2'
        year: '2016'
        notes: Example modified from Jabref

# Conference

    Code
      bib_cff
    Output
      - type: conference-paper
        title: On Notions of Information Transfer in VLSI Circuits
        authors:
        - family-names: Oaho
          given-names: Alfred V.
        - family-names: Ullman
          given-names: Jeffrey D.
        - family-names: Yannakakis
          given-names: Mihalis
        collection-title: Proc. Fifteenth Annual ACM STOC
        collection-type: conference
        year: '1983'
        editors:
        - family-names: Oz
          given-names: Wizard V.
        - family-names: Yannakakis
          given-names: Mihalis
        volume: '41'
        issue: '17'
        institution:
          name: ACM
        publisher:
          name: Academic Press
        notes: Example modified for testing purposes
        start: '133'
        end: '139'
        conference:
          name: Proc. Fifteenth Annual ACM STOC
          address: Boston

# InBook

    Code
      bib_cff
    Output
      - type: book
        title: A Framework for Freeness Analysis
        authors:
        - family-names: King
          given-names: A.
        editors:
        - family-names: Tick
          given-names: E
        - family-names: Succi
          given-names: G
        section: 7, 14
        publisher:
          name: Kluwer Academic Publishers
          address: Dordrecht
        year: '1994'
        volume: '27'
        issue: '2'
        collection-title: Implementations of Logic Programming Systems
        collection-type: book
        edition: Second
        month: '1'
        notes: Example modified for testing purposes
        start: '137'
        end: '149'

# InCollection

    Code
      bib_cff
    Output
      - type: generic
        title: Knowledge-Based Methods for WSD
        authors:
        - family-names: Mihalcea
          given-names: Rada
        collection-title: 'Word Sense Disambiguation: Algorithms and Applications'
        collection-type: collection
        publisher:
          name: Springer
          address: 107--132
        year: '2006'
        editors:
        - family-names: Agirre
          given-names: Eneko
        - family-names: Edmonds
          given-names: Philip
        volume: '23'
        issue: '3'
        section: '1,2,3'
        edition: Third
        month: '8'
        notes: A note
        start: '24'
        end: '57'

# InProceedings

    Code
      bib_cff
    Output
      - type: conference-paper
        title: On Notions of Information Transfer in VLSI Circuits
        authors:
        - family-names: Oaho
          given-names: Alfred V.
        - family-names: Ullman
          given-names: Jeffrey D.
        - family-names: Yannakakis
          given-names: Mihalis
        collection-title: Proc. Fifteenth Annual ACM STOC
        collection-type: proceedings
        year: '1983'
        editors:
        - family-names: Oz
          given-names: Wizard V.
        - family-names: Yannakakis
          given-names: Mihalis
        volume: '41'
        issue: '17'
        institution:
          name: ACM
        publisher:
          name: Academic Press
        notes: Example modified for testing purposes
        start: '133'
        end: '139'
        conference:
          name: Proc. Fifteenth Annual ACM STOC
          address: Boston

---

    Code
      toBibtex(bib)
    Output
      @InProceedings{oaho_etall:1983,
        title = {On Notions of Information Transfer in VLSI Circuits},
        author = {Alfred V. Oaho and Jeffrey D. Ullman and Mihalis Yannakakis},
        year = {1983},
        booktitle = {I Am a conference},
        publisher = {Academic Press},
        address = {Boston},
        editor = {Wizard V. Oz and Mihalis Yannakakis},
        volume = {41},
        number = {17},
        pages = {133--139},
        note = {Example modified for testing purposes},
        organization = {ACM},
      }

# Manual

    Code
      bib_cff
    Output
      - type: manual
        title: A Language and Environment for Statistical Computing
        authors:
        - name: R Core Team
        institution:
          name: R Foundation for Statistical Computing
          address: Vienna, Austria
        edition: Fourth
        month: '8'
        year: '2021'
        notes: Example modified for testing purposes

# MastersThesis

    Code
      bib_cff
    Output
      - type: thesis
        title: An examination of keystroke dynamics for continuous user authentication
        authors:
        - family-names: Alsolami
          given-names: Eesa
        institution:
          name: Queensland University of Technology
          address: Queensland, NZ
        year: '2012'
        month: '8'
        notes: Example modified for testing purposes
        thesis-type: Master's Thesis

# Misc

    Code
      bib_cff
    Output
      - type: generic
        title: A Language and Environment for Statistical Computing
        authors:
        - name: R Core Team
        medium: CD-ROM
        month: '1'
        year: '2021'
        notes: A note

# PhdThesis

    Code
      bib_cff
    Output
      - type: thesis
        title: An examination of keystroke dynamics for continuous user authentication
        authors:
        - family-names: Alsolami
          given-names: Eesa
        institution:
          name: Queensland University of Technology
          address: Queensland, NZ
        year: '2012'
        month: '8'
        notes: Example modified for testing purposes
        thesis-type: PhD Thesis

# Proceedings

    Code
      bib_cff
    Output
      - type: proceedings
        title: Proc. Fifteenth Annual STOC
        authors:
        - name: anonymous
        year: '1983'
        editors:
        - family-names: Oz
          given-names: Wizard V.
        - family-names: Yannakakis
          given-names: Mihalis
        volume: '1'
        issue: '17'
        collection-title: All ACM Conferences
        collection-type: proceedings
        month: '8'
        institution:
          name: The OX Association for Computing Machinery
        publisher:
          name: Academic Press
        notes: Example modified for testing purposes
        conference:
          name: All ACM Conferences
          address: Boston, US

# TechReport

    Code
      bib_cff
    Output
      - type: report
        title: Naive tools for studying compilation histories
        authors:
        - family-names: Jadud
          given-names: Matthew C.
        - family-names: Fincher
          given-names: Sally A.
        institution:
          name: University of Kent Canterbury
          address: Computing Laboratory, University of Kent, Canterbury, Kent, CT2 7NF
        year: '2003'
        issue: 3-03
        month: '3'
        notes: Example modified for testing purposes

# Unpublished

    Code
      bib_cff
    Output
      - type: unpublished
        title: Demonstratives
        authors:
        - family-names: Kaplan
          given-names: D.
        notes: Unpublished manuscript, UCLA
        year: '1977'
        month: '8'

---

    Code
      toBibtex(bib)
    Output
      @Unpublished{kaplan:1977,
        title = {Demonstratives},
        author = {D. Kaplan},
        year = {1977},
        month = {aug},
        note = {Extracted with cffr R package},
      }

# InBook with booktitle

    Code
      bib_cff
    Output
      - type: generic
        title: Bibliographies and citations
        authors:
        - family-names: Xie
          given-names: Yihui
        - family-names: Dervieux
          given-names: Christophe
        - family-names: Riederer
          given-names: Emily
        year: '2020'
        collection-title: R Markdown Cookbook
        collection-type: collection
        publisher:
          name: Chapman and Hall/CRC
          address: Boca Raton, Florida
        isbn: '9780367563837'
        url: https://yihui.org/rmarkdown-cookbook/
        section: '4.5'

# Test entry without author

    Code
      bib_cff
    Output
      - type: proceedings
        title: Proceedings of the 6th European Conference on Computer Systems
        authors:
        - name: anonymous
        editors:
        - family-names: Berbers
          given-names: Yolande
        - family-names: Zwaenepoel
          given-names: Willy
        collection-title: Proceedings of the 6th European Conference on Computer Systems
        collection-type: proceedings
        publisher:
          name: ACM
        month: '4'
        year: '2006'
        isbn: 1-59593-322-02
        conference:
          name: Proceedings of the 6th European Conference on Computer Systems

# Test entry without author but has a key

    Code
      bib_cff
    Output
      - type: generic
        title: Proceedings of the 6th European Conference on Computer Systems
        authors:
        - name: anonymous
        collection-title: Proceedings of the 6th European Conference on Computer Systems
        collection-type: misc
        publisher:
          name: ACM
        month: '4'
        year: '2006'
        isbn: 1-59593-322-02

# Test entry without author and key

    Code
      bib_cff
    Output
      - type: generic
        title: Proceedings of the 6th European Conference on Computer Systems
        authors:
        - name: anonymous
        collection-title: Proceedings of the 6th European Conference on Computer Systems
        collection-type: misc
        publisher:
          name: ACM
        month: '4'
        year: '2006'
        isbn: 1-59593-322-02

# Skip misc without title

    Code
      cffobj
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.
      title: My Research Software
      authors:
      - family-names: Doe
        given-names: John

# Skip misc without title, not skipping the good one

    Code
      cffobj
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.
      title: My Research Software
      authors:
      - family-names: Doe
        given-names: John
      references:
      - type: generic
        title: 'rromeo: An R Client for SHERPA/RoMEO API'
        authors:
        - family-names: Grenié
          given-names: Matthias
        - family-names: Gruson
          given-names: Hugo
        year: '2019'
        url: https://CRAN.R-project.org/package=rromeo

# Check extended BibLatex Fields

    Code
      bib_cff
    Output
      - type: article
        title: Computation of methodology hyphen independent ionic solvation free energies
          from molecular simulations
        authors:
        - family-names: Kastenholz
          given-names: M. A.
        - family-names: Hünenbergerb
          given-names: Philippe H.
        journal: J. Chem. Phys.
        year: '2006'
        notes: Example modified for testing purposes
        date-published: '2006-03-15'
        filename: a_file.pdf
        issue-title: Semantic 3D Media and Content
        translators:
        - family-names: Wicksteed
          given-names: P. H.
        - family-names: Cornford
          given-names: F. M.
        date-accessed: '2006-10-01'
        pages: '528'
        abstract: The computation of ionic solvation free energies from atomistic simulations
          is a surprisingly difficult problem that has found no satisfactory solution for
          more than 15 years.
        doi: 10.1063/1.2172593
        isbn: 0-816-52066-6
        issn: 0097-8493
        url: http://www.ctan.org
        start: '55'
        end: '65'
        month: '3'

# Duplicate entries

    Code
      bib_cff <- as_cff(rep(bib, 2))
    Message
      i Removing duplicate <bibentry> objects.

# Identifiers and dois

    Code
      as_cff(checf)
    Output
      url: https://www.R-project.org/
      doi: 10.5281/zenodo.5366600
      identifiers:
      - type: doi
        value: 10.5281/zenodo.5366601
      - type: doi
        value: 10.5281/zenodo.5366602
      - type: url
        value: https://google.com/

---

    Code
      cffobj
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.
      title: My Research Software
      authors:
      - family-names: Doe
        given-names: John
      references:
      - type: manual
        title: A Language and Environment for Statistical Computing
        authors:
        - name: R Core Team
        year: '2022'
        url: https://www.R-project.org/
        doi: 10.5281/zenodo.5366600
        identifiers:
        - type: doi
          value: 10.5281/zenodo.5366601
        - type: doi
          value: 10.5281/zenodo.5366602
        - type: url
          value: https://google.com/

# Test R 2026

    Code
      as_cff(sin)
    Output
      type: manual
      title: 'R: A Language and Environment for Statistical Computing'
      authors:
      - name: R Core Team
        website: https://ror.org/02zz1nj61
      institution:
        name: R Foundation for Statistical Computing
        website: https://ror.org/05qewa988
        address: Vienna, Austria
      year: '2026'
      doi: 10.32614/R.manuals
      url: https://www.R-project.org/

---

    Code
      cffobj
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.
      title: My Research Software
      authors:
      - family-names: Doe
        given-names: John
      references:
      - type: manual
        title: 'R: A Language and Environment for Statistical Computing'
        authors:
        - name: R Core Team
          website: https://ror.org/02zz1nj61
        institution:
          name: R Foundation for Statistical Computing
          website: https://ror.org/05qewa988
          address: Vienna, Austria
        year: '2026'
        doi: 10.32614/R.manuals
        url: https://www.R-project.org/

# as.cff still works

    Code
      l2
    Output
      cff-version: 1.2.0
      title: Manipulating files

# as_cff.person

    Code
      as_cff(pers)
    Output
      - family-names: person
        given-names: A
        email: fake@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
        affiliation: Real Madrid
        website: https://www.google.com/

---

    Code
      single_cff
    Output
      family-names: person
      given-names: A
      email: fake@gmail.com
      orcid: https://orcid.org/0000-0001-8457-4658
      affiliation: Real Madrid
      website: https://www.google.com/

# as_cff.bibentry, toBibtex

    Code
      bbb
    Output
      - type: generic
        title: title
        authors:
        - name: Author
        editors:
        - name: Editor

---

    Code
      b_single_cff
    Output
      type: generic
      title: title
      authors:
      - name: Author
      editors:
      - name: Editor

# as_cff.default

    Code
      as_cff(b)
    Output
      a: '1'

# Other convertes

    Code
      noadd <- cff(chocolate = "New York", version = 5)
    Message
      i Found misspelled keys. Trying to map:
      x chocolate: No match, removing.

# Reading full cff

    Code
      df
    Output
                                    class
      authors            cff_pers_lst|cff
      contact            cff_pers_lst|cff
      identifiers         cff_ref_lst|cff
      references          cff_ref_lst|cff
      preferred-citation      cff_ref|cff

---

    Code
      df2
    Output
                                   class
      authors           cff_pers_lst|cff
      contact           cff_pers_lst|cff
      editors           cff_pers_lst|cff
      editors-series    cff_pers_lst|cff
      recipients        cff_pers_lst|cff
      senders           cff_pers_lst|cff
      translators       cff_pers_lst|cff
      conference            cff_pers|cff
      database-provider     cff_pers|cff
      institution           cff_pers|cff
      location              cff_pers|cff
      publisher             cff_pers|cff
      identifiers        cff_ref_lst|cff

# Merge all DESCRIPTION files with CITATION_basic

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdesc" in publications use:'
      type: software
      title: 'basicdesc: A Basic Description'
      version: 0.1.6
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      abstract: A very basic description. Should parse without problems.
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdesc" in publications use:'
      type: software
      title: 'basicdesc: A Basic Description'
      version: 0.1.6
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      abstract: A very basic description. Should parse without problems.
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdescdate" in publications use:'
      type: software
      title: 'basicdescdate: A Basic Description with Date'
      version: 0.1.6
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      abstract: A very basic description. Should parse without problems. I have a Date
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      date-released: '1999-01-01'
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "GenomicRanges" in publications use:'
      type: software
      title: 'GenomicRanges: Representation and manipulation of genomic intervals'
      version: 1.51.4
      authors:
      - family-names: Aboyoun
        given-names: Patrick
      - family-names: Pagès
        given-names: Hervé
        email: hpages.on.github@gmail.com
      - family-names: Lawrence
        given-names: Michael
      abstract: The ability to efficiently represent and manipulate genomic annotations
        and alignments is playing a central role when it comes to analyzing high-throughput
        sequencing data (a.k.a. NGS data). The GenomicRanges package defines general purpose
        containers for storing and manipulating genomic intervals and variables defined
        along a genome. More specialized containers for representing and manipulating short
        alignments against a reference genome, or a matrix-like summarization of an experiment,
        are defined in the GenomicAlignments and SummarizedExperiment packages, respectively.
        Both packages build on top of the GenomicRanges infrastructure.
      repository: https://bioconductor.org/
      repository-code: https://github.com/Bioconductor/GenomicRanges
      url: https://bioconductor.org/packages/GenomicRanges
      date-released: '2022-12-15'
      contact:
      - family-names: Pagès
        given-names: Hervé
        email: hpages.on.github@gmail.com
      license: Artistic-2.0
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "options" in publications use:'
      type: software
      title: 'options: Simple, Consistent Package Options'
      version: 0.3.0
      authors:
      - family-names: Kelkhoff
        given-names: Doug
        email: doug.kelkhoff@gmail.com
      abstract: Simple mechanisms for defining and interpreting package options. Provides
        helpers for interpreting environment variables, global options, defining default
        values and more.
      repository: https://CRAN.R-project.org/package=options
      repository-code: https://codeberg.org/dgkf/options
      url: https://dgkf.github.io/options/
      contact:
      - family-names: Kelkhoff
        given-names: Doug
        email: doug.kelkhoff@gmail.com
      license: MIT
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.options
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.1.6
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      - family-names: Salmon
        given-names: Maëlle
        orcid: https://orcid.org/0000-0002-2815-0399
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      repository: https://CRAN.R-project.org/package=codemetar
      repository-code: https://gitlab.com/ninijay/methoden
      url: https://ropensci.github.io/codemetar
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.codemetar
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "manypersons" in publications use:'
      type: software
      title: 'manypersons: A lot of persons'
      version: 0.1.6
      authors:
      - family-names: Hernangómez
        given-names: Diego
        email: fake@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      - family-names: Doe
        given-names: Joe
        affiliation: This One
        country: ES
      - family-names: Doe
        given-names: Pepe
        email: fake@gmail.com
      - name: I am an entity
        date-end: '2020-01-01'
      abstract: Overkill desc with many persons. Try this
      repository-code: https://github.com/many/persons
      url: https://many.github.io/persons
      contact:
      - family-names: Hernangómez
        given-names: Diego
        email: fake@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      - name: I am an entity
        date-end: '2020-01-01'
      keywords:
      - metadata
      - cffr
      - ropensci
      - citation
      - credit
      - linked-data
      - one
      - two
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "manyurls" in publications use:'
      type: software
      title: 'manyurls: A lot of urls'
      version: 0.1.6
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      abstract: This package has many urls. Specifically, 1 Bug Reports and 6 URLs. Expected
        is to have 1 repository-code, 1 url and 3 URLs, since there is 1 duplicate and 1
        invalid url.
      repository-code: https://github.com/test/package
      url: https://test.github.io/package/
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: url
        value: https://r-forge.r-project.org/projects/test/
      - type: url
        value: http://google.ru
      - type: url
        value: https://gitlab.com/r-packages/behaviorchange
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.1.6
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      - family-names: Salmon
        given-names: Maëlle
        orcid: https://orcid.org/0000-0002-2815-0399
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      repository: https://CRAN.R-project.org/package=codemetar
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.codemetar
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "idonthavencoding" in publications use:'
      type: software
      title: 'idonthavencoding: No Encoding'
      version: 0.1.6
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
        orcid: https://orcid.org/0000-0000-0000-001X
      abstract: I don't have Encoding field. Should parse without problems. Maybe some warning,
        but because the missing Encoding field.
      repository-code: https://github.com/idonthavencoding/package
      url: https://idonthavencoding.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
        orcid: https://orcid.org/0000-0000-0000-001X
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "resmush" in publications use:'
      type: software
      title: 'resmush: Optimize and Compress Image Files with ''reSmush.it'''
      version: 0.1.0
      authors:
      - family-names: Hernangómez
        given-names: Diego
        email: diego.hernangomezherrero@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      abstract: Compress local and online images using the 'reSmush.it' API service <https://resmush.it/>.
      repository: https://CRAN.R-project.org/package=resmush
      repository-code: https://github.com/dieghernan/resmush
      url: https://dieghernan.github.io/resmush/
      date-released: '2024-02-02'
      contact:
      - family-names: Hernangómez
        given-names: Diego
        email: diego.hernangomezherrero@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      keywords:
      - r
      - compress-images
      - optimize-images
      - resmushit
      - api
      license: MIT
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.resmush
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "quint" in publications use:'
      type: software
      title: 'quint: Qualitative Interaction Trees'
      version: 2.2.2
      authors:
      - family-names: Dusseldorp
        given-names: Elise
        email: elise.dusseldorp@fsw.leidenuniv.nl
      - family-names: Doove
        given-names: Lisa
        email: lisa.doove@kuleuven.be
      - family-names: Put
        given-names: Jeanne
        name-particle: van de
        email: jmms1991@gmail.com
      - family-names: Van Mechelen
        given-names: Iven
        email: iven.vanmechelen@kuleuven.be
      - family-names: Claramunt Gonzalez
        given-names: Juan
        email: j.claramunt.gonzalez@fsw.leidenuniv.nl
      abstract: Grows a qualitative interaction tree. Quint is a tool for subgroup analysis,
        suitable for data from a two-arm randomized controlled trial. More information in
        Dusseldorp, E., Doove, L., & Van Mechelen, I. (2016) <https://doi.org/10.3758/s13428-015-0594-z>.
      repository: https://CRAN.R-project.org/package=quint
      date-released: '2022-07-01'
      contact:
      - family-names: Dusseldorp
        given-names: Elise
        email: elise.dusseldorp@fsw.leidenuniv.nl
      license: GPL-3.0-or-later
      commit: 2.2.2
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.quint
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.3.5
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      - family-names: Salmon
        given-names: Maëlle
        orcid: https://orcid.org/0000-0002-2815-0399
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      repository: https://ropensci.r-universe.dev
      repository-code: https://github.com/ropensci/codemetar
      url: https://docs.ropensci.org/codemetar/
      date-released: '2024-02-09'
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data
      license: GPL-3.0-only
      commit: bdd9a29d7eabcc43c3195fe461f884932eba763c
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.codemetar
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "rgeos" in publications use:'
      type: software
      title: 'rgeos: Interface to Geometry Engine - Open Source (''GEOS'')'
      version: 0.5-7
      authors:
      - family-names: Bivand
        given-names: Roger
        email: Roger.Bivand@nhh.no
        orcid: https://orcid.org/0000-0003-2392-6140
      - family-names: Rundel
        given-names: Colin
      abstract: 'Interface to Geometry Engine - Open Source (''GEOS'') using the C ''API''
        for topology operations on geometries. Please note that ''rgeos'' will be retired
        by the end of 2023, plan transition to sf functions using ''GEOS'' at your earliest
        convenience. The ''GEOS'' library is external to the package, and, when installing
        the package from source, must be correctly installed first. Windows and Mac Intel
        OS X binaries are provided on ''CRAN''. (''rgeos'' >= 0.5-1): Up to and including
        ''GEOS'' 3.7.1, topological operations succeeded with some invalid geometries for
        which the same operations fail from and including ''GEOS'' 3.7.2. The ''checkValidity=''
        argument defaults and structure have been changed, from default FALSE to integer
        default ''0L'' for ''GEOS'' < 3.7.2 (no check), ''1L'' ''GEOS'' >= 3.7.2 (check
        and warn). A value of ''2L'' is also provided that may be used, assigned globally
        using ''set_RGEOS_CheckValidity(2L)'', or locally using the ''checkValidity=2L''
        argument, to attempt zero-width buffer repair if invalid geometries are found. The
        previous default (FALSE, now ''0L'') is fastest and used for ''GEOS'' < 3.7.2, but
        will not warn users of possible problems before the failure of topological operations
        that previously succeeded. From ''GEOS'' 3.8.0, repair of geometries may also be
        attempted using ''gMakeValid()'', which may, however, return a collection of geometries
        of different types.'
      repository: https://CRAN.R-project.org/package=rgeos
      repository-code: https://r-forge.r-project.org/projects/rgeos/
      url: https://trac.osgeo.org/geos/
      date-released: '2020-09-07'
      contact:
      - family-names: Bivand
        given-names: Roger
        email: Roger.Bivand@nhh.no
        orcid: https://orcid.org/0000-0003-2392-6140
      license: GPL-2.0-or-later
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: url
        value: http://rgeos.r-forge.r-project.org/index.html
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "surveillance" in publications use:'
      type: software
      title: 'surveillance: Temporal and Spatio-Temporal Modeling and Monitoring of Epidemic
        Phenomena'
      version: 1.19.1
      authors:
      - family-names: Höhle
        given-names: Michael
        email: hoehle@math.su.se
        orcid: https://orcid.org/0000-0002-0423-6702
      - family-names: Meyer
        given-names: Sebastian
        email: seb.meyer@fau.de
        orcid: https://orcid.org/0000-0002-1791-9449
      - family-names: Paul
        given-names: Michaela
      abstract: Statistical methods for the modeling and monitoring of time series of counts,
        proportions and categorical data, as well as for the modeling of continuous-time
        point processes of epidemic phenomena. The monitoring methods focus on aberration
        detection in count data time series from public health surveillance of communicable
        diseases, but applications could just as well originate from environmetrics, reliability
        engineering, econometrics, or social sciences. The package implements many typical
        outbreak detection procedures such as the (improved) Farrington algorithm, or the
        negative binomial GLR-CUSUM method of Höhle and Paul (2008) <https://doi.org/10.1016/j.csda.2008.02.015>.
        A novel CUSUM approach combining logistic and multinomial logistic modeling is also
        included. The package contains several real-world data sets, the ability to simulate
        outbreak data, and to visualize the results of the monitoring in a temporal, spatial
        or spatio-temporal fashion. A recent overview of the available monitoring procedures
        is given by Salmon et al. (2016) <https://doi.org/10.18637/jss.v070.i10>. For the
        retrospective analysis of epidemic spread, the package provides three endemic-epidemic
        modeling frameworks with tools for visualization, likelihood inference, and simulation.
        hhh4() estimates models for (multivariate) count time series following Paul and
        Held (2011) <https://doi.org/10.1002/sim.4177> and Meyer and Held (2014) <https://doi.org/10.1214/14-AOAS743>.
        twinSIR() models the susceptible-infectious-recovered (SIR) event history of a fixed
        population, e.g, epidemics across farms or networks, as a multivariate point process
        as proposed by Höhle (2009) <https://doi.org/10.1002/bimj.200900050>. twinstim()
        estimates self-exciting point process models for a spatio-temporal point pattern
        of infective events, e.g., time-stamped geo-referenced surveillance data, as proposed
        by Meyer et al. (2012) <https://doi.org/10.1111/j.1541-0420.2011.01684.x>. A recent
        overview of the implemented space-time modeling frameworks for epidemic phenomena
        is given by Meyer et al. (2017) <https://doi.org/10.18637/jss.v077.i11>.
      repository: https://CRAN.R-project.org/package=surveillance
      repository-code: https://surveillance.R-Forge.R-project.org/
      url: https://surveillance.R-Forge.R-project.org/
      date-released: '2021-03-30'
      contact:
      - family-names: Meyer
        given-names: Sebastian
        email: seb.meyer@fau.de
        orcid: https://orcid.org/0000-0002-1791-9449
      license: GPL-2.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.surveillance
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "MomTrunc" in publications use:'
      type: software
      title: 'MomTrunc: Moments of Folded and Doubly Truncated Multivariate Distributions'
      version: '6.0'
      authors:
      - family-names: Galarza
        given-names: Christian E.
        email: cgalarza88@gmail.com
        orcid: https://orcid.org/0000-0002-4818-6006
      - family-names: Lachos
        given-names: Victor H.
        email: hlachos@gmail.com
        orcid: https://orcid.org/0000-0002-7239-2459
      abstract: It computes arbitrary products moments (mean vector and variance-covariance
        matrix), for some double truncated (and folded) multivariate distributions. These
        distributions belong to the family of selection elliptical distributions, which
        includes well known skewed distributions as the unified skew-t distribution (SUT)
        and its particular cases as the extended skew-t (EST), skew-t (ST) and the symmetric
        student-t (T) distribution. Analogous normal cases unified skew-normal (SUN), extended
        skew-normal (ESN), skew-normal (SN), and symmetric normal (N) are also included.
        Density, probabilities and random deviates are also offered for these members.
      repository: https://CRAN.R-project.org/package=MomTrunc
      date-released: '2022-06-15'
      contact:
      - family-names: Galarza
        given-names: Christian E.
        email: cgalarza88@gmail.com
        orcid: https://orcid.org/0000-0002-4818-6006
      license: GPL-2.0-or-later
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.MomTrunc
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.1.6
      authors:
      - family-names: Ok
        given-names: John
        email: email@email.edu
      - family-names: Doe
        given-names: Jane
        email: email2@email.edu
      - family-names: Doo
        given-names: Jane
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      repository: https://CRAN.R-project.org/package=codemetar
      repository-code: https://github.com/ropensci/codemetar
      url: https://ropensci.github.io/codemetar
      contact:
      - family-names: Ok
        given-names: John
        email: email@email.edu
      - family-names: Doe
        given-names: Jane
        email: email2@email.edu
      license: GPL-3.0-only
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: doi
        value: 10.32614/CRAN.package.codemetar
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

---

    Code
      merged
    Output
      cff-version: 1.2.0
      message: 'To cite package "wrongurls" in publications use:'
      type: software
      title: 'wrongurls: Generate CodeMeta Metadata for R Packages'
      version: 0.1.0
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
      abstract: Codemeta defines a 'JSON-LD' format for describing software metadata. This
        package provides utilities to generate, parse, and modify codemeta.jsonld files
        automatically for R packages.
      url: https://httpbin.org/status/404
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data
      license: MIT
      doi: 10.1111/2041-210X.12469
      preferred-citation:
        type: article
        title: 'RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic,
          Character, and Trait Data in R'
        authors:
        - family-names: Boettiger
          given-names: Carl
        - family-names: Chamberlain
          given-names: Scott
        - family-names: Vos
          given-names: Rutger
        - family-names: Lapp
          given-names: Hilmar
        journal: Methods in Ecology and Evolution
        year: '2016'
        volume: '7'
        doi: 10.1111/2041-210X.12469
        start: '352'
        end: '357'
      identifiers:
      - type: url
        value: https://httpbin.org/status/429
      - type: url
        value: https://www.github.es/ropensci/codemeta
      references:
      - type: book
        title: 'ggplot2: Elegant Graphics for Data Analysis'
        authors:
        - family-names: Wickham
          given-names: Hadley
        publisher:
          name: Springer-Verlag New York
        year: '2016'
        isbn: 978-3-319-24277-4
        url: https://ggplot2.tidyverse.org

# Check dependencies

    Code
      print(selected)
    Output
      - title: 'R: A Language and Environment for Statistical Computing'
        url: https://www.R-project.org/
      - title: cli
        url: https://cli.r-lib.org
        repository: https://CRAN.R-project.org/package=cli
      - title: desc
        url: https://desc.r-lib.org/
        repository: https://CRAN.R-project.org/package=desc
      - title: jsonlite
        url: https://jeroen.r-universe.dev/jsonlite
        repository: https://CRAN.R-project.org/package=jsonlite
      - title: jsonvalidate
        url: https://docs.ropensci.org/jsonvalidate/
        repository: https://CRAN.R-project.org/package=jsonvalidate
      - title: tools
      - title: utils
      - title: yaml
        url: https://yaml.r-lib.org
        repository: https://CRAN.R-project.org/package=yaml
      - title: bibtex
        url: https://docs.ropensci.org/bibtex/
        repository: https://CRAN.R-project.org/package=bibtex
      - title: knitr
        url: https://yihui.org/knitr/
        repository: https://CRAN.R-project.org/package=knitr
      - title: lifecycle
        url: https://lifecycle.r-lib.org/
        repository: https://CRAN.R-project.org/package=lifecycle
      - title: quarto
        url: https://quarto-dev.github.io/quarto-r/
        repository: https://CRAN.R-project.org/package=quarto
      - title: rmarkdown
        url: https://pkgs.rstudio.com/rmarkdown/
        repository: https://CRAN.R-project.org/package=rmarkdown
      - title: testthat
        url: https://testthat.r-lib.org
        repository: https://CRAN.R-project.org/package=testthat
      - title: usethis
        url: https://usethis.r-lib.org
        repository: https://CRAN.R-project.org/package=usethis

# cff_extract_to_bibtex

    Code
      old1 <- cff_extract_to_bibtex(a_cff)
    Condition
      Warning:
      `cff_extract_to_bibtex()` was deprecated in cffr 0.5.0.
      i Function renamed, use `as_bibentry()` instead.

# cff_to_bibtex

    Code
      old1 <- cff_to_bibtex(a_cff)
    Condition
      Warning:
      `cff_extract_to_bibtex()` was deprecated in cffr 1.0.0.
      i Function renamed, use `as_bibentry()` instead.

# cff_from_bibtex

    Code
      ffile <- cff_from_bibtex(x2)
    Condition
      Warning:
      `cff_from_bibtex()` was deprecated in cffr 1.0.0.
      i Please use `cff_read_bib()` instead.

---

    Code
      flines <- cff_from_bibtex(x)
    Condition
      Warning:
      `cff_from_bibtex()` was deprecated in cffr 1.0.0.
      i Please use `cff_read_bib_text()` instead.

# write_bib

    Code
      write_bib(bib, tmp, verbose = FALSE)
    Condition
      Warning:
      `write_bib()` was deprecated in cffr 1.0.0.
      i Please use `cff_write_bib()` instead.

---

    Code
      cat(readLines(tmp), sep = "\n")
    Output
      @Misc{,
        title = {My title},
        author = {Fran Pérez},
      }

# write_citation

    Code
      write_citation(bib, tmp, verbose = FALSE)
    Condition
      Warning:
      `write_citation()` was deprecated in cffr 1.0.0.
      i Please use `cff_write_citation()` instead.

---

    Code
      cat(readLines(tmp), sep = "\n")
    Output
      
      bibentry(bibtype = "Misc",
               title = "My title",
               author = person(given = "Fran",
                               family = "Pérez"))

# cff_parse_person

    Code
      pend <- cff_parse_person(p)
    Condition
      Warning:
      `cff_parse_person()` was deprecated in cffr 1.0.0.
      i Please use `as_cff_person()` instead.

# cff_parse_person_bibtex

    Code
      pend <- cff_parse_person_bibtex(p)
    Condition
      Warning:
      `cff_parse_person_bibtex()` was deprecated in cffr 1.0.0.
      i Please use `as_cff_person()` instead.

# cff_parse_citation

    Code
      pend <- cff_parse_citation(p)
    Condition
      Warning:
      `cff_parse_citation()` was deprecated in cffr 1.0.0.
      i Please use `as_cff.bibentry()` instead.

# Errors and messages

    Code
      cff_read_bib_text(a_cff)
    Condition
      Error in `cff_read_bib_text()`:
      ! `x` should be a <character>, not a <cff>.

---

    Code
      cff_read_bib_text("a bad line")
    Message
      ! `x` doesn't look as a BibTeX entry. Check the results.
    Condition
      Error:
      ! Invalid bib file

# Read lines

    Code
      list
    Output
      - type: book
        title: 'Relativity: The Special and the General Theory'
        authors:
        - family-names: Einstein
          given-names: Albert
        year: '1920'
        publisher:
          name: Henry Holt and Company
          address: London, United Kingdom
        isbn: '9781587340925'
      - type: generic
        title: Handing out random pamphlets in airports
        authors:
        - family-names: Missilany
          given-names: Joe-Bob
        year: '1984'
        month: '10'
        notes: This is a full MISC entry
        medium: Handed out at O'Hare

---

    Code
      fromfile <- cff_read_bib_text(tmpbib)
    Message
      ! `x` seems to be a ".bib" file, not a BibTeX entry.
      i Reading `x` with `cffr:cff_read_bib()`

# Test indev

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdesc" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'basicdesc: A Basic Description'
      version: 0.1.6
      abstract: A very basic description. Should parse without problems.
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

# No auto generate preferred citations

    Code
      cff_create(rgeos, gh_keywords = FALSE, keys = list(references = NULL))
    Output
      cff-version: 1.2.0
      message: 'To cite package "rgeos" in publications use:'
      type: software
      license: GPL-2.0-or-later
      title: 'rgeos: Interface to Geometry Engine - Open Source (''GEOS'')'
      version: 0.5-7
      identifiers:
      - type: url
        value: http://rgeos.r-forge.r-project.org/index.html
      abstract: 'Interface to Geometry Engine - Open Source (''GEOS'') using the C ''API''
        for topology operations on geometries. Please note that ''rgeos'' will be retired
        by the end of 2023, plan transition to sf functions using ''GEOS'' at your earliest
        convenience. The ''GEOS'' library is external to the package, and, when installing
        the package from source, must be correctly installed first. Windows and Mac Intel
        OS X binaries are provided on ''CRAN''. (''rgeos'' >= 0.5-1): Up to and including
        ''GEOS'' 3.7.1, topological operations succeeded with some invalid geometries for
        which the same operations fail from and including ''GEOS'' 3.7.2. The ''checkValidity=''
        argument defaults and structure have been changed, from default FALSE to integer
        default ''0L'' for ''GEOS'' < 3.7.2 (no check), ''1L'' ''GEOS'' >= 3.7.2 (check
        and warn). A value of ''2L'' is also provided that may be used, assigned globally
        using ''set_RGEOS_CheckValidity(2L)'', or locally using the ''checkValidity=2L''
        argument, to attempt zero-width buffer repair if invalid geometries are found. The
        previous default (FALSE, now ''0L'') is fastest and used for ''GEOS'' < 3.7.2, but
        will not warn users of possible problems before the failure of topological operations
        that previously succeeded. From ''GEOS'' 3.8.0, repair of geometries may also be
        attempted using ''gMakeValid()'', which may, however, return a collection of geometries
        of different types.'
      authors:
      - family-names: Bivand
        given-names: Roger
        email: Roger.Bivand@nhh.no
        orcid: https://orcid.org/0000-0003-2392-6140
      - family-names: Rundel
        given-names: Colin
      repository: https://CRAN.R-project.org/package=rgeos
      repository-code: https://r-forge.r-project.org/projects/rgeos/
      url: https://trac.osgeo.org/geos/
      date-released: '2020-09-07'
      contact:
      - family-names: Bivand
        given-names: Roger
        email: Roger.Bivand@nhh.no
        orcid: https://orcid.org/0000-0003-2392-6140

---

    Code
      cff_create(basicdate, gh_keywords = FALSE, keys = list(references = NULL))
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdescdate" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'basicdescdate: A Basic Description with Date'
      version: 0.1.6
      abstract: A very basic description. Should parse without problems. I have a Date
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      date-released: '1999-01-01'
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

# Fuzzy match on cff_create

    Code
      print_snapshot("Fuzzy match on cff_create", modobject)
    Output
      
      
      ## Fuzzy match on cff_create 
      
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.
      title: Modifying a 'cff' object
      version: 0.0.1
      authors:
      - family-names: Doe
        given-names: John
      repository: https://github.com/user/repo
      url: https://ropensci.org/
      
      ---

# Coerce date

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "rgeos" in publications use:'
      type: software
      license: GPL-2.0-or-later
      title: 'rgeos: Interface to Geometry Engine - Open Source (''GEOS'')'
      version: 0.5-7
      identifiers:
      - type: url
        value: http://rgeos.r-forge.r-project.org/index.html
      abstract: 'Interface to Geometry Engine - Open Source (''GEOS'') using the C ''API''
        for topology operations on geometries. Please note that ''rgeos'' will be retired
        by the end of 2023, plan transition to sf functions using ''GEOS'' at your earliest
        convenience. The ''GEOS'' library is external to the package, and, when installing
        the package from source, must be correctly installed first. Windows and Mac Intel
        OS X binaries are provided on ''CRAN''. (''rgeos'' >= 0.5-1): Up to and including
        ''GEOS'' 3.7.1, topological operations succeeded with some invalid geometries for
        which the same operations fail from and including ''GEOS'' 3.7.2. The ''checkValidity=''
        argument defaults and structure have been changed, from default FALSE to integer
        default ''0L'' for ''GEOS'' < 3.7.2 (no check), ''1L'' ''GEOS'' >= 3.7.2 (check
        and warn). A value of ''2L'' is also provided that may be used, assigned globally
        using ''set_RGEOS_CheckValidity(2L)'', or locally using the ''checkValidity=2L''
        argument, to attempt zero-width buffer repair if invalid geometries are found. The
        previous default (FALSE, now ''0L'') is fastest and used for ''GEOS'' < 3.7.2, but
        will not warn users of possible problems before the failure of topological operations
        that previously succeeded. From ''GEOS'' 3.8.0, repair of geometries may also be
        attempted using ''gMakeValid()'', which may, however, return a collection of geometries
        of different types.'
      authors:
      - family-names: Bivand
        given-names: Roger
        email: Roger.Bivand@nhh.no
        orcid: https://orcid.org/0000-0003-2392-6140
      - family-names: Rundel
        given-names: Colin
      repository: https://CRAN.R-project.org/package=rgeos
      repository-code: https://r-forge.r-project.org/projects/rgeos/
      url: https://trac.osgeo.org/geos/
      date-released: '2020-09-07'
      contact:
      - family-names: Bivand
        given-names: Roger
        email: Roger.Bivand@nhh.no
        orcid: https://orcid.org/0000-0003-2392-6140

# Coerce date in another format

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdescdate" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'basicdescdate: A Basic Description with Date'
      version: 0.1.6
      abstract: A very basic description. Should parse without problems. I have a Date
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      date-released: '1999-01-01'
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

# Parsing many urls

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "manyurls" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'manyurls: A lot of urls'
      version: 0.1.6
      identifiers:
      - type: url
        value: https://r-forge.r-project.org/projects/test/
      - type: url
        value: http://google.ru
      - type: url
        value: https://gitlab.com/r-packages/behaviorchange
      abstract: This package has many urls. Specifically, 1 Bug Reports and 6 URLs. Expected
        is to have 1 repository-code, 1 url and 3 URLs, since there is 1 duplicate and 1
        invalid url.
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      repository-code: https://github.com/test/package
      url: https://test.github.io/package/
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

# Parsing Gitlab

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.1.6
      doi: 10.32614/CRAN.package.codemetar
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      - family-names: Salmon
        given-names: Maëlle
        orcid: https://orcid.org/0000-0002-2815-0399
      repository: https://CRAN.R-project.org/package=codemetar
      repository-code: https://gitlab.com/ninijay/methoden
      url: https://ropensci.github.io/codemetar
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data

# Parsing many persons

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "manypersons" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'manypersons: A lot of persons'
      version: 0.1.6
      abstract: Overkill desc with many persons. Try this
      authors:
      - family-names: Hernangómez
        given-names: Diego
        email: fake@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      - family-names: Doe
        given-names: Joe
        affiliation: This One
        country: ES
      - family-names: Doe
        given-names: Pepe
        email: fake@gmail.com
      - name: I am an entity
        date-end: '2020-01-01'
      repository-code: https://github.com/many/persons
      url: https://many.github.io/persons
      contact:
      - family-names: Hernangómez
        given-names: Diego
        email: fake@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      - name: I am an entity
        date-end: '2020-01-01'
      keywords:
      - metadata
      - cffr
      - ropensci
      - citation
      - credit
      - linked-data
      - one
      - two

# Parsing wrong urls

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "wrongurls" in publications use:'
      type: software
      license: MIT
      title: 'wrongurls: Generate CodeMeta Metadata for R Packages'
      version: 0.1.0
      identifiers:
      - type: url
        value: https://httpbin.org/status/429
      - type: url
        value: https://www.github.es/ropensci/codemeta
      abstract: Codemeta defines a 'JSON-LD' format for describing software metadata. This
        package provides utilities to generate, parse, and modify codemeta.jsonld files
        automatically for R packages.
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
      url: https://httpbin.org/status/404
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data

# Parsing two maintainers

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.1.6
      doi: 10.32614/CRAN.package.codemetar
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      authors:
      - family-names: Ok
        given-names: John
        email: email@email.edu
      - family-names: Doe
        given-names: Jane
        email: email2@email.edu
      - family-names: Doo
        given-names: Jane
      repository: https://CRAN.R-project.org/package=codemetar
      repository-code: https://github.com/ropensci/codemetar
      url: https://ropensci.github.io/codemetar
      contact:
      - family-names: Ok
        given-names: John
        email: email@email.edu
      - family-names: Doe
        given-names: Jane
        email: email2@email.edu

# Parsing r-universe

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "codemetar" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'codemetar: Generate ''CodeMeta'' Metadata for R Packages'
      version: 0.3.5
      doi: 10.32614/CRAN.package.codemetar
      abstract: The 'Codemeta' Project defines a 'JSON-LD' format for describing software
        metadata, as detailed at <https://codemeta.github.io>. This package provides utilities
        to generate, parse, and modify 'codemeta.json' files automatically for R packages,
        as well as tools and examples for working with 'codemeta.json' 'JSON-LD' more generally.
      authors:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      - family-names: Salmon
        given-names: Maëlle
        orcid: https://orcid.org/0000-0002-2815-0399
      repository: https://ropensci.r-universe.dev
      repository-code: https://github.com/ropensci/codemetar
      commit: bdd9a29d7eabcc43c3195fe461f884932eba763c
      url: https://docs.ropensci.org/codemetar/
      date-released: '2024-02-09'
      contact:
      - family-names: Boettiger
        given-names: Carl
        email: cboettig@gmail.com
        orcid: https://orcid.org/0000-0002-1642-628X
      keywords:
      - metadata
      - codemeta
      - ropensci
      - citation
      - credit
      - linked-data

# Parsing Bioconductor

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "GenomicRanges" in publications use:'
      type: software
      license: Artistic-2.0
      title: 'GenomicRanges: Representation and manipulation of genomic intervals'
      version: 1.51.4
      abstract: The ability to efficiently represent and manipulate genomic annotations
        and alignments is playing a central role when it comes to analyzing high-throughput
        sequencing data (a.k.a. NGS data). The GenomicRanges package defines general purpose
        containers for storing and manipulating genomic intervals and variables defined
        along a genome. More specialized containers for representing and manipulating short
        alignments against a reference genome, or a matrix-like summarization of an experiment,
        are defined in the GenomicAlignments and SummarizedExperiment packages, respectively.
        Both packages build on top of the GenomicRanges infrastructure.
      authors:
      - family-names: Aboyoun
        given-names: Patrick
      - family-names: Pagès
        given-names: Hervé
        email: hpages.on.github@gmail.com
      - family-names: Lawrence
        given-names: Michael
      repository: https://bioconductor.org/
      repository-code: https://github.com/Bioconductor/GenomicRanges
      url: https://bioconductor.org/packages/GenomicRanges
      date-released: '2022-12-15'
      contact:
      - family-names: Pagès
        given-names: Hervé
        email: hpages.on.github@gmail.com

# Parsing Posit Package Manager

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "resmush" in publications use:'
      type: software
      license: MIT
      title: 'resmush: Optimize and Compress Image Files with ''reSmush.it'''
      version: 0.1.0
      doi: 10.32614/CRAN.package.resmush
      abstract: Compress local and online images using the 'reSmush.it' API service <https://resmush.it/>.
      authors:
      - family-names: Hernangómez
        given-names: Diego
        email: diego.hernangomezherrero@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      repository: https://CRAN.R-project.org/package=resmush
      repository-code: https://github.com/dieghernan/resmush
      url: https://dieghernan.github.io/resmush/
      date-released: '2024-02-02'
      contact:
      - family-names: Hernangómez
        given-names: Diego
        email: diego.hernangomezherrero@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
      keywords:
      - r
      - compress-images
      - optimize-images
      - resmushit
      - api

# Search package on CRAN

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: 'To cite package "ggplot2" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'ggplot2: A Basic Description'
      version: 0.1.6
      doi: 10.32614/CRAN.package.ggplot2
      abstract: A very basic description. Should parse without problems.
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      repository: https://CRAN.R-project.org/package=ggplot2
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

# Add new keys

    Code
      s
    Output
      cff-version: 1.2.0
      message: This overwrites fields
      type: software
      license: GPL-3.0-only
      title: 'basicdesc: A Basic Description'
      version: 0.1.6
      abstract: New abstract
      authors:
      - family-names: Nadie
        given-names: Don
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      date-released: '1900-01-01'
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      keywords:
      - A
      - new
      - list
      - of
      - keywords

# Append keys

    Code
      desc_file
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdesc" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'basicdesc: A Basic Description'
      version: 0.1.6
      abstract: A very basic description. Should parse without problems.
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

---

    Code
      s
    Output
      cff-version: 1.2.0
      message: 'To cite package "basicdesc" in publications use:'
      type: software
      license: GPL-3.0-only
      title: 'basicdesc: A Basic Description'
      version: 0.1.6
      abstract: A very basic description. Should parse without problems.
      authors:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com
      - family-names: author
        given-names: New
        website: https://stackoverflow.com/
        country: IT
      repository-code: https://github.com/basic/package
      url: https://basic.github.io/package
      contact:
      - family-names: Basic
        given-names: Marc
        email: marcbasic@gmail.com

# Check abort_if_not_cff

    Code
      abort_if_not_cff(err)
    Condition
      Error in `abort_if_not_cff()`:
      ! `x` is not a '*.cff' file.

---

    Code
      abort_if_not_cff(x)
    Condition
      Error in `abort_if_not_cff()`:
      ! `x` is an object of class <data.frame>, not <cff>.

# Check file_exist_abort

    Code
      file_exist_abort(nonexits, abort = TRUE)
    Condition
      Error in `file_exist_abort()`:
      ! 'aaaaaaaa' doesn't exist. Check the '.' directory

---

    Code
      file_exist_abort(non_with_path, abort = TRUE)
    Condition
      Error in `file_exist_abort()`:
      ! './R/no/existe' doesn't exist. Check the './R/no' directory

# Check match_cff_arg

    Code
      match_cff_arg("a", "b", "...")
    Condition
      Error in `match_cff_arg()`:
      ! `...` should be "b", not "a".

---

    Code
      match_cff_arg("a", c("b", "c", "d"), "what")
    Condition
      Error in `match_cff_arg()`:
      ! `what` should be "b", "c", or "d", not "a".

---

    Code
      my_fun()
    Condition
      Error in `my_fun()`:
      ! `aa` should be "A", "B", "C", "D", or "E", not "a".

# Errors and messages

    Code
      cff_modify(a_list, abstract = "An abstract")
    Condition
      Error in `cff_modify()`:
      ! `x` should be a <cff> object, not <list>.

---

    Code
      xend <- cff_modify(a_cff)
    Message
      i Args `...` empty. Returning `x`.

---

    Code
      dup <- cff_modify(a_cff, abstract = "a", abstract = "b")
    Message
      ! Removing duplicated keys.

---

    Code
      cff_modify(a_cff, "a", "b")
    Condition
      Error in `validate_extra_keys()`:
      ! Elements in `...` should be named.

---

    Code
      mod <- cff_modify(a_cff, "a", abstract = "b")
    Message
      ! Found 1 not-named argument in position 1.
      i Removing unnamed arguments

# Test first von last

    Code
      unlist(res)
    Output
      family-names  given-names 
              "BB"         "AA" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      name 
      "AA" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
              "bb"         "AA" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      name 
      "aa" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "CC"          "AA"          "bb" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "EE"          "AA"    "bb CC dd" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "dd"       "AA bB"          "cc" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "dd"    "AA \\BBb"          "cc" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "DD"       "AA bb"          "cc" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
            "cc DD"          "AA"          "bb" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
              "CC"      "AA bb" 

# Testing with random names First von Last

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
      "La Fontaine"        "Jean"          "de" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
          family-names      given-names 
      "Hernandez Sanz"          "Diego" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names 
       "Miramontes" "Juan Manuel" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
             family-names         given-names 
      "Miramontes Garcia"       "Juan Manuel" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
            "Halen" "Juan Manuel"         "van" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
              family-names          given-names 
      "de la Cruz y Ochoa"              "Bosco" 

# Test von Last, First

    Code
      unlist(res)
    Output
       family-names   given-names name-particle 
               "CC"          "AA"          "bb" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "CC"          "aa"          "bb" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
               "EE"          "AA"    "bb CC dd" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
              "bb"         "AA" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      name 
      "BB" 

# Test von Last, First with brackets, etc

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
            "Armas"         "Ana"          "de" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
        "de Armas"        "Ana" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
            family-names        given-names 
      "de Armas, Aguero"              "Ana" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
            family-names        given-names 
      "de Armas, Aguero"        "Ana Maria" 

# Test von Last, Jr,  First

    Code
      unlist(res)
    Output
       family-names   given-names name-particle   name-suffix 
               "CC"          "AA"          "bb"          "XX" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
              "BB"         "AA" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
         name 
      "BB AA" 

# Test von Last, Jr,  First with masking

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names  name-suffix 
           "Davis"      "Sammy"         "Jr" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names   name-suffix 
       "Davis, and"       "Sammy" "Jr, another" 

# Rest of cases

    Code
      unlist(res)
    Output
                                  name 
      "David, and, Jr, another, Sammy" 

# tames da beast

    Code
      unlist(create_person_from_txt(x))
    Output
                       name 
      "jean de la fontaine" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
         "fontaine"        "Jean"       "de la" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
         "fontaine"     "Jean de"          "la" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
                       name 
      "jean de la fontaine" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
        "fontaine" "Jean de la" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
      family-names  given-names 
        "Fontaine" "Jean De La" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
                       name 
      "jean De la Fontaine" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
      "La Fontaine"        "Jean"          "de" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
                       name 
      "jean de la fontaine" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
         "fontaine"        "Jean"       "de la" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
          family-names      given-names 
      "De La Fontaine"           "Jean" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
         "Fontaine"        "Jean"       "De la" 

---

    Code
      unlist(create_person_from_txt(x))
    Output
       family-names   given-names name-particle 
      "La Fontaine"        "Jean"          "de" 

# as_bibentry default

    Code
      s <- as_bibentry(a = 1)
    Message
      x Can't convert to `bibentry()`: 
      i argument "bibtype" is missing, with no default
      ! Returning empty <bibentry>

# as_bibentry NULL

    Code
      toBibtex(a_bib)
    Output
      @Misc{basic,
        title = {basicdesc: A Basic Description},
        author = {Marc Basic},
        url = {https://basic.github.io/package},
        abstract = {A very basic description. Should parse without problems.},
        version = {0.1.6},
      }

# as_bibentry character

    Code
      as_bibentry("invented_package")
    Condition
      Error in `as_bibentry()`:
      ! Don't know how to extract a <bibentry> from "invented_package". If it is a package run `install.packages("invented_package")` first.

# as_bibentry cff

    Code
      end <- as_bibentry(a_cff, what = "references")
    Message
      ! In `x` didn't find anything with `what` = "references". Returning empty <bibentry>.

---

    Code
      toBibtex(as_bibentry(mod_ref))
    Output
      @Manual{bootstrapmetho:1997,
        title = {Bootstrap Methods and Their Applications},
        year = {1997},
        publisher = {Cambridge University Press (Madrid)},
        url = {http://statwww.epfl.ch/davison/BMA/},
        translator = {{Research Translators Ltd.}},
      }

# Validate full CITATION.cff

    Code
      ok <- cff_validate(fcff)
    Output
      == Validating cff ==============================================================
    Message
      v Congratulations! This <cff> is valid

# Validate error CITATION.cff

    Code
      tab <- cff_validate(ferr)
    Output
      == Validating cff ==============================================================
    Message
      x Oops! This <cff> has the following errors:
      * cff/: must NOT have additional properties
      * cff/authors/0: must NOT have additional properties
      * cff/authors/0/orcid: must match pattern "https://orcid\.org/[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{3}[0-9X]{1}"
      * cff/authors/0/orcid: must match format "uri"
      * cff/authors/0: must have required property 'name'
      * cff/authors/0: must NOT have additional properties
      * cff/authors/0: must NOT have additional properties
      * cff/authors/0: must NOT have additional properties
      * cff/authors/0/orcid: must match pattern "https://orcid\.org/[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{3}[0-9X]{1}"
      * cff/authors/0/orcid: must match format "uri"
      * cff/authors/0: must match a schema in anyOf
      * cff/doi: must match pattern "^10\.\d{4,9}(\.\d+)?/[A-Za-z0-9:/_;\-\.\(\)\[\]\\]+$"
      * cff/keywords/0: must be string
      * cff/license: must be equal to one of the allowed values
      * cff/license: must be array
      * cff/license: must match exactly one schema in oneOf
      * cff/url: must match pattern "^(https|http|ftp|sftp)://.+"
      * cff/url: must match format "uri"

---

    Code
      df[, snap_fields == "character"]
    Output
                       field
      1                 cff/
      2        cff/authors/0
      3  cff/authors/0/orcid
      4  cff/authors/0/orcid
      5        cff/authors/0
      6        cff/authors/0
      7        cff/authors/0
      8        cff/authors/0
      9  cff/authors/0/orcid
      10 cff/authors/0/orcid
      11       cff/authors/0
      12             cff/doi
      13      cff/keywords/0
      14         cff/license
      15         cff/license
      16         cff/license
      17             cff/url
      18             cff/url
                                                                                       message
      1                                                    must NOT have additional properties
      2                                                    must NOT have additional properties
      3  must match pattern "https://orcid\\.org/[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{3}[0-9X]{1}"
      4                                                                must match format "uri"
      5                                                     must have required property 'name'
      6                                                    must NOT have additional properties
      7                                                    must NOT have additional properties
      8                                                    must NOT have additional properties
      9  must match pattern "https://orcid\\.org/[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{3}[0-9X]{1}"
      10                                                               must match format "uri"
      11                                                          must match a schema in anyOf
      12 must match pattern "^10\\.\\d{4,9}(\\.\\d+)?/[A-Za-z0-9:/_;\\-\\.\\(\\)\\[\\]\\\\]+$"
      13                                                                        must be string
      14                                            must be equal to one of the allowed values
      15                                                                         must be array
      16                                                must match exactly one schema in oneOf
      17                                      must match pattern "^(https|http|ftp|sftp)://.+"
      18                                                               must match format "uri"
             instancePath                       schemaPath              keyword
      1                             #/additionalProperties additionalProperties
      2        /authors/0           #/additionalProperties additionalProperties
      3  /authors/0/orcid      #/definitions/orcid/pattern              pattern
      4  /authors/0/orcid       #/definitions/orcid/format               format
      5        /authors/0                       #/required             required
      6        /authors/0           #/additionalProperties additionalProperties
      7        /authors/0           #/additionalProperties additionalProperties
      8        /authors/0           #/additionalProperties additionalProperties
      9  /authors/0/orcid      #/definitions/orcid/pattern              pattern
      10 /authors/0/orcid       #/definitions/orcid/format               format
      11       /authors/0 #/properties/authors/items/anyOf                anyOf
      12             /doi        #/definitions/doi/pattern              pattern
      13      /keywords/0 #/properties/keywords/items/type                 type
      14         /license  #/definitions/license-enum/enum                 enum
      15         /license                   #/oneOf/1/type                 type
      16         /license                          #/oneOf                oneOf
      17             /url        #/definitions/url/pattern              pattern
      18             /url         #/definitions/url/format               format
                 dataPath
      1                  
      2        /authors/0
      3  /authors/0/orcid
      4  /authors/0/orcid
      5        /authors/0
      6        /authors/0
      7        /authors/0
      8        /authors/0
      9  /authors/0/orcid
      10 /authors/0/orcid
      11       /authors/0
      12             /doi
      13      /keywords/0
      14         /license
      15         /license
      16         /license
      17             /url
      18             /url

# Test in mock package

    Code
      cff_gha_update()
    Message
      i Creating directory './.github/workflows'.
      v Installing './.github/workflows/update-citation-cff.yaml'
      i Adding ".github" to '.Rbuildignore'

---

    Code
      cff_gha_update()
    Message
      ! File './.github/workflows/update-citation-cff.yaml' already installed. Use `overwrite = TRUE`  for overwrite

---

    Code
      cff_gha_update(overwrite = TRUE)
    Message
      v Installing './.github/workflows/update-citation-cff.yaml'

# preferred-citation-book-missing

    Code
      s <- as_bibentry(x)
    Message
      x Can't convert to `bibentry()`: 
      i A bibentry of bibtype 'Book' has to specify the field: publisher
      ! Returning empty <bibentry>

# preferred-citation-book

    Code
      toBibtex(bib)
    Output
      @Book{bueler:2021,
        title = {PETSc for Partial Differential Equations: Numerical Solutions in C and Python},
        author = {Ed Bueler},
        year = {2021},
        publisher = {SIAM Press},
        address = {Philadelphia},
        isbn = {978111976304},
        url = {https://github.com/bueler/p4pdes},
      }

# preferred-citation-conference-paper-2

    Code
      toBibtex(bib)
    Output
      @InProceedings{gamblin_etall:2016,
        title = {The Spack Package Manager: Bringing Order to HPC Software Chaos},
        author = {Todd Gamblin and Matthew LeGendre and Michael R. Collette and Gregory L. Lee and Adam Moody and Bronis R. {de Supinski} and Scott Futral},
        year = {2016},
        month = {dec},
        booktitle = {Proceedings of Supercomputing 2015},
        publisher = {ACM/IEEE},
        address = {Austin, Texas, USA},
        doi = {10.1244/2886907.2879400},
      }

# preferred-citation-conference-paper-missing

    Code
      toBibtex(bib)
    Output
      @InProceedings{gamblin_etall:2015,
        title = {The Spack Package Manager: Bringing Order to HPC Software Chaos},
        author = {Todd Gamblin and Matthew LeGendre and Michael R. Collette and Gregory L. Lee and Adam Moody and Bronis R. {de Supinski} and Scott Futral},
        year = {2015},
        month = {nov},
        booktitle = {Proceedings of Supercomputing 2015},
        doi = {10.1244/2886907.2879400},
      }

# preferred-citation-conference-paper

    Code
      toBibtex(bib)
    Output
      @InProceedings{rampin_etall:2016,
        title = {ReproZip: Computational Reproducibility With Ease},
        author = {Rémi Rampin and Juliana Freire and Fernando Chirigati and Dennis Shasha},
        year = {2016},
        month = {jun},
        booktitle = {Proceedings of the 2016 ACM SIGMOD International Conference on Management of Data (SIGMOD)},
        publisher = {ACM},
        address = {San Francisco, US},
        pages = {2085--2088},
        doi = {10.1145/2882903.2899401},
        date = {2016-06-26},
      }

# preferred-citation-manual

    Code
      toBibtex(bib)
    Output
      @Manual{hernangomez:2021,
        title = {cffr: Generate Citation File Format ('cff') Metadata for R Packages},
        author = {Diego Hernangómez},
        year = {2021},
        doi = {10.5281/zenodo.5509766},
        url = {https://dieghernan.github.io/cffr/},
      }

# preferred-citation-no-month

    Code
      toBibtex(bib)
    Output
      @Article{lisa:2021,
        title = {My awesome research software},
        author = {Mona Lisa},
        year = {2021},
        journal = {Journal Title},
      }

# preferred-citation-no-vol

    Code
      toBibtex(bib)
    Output
      @Article{hartmann_etall:2020,
        title = {An image-based data-driven analysis of cellular architecture in a developing tissue},
        author = {Jonas Hartmann and Mie Wong and Elisa Gallo and Darren Gilmour},
        year = {2020},
        journal = {eLife},
        pages = {888},
        doi = {10.7554/eLife.55913},
        url = {https://elifesciences.org/articles/55913},
      }

# preferred-citation-pamphlet

    Code
      toBibtex(bib)
    Output
      @Booklet{haines:2021,
        title = {Why and how to use CFF},
        author = {Robert Haines},
        year = {2021},
        month = {sep},
        doi = {10.5281/zenodo.1184077},
        url = {https://github.com/citation-file-format/ruby-cff},
      }

# preferred-citation-report-no-institution

    Code
      toBibtex(bib)
    Output
      @TechReport{haines:2021,
        title = {The benefits of using CFF files},
        author = {Robert Haines},
        year = {2021},
        month = {sep},
        doi = {10.5281/zenodo.1184077},
        url = {https://github.com/citation-file-format/ruby-cff},
        institution = {The University of Manchester},
      }

# preferred-citation-report

    Code
      toBibtex(bib)
    Output
      @TechReport{haines:2021,
        title = {The benefits of using CFF files},
        author = {Robert Haines},
        year = {2021},
        month = {sep},
        doi = {10.5281/zenodo.1184077},
        url = {https://github.com/citation-file-format/ruby-cff},
        institution = {The CFF Institute},
      }

# preferred-citation-unpublished

    Code
      toBibtex(bib)
    Output
      @Unpublished{haines:2021,
        title = {Why and how to use CFF},
        author = {Robert Haines},
        year = {2021},
        month = {sep},
        doi = {10.5281/zenodo.1184077},
        url = {https://github.com/citation-file-format/ruby-cff},
        note = {Self-published by the author.},
      }

# reprozip

    Code
      toBibtex(bib)
    Output
      @Proceedings{rampin_etall:2016,
        title = {ReproZip: Computational Reproducibility With Ease},
        author = {Remi Rampin and Juliana Freire and Fernando Chirigati and Dennis Shasha},
        year = {2016},
        month = {jun},
        address = {San Francisco, US},
        doi = {10.1145/2882903.2899401},
        abstract = {We present ReproZip, the recommended packaging tool for the SIGMOD Reproducibility Review. ReproZip was designed to simplify the process of making an existing computational experiment reproducible across platforms, even when the experiment was put together without reproducibility in mind. The tool creates a self-contained package for an experiment by automatically tracking and identifying all its required dependencies. The researcher can share the package with others, who can then use ReproZip to unpack the experiment, reproduce the findings on their favorite operating system, as well as modify the original experiment for reuse in new research, all with little effort. The demo will consist of examples of non-trivial experiments, showing how these can be packed in a Linux machine and reproduced on different machines and operating systems. Demo visitors will also be able to pack and reproduce their own experiments.},
        date = {2016-06-26},
      }

# smith-et-al

    Code
      toBibtex(bib)
    Output
      @Article{smith_etall:2016,
        title = {Software citation principles},
        author = {A. M. Smith and D. S. Katz and K. E. Niemeyer and {FORCE11 Software Citation Working Group}},
        year = {2016},
        month = {sep},
        journal = {PeerJ Computer Science},
        volume = {2},
        number = {123},
        pages = {e86},
        doi = {10.7717/peerj-cs.86},
      }

# tidyverse-joss-paper

    Code
      toBibtex(bib)
    Output
      @Article{wickham:2019,
        title = {Welcome to the Tidyverse},
        author = {Hadley Wickham},
        year = {2019},
        month = {nov},
        journal = {Journal of Open Source Software},
        volume = {4},
        number = {43},
        pages = {1686},
        doi = {10.21105/joss.01686},
      }

# Coerce one person

    Code
      as_cff_person(p)
    Output
      - family-names: person
        given-names: one

# Coerce several persons

    Code
      as_cff_person(p)
    Output
      - family-names: person
        given-names: one
      - family-names: human
        given-names: another
      - family-names: more
        given-names: and one

# Coerce bibtex persons

    Code
      as_cff_person(s)
    Output
      - family-names: Wright
        given-names: Frank Edwin
        name-suffix: III

---

    Code
      as_cff_person(s)
    Output
      - family-names: person
        given-names: A
      - name: another
      - family-names: one
        given-names: Another

# Coerce bibtex persons with masks

    Code
      as_cff_person(s)
    Output
      - name: Elephant
      - name: Castle

---

    Code
      as_cff_person(s)
    Output
      - name: Elephant and Castle

---

    Code
      as_cff_person(s)
    Output
      - name: Elephant and Castle
      - name: this
      - name: Ltd.

---

    Code
      as_cff_person(s)
    Output
      - name: Elephant and Castle
      - name: this AND Ltd.

# Can extract comments from format

    Code
      pp2
    Output
      - family-names: Doe
        given-names: John
        email: first_mail@gmail.com
        orcid: https://orcid.org/0000-0001-8457-4658
        website: https://www.google.com/
        address: Sweet Home Chicago

# as data frame complete

    Code
      names(the_df)
    Output
        [1] "cff_version"                                       
        [2] "message"                                           
        [3] "abstract"                                          
        [4] "authors.00.family_names"                           
        [5] "authors.00.given_names"                            
        [6] "authors.00.name_particle"                          
        [7] "authors.00.name_suffix"                            
        [8] "authors.00.alias"                                  
        [9] "authors.00.affiliation"                            
       [10] "authors.00.address"                                
       [11] "authors.00.city"                                   
       [12] "authors.00.region"                                 
       [13] "authors.00.post_code"                              
       [14] "authors.00.country"                                
       [15] "authors.00.orcid"                                  
       [16] "authors.00.email"                                  
       [17] "authors.00.tel"                                    
       [18] "authors.00.fax"                                    
       [19] "authors.00.website"                                
       [20] "authors.01.name"                                   
       [21] "authors.01.address"                                
       [22] "authors.01.city"                                   
       [23] "authors.01.region"                                 
       [24] "authors.01.post_code"                              
       [25] "authors.01.country"                                
       [26] "authors.01.orcid"                                  
       [27] "authors.01.email"                                  
       [28] "authors.01.tel"                                    
       [29] "authors.01.fax"                                    
       [30] "authors.01.website"                                
       [31] "authors.01.date_start"                             
       [32] "authors.01.date_end"                               
       [33] "authors.01.location"                               
       [34] "commit"                                            
       [35] "contact.00.family_names"                           
       [36] "contact.00.given_names"                            
       [37] "contact.00.name_particle"                          
       [38] "contact.00.name_suffix"                            
       [39] "contact.00.alias"                                  
       [40] "contact.00.affiliation"                            
       [41] "contact.00.address"                                
       [42] "contact.00.city"                                   
       [43] "contact.00.region"                                 
       [44] "contact.00.post_code"                              
       [45] "contact.00.country"                                
       [46] "contact.00.orcid"                                  
       [47] "contact.00.email"                                  
       [48] "contact.00.tel"                                    
       [49] "contact.00.fax"                                    
       [50] "contact.00.website"                                
       [51] "contact.01.name"                                   
       [52] "contact.01.address"                                
       [53] "contact.01.city"                                   
       [54] "contact.01.region"                                 
       [55] "contact.01.post_code"                              
       [56] "contact.01.country"                                
       [57] "contact.01.orcid"                                  
       [58] "contact.01.email"                                  
       [59] "contact.01.tel"                                    
       [60] "contact.01.fax"                                    
       [61] "contact.01.website"                                
       [62] "contact.01.date_start"                             
       [63] "contact.01.date_end"                               
       [64] "contact.01.location"                               
       [65] "date_released"                                     
       [66] "doi"                                               
       [67] "identifiers.00.type"                               
       [68] "identifiers.00.value"                              
       [69] "identifiers.01.type"                               
       [70] "identifiers.01.value"                              
       [71] "identifiers.02.type"                               
       [72] "identifiers.02.value"                              
       [73] "identifiers.03.type"                               
       [74] "identifiers.03.value"                              
       [75] "keywords.00"                                       
       [76] "keywords.01"                                       
       [77] "keywords.02"                                       
       [78] "keywords.03"                                       
       [79] "license"                                           
       [80] "license_url"                                       
       [81] "repository"                                        
       [82] "repository_code"                                   
       [83] "repository_artifact"                               
       [84] "title"                                             
       [85] "type"                                              
       [86] "url"                                               
       [87] "version"                                           
       [88] "preferred_citation.type"                           
       [89] "preferred_citation.title"                          
       [90] "preferred_citation.abbreviation"                   
       [91] "preferred_citation.abstract"                       
       [92] "preferred_citation.collection_doi"                 
       [93] "preferred_citation.collection_title"               
       [94] "preferred_citation.collection_type"                
       [95] "preferred_citation.commit"                         
       [96] "preferred_citation.copyright"                      
       [97] "preferred_citation.data_type"                      
       [98] "preferred_citation.database"                       
       [99] "preferred_citation.date_accessed"                  
      [100] "preferred_citation.date_downloaded"                
      [101] "preferred_citation.date_released"                  
      [102] "preferred_citation.date_published"                 
      [103] "preferred_citation.department"                     
      [104] "preferred_citation.doi"                            
      [105] "preferred_citation.edition"                        
      [106] "preferred_citation.end"                            
      [107] "preferred_citation.entry"                          
      [108] "preferred_citation.filename"                       
      [109] "preferred_citation.format"                         
      [110] "preferred_citation.identifiers.00.type"            
      [111] "preferred_citation.identifiers.00.value"           
      [112] "preferred_citation.identifiers.01.type"            
      [113] "preferred_citation.identifiers.01.value"           
      [114] "preferred_citation.identifiers.02.type"            
      [115] "preferred_citation.identifiers.02.value"           
      [116] "preferred_citation.identifiers.03.type"            
      [117] "preferred_citation.identifiers.03.value"           
      [118] "preferred_citation.isbn"                           
      [119] "preferred_citation.issn"                           
      [120] "preferred_citation.issue"                          
      [121] "preferred_citation.issue_date"                     
      [122] "preferred_citation.issue_title"                    
      [123] "preferred_citation.journal"                        
      [124] "preferred_citation.keywords.00"                    
      [125] "preferred_citation.keywords.01"                    
      [126] "preferred_citation.languages.00"                   
      [127] "preferred_citation.languages.01"                   
      [128] "preferred_citation.license"                        
      [129] "preferred_citation.license_url"                    
      [130] "preferred_citation.loc_start"                      
      [131] "preferred_citation.loc_end"                        
      [132] "preferred_citation.medium"                         
      [133] "preferred_citation.month"                          
      [134] "preferred_citation.nihmsid"                        
      [135] "preferred_citation.notes"                          
      [136] "preferred_citation.number"                         
      [137] "preferred_citation.number_volumes"                 
      [138] "preferred_citation.pages"                          
      [139] "preferred_citation.patent_states.00"               
      [140] "preferred_citation.patent_states.01"               
      [141] "preferred_citation.patent_states.02"               
      [142] "preferred_citation.patent_states.03"               
      [143] "preferred_citation.patent_states.04"               
      [144] "preferred_citation.pmcid"                          
      [145] "preferred_citation.repository"                     
      [146] "preferred_citation.repository_code"                
      [147] "preferred_citation.repository_artifact"            
      [148] "preferred_citation.scope"                          
      [149] "preferred_citation.section"                        
      [150] "preferred_citation.status"                         
      [151] "preferred_citation.start"                          
      [152] "preferred_citation.thesis_type"                    
      [153] "preferred_citation.url"                            
      [154] "preferred_citation.version"                        
      [155] "preferred_citation.volume"                         
      [156] "preferred_citation.volume_title"                   
      [157] "preferred_citation.year"                           
      [158] "preferred_citation.year_original"                  
      [159] "preferred_citation.conference.name"                
      [160] "preferred_citation.conference.address"             
      [161] "preferred_citation.conference.city"                
      [162] "preferred_citation.conference.region"              
      [163] "preferred_citation.conference.post_code"           
      [164] "preferred_citation.conference.country"             
      [165] "preferred_citation.conference.orcid"               
      [166] "preferred_citation.conference.email"               
      [167] "preferred_citation.conference.tel"                 
      [168] "preferred_citation.conference.fax"                 
      [169] "preferred_citation.conference.website"             
      [170] "preferred_citation.conference.date_start"          
      [171] "preferred_citation.conference.date_end"            
      [172] "preferred_citation.conference.location"            
      [173] "preferred_citation.authors.00.family_names"        
      [174] "preferred_citation.authors.00.given_names"         
      [175] "preferred_citation.authors.00.name_particle"       
      [176] "preferred_citation.authors.00.name_suffix"         
      [177] "preferred_citation.authors.00.alias"               
      [178] "preferred_citation.authors.00.affiliation"         
      [179] "preferred_citation.authors.00.address"             
      [180] "preferred_citation.authors.00.city"                
      [181] "preferred_citation.authors.00.region"              
      [182] "preferred_citation.authors.00.post_code"           
      [183] "preferred_citation.authors.00.country"             
      [184] "preferred_citation.authors.00.orcid"               
      [185] "preferred_citation.authors.00.email"               
      [186] "preferred_citation.authors.00.tel"                 
      [187] "preferred_citation.authors.00.fax"                 
      [188] "preferred_citation.authors.00.website"             
      [189] "preferred_citation.authors.01.name"                
      [190] "preferred_citation.authors.01.address"             
      [191] "preferred_citation.authors.01.city"                
      [192] "preferred_citation.authors.01.region"              
      [193] "preferred_citation.authors.01.post_code"           
      [194] "preferred_citation.authors.01.country"             
      [195] "preferred_citation.authors.01.orcid"               
      [196] "preferred_citation.authors.01.email"               
      [197] "preferred_citation.authors.01.tel"                 
      [198] "preferred_citation.authors.01.fax"                 
      [199] "preferred_citation.authors.01.website"             
      [200] "preferred_citation.authors.01.date_start"          
      [201] "preferred_citation.authors.01.date_end"            
      [202] "preferred_citation.authors.01.location"            
      [203] "preferred_citation.contact.00.family_names"        
      [204] "preferred_citation.contact.00.given_names"         
      [205] "preferred_citation.contact.00.name_particle"       
      [206] "preferred_citation.contact.00.name_suffix"         
      [207] "preferred_citation.contact.00.alias"               
      [208] "preferred_citation.contact.00.affiliation"         
      [209] "preferred_citation.contact.00.address"             
      [210] "preferred_citation.contact.00.city"                
      [211] "preferred_citation.contact.00.region"              
      [212] "preferred_citation.contact.00.post_code"           
      [213] "preferred_citation.contact.00.country"             
      [214] "preferred_citation.contact.00.orcid"               
      [215] "preferred_citation.contact.00.email"               
      [216] "preferred_citation.contact.00.tel"                 
      [217] "preferred_citation.contact.00.fax"                 
      [218] "preferred_citation.contact.00.website"             
      [219] "preferred_citation.contact.01.name"                
      [220] "preferred_citation.contact.01.address"             
      [221] "preferred_citation.contact.01.city"                
      [222] "preferred_citation.contact.01.region"              
      [223] "preferred_citation.contact.01.post_code"           
      [224] "preferred_citation.contact.01.country"             
      [225] "preferred_citation.contact.01.orcid"               
      [226] "preferred_citation.contact.01.email"               
      [227] "preferred_citation.contact.01.tel"                 
      [228] "preferred_citation.contact.01.fax"                 
      [229] "preferred_citation.contact.01.website"             
      [230] "preferred_citation.contact.01.date_start"          
      [231] "preferred_citation.contact.01.date_end"            
      [232] "preferred_citation.contact.01.location"            
      [233] "preferred_citation.database_provider.name"         
      [234] "preferred_citation.database_provider.address"      
      [235] "preferred_citation.database_provider.city"         
      [236] "preferred_citation.database_provider.region"       
      [237] "preferred_citation.database_provider.post_code"    
      [238] "preferred_citation.database_provider.country"      
      [239] "preferred_citation.database_provider.orcid"        
      [240] "preferred_citation.database_provider.email"        
      [241] "preferred_citation.database_provider.tel"          
      [242] "preferred_citation.database_provider.fax"          
      [243] "preferred_citation.database_provider.website"      
      [244] "preferred_citation.database_provider.date_start"   
      [245] "preferred_citation.database_provider.date_end"     
      [246] "preferred_citation.database_provider.location"     
      [247] "preferred_citation.editors.00.family_names"        
      [248] "preferred_citation.editors.00.given_names"         
      [249] "preferred_citation.editors.00.name_particle"       
      [250] "preferred_citation.editors.00.name_suffix"         
      [251] "preferred_citation.editors.00.alias"               
      [252] "preferred_citation.editors.00.affiliation"         
      [253] "preferred_citation.editors.00.address"             
      [254] "preferred_citation.editors.00.city"                
      [255] "preferred_citation.editors.00.region"              
      [256] "preferred_citation.editors.00.post_code"           
      [257] "preferred_citation.editors.00.country"             
      [258] "preferred_citation.editors.00.orcid"               
      [259] "preferred_citation.editors.00.email"               
      [260] "preferred_citation.editors.00.tel"                 
      [261] "preferred_citation.editors.00.fax"                 
      [262] "preferred_citation.editors.00.website"             
      [263] "preferred_citation.editors.01.name"                
      [264] "preferred_citation.editors.01.address"             
      [265] "preferred_citation.editors.01.city"                
      [266] "preferred_citation.editors.01.region"              
      [267] "preferred_citation.editors.01.post_code"           
      [268] "preferred_citation.editors.01.country"             
      [269] "preferred_citation.editors.01.orcid"               
      [270] "preferred_citation.editors.01.email"               
      [271] "preferred_citation.editors.01.tel"                 
      [272] "preferred_citation.editors.01.fax"                 
      [273] "preferred_citation.editors.01.website"             
      [274] "preferred_citation.editors.01.date_start"          
      [275] "preferred_citation.editors.01.date_end"            
      [276] "preferred_citation.editors.01.location"            
      [277] "preferred_citation.editors_series.00.family_names" 
      [278] "preferred_citation.editors_series.00.given_names"  
      [279] "preferred_citation.editors_series.00.name_particle"
      [280] "preferred_citation.editors_series.00.name_suffix"  
      [281] "preferred_citation.editors_series.00.alias"        
      [282] "preferred_citation.editors_series.00.affiliation"  
      [283] "preferred_citation.editors_series.00.address"      
      [284] "preferred_citation.editors_series.00.city"         
      [285] "preferred_citation.editors_series.00.region"       
      [286] "preferred_citation.editors_series.00.post_code"    
      [287] "preferred_citation.editors_series.00.country"      
      [288] "preferred_citation.editors_series.00.orcid"        
      [289] "preferred_citation.editors_series.00.email"        
      [290] "preferred_citation.editors_series.00.tel"          
      [291] "preferred_citation.editors_series.00.fax"          
      [292] "preferred_citation.editors_series.00.website"      
      [293] "preferred_citation.editors_series.01.name"         
      [294] "preferred_citation.editors_series.01.address"      
      [295] "preferred_citation.editors_series.01.city"         
      [296] "preferred_citation.editors_series.01.region"       
      [297] "preferred_citation.editors_series.01.post_code"    
      [298] "preferred_citation.editors_series.01.country"      
      [299] "preferred_citation.editors_series.01.orcid"        
      [300] "preferred_citation.editors_series.01.email"        
      [301] "preferred_citation.editors_series.01.tel"          
      [302] "preferred_citation.editors_series.01.fax"          
      [303] "preferred_citation.editors_series.01.website"      
      [304] "preferred_citation.editors_series.01.date_start"   
      [305] "preferred_citation.editors_series.01.date_end"     
      [306] "preferred_citation.editors_series.01.location"     
      [307] "preferred_citation.institution.name"               
      [308] "preferred_citation.institution.address"            
      [309] "preferred_citation.institution.city"               
      [310] "preferred_citation.institution.region"             
      [311] "preferred_citation.institution.post_code"          
      [312] "preferred_citation.institution.country"            
      [313] "preferred_citation.institution.orcid"              
      [314] "preferred_citation.institution.email"              
      [315] "preferred_citation.institution.tel"                
      [316] "preferred_citation.institution.fax"                
      [317] "preferred_citation.institution.website"            
      [318] "preferred_citation.institution.date_start"         
      [319] "preferred_citation.institution.date_end"           
      [320] "preferred_citation.institution.location"           
      [321] "preferred_citation.location.name"                  
      [322] "preferred_citation.location.address"               
      [323] "preferred_citation.location.city"                  
      [324] "preferred_citation.location.region"                
      [325] "preferred_citation.location.post_code"             
      [326] "preferred_citation.location.country"               
      [327] "preferred_citation.location.orcid"                 
      [328] "preferred_citation.location.email"                 
      [329] "preferred_citation.location.tel"                   
      [330] "preferred_citation.location.fax"                   
      [331] "preferred_citation.location.website"               
      [332] "preferred_citation.location.date_start"            
      [333] "preferred_citation.location.date_end"              
      [334] "preferred_citation.location.location"              
      [335] "preferred_citation.publisher.name"                 
      [336] "preferred_citation.publisher.address"              
      [337] "preferred_citation.publisher.city"                 
      [338] "preferred_citation.publisher.region"               
      [339] "preferred_citation.publisher.post_code"            
      [340] "preferred_citation.publisher.country"              
      [341] "preferred_citation.publisher.orcid"                
      [342] "preferred_citation.publisher.email"                
      [343] "preferred_citation.publisher.tel"                  
      [344] "preferred_citation.publisher.fax"                  
      [345] "preferred_citation.publisher.website"              
      [346] "preferred_citation.publisher.date_start"           
      [347] "preferred_citation.publisher.date_end"             
      [348] "preferred_citation.publisher.location"             
      [349] "preferred_citation.recipients.00.family_names"     
      [350] "preferred_citation.recipients.00.given_names"      
      [351] "preferred_citation.recipients.00.name_particle"    
      [352] "preferred_citation.recipients.00.name_suffix"      
      [353] "preferred_citation.recipients.00.alias"            
      [354] "preferred_citation.recipients.00.affiliation"      
      [355] "preferred_citation.recipients.00.address"          
      [356] "preferred_citation.recipients.00.city"             
      [357] "preferred_citation.recipients.00.region"           
      [358] "preferred_citation.recipients.00.post_code"        
      [359] "preferred_citation.recipients.00.country"          
      [360] "preferred_citation.recipients.00.orcid"            
      [361] "preferred_citation.recipients.00.email"            
      [362] "preferred_citation.recipients.00.tel"              
      [363] "preferred_citation.recipients.00.fax"              
      [364] "preferred_citation.recipients.00.website"          
      [365] "preferred_citation.recipients.01.name"             
      [366] "preferred_citation.recipients.01.address"          
      [367] "preferred_citation.recipients.01.city"             
      [368] "preferred_citation.recipients.01.region"           
      [369] "preferred_citation.recipients.01.post_code"        
      [370] "preferred_citation.recipients.01.country"          
      [371] "preferred_citation.recipients.01.orcid"            
      [372] "preferred_citation.recipients.01.email"            
      [373] "preferred_citation.recipients.01.tel"              
      [374] "preferred_citation.recipients.01.fax"              
      [375] "preferred_citation.recipients.01.website"          
      [376] "preferred_citation.recipients.01.date_start"       
      [377] "preferred_citation.recipients.01.date_end"         
      [378] "preferred_citation.recipients.01.location"         
      [379] "preferred_citation.senders.00.family_names"        
      [380] "preferred_citation.senders.00.given_names"         
      [381] "preferred_citation.senders.00.name_particle"       
      [382] "preferred_citation.senders.00.name_suffix"         
      [383] "preferred_citation.senders.00.alias"               
      [384] "preferred_citation.senders.00.affiliation"         
      [385] "preferred_citation.senders.00.address"             
      [386] "preferred_citation.senders.00.city"                
      [387] "preferred_citation.senders.00.region"              
      [388] "preferred_citation.senders.00.post_code"           
      [389] "preferred_citation.senders.00.country"             
      [390] "preferred_citation.senders.00.orcid"               
      [391] "preferred_citation.senders.00.email"               
      [392] "preferred_citation.senders.00.tel"                 
      [393] "preferred_citation.senders.00.fax"                 
      [394] "preferred_citation.senders.00.website"             
      [395] "preferred_citation.senders.01.name"                
      [396] "preferred_citation.senders.01.address"             
      [397] "preferred_citation.senders.01.city"                
      [398] "preferred_citation.senders.01.region"              
      [399] "preferred_citation.senders.01.post_code"           
      [400] "preferred_citation.senders.01.country"             
      [401] "preferred_citation.senders.01.orcid"               
      [402] "preferred_citation.senders.01.email"               
      [403] "preferred_citation.senders.01.tel"                 
      [404] "preferred_citation.senders.01.fax"                 
      [405] "preferred_citation.senders.01.website"             
      [406] "preferred_citation.senders.01.date_start"          
      [407] "preferred_citation.senders.01.date_end"            
      [408] "preferred_citation.senders.01.location"            
      [409] "preferred_citation.translators.00.family_names"    
      [410] "preferred_citation.translators.00.given_names"     
      [411] "preferred_citation.translators.00.name_particle"   
      [412] "preferred_citation.translators.00.name_suffix"     
      [413] "preferred_citation.translators.00.alias"           
      [414] "preferred_citation.translators.00.affiliation"     
      [415] "preferred_citation.translators.00.address"         
      [416] "preferred_citation.translators.00.city"            
      [417] "preferred_citation.translators.00.region"          
      [418] "preferred_citation.translators.00.post_code"       
      [419] "preferred_citation.translators.00.country"         
      [420] "preferred_citation.translators.00.orcid"           
      [421] "preferred_citation.translators.00.email"           
      [422] "preferred_citation.translators.00.tel"             
      [423] "preferred_citation.translators.00.fax"             
      [424] "preferred_citation.translators.00.website"         
      [425] "preferred_citation.translators.01.name"            
      [426] "preferred_citation.translators.01.address"         
      [427] "preferred_citation.translators.01.city"            
      [428] "preferred_citation.translators.01.region"          
      [429] "preferred_citation.translators.01.post_code"       
      [430] "preferred_citation.translators.01.country"         
      [431] "preferred_citation.translators.01.orcid"           
      [432] "preferred_citation.translators.01.email"           
      [433] "preferred_citation.translators.01.tel"             
      [434] "preferred_citation.translators.01.fax"             
      [435] "preferred_citation.translators.01.website"         
      [436] "preferred_citation.translators.01.date_start"      
      [437] "preferred_citation.translators.01.date_end"        
      [438] "preferred_citation.translators.01.location"        
      [439] "references.00.type"                                
      [440] "references.00.title"                               
      [441] "references.00.abbreviation"                        
      [442] "references.00.abstract"                            
      [443] "references.00.collection_doi"                      
      [444] "references.00.collection_title"                    
      [445] "references.00.collection_type"                     
      [446] "references.00.commit"                              
      [447] "references.00.copyright"                           
      [448] "references.00.data_type"                           
      [449] "references.00.database"                            
      [450] "references.00.date_accessed"                       
      [451] "references.00.date_downloaded"                     
      [452] "references.00.date_released"                       
      [453] "references.00.date_published"                      
      [454] "references.00.department"                          
      [455] "references.00.doi"                                 
      [456] "references.00.edition"                             
      [457] "references.00.end"                                 
      [458] "references.00.entry"                               
      [459] "references.00.filename"                            
      [460] "references.00.format"                              
      [461] "references.00.identifiers.00.type"                 
      [462] "references.00.identifiers.00.value"                
      [463] "references.00.identifiers.01.type"                 
      [464] "references.00.identifiers.01.value"                
      [465] "references.00.identifiers.02.type"                 
      [466] "references.00.identifiers.02.value"                
      [467] "references.00.identifiers.03.type"                 
      [468] "references.00.identifiers.03.value"                
      [469] "references.00.isbn"                                
      [470] "references.00.issn"                                
      [471] "references.00.issue"                               
      [472] "references.00.issue_date"                          
      [473] "references.00.issue_title"                         
      [474] "references.00.journal"                             
      [475] "references.00.keywords.00"                         
      [476] "references.00.keywords.01"                         
      [477] "references.00.languages.00"                        
      [478] "references.00.languages.01"                        
      [479] "references.00.license"                             
      [480] "references.00.license_url"                         
      [481] "references.00.loc_start"                           
      [482] "references.00.loc_end"                             
      [483] "references.00.medium"                              
      [484] "references.00.month"                               
      [485] "references.00.nihmsid"                             
      [486] "references.00.notes"                               
      [487] "references.00.number"                              
      [488] "references.00.number_volumes"                      
      [489] "references.00.pages"                               
      [490] "references.00.patent_states.00"                    
      [491] "references.00.patent_states.01"                    
      [492] "references.00.patent_states.02"                    
      [493] "references.00.patent_states.03"                    
      [494] "references.00.patent_states.04"                    
      [495] "references.00.pmcid"                               
      [496] "references.00.repository"                          
      [497] "references.00.repository_code"                     
      [498] "references.00.repository_artifact"                 
      [499] "references.00.scope"                               
      [500] "references.00.section"                             
      [501] "references.00.status"                              
      [502] "references.00.start"                               
      [503] "references.00.thesis_type"                         
      [504] "references.00.url"                                 
      [505] "references.00.version"                             
      [506] "references.00.volume"                              
      [507] "references.00.volume_title"                        
      [508] "references.00.year"                                
      [509] "references.00.year_original"                       
      [510] "references.00.conference.name"                     
      [511] "references.00.conference.address"                  
      [512] "references.00.conference.city"                     
      [513] "references.00.conference.region"                   
      [514] "references.00.conference.post_code"                
      [515] "references.00.conference.country"                  
      [516] "references.00.conference.orcid"                    
      [517] "references.00.conference.email"                    
      [518] "references.00.conference.tel"                      
      [519] "references.00.conference.fax"                      
      [520] "references.00.conference.website"                  
      [521] "references.00.conference.date_start"               
      [522] "references.00.conference.date_end"                 
      [523] "references.00.conference.location"                 
      [524] "references.00.authors.00.family_names"             
      [525] "references.00.authors.00.given_names"              
      [526] "references.00.authors.00.name_particle"            
      [527] "references.00.authors.00.name_suffix"              
      [528] "references.00.authors.00.alias"                    
      [529] "references.00.authors.00.affiliation"              
      [530] "references.00.authors.00.address"                  
      [531] "references.00.authors.00.city"                     
      [532] "references.00.authors.00.region"                   
      [533] "references.00.authors.00.post_code"                
      [534] "references.00.authors.00.country"                  
      [535] "references.00.authors.00.orcid"                    
      [536] "references.00.authors.00.email"                    
      [537] "references.00.authors.00.tel"                      
      [538] "references.00.authors.00.fax"                      
      [539] "references.00.authors.00.website"                  
      [540] "references.00.authors.01.name"                     
      [541] "references.00.authors.01.address"                  
      [542] "references.00.authors.01.city"                     
      [543] "references.00.authors.01.region"                   
      [544] "references.00.authors.01.post_code"                
      [545] "references.00.authors.01.country"                  
      [546] "references.00.authors.01.orcid"                    
      [547] "references.00.authors.01.email"                    
      [548] "references.00.authors.01.tel"                      
      [549] "references.00.authors.01.fax"                      
      [550] "references.00.authors.01.website"                  
      [551] "references.00.authors.01.date_start"               
      [552] "references.00.authors.01.date_end"                 
      [553] "references.00.authors.01.location"                 
      [554] "references.00.contact.00.family_names"             
      [555] "references.00.contact.00.given_names"              
      [556] "references.00.contact.00.name_particle"            
      [557] "references.00.contact.00.name_suffix"              
      [558] "references.00.contact.00.alias"                    
      [559] "references.00.contact.00.affiliation"              
      [560] "references.00.contact.00.address"                  
      [561] "references.00.contact.00.city"                     
      [562] "references.00.contact.00.region"                   
      [563] "references.00.contact.00.post_code"                
      [564] "references.00.contact.00.country"                  
      [565] "references.00.contact.00.orcid"                    
      [566] "references.00.contact.00.email"                    
      [567] "references.00.contact.00.tel"                      
      [568] "references.00.contact.00.fax"                      
      [569] "references.00.contact.00.website"                  
      [570] "references.00.contact.01.name"                     
      [571] "references.00.contact.01.address"                  
      [572] "references.00.contact.01.city"                     
      [573] "references.00.contact.01.region"                   
      [574] "references.00.contact.01.post_code"                
      [575] "references.00.contact.01.country"                  
      [576] "references.00.contact.01.orcid"                    
      [577] "references.00.contact.01.email"                    
      [578] "references.00.contact.01.tel"                      
      [579] "references.00.contact.01.fax"                      
      [580] "references.00.contact.01.website"                  
      [581] "references.00.contact.01.date_start"               
      [582] "references.00.contact.01.date_end"                 
      [583] "references.00.contact.01.location"                 
      [584] "references.00.database_provider.name"              
      [585] "references.00.database_provider.address"           
      [586] "references.00.database_provider.city"              
      [587] "references.00.database_provider.region"            
      [588] "references.00.database_provider.post_code"         
      [589] "references.00.database_provider.country"           
      [590] "references.00.database_provider.orcid"             
      [591] "references.00.database_provider.email"             
      [592] "references.00.database_provider.tel"               
      [593] "references.00.database_provider.fax"               
      [594] "references.00.database_provider.website"           
      [595] "references.00.database_provider.date_start"        
      [596] "references.00.database_provider.date_end"          
      [597] "references.00.database_provider.location"          
      [598] "references.00.editors.00.family_names"             
      [599] "references.00.editors.00.given_names"              
      [600] "references.00.editors.00.name_particle"            
      [601] "references.00.editors.00.name_suffix"              
      [602] "references.00.editors.00.alias"                    
      [603] "references.00.editors.00.affiliation"              
      [604] "references.00.editors.00.address"                  
      [605] "references.00.editors.00.city"                     
      [606] "references.00.editors.00.region"                   
      [607] "references.00.editors.00.post_code"                
      [608] "references.00.editors.00.country"                  
      [609] "references.00.editors.00.orcid"                    
      [610] "references.00.editors.00.email"                    
      [611] "references.00.editors.00.tel"                      
      [612] "references.00.editors.00.fax"                      
      [613] "references.00.editors.00.website"                  
      [614] "references.00.editors.01.name"                     
      [615] "references.00.editors.01.address"                  
      [616] "references.00.editors.01.city"                     
      [617] "references.00.editors.01.region"                   
      [618] "references.00.editors.01.post_code"                
      [619] "references.00.editors.01.country"                  
      [620] "references.00.editors.01.orcid"                    
      [621] "references.00.editors.01.email"                    
      [622] "references.00.editors.01.tel"                      
      [623] "references.00.editors.01.fax"                      
      [624] "references.00.editors.01.website"                  
      [625] "references.00.editors.01.date_start"               
      [626] "references.00.editors.01.date_end"                 
      [627] "references.00.editors.01.location"                 
      [628] "references.00.editors_series.00.family_names"      
      [629] "references.00.editors_series.00.given_names"       
      [630] "references.00.editors_series.00.name_particle"     
      [631] "references.00.editors_series.00.name_suffix"       
      [632] "references.00.editors_series.00.alias"             
      [633] "references.00.editors_series.00.affiliation"       
      [634] "references.00.editors_series.00.address"           
      [635] "references.00.editors_series.00.city"              
      [636] "references.00.editors_series.00.region"            
      [637] "references.00.editors_series.00.post_code"         
      [638] "references.00.editors_series.00.country"           
      [639] "references.00.editors_series.00.orcid"             
      [640] "references.00.editors_series.00.email"             
      [641] "references.00.editors_series.00.tel"               
      [642] "references.00.editors_series.00.fax"               
      [643] "references.00.editors_series.00.website"           
      [644] "references.00.editors_series.01.name"              
      [645] "references.00.editors_series.01.address"           
      [646] "references.00.editors_series.01.city"              
      [647] "references.00.editors_series.01.region"            
      [648] "references.00.editors_series.01.post_code"         
      [649] "references.00.editors_series.01.country"           
      [650] "references.00.editors_series.01.orcid"             
      [651] "references.00.editors_series.01.email"             
      [652] "references.00.editors_series.01.tel"               
      [653] "references.00.editors_series.01.fax"               
      [654] "references.00.editors_series.01.website"           
      [655] "references.00.editors_series.01.date_start"        
      [656] "references.00.editors_series.01.date_end"          
      [657] "references.00.editors_series.01.location"          
      [658] "references.00.institution.name"                    
      [659] "references.00.institution.address"                 
      [660] "references.00.institution.city"                    
      [661] "references.00.institution.region"                  
      [662] "references.00.institution.post_code"               
      [663] "references.00.institution.country"                 
      [664] "references.00.institution.orcid"                   
      [665] "references.00.institution.email"                   
      [666] "references.00.institution.tel"                     
      [667] "references.00.institution.fax"                     
      [668] "references.00.institution.website"                 
      [669] "references.00.institution.date_start"              
      [670] "references.00.institution.date_end"                
      [671] "references.00.institution.location"                
      [672] "references.00.location.name"                       
      [673] "references.00.location.address"                    
      [674] "references.00.location.city"                       
      [675] "references.00.location.region"                     
      [676] "references.00.location.post_code"                  
      [677] "references.00.location.country"                    
      [678] "references.00.location.orcid"                      
      [679] "references.00.location.email"                      
      [680] "references.00.location.tel"                        
      [681] "references.00.location.fax"                        
      [682] "references.00.location.website"                    
      [683] "references.00.location.date_start"                 
      [684] "references.00.location.date_end"                   
      [685] "references.00.location.location"                   
      [686] "references.00.publisher.name"                      
      [687] "references.00.publisher.address"                   
      [688] "references.00.publisher.city"                      
      [689] "references.00.publisher.region"                    
      [690] "references.00.publisher.post_code"                 
      [691] "references.00.publisher.country"                   
      [692] "references.00.publisher.orcid"                     
      [693] "references.00.publisher.email"                     
      [694] "references.00.publisher.tel"                       
      [695] "references.00.publisher.fax"                       
      [696] "references.00.publisher.website"                   
      [697] "references.00.publisher.date_start"                
      [698] "references.00.publisher.date_end"                  
      [699] "references.00.publisher.location"                  
      [700] "references.00.recipients.00.family_names"          
      [701] "references.00.recipients.00.given_names"           
      [702] "references.00.recipients.00.name_particle"         
      [703] "references.00.recipients.00.name_suffix"           
      [704] "references.00.recipients.00.alias"                 
      [705] "references.00.recipients.00.affiliation"           
      [706] "references.00.recipients.00.address"               
      [707] "references.00.recipients.00.city"                  
      [708] "references.00.recipients.00.region"                
      [709] "references.00.recipients.00.post_code"             
      [710] "references.00.recipients.00.country"               
      [711] "references.00.recipients.00.orcid"                 
      [712] "references.00.recipients.00.email"                 
      [713] "references.00.recipients.00.tel"                   
      [714] "references.00.recipients.00.fax"                   
      [715] "references.00.recipients.00.website"               
      [716] "references.00.recipients.01.name"                  
      [717] "references.00.recipients.01.address"               
      [718] "references.00.recipients.01.city"                  
      [719] "references.00.recipients.01.region"                
      [720] "references.00.recipients.01.post_code"             
      [721] "references.00.recipients.01.country"               
      [722] "references.00.recipients.01.orcid"                 
      [723] "references.00.recipients.01.email"                 
      [724] "references.00.recipients.01.tel"                   
      [725] "references.00.recipients.01.fax"                   
      [726] "references.00.recipients.01.website"               
      [727] "references.00.recipients.01.date_start"            
      [728] "references.00.recipients.01.date_end"              
      [729] "references.00.recipients.01.location"              
      [730] "references.00.senders.00.family_names"             
      [731] "references.00.senders.00.given_names"              
      [732] "references.00.senders.00.name_particle"            
      [733] "references.00.senders.00.name_suffix"              
      [734] "references.00.senders.00.alias"                    
      [735] "references.00.senders.00.affiliation"              
      [736] "references.00.senders.00.address"                  
      [737] "references.00.senders.00.city"                     
      [738] "references.00.senders.00.region"                   
      [739] "references.00.senders.00.post_code"                
      [740] "references.00.senders.00.country"                  
      [741] "references.00.senders.00.orcid"                    
      [742] "references.00.senders.00.email"                    
      [743] "references.00.senders.00.tel"                      
      [744] "references.00.senders.00.fax"                      
      [745] "references.00.senders.00.website"                  
      [746] "references.00.senders.01.name"                     
      [747] "references.00.senders.01.address"                  
      [748] "references.00.senders.01.city"                     
      [749] "references.00.senders.01.region"                   
      [750] "references.00.senders.01.post_code"                
      [751] "references.00.senders.01.country"                  
      [752] "references.00.senders.01.orcid"                    
      [753] "references.00.senders.01.email"                    
      [754] "references.00.senders.01.tel"                      
      [755] "references.00.senders.01.fax"                      
      [756] "references.00.senders.01.website"                  
      [757] "references.00.senders.01.date_start"               
      [758] "references.00.senders.01.date_end"                 
      [759] "references.00.senders.01.location"                 
      [760] "references.00.translators.00.family_names"         
      [761] "references.00.translators.00.given_names"          
      [762] "references.00.translators.00.name_particle"        
      [763] "references.00.translators.00.name_suffix"          
      [764] "references.00.translators.00.alias"                
      [765] "references.00.translators.00.affiliation"          
      [766] "references.00.translators.00.address"              
      [767] "references.00.translators.00.city"                 
      [768] "references.00.translators.00.region"               
      [769] "references.00.translators.00.post_code"            
      [770] "references.00.translators.00.country"              
      [771] "references.00.translators.00.orcid"                
      [772] "references.00.translators.00.email"                
      [773] "references.00.translators.00.tel"                  
      [774] "references.00.translators.00.fax"                  
      [775] "references.00.translators.00.website"              
      [776] "references.00.translators.01.name"                 
      [777] "references.00.translators.01.address"              
      [778] "references.00.translators.01.city"                 
      [779] "references.00.translators.01.region"               
      [780] "references.00.translators.01.post_code"            
      [781] "references.00.translators.01.country"              
      [782] "references.00.translators.01.orcid"                
      [783] "references.00.translators.01.email"                
      [784] "references.00.translators.01.tel"                  
      [785] "references.00.translators.01.fax"                  
      [786] "references.00.translators.01.website"              
      [787] "references.00.translators.01.date_start"           
      [788] "references.00.translators.01.date_end"             
      [789] "references.00.translators.01.location"             

# Convert a citation only

    Code
      names(the_df)
    Output
       [1] "references.00.type"                   
       [2] "references.00.title"                  
       [3] "references.00.authors.00.family_names"
       [4] "references.00.authors.00.given_names" 
       [5] "references.00.authors.00.email"       
       [6] "references.00.authors.00.orcid"       
       [7] "references.00.authors.01.family_names"
       [8] "references.00.authors.01.given_names" 
       [9] "references.00.authors.01.affiliation" 
      [10] "references.00.authors.01.country"     
      [11] "references.00.authors.02.family_names"
      [12] "references.00.authors.02.given_names" 
      [13] "references.00.authors.02.email"       
      [14] "references.00.authors.03.name"        
      [15] "references.00.authors.03.date_end"    
      [16] "references.00.url"                    
      [17] "references.00.abstract"               
      [18] "references.00.keywords.00"            
      [19] "references.00.keywords.01"            
      [20] "references.00.keywords.02"            
      [21] "references.00.keywords.03"            
      [22] "references.00.keywords.04"            
      [23] "references.00.keywords.05"            
      [24] "references.00.keywords.06"            
      [25] "references.00.keywords.07"            
      [26] "references.00.version"                

# Convert authors only

    Code
      names(the_df)
    Output
      [1] "family_names" "given_names" 

# Convert list of authors

    Code
      names(the_df)
    Output
       [1] "person.00.family_names"  "person.00.given_names"  
       [3] "person.00.name_particle" "person.00.name_suffix"  
       [5] "person.00.alias"         "person.00.affiliation"  
       [7] "person.00.address"       "person.00.city"         
       [9] "person.00.region"        "person.00.post_code"    
      [11] "person.00.country"       "person.00.orcid"        
      [13] "person.00.email"         "person.00.tel"          
      [15] "person.00.fax"           "person.00.website"      
      [17] "person.01.name"          "person.01.address"      
      [19] "person.01.city"          "person.01.region"       
      [21] "person.01.post_code"     "person.01.country"      
      [23] "person.01.orcid"         "person.01.email"        
      [25] "person.01.tel"           "person.01.fax"          
      [27] "person.01.website"       "person.01.date_start"   
      [29] "person.01.date_end"      "person.01.location"     

# as.person method

    Code
      dput(pub)
    Output
      structure(list(list(given = "Entity Project Team Conference entity", 
          family = NULL, role = NULL, email = "project@entity.com", 
          comment = c(address = "22 Acacia Avenue", city = "Citationburgh", 
          region = "Renfrewshire", `post-code` = "C13 7X7", country = "GB", 
          ORCID = "0000-0001-2345-6789", tel = "+44(0)141-323 4567", 
          fax = "+44(0)141-323 45678", website = "https://www.entity-project-team.io", 
          `date-start` = "2017-01-01", `date-end` = "2017-01-31", location = "The team garage"
          ))), class = "person")

---

    Code
      format(pub, include = c("given", "family", "email", "role", "comment"))
    Output
      [1] "Entity Project Team Conference entity <project@entity.com> (address: 22 Acacia Avenue, city: Citationburgh, region: Renfrewshire, post-code: C13 7X7, country: GB, ORCID: <https://orcid.org/0000-0001-2345-6789>, tel: +44(0)141-323 4567, fax: +44(0)141-323 45678, website: https://www.entity-project-team.io, date-start: 2017-01-01, date-end: 2017-01-31, location: The team garage)"

---

    Code
      dput(aut)
    Output
      structure(list(list(given = "One Truly", family = "van der Real Person, IV", 
          role = NULL, email = "project@entity.com", comment = c(alias = "Citey", 
          affiliation = "Excellent University, Niceplace, Arcadia", 
          address = "22 Acacia Avenue", city = "Citationburgh", region = "Renfrewshire", 
          `post-code` = "C13 7X7", country = "GB", ORCID = "0000-0001-2345-6789", 
          tel = "+44(0)141-323 4567", fax = "+44(0)141-323 45678", 
          website = "https://www.entity-project-team.io"))), class = "person")

---

    Code
      format(aut, include = c("given", "family", "email", "role", "comment"))
    Output
      [1] "One Truly van der Real Person, IV <project@entity.com> (alias: Citey, affiliation: Excellent University, Niceplace, Arcadia, address: 22 Acacia Avenue, city: Citationburgh, region: Renfrewshire, post-code: C13 7X7, country: GB, ORCID: <https://orcid.org/0000-0001-2345-6789>, tel: +44(0)141-323 4567, fax: +44(0)141-323 45678, website: https://www.entity-project-team.io)"

---

    Code
      dput(aut2)
    Output
      structure(list(list(given = "One Truly", family = "van der Real Person, IV", 
          role = NULL, email = "project@entity.com", comment = c(alias = "Citey", 
          affiliation = "Excellent University, Niceplace, Arcadia", 
          address = "22 Acacia Avenue", city = "Citationburgh", region = "Renfrewshire", 
          `post-code` = "C13 7X7", country = "GB", ORCID = "0000-0001-2345-6789", 
          tel = "+44(0)141-323 4567", fax = "+44(0)141-323 45678", 
          website = "https://www.entity-project-team.io")), list(given = "Entity Project Team Conference entity", 
          family = NULL, role = NULL, email = "project@entity.com", 
          comment = c(address = "22 Acacia Avenue", city = "Citationburgh", 
          region = "Renfrewshire", `post-code` = "C13 7X7", country = "GB", 
          ORCID = "0000-0001-2345-6789", tel = "+44(0)141-323 4567", 
          fax = "+44(0)141-323 45678", website = "https://www.entity-project-team.io", 
          `date-start` = "2017-01-01", `date-end` = "2017-01-31", location = "The team garage"
          ))), class = "person")

---

    Code
      end <- as.person(malf)
    Message
      i Can't create <person> for some elements of `x`.

---

    Code
      aa2 <- as.person(aa)
    Message
      i Removing duplicate <person> objects.

# as.person method names and particles

    Code
      cf
    Output
      - family-names: Wicksteed
        given-names: P. H.
        name-particle: von
        name-suffix: III
      - name: The translator factory

# Errors on other as.person methods

    Code
      as.person(the_cff)
    Condition
      Error in `as.person()`:
      ! (cffr) `as.person.cff()` method not implemented yet.

---

    Code
      as.person(key)
    Condition
      Error in `as.person()`:
      ! (cffr) `as.person.cff_ref_lst()` method not implemented yet.

---

    Code
      as.person(key)
    Condition
      Error in `as.person()`:
      ! (cffr) `as.person.cff_ref()` method not implemented yet.

# head and tail

    Code
      a_cff
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.
      title: My Research Software
      authors:
      - family-names: Doe
        given-names: John

---

    Code
      head(a_cff, 2)
    Output
      cff-version: 1.2.0
      message: If you use this software, please cite it using these metadata.

---

    Code
      tail(a_cff, 2)
    Output
      title: My Research Software
      authors:
      - family-names: Doe
        given-names: John

# toBibtex

    Code
      toBibtex(full_cff)
    Output
      @Misc{druskat_etall:2021,
        title = {Citation File Format},
        author = {Stephan Druskat and Jurriaan H. Spaaks and Neil {Chue Hong} and Robert Haines and James Baker and Spencer Bliven and Egon Willighagen and David Pérez-Suárez and Alexander Konovalov},
        year = {2021},
        month = {aug},
        doi = {10.5281/zenodo.5171937},
        url = {https://github.com/citation-file-format/citation-file-format},
        abstract = {The Citation File Format lets you provide citation metadata for software or datasets in plaintext files that are easy to read by both humans and machines.},
        urldate = {2021-11-07},
      }

---

    Code
      toBibtex(full_cff, what = "references")
    Output
      @Article{hernangomez:2021,
        title = {cffr: Generate Citation File Format Metadata for R Packages},
        author = {Diego Hernangómez},
        year = {2021},
        journal = {Journal of Open Source Software},
        publisher = {The Open Journal},
        volume = {6},
        number = {67},
        pages = {3900},
        doi = {10.21105/joss.03900},
        url = {https://doi.org/10.21105/joss.03900},
        note = {Publisher: The Open Journal},
      }
      
      @Article{boettiger_etall:2016,
        title = {RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic, Character, and Trait Data in R},
        author = {Carl Boettiger and Scott Chamberlain and Rutger Vos and Hilmar Lapp},
        year = {2016},
        journal = {Methods in Ecology and Evolution},
        volume = {7},
        pages = {352--357},
        doi = {10.1111/2041-210X.12469},
      }
      
      @Book{wickham:2016,
        title = {ggplot2: Elegant Graphics for Data Analysis},
        author = {Hadley Wickham},
        year = {2016},
        publisher = {Springer-Verlag New York},
        isbn = {978-3-319-24277-4},
        url = {https://ggplot2.tidyverse.org},
      }

---

    Code
      toBibtex(full_cff, what = "all")
    Output
      @Misc{druskat_etall:2021,
        title = {Citation File Format},
        author = {Stephan Druskat and Jurriaan H. Spaaks and Neil {Chue Hong} and Robert Haines and James Baker and Spencer Bliven and Egon Willighagen and David Pérez-Suárez and Alexander Konovalov},
        year = {2021},
        month = {aug},
        doi = {10.5281/zenodo.5171937},
        url = {https://github.com/citation-file-format/citation-file-format},
        abstract = {The Citation File Format lets you provide citation metadata for software or datasets in plaintext files that are easy to read by both humans and machines.},
        urldate = {2021-11-07},
      }
      
      @Article{hernangomez:2021,
        title = {cffr: Generate Citation File Format Metadata for R Packages},
        author = {Diego Hernangómez},
        year = {2021},
        journal = {Journal of Open Source Software},
        publisher = {The Open Journal},
        volume = {6},
        number = {67},
        pages = {3900},
        doi = {10.21105/joss.03900},
        url = {https://doi.org/10.21105/joss.03900},
        note = {Publisher: The Open Journal},
      }
      
      @Article{boettiger_etall:2016,
        title = {RNeXML: A Package for Reading and Writing Richly Annotated Phylogenetic, Character, and Trait Data in R},
        author = {Carl Boettiger and Scott Chamberlain and Rutger Vos and Hilmar Lapp},
        year = {2016},
        journal = {Methods in Ecology and Evolution},
        volume = {7},
        pages = {352--357},
        doi = {10.1111/2041-210X.12469},
      }
      
      @Book{wickham:2016,
        title = {ggplot2: Elegant Graphics for Data Analysis},
        author = {Hadley Wickham},
        year = {2016},
        publisher = {Springer-Verlag New York},
        isbn = {978-3-319-24277-4},
        url = {https://ggplot2.tidyverse.org},
      }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_read.R
\encoding{UTF-8}
\name{cff_read}
\alias{cff_read}
\alias{cff_read_cff_citation}
\alias{cff_read_description}
\alias{cff_read_citation}
\alias{cff_read_bib}
\title{Read an external file as a \code{\link{cff}} object}
\usage{
cff_read(path, ...)

cff_read_cff_citation(path, ...)

cff_read_description(
  path,
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  authors_roles = c("aut", "cre"),
  ...
)

cff_read_citation(path, meta = NULL, ...)

cff_read_bib(path, encoding = "UTF-8", ...)
}
\arguments{
\item{path}{Path to a file.}

\item{...}{Arguments to be passed to other functions, for example to
\code{\link[yaml:read_yaml]{yaml::read_yaml()}} or \code{\link[bibtex:read.bib]{bibtex::read.bib()}}.}

\item{cff_version}{The Citation File Format schema version that the
\code{CITATION.cff} file adheres to for providing the citation metadata.}

\item{gh_keywords}{Logical \code{TRUE/FALSE}. If the package is hosted on
GitHub, would you like to add the repo topics as keywords?}

\item{authors_roles}{Roles to be considered as authors of the package when
generating the \code{CITATION.cff} file. See \strong{Details}.}

\item{meta}{A list of package metadata as obtained by
\code{\link[utils:packageDescription]{utils::packageDescription()}} or \code{NULL} (the default). See \strong{Details}.}

\item{encoding}{Encoding to be assumed for \code{path}. See \code{\link[=readLines]{readLines()}}.}
}
\value{
\itemize{
\item \code{cff_read_cff_citation()} and \code{cff_read_description()} return an object
with class \code{cff}.
\item \code{cff_read_citation()} and \code{cff_read_bib()} return an object of classes
\code{\link[=cff_ref_lst]{cff_ref_lst, cff}} according to the \code{definitions.references}
specified in the \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Citation File Format schema}.
}

Learn more about the \CRANpkg{cffr} class system in \link{cff_class}.
}
\description{
Read files and convert them to \code{\link{cff}} objects. Files supported
are:
\itemize{
\item \code{CITATION.cff} files.
\item \code{DESCRIPTION} files.
\item \strong{R} citation files (usually located in \code{inst/CITATION}).
\item BibTeX files (with extension \verb{*.bib}).
}

\code{\link[=cff_read]{cff_read()}} attempts to guess the type of file provided in \code{path}. However,
we provide aliases for each specific file type:
\itemize{
\item \code{\link[=cff_read_cff_citation]{cff_read_cff_citation()}}, which uses \code{\link[yaml:read_yaml]{yaml::read_yaml()}}.
\item \code{\link[=cff_read_description]{cff_read_description()}}, which uses \code{\link[desc:desc]{desc::desc()}}.
\item \code{\link[=cff_read_citation]{cff_read_citation()}}, which uses \code{\link[utils:readCitationFile]{utils::readCitationFile()}}.
\item \code{\link[=cff_read_bib]{cff_read_bib()}}, which requires \CRANpkg{bibtex} (>= 0.5.0) and uses
\code{\link[bibtex:read.bib]{bibtex::read.bib()}}.
}
}
\details{
For details of \code{cff_read_description()} see \code{\link[=cff_create]{cff_create()}}.
\subsection{The \code{meta} object}{

Section 1.9 CITATION files of \emph{Writing R Extensions} (R Core Team 2023)
specifies how to create dynamic \code{CITATION} files using \code{meta} object, hence
the \code{meta} argument in \code{\link[=cff_read_citation]{cff_read_citation()}} may be needed for reading
some files correctly.
}
}
\examples{

# Create cff object from cff file

from_cff_file <- cff_read(system.file("examples/CITATION_basic.cff",
  package = "cffr"
))

head(from_cff_file, 7)

# Create cff object from DESCRIPTION
from_desc <- cff_read(system.file("examples/DESCRIPTION_basic",
  package = "cffr"
))

from_desc

# Create cff object from BibTex

if (requireNamespace("bibtex", quietly = TRUE)) {
  from_bib <- cff_read(system.file("examples/example.bib",
    package = "cffr"
  ))

  # First item only
  from_bib[[1]]
}
# Create cff object from CITATION
from_citation <- cff_read(system.file("CITATION", package = "cffr"))

# First item only
from_citation[[1]]

}
\references{
\itemize{
\item R Core Team (2023). \emph{Writing R Extensions}.
\url{https://cran.r-project.org/doc/manuals/r-release/R-exts.html}
\item Hernangomez D (2022). "BibTeX and CFF, a potential crosswalk."
\emph{The cffr package, Vignettes}. \doi{10.21105/joss.03900},
\url{https://docs.ropensci.org/cffr/articles/bibtex-cff.html}.
}
}
\seealso{
The underlying functions used for reading external files:
\itemize{
\item \code{\link[yaml:read_yaml]{yaml::read_yaml()}} for \code{CITATION.cff} files.
\item \code{\link[desc:desc]{desc::desc()}} for \code{DESCRIPTION} files.
\item \code{\link[utils:readCitationFile]{utils::readCitationFile()}} for \strong{R} citation files.
\item \code{\link[bibtex:read.bib]{bibtex::read.bib()}} for BibTeX files (extension \verb{*.bib}).
}

Other functions for reading external files:
\code{\link[=cff_read_bib_text]{cff_read_bib_text()}}

Other functions for working with BibTeX format:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=cff_read_bib_text]{cff_read_bib_text()}},
\code{\link[=cff_write_bib]{cff_write_bib()}},
\code{\link[=encoded_utf_to_latex]{encoded_utf_to_latex()}}
}
\concept{bibtex}
\concept{reading}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cffr-package.R
\docType{package}
\name{cffr-package}
\alias{cffr}
\alias{cffr-package}
\title{cffr: Generate Citation File Format ('cff') Metadata for R Packages}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The Citation File Format version 1.2.0 \doi{10.5281/zenodo.5171937} is a human and machine readable file format which provides citation metadata for software. This package provides core utilities to generate and validate this metadata.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://docs.ropensci.org/cffr/}
  \item \url{https://github.com/ropensci/cffr}
  \item Report bugs at \url{https://github.com/ropensci/cffr/issues}
}

}
\author{
\strong{Maintainer}: Diego Hernangómez \email{diego.hernangomezherrero@gmail.com} (\href{https://orcid.org/0000-0001-8457-4658}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Diego Hernangómez \email{diego.hernangomezherrero@gmail.com} (\href{https://orcid.org/0000-0001-8457-4658}{ORCID}) [copyright holder]
}

Other contributors:
\itemize{
  \item João Martins (\href{https://orcid.org/0000-0001-7961-4280}{ORCID}) [reviewer]
  \item Scott Chamberlain (\href{https://orcid.org/0000-0003-1444-9135}{ORCID}) [reviewer]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\encoding{UTF-8}
\name{cran_to_spdx}
\alias{cran_to_spdx}
\title{Mapping between \code{License} fields and SPDX}
\format{
A data frame with 86 rows and 2 variables:
\describe{
\item{\code{LICENSE}}{A valid \code{License} string on CRAN.}
\item{\code{SPDX}}{A valid SPDX License Identifier.}
}
}
\source{
\url{https://spdx.org/licenses/}
}
\description{
A dataset containing the mapping between the \code{License} strings observed
on CRAN packages and its (approximate) match on the
\href{https://spdx.org/licenses/}{SPDX License List}.
}
\examples{

data("cran_to_spdx")

head(cran_to_spdx, 20)
}
\seealso{
\emph{Writing R Extensions}, \href{https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Licensing}{Licensing section}.
}
\concept{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_cff_person.R
\encoding{UTF-8}
\name{as_cff_person}
\alias{as_cff_person}
\alias{as_cff_person.default}
\alias{as_cff_person.person}
\alias{as_cff_person.character}
\title{Coerce \strong{R} objects to \code{\link{cff_pers_lst}} objects (\code{cff} persons)}
\usage{
as_cff_person(x, ...)

\method{as_cff_person}{default}(x, ...)

\method{as_cff_person}{person}(x, ...)

\method{as_cff_person}{character}(x, ...)
}
\arguments{
\item{x}{Any \strong{R} object.}

\item{...}{Ignored by this method.}
}
\value{
\code{as_cff_person()} returns an object of classes
\code{\link[=cff_pers_lst]{cff_pers_lst, cff}} according to the \code{definitions.person}
or \code{definitions.entity} specified in the \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Citation File Format schema}.
Each element of the \code{cff_pers_lst} object has classes
\code{\link[=cff_pers]{cff_pers, cff}}.
}
\description{
\code{as_cff_person()} turns an existing list-like \strong{R} object into a
\code{\link{cff_pers_lst}} object representing a list of \code{definitions.person} or
\code{definitions.entity}, as defined by the \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Citation File Format schema}.

\code{as_cff_person} is an S3 generic, with methods for:
\itemize{
\item \code{person}: Objects created with \code{\link[=person]{person()}}.
\item \code{character}: Strings with the definition of an author or multiple authors,
using the standard BibTeX notation (see Markey, 2007) and others, such as
the output of \code{\link[=format]{format()}} for person (see \code{\link[=person]{format.person()}}).
\item Default: Other inputs are first coerced with \code{\link[=as.character]{as.character()}}.
}

The inverse transformation (\code{cff_pers_lst} to \code{person}) can be done using
the methods \code{\link[=as.person.cff_pers]{as.person.cff_pers()}} and \code{\link[=as.person.cff_pers_lst]{as.person.cff_pers_lst()}}.
}
\details{
\code{as_cff_person()} recognizes whether the input should be converted using the
CFF reference for \code{definitions.person} or \code{definitions.entity}.

\code{as_cff_person()} uses a custom algorithm that breaks a name as
explained in Section 11 of "Tame the BeaST" (Markey, 2007) (see also
Decoret, 2007):
\itemize{
\item \verb{First von Last}.
\item \verb{von Last, First}.
\item \verb{von Last, Jr, First}.
}

Mapping is performed as follows:
\itemize{
\item \code{First} is mapped to the CFF field \code{given-names}.
\item \code{von} is mapped to the CFF field \code{name-particle}.
\item \code{Last} is mapped to the CFF field \code{family-names}.
\item \code{Jr} is mapped to the CFF field \code{name-suffix}.
}

For entities, the entire \code{character} is mapped to \code{name}.
It is recommended to "protect" entity names with \code{{}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Don't do
entity <- "Elephant and Castle"
as_cff_person(entity)
- name: Elephant
- name: Castle

# Do
entity_protect <- "\{Elephant and Castle\}"
as_cff_person(entity_protect)
- name: Elephant and Castle
}\if{html}{\out{</div>}}

\code{as_cff_person()} attempts to extract as much information as possible.
For \code{character} strings from \code{\link[utils:person]{format.person()}}, the
email and ORCID are also extracted.
}
\examples{
# Create a person object
a_person <- person(
  given = "First", family = "Author",
  role = c("aut", "cre"),
  email = "first.last@example.com", comment = c(
    ORCID = "0000-0001-8457-4658",
    affiliation = "An affiliation"
  )
)

a_person

cff_person <- as_cff_person(a_person)

# Class cff_pers_lst / cff
class(cff_person)

# With each element with class cff_pers / cff
class(cff_person[[1]])

# Print
cff_person

# Back to person object with S3 Method
as.person(cff_person)

# Coerce a string
a_str <- paste0(
  "Julio Iglesias <fake@email.com> ",
  "(city: Miami, region: California, country: US)"
)
as_cff_person(a_str)

# Several persons
persons <- c(
  person("Clark", "Kent", comment = c(affiliation = "Daily Planet")),
  person("Lois", "Lane"), person("Oscorp Inc.")
)

a_cff <- as_cff_person(persons)

a_cff

# Printed as Bibtex thanks to the method
toBibtex(a_cff)

# Or as person object
as.person(a_cff)

# Or you can use BibTeX style as input if you prefer

x <- "Frank Sinatra and Dean Martin and Davis, Jr., Sammy and Joey Bishop"

as_cff_person(x)

as_cff_person("Herbert von Karajan")

toBibtex(as_cff_person("Herbert von Karajan"))
}
\references{
\itemize{
\item Patashnik, Oren. "BIBTEXTING" February 1988.
\url{https://osl.ugr.es/CTAN/biblio/bibtex/base/btxdoc.pdf}.
\item Markey, Nicolas. "Tame the BeaST"
\emph{The B to X of BibTeX, Version 1.4} (October 2007).
\url{https://osl.ugr.es/CTAN/info/bibtex/tamethebeast/ttb_en.pdf}.
\item Decoret X (2007). "A summary of BibTex."\url{https://maverick.inria.fr/~Xavier.Decoret/resources/xdkbibtex/bibtex_summary.html#names}
}

See \strong{Examples} for more information.
}
\seealso{
Examples in \code{vignette("cffr", "cffr")} and \code{\link[utils:person]{utils::person()}}.

Learn more about the \code{cff_pers_lst} and \code{cff_pers} classes in \link{cff_class}.

Coercing between \strong{R} classes with \strong{S3 Methods}:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=as_cff]{as_cff()}},
\code{\link{cff_class}}
}
\concept{s3method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\encoding{UTF-8}
\name{cff_extract_to_bibtex}
\alias{cff_extract_to_bibtex}
\alias{cff_to_bibtex}
\title{Previous API: Create BibTeX entries from several sources}
\usage{
cff_extract_to_bibtex(x, what = c("preferred", "references", "all"))

cff_to_bibtex(x, what = c("preferred", "references", "all"))
}
\arguments{
\item{x}{The source used to generate
the \code{bibentry} object via \CRANpkg{cffr}. It can be:
\itemize{
\item A missing value, which retrieves the \code{DESCRIPTION}
file from your in-development package.
\item An existing \code{cff} object created with \code{\link[=cff]{cff()}}, \code{\link[=cff_create]{cff_create()}}, or
\code{\link[=as_cff]{as_cff()}}.
\item Path to a CITATION.cff file (\code{"CITATION.cff"}).
\item The name of an installed package (\code{"jsonlite"}).
\item Path to a DESCRIPTION file (\code{"DESCRIPTION"}).
}}

\item{what}{Fields to extract from a full \code{cff} object. The value could be:
\itemize{
\item \code{preferred}: This would create a single entry with the main citation
info of the package (key \code{preferred-citation}).
\item \code{references}: Extract all the entries of \code{references} key.
\item \code{all}: A combination of the previous two options. This would extract
both the \code{preferred-citation} and the \code{references} key.
}

See \code{vignette("crosswalk", package = "cffr")}.}
}
\value{
See \code{\link[=as_bibentry]{as_bibentry()}}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Please use \code{\link[=as_bibentry]{as_bibentry()}} instead.
}
\examples{
\donttest{
# From a cff object
cff_object <- cff()

cff_object

# bibentry object
bib <- as_bibentry(cff_object)
}
}
\seealso{
Other deprecated functions:
\code{\link[=cff_from_bibtex]{cff_from_bibtex()}},
\code{\link[=cff_parse_citation]{cff_parse_citation()}},
\code{\link[=cff_parse_person]{cff_parse_person()}},
\code{\link[=write_bib]{write_bib()}}
}
\concept{deprecated}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-schema.R
\encoding{UTF-8}
\name{cff_schema}
\alias{cff_schema}
\alias{cff_schema_keys}
\alias{cff_schema_keys_license}
\alias{cff_schema_definitions_person}
\alias{cff_schema_definitions_entity}
\alias{cff_schema_definitions_refs}
\title{Schema utils}
\source{
\href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Guide to Citation File Format schema version 1.2.0}.
}
\usage{
cff_schema_keys(sorted = FALSE)

cff_schema_keys_license()

cff_schema_definitions_person()

cff_schema_definitions_entity()

cff_schema_definitions_refs()
}
\arguments{
\item{sorted}{Logical \code{TRUE/FALSE}. Should the keys be arranged
alphabetically?}
}
\value{
A character vector with the names of valid keys for Citation File Format
version 1.2.0.
}
\description{
Helper functions with the valid values of different fields, according to the
\href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Citation File Format schema version 1.2.0}.
\itemize{
\item \code{\link[=cff_schema_keys]{cff_schema_keys()}} provides the valid high-level keys of the Citation
File Format.
\item \code{\link[=cff_schema_keys_license]{cff_schema_keys_license()}} provides the valid
\href{https://spdx.org/licenses/}{SPDX license identifier(s)} to be used on the
\code{CITATION.cff} file.
\item \code{\link[=cff_schema_definitions_person]{cff_schema_definitions_person()}} and \code{\link[=cff_schema_definitions_entity]{cff_schema_definitions_entity()}}
return the valid fields to include when defining a
person or entity.
\item \code{\link[=cff_schema_definitions_refs]{cff_schema_definitions_refs()}} provides the valid
keys to be used on the \code{preferred-citation} and \code{references} keys.
}
}
\examples{

cff_schema_keys(sorted = TRUE)

# Valid Licenses keys
head(cff_schema_keys_license(), 20)

cff_schema_definitions_person()

cff_schema_definitions_entity()

cff_schema_definitions_refs()
}
\concept{schemas}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encoded_utf_to_latex.R
\encoding{UTF-8}
\name{encoded_utf_to_latex}
\alias{encoded_utf_to_latex}
\title{Encode UTF-8 text to LaTeX}
\usage{
encoded_utf_to_latex(x)
}
\arguments{
\item{x}{A string, possibly encoded in UTF-8 encoding system.}
}
\value{
A string with the corresponding transformations.
}
\description{
Transform a UTF-8 string into LaTeX special characters.
}
\details{
This is a variation of \code{\link[tools:encoded_text_to_latex]{tools::encoded_text_to_latex()}} with some
additional replacements for better compatibility.
}
\examples{
# Full range of supported characters on R
library(tools)

range <- 1:511

ascii_table <- data.frame(
  dec = range,
  utf8 = intToUtf8(range, multiple = TRUE)
)

# Add latex using base approach
ascii_table$latex_base <- encoded_text_to_latex(ascii_table$utf8,
  encoding = "UTF-8"
)

# With cffr
ascii_table$latex_cffr <- encoded_utf_to_latex(ascii_table$utf8)

ascii_table
}
\seealso{
\code{\link[tools:encoded_text_to_latex]{tools::encoded_text_to_latex()}}

Other functions for working with BibTeX format:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=cff_read]{cff_read()}},
\code{\link[=cff_read_bib_text]{cff_read_bib_text()}},
\code{\link[=cff_write_bib]{cff_write_bib()}}
}
\concept{bibtex}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_create.R
\encoding{UTF-8}
\name{cff_create}
\alias{cff_create}
\title{Create a \code{\link{cff}} object from several sources}
\usage{
cff_create(
  x,
  keys = list(),
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  dependencies = TRUE,
  authors_roles = c("aut", "cre")
)
}
\arguments{
\item{x}{The source used to generate
the \code{\link{cff}} object. It can be:
\itemize{
\item A missing value, which retrieves the \code{DESCRIPTION} file from your
in-development \strong{R} package.
\item An existing \code{\link{cff}} object.
\item The name of an installed package (\code{"jsonlite"}).
\item Path to a \code{DESCRIPTION} file (\code{"./DESCRIPTION"}).
}}

\item{keys}{List of additional keys to add to the \code{\link{cff}} object. See
\code{\link[=cff_modify]{cff_modify()}}.}

\item{cff_version}{The Citation File Format schema version that the
\code{CITATION.cff} file adheres to for providing the citation metadata.}

\item{gh_keywords}{Logical \code{TRUE/FALSE}. If the package is hosted on
GitHub, would you like to add the repo topics as keywords?}

\item{dependencies}{Logical \code{TRUE/FALSE}. Should the dependencies of your
package be added to the \code{references} CFF key?}

\item{authors_roles}{Roles to be considered as authors of the package when
generating the \code{CITATION.cff} file. See \strong{Details}.}
}
\value{
A \code{\link{cff}} object.
}
\description{
Create a full and possibly valid \code{\link{cff}} object from a given source. This
object can be written to a \verb{*.cff} file with \code{\link[=cff_write]{cff_write()}},
see \strong{Examples}.

Most of the heavy lifting of \CRANpkg{cffr} is done by this function.
}
\details{
If \code{x} is a path to a \code{DESCRIPTION} file or if \code{inst/CITATION} is not present
on your package, \CRANpkg{cffr} would auto-generate a \code{preferred-citation}
key using the information provided on that file.

By default, only persons whose role in the \code{DESCRIPTION} file of the package
is author (\code{"aut"}) or maintainer (\code{"cre"}) are considered to be authors
of the package. The default setting can be controlled via the \code{authors_roles}
argument. See \strong{Details} on \code{\link[=person]{person()}} to get additional insights
on person roles.
}
\examples{
\donttest{
# Installed package
cff_create("jsonlite")

# Demo file
demo_file <- system.file("examples/DESCRIPTION_basic", package = "cffr")
cff_create(demo_file)

# Add additional keys

newkeys <- list(
  message = "This overwrites fields",
  abstract = "New abstract",
  keywords = c("A", "new", "list", "of", "keywords"),
  authors = as_cff_person("New author")
)

cff_create(demo_file, keys = newkeys)

# Update a field on a list, for example authors or contacts.
# We are adding a new contact here

old <- cff_create(demo_file)

new_contact <- append(
  old$contact,
  as_cff_person(person(
    given = "I am",
    family = "New Contact"
  ))
)

cff_create(demo_file, keys = list("contact" = new_contact))
}
}
\seealso{
\href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Guide to Citation File Format schema version 1.2.0}.
\itemize{
\item \code{\link[=cff_modify]{cff_modify()}} as the recommended way to modify a \code{cff} object.
\item \code{\link[=cff_write]{cff_write()}} for creating a CFF file.
\item \code{vignette("cffr", "cffr")} shows an introduction on how manipulate
\code{\link{cff}} objects.
\item \code{vignette("crosswalk", package = "cffr")} provides details on how the
metadata of a package is mapped to produce a \code{cff} object.
}

Other core functions of \CRANpkg{cffr}:
\code{\link[=cff]{cff()}},
\code{\link[=cff_modify]{cff_modify()}},
\code{\link[=cff_validate]{cff_validate()}}
}
\concept{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\encoding{UTF-8}
\name{write_bib}
\alias{write_bib}
\alias{write_citation}
\title{Previous API: Write files}
\usage{
write_bib(x, file = tempfile(), append = FALSE, verbose = TRUE, ascii = FALSE)

write_citation(
  x,
  file = "./inst/CITATION",
  append = FALSE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{\link{bibentry}} or a \code{\link{cff}} object.}

\item{file}{Name of the file to be created. If \code{NULL}, the lines are
displayed instead.}

\item{append}{Logical. Should entries be appended to an existing file?}

\item{verbose}{Logical. Display informative messages.}

\item{ascii}{Logical. Should entries be written using ASCII characters only?}
}
\value{
Write a file.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Please use \code{\link[=cff_write_bib]{cff_write_bib()}} or
\code{\link[=cff_write_citation]{cff_write_citation()}} instead.
}
\examples{

bib <- bibentry("Misc",
  title = "My title",
  author = "Fran Pérez"
)

my_temp_bib <- tempfile(fileext = ".bib")

cff_write_bib(bib, file = my_temp_bib)

cat(readLines(my_temp_bib), sep = "\n")

cff_write_bib(bib, file = my_temp_bib, ascii = TRUE, append = TRUE)

cat(readLines(my_temp_bib), sep = "\n")
}
\seealso{
\itemize{
\item \code{\link[=cff_write_bib]{cff_write_bib()}} for writing \verb{*.bib} files.
\item \code{\link[=cff_write_citation]{cff_write_citation()}} for writing \strong{R} \code{CITATION} files.
}

Other deprecated functions:
\code{\link[=cff_extract_to_bibtex]{cff_extract_to_bibtex()}},
\code{\link[=cff_from_bibtex]{cff_from_bibtex()}},
\code{\link[=cff_parse_citation]{cff_parse_citation()}},
\code{\link[=cff_parse_person]{cff_parse_person()}}
}
\concept{deprecated}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_git_hook.R
\encoding{UTF-8}
\name{cff_git_hook}
\alias{cff_git_hook}
\alias{cff_git_hook_install}
\alias{cff_git_hook_remove}
\title{Use a git pre-commit hook \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}}
\usage{
cff_git_hook_install()

cff_git_hook_remove()
}
\value{
Invisible. This function is called for its side effects.
}
\description{
Install a
\href{https://git-scm.com/book/en/v2/Customizing-Git-Git-Hooks#_committing_workflow_hooks}{pre-commit hook}
that reminds you to update your \code{CITATION.cff} file. This is a wrapper around
\code{\link[usethis:use_git_hook]{usethis::use_git_hook()}}.
}
\details{
This function installs a pre-commit hook using
\code{\link[usethis:use_git_hook]{usethis::use_git_hook()}}.

A pre-commit hook is a script that identifies simple issues before
submission to code review. This pre-commit hook warns you if any of the
following conditions are met:
\itemize{
\item You included your \code{DESCRIPTION} or \code{inst/CITATION} file in a commit but
did not include your \code{CITATION.cff}, and the \code{CITATION.cff} file is
"older" than your \code{DESCRIPTION} or \code{inst/CITATION} file.
\item You updated your \code{CITATION.cff} but did not include it in
your commit.
}
}
\section{A word of caution}{
The pre-commit hook may prevent you from committing if you are not updating
your \code{CITATION.cff}. However, the detection mechanism is not perfect and may
be triggered even if you have attempted to update your \code{CITATION.cff} file.

This typically occurs when you have updated your \code{DESCRIPTION} or
\code{inst/CITATION} files, but those changes do not affect your
\code{CITATION.cff} file (for example, you are adding new dependencies).

In those cases, you can override the check by running
\verb{git commit --no-verify} in the terminal.

If you are using \strong{RStudio}, you can also run this command from an \strong{R}
script by selecting that line and sending it to the terminal using:
\itemize{
\item Windows & Linux: \code{Ctrl+Alt+Enter}.
\item Mac: \code{Cmd+Option+Return}.
}
}

\section{Removing the git pre-commit hook}{
You can remove the pre-commit hook using \code{cff_git_hook_remove()}.
}

\examples{
\dontrun{
cff_git_hook_install()
}

}
\seealso{
\itemize{
\item \code{\link[usethis:use_git_hook]{usethis::use_git_hook()}}, which is the underlying function used by
\code{cff_git_hook_install()}.
\item \code{\link[usethis:use_git]{usethis::use_git()}} and related function of \CRANpkg{usethis} for using
Git with \strong{R} packages.
}

Other Git/GitHub helpers provided by \CRANpkg{cffr}:
\code{\link[=cff_gha_update]{cff_gha_update()}}
}
\concept{git}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_bibentry.R
\encoding{UTF-8}
\name{as_bibentry}
\alias{as_bibentry}
\alias{as_bibentry.default}
\alias{as_bibentry.character}
\alias{as_bibentry.NULL}
\alias{as_bibentry.list}
\alias{as_bibentry.cff}
\alias{as_bibentry.cff_ref_lst}
\alias{as_bibentry.cff_ref}
\title{Create \code{\link{bibentry}} objects from several sources}
\usage{
as_bibentry(x, ...)

\method{as_bibentry}{default}(x, ...)

\method{as_bibentry}{character}(x, ..., what = c("preferred", "references", "all"))

\method{as_bibentry}{`NULL`}(x, ...)

\method{as_bibentry}{list}(x, ...)

\method{as_bibentry}{cff}(x, ..., what = c("preferred", "references", "all"))

\method{as_bibentry}{cff_ref_lst}(x, ...)

\method{as_bibentry}{cff_ref}(x, ...)
}
\arguments{
\item{x}{The source used to generate
the \code{bibentry} object via \CRANpkg{cffr}. It can be:
\itemize{
\item A missing value, which retrieves the \code{DESCRIPTION}
file from your in-development package.
\item An existing \code{cff} object created with \code{\link[=cff]{cff()}}, \code{\link[=cff_create]{cff_create()}}, or
\code{\link[=as_cff]{as_cff()}}.
\item Path to a CITATION.cff file (\code{"CITATION.cff"}).
\item The name of an installed package (\code{"jsonlite"}).
\item Path to a DESCRIPTION file (\code{"DESCRIPTION"}).
}}

\item{...}{Additional arguments to be passed to or from methods.}

\item{what}{Fields to extract from a full \code{cff} object. The value could be:
\itemize{
\item \code{preferred}: This would create a single entry with the main citation
info of the package (key \code{preferred-citation}).
\item \code{references}: Extract all the entries of \code{references} key.
\item \code{all}: A combination of the previous two options. This would extract
both the \code{preferred-citation} and the \code{references} key.
}

See \code{vignette("crosswalk", package = "cffr")}.}
}
\value{
\code{as_bibentry()} returns a \code{bibentry} object with one or more entries.
}
\description{
This function creates \code{\link{bibentry}} objects from different metadata sources
(\code{\link{cff}} objects, \code{DESCRIPTION} files and other sources). The inverse
transformation (\code{bibentry} object to \code{\link{cff_ref_lst}}) can be done with the
corresponding \code{\link[=as_cff.bibentry]{as_cff.bibentry()}} method.

With \code{\link[=toBibtex.cff]{toBibtex()}} it is possible to convert \code{\link{cff}} objects
to BibTeX markup on the fly, see \strong{Examples}.
}
\details{
An \strong{R} \code{bibentry} object is the representation of a BibTeX entry. These
objects can be converted to BibTeX markup with \code{\link[=toBibtex]{toBibtex()}}, that creates an
object of class \code{Bibtex} and can be printed and exported as a valid BibTeX
entry.

\code{as_bibtex()} tries to map the information of the source \code{x} into a \code{\link{cff}}
object and performs a mapping of the metadata to BibTeX, according to
\code{vignette("bibtex_cff", "cffr")}.
}
\examples{
\donttest{
# From a cff object ----
cff_object <- cff()

cff_object

# bibentry object
bib <- as_bibentry(cff_object)

class(bib)

bib

# Print as bibtex
toBibtex(bib)

# Thanks to the S3 Method we can also do
toBibtex(cff_object)

# Other sources ----
# From a CITATION.cff

path <- system.file("examples/CITATION_complete.cff", package = "cffr")
cff_file <- as_bibentry(path)

cff_file

# For an installed package with options
installed_package <- as_bibentry("jsonvalidate", what = "all")

installed_package

# Use a DESCRIPTION file
path2 <- system.file("examples/DESCRIPTION_gitlab", package = "cffr")
desc_file <- as_bibentry(path2)

toBibtex(desc_file)
}

}
\references{
\itemize{
\item Patashnik, Oren. "BIBTEXTING" February 1988.
\url{https://osl.ugr.es/CTAN/biblio/bibtex/base/btxdoc.pdf}.
\item Haines, R., & The Ruby Citation File Format Developers. (2021).
\emph{Ruby CFF Library (Version 0.9.0)} (Computer software).
\doi{10.5281/zenodo.1184077}.
\item Hernangomez D (2022). "BibTeX and CFF, a potential crosswalk."
\emph{The cffr package, Vignettes}. \doi{10.21105/joss.03900},
\url{https://docs.ropensci.org/cffr/articles/bibtex-cff.html}.
}
}
\seealso{
\code{\link[utils:bibentry]{utils::bibentry()}} to understand more about the \code{bibentry} class.
\itemize{
\item \code{vignette("crosswalk", package = "cffr")} provides details on how the
metadata of a package is mapped to produce a \code{cff} object.
\item \code{vignette("bibtex_cff", "cffr")} provides detailed information about the
internal mapping performed between \code{cff} objects and BibTeX markup (
both \code{cff} to BibTeX and BibTeX to \code{cff}).
}

Other related functions:
\itemize{
\item \code{\link[utils:toBibtex]{utils::toBibtex()}}.
}

Other functions for working with BibTeX format:
\code{\link[=cff_read]{cff_read()}},
\code{\link[=cff_read_bib_text]{cff_read_bib_text()}},
\code{\link[=cff_write_bib]{cff_write_bib()}},
\code{\link[=encoded_utf_to_latex]{encoded_utf_to_latex()}}

Coercing between \strong{R} classes with \strong{S3 Methods}:
\code{\link[=as_cff]{as_cff()}},
\code{\link[=as_cff_person]{as_cff_person()}},
\code{\link{cff_class}}
}
\concept{bibtex}
\concept{s3method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\encoding{UTF-8}
\name{cff_from_bibtex}
\alias{cff_from_bibtex}
\title{Previous API: Create a \code{\link{cff}} object from BibTeX entries}
\usage{
cff_from_bibtex(x, encoding = "UTF-8", ...)
}
\arguments{
\item{x}{The source used to generate the
\code{\link{cff}} object. Must be a \code{character} object indicating either:
\itemize{
\item The path to a BibTeX file.
\item A vector of characters with a complete BibTeX string. See \strong{Examples}.
}}

\item{encoding}{Encoding to be assumed for \code{x}. See \code{\link[=readLines]{readLines()}}.}

\item{...}{Other arguments passed to \code{\link[bibtex:read.bib]{bibtex::read.bib()}}.}
}
\value{
See \code{\link[=cff_read_bib]{cff_read_bib()}} for reading \verb{*.bib} files and \code{\link[=cff_read_bib_text]{cff_read_bib_text()}}
for reading a \code{character} object representing a BibTeX entry.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Please use either \code{\link[=cff_read_bib]{cff_read_bib()}} or
\code{\link[=cff_read_bib_text]{cff_read_bib_text()}} instead.
}
\examples{
if (requireNamespace("bibtex", quietly = TRUE)) {
  x <- c(
    "@book{einstein1921,
    title        = {Relativity: The Special and the General Theory},
    author       = {Einstein, Albert},
    year         = 1920,
    publisher    = {Henry Holt and Company},
    address      = {London, United Kingdom},
    isbn         = 9781587340925
}",
    "@misc{misc-full,
    title        = {Handing out random pamphlets in airports},
    author       = {Joe-Bob Missilany},
    year         = 1984,
    month        = oct,
    note         = {This is a full MISC entry},
    howpublished = {Handed out at O'Hare}
}"
  )

  cff_read_bib_text(x)

  # From a file

  x2 <- system.file("examples/example.bib", package = "cffr")
  cff_read_bib(x2)
}
}
\seealso{
Other deprecated functions:
\code{\link[=cff_extract_to_bibtex]{cff_extract_to_bibtex()}},
\code{\link[=cff_parse_citation]{cff_parse_citation()}},
\code{\link[=cff_parse_person]{cff_parse_person()}},
\code{\link[=write_bib]{write_bib()}}
}
\concept{deprecated}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docs.R, R/methods.R
\encoding{UTF-8}
\name{cff_class}
\alias{cff_class}
\alias{cff_ref}
\alias{cff_ref_lst}
\alias{cff_pers}
\alias{cff_pers_lst}
\alias{print.cff}
\alias{c.cff}
\alias{[.cff_ref_lst}
\alias{[.cff_pers_lst}
\alias{as.data.frame.cff}
\alias{as.data.frame.cff_pers_lst}
\alias{as.data.frame.cff_pers}
\alias{as.data.frame.cff_ref_lst}
\alias{as.data.frame.cff_ref}
\alias{head.cff}
\alias{tail.cff}
\alias{as.list.cff}
\alias{toBibtex.cff}
\alias{toBibtex.cff_ref_lst}
\alias{toBibtex.cff_ref}
\alias{toBibtex.cff_pers_lst}
\alias{toBibtex.cff_pers}
\alias{as.person.cff_pers}
\alias{as.person.cff_pers_lst}
\alias{as.person.cff}
\alias{as.person.cff_ref}
\alias{as.person.cff_ref_lst}
\title{The \code{cff} class}
\description{
\href{https://cran.r-project.org/package=cffr}{\strong{cffr}} implements an S3 object with
\code{\link{class}} \code{cff}, which is used to represent the information of a \code{CITATION.cff}
file in \strong{R}.

Under the hood, a \code{cff} object is simply a named \code{\link{list}} to which we added
additional methods, most notably \code{\link{print}} and \code{\link[=as_cff]{as_cff()}}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{a_named_list <- list(
  first = "I", second = "am", third = "a", fourth = "list",
  fifth = "with", sixth = "names", "none" = NULL
)

# As cff
a_cff_object <- as_cff(a_named_list)

class(a_cff_object)
#> [1] "cff"

a_cff_object
#> first: I
#> second: am
#> third: a
#> fourth: list
#> fifth: with
#> sixth: names

is.list(a_cff_object)
#> [1] TRUE
}\if{html}{\out{</div>}}

\code{\link[=as_cff]{as_cff()}} not only converts a \code{list} to \code{cff} but also removes items (known as
\code{keys} in CFF terminology) that are \code{NULL} or \code{NA}.
\subsection{Sub-classes}{

\strong{cffr} implements two special sub-classes of \code{cff}, with the aim of
representing two special types of objects defined in the \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{CFF Schema}:
\itemize{
\item \code{definition.person} and \code{definition.entity}: CFF definitions for sub-lists
representing persons or entities. In \strong{cffr}, the sub-class \code{cff_pers_lst}
has been implemented to collect an array of \code{definition.person/entity},
where each individual person or entity has a sub-class \code{cff_pers}.
\item Similarly, \code{definition.reference} is the CFF definition for collecting
references to related works and other articles of software used in the
development of the main work of the CFF file. This is implemented in
\strong{cffr} as a sub-class named \code{cff_ref_lst} for arrays of
\code{definition.reference}, where each element has a sub-class named \code{cff_ref}.
}

These two sub-classes do not provide full valid \code{cff} objects, but adapt
information to the schema required by CFF:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Using the utils::person() function

## Array
two_persons <- as_cff_person(
  c(
    person("A", "person", comment = c(ORCID = "0000-0001-8457-4658")),
    person("An entity", email = "fake@gmail.com")
  )
)

two_persons
#> - family-names: person
#>   given-names: A
#>   orcid: https://orcid.org/0000-0001-8457-4658
#> - name: An entity
#>   email: fake@gmail.com

class(two_persons)
#> [1] "cff_pers_lst" "cff"

# Single element

two_persons[[1]]
#> family-names: person
#> given-names: A
#> orcid: https://orcid.org/0000-0001-8457-4658

class(two_persons[[1]])
#> [1] "cff_pers" "cff"

# Array of references

cit <- c(citation(), citation("yaml"))

ref_list <- as_cff(cit)

ref_list
#> - type: manual
#>   title: 'R: A Language and Environment for Statistical Computing'
#>   authors:
#>   - name: R Core Team
#>     website: https://ror.org/02zz1nj61
#>   institution:
#>     name: R Foundation for Statistical Computing
#>     website: https://ror.org/05qewa988
#>     address: Vienna, Austria
#>   year: '2026'
#>   doi: 10.32614/R.manuals
#>   url: https://www.R-project.org/
#> - type: manual
#>   title: 'yaml: Methods to Convert R Data to YAML and Back'
#>   authors:
#>   - family-names: Stephens
#>     given-names: Jeremy
#>   - family-names: Simonov
#>     given-names: Kirill
#>   year: '2025'
#>   notes: R package version 2.3.12
#>   url: https://CRAN.R-project.org/package=yaml
#>   doi: 10.32614/CRAN.package.yaml

class(ref_list)
#> [1] "cff_ref_lst" "cff"

# Single element

ref_list[[1]]
#> type: manual
#> title: 'R: A Language and Environment for Statistical Computing'
#> authors:
#> - name: R Core Team
#>   website: https://ror.org/02zz1nj61
#> institution:
#>   name: R Foundation for Statistical Computing
#>   website: https://ror.org/05qewa988
#>   address: Vienna, Austria
#> year: '2026'
#> doi: 10.32614/R.manuals
#> url: https://www.R-project.org/

class(ref_list[[1]])
#> [1] "cff_ref" "cff"
}\if{html}{\out{</div>}}
}

\subsection{Valid \code{cff} objects}{

Creating a \code{cff} object does not ensure its validity according to the \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{CFF Schema}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{class(a_cff_object)
#> [1] "cff"

cff_validate(a_cff_object)
#> == Validating cff ==============================================================
#> x Oops! This <cff> has the following errors:
#> * cff/: must have required property 'authors'
#> * cff/: must have required property 'cff-version'
#> * cff/: must have required property 'message'
#> * cff/: must have required property 'title'
#> * cff/: must NOT have additional properties
#> * cff/: must NOT have additional properties
#> * cff/: must NOT have additional properties
#> * cff/: must NOT have additional properties
#> * cff/: must NOT have additional properties
#> * cff/: must NOT have additional properties
}\if{html}{\out{</div>}}

\code{\link[=cff_validate]{cff_validate()}} gives minimal messages about what's wrong with our \code{cff} and
(invisibly) returns the result of the validation (\code{TRUE/FALSE}).

We can use \code{\link[=cff_modify]{cff_modify()}} to add more keys:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{cff_valid <- cff_modify(a_cff_object,
  authors = as_cff_person("\{James and James\}"),
  cff_version = "1.2.0",
  message = "Hi there",
  title = "My title"
)


# Remove invalid keys
cff_valid <- as_cff(cff_valid[names(cff_valid) \%in\% cff_schema_keys()])

cff_valid
#> authors:
#> - name: James and James
#> cff-version: 1.2.0
#> message: Hi there
#> title: My title

cff_validate(cff_valid)
#> == Validating cff ==============================================================
#> v Congratulations! This <cff> is valid
}\if{html}{\out{</div>}}
}

\subsection{Base methods provided by \strong{cffr}}{

\strong{cffr} version 1.0.0 provides additional S3 methods for common coercion
functions of the \strong{base} and \strong{utils} packages.
\subsection{\code{\link[=as.data.frame]{as.data.frame()}}}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{minimal_cff <- cff()

minimal_cff
#> cff-version: 1.2.0
#> message: If you use this software, please cite it using these metadata.
#> title: My Research Software
#> authors:
#> - family-names: Doe
#>   given-names: John

as_df <- as.data.frame(minimal_cff)

class(as_df)
#> [1] "data.frame"

t(as_df)
#>                         [,1]                                                            
#> cff_version             "1.2.0"                                                         
#> message                 "If you use this software, please cite it using these metadata."
#> title                   "My Research Software"                                          
#> authors.00.family_names "Doe"                                                           
#> authors.00.given_names  "John"
}\if{html}{\out{</div>}}
}

\subsection{\code{\link[=c]{c()}}}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{new_keys <- c("date-released" = "2020-01-31", abstract = "Minimal example")

c(minimal_cff, new_keys)
#> cff-version: 1.2.0
#> message: If you use this software, please cite it using these metadata.
#> title: My Research Software
#> authors:
#> - family-names: Doe
#>   given-names: John
#> date-released: '2020-01-31'
#> abstract: Minimal example
}\if{html}{\out{</div>}}
}

\subsection{\code{\link[=as.list]{as.list()}}}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{as.list(minimal_cff)
#> $`cff-version`
#> [1] "1.2.0"
#> 
#> $message
#> [1] "If you use this software, please cite it using these metadata."
#> 
#> $title
#> [1] "My Research Software"
#> 
#> $authors
#> $authors[[1]]
#> $authors[[1]]$`family-names`
#> [1] "Doe"
#> 
#> $authors[[1]]$`given-names`
#> [1] "John"
}\if{html}{\out{</div>}}
}

\subsection{\code{\link[=as.person]{as.person()}}}{

Only for \code{cff_pers_lst} and \code{cff_pers} objects:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{as.person(two_persons)
#> [1] "A person (ORCID: <https://orcid.org/0000-0001-8457-4658>)"
#> [2] "An entity <fake@gmail.com>"
}\if{html}{\out{</div>}}
}

\subsection{\code{\link[=toBibtex]{toBibtex()}}}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# For cff
toBibtex(minimal_cff)
#> @Misc\{doe,
#>   title = \{My Research Software\},
#>   author = \{John Doe\},
#> \}

# cff_ref, cff_ref_lst
toBibtex(cit)
#> @Manual\{,
#>   title = \{R: A Language and Environment for Statistical Computing\},
#>   author = \{\{R Core Team\}\},
#>   organization = \{R Foundation for Statistical Computing\},
#>   address = \{Vienna, Austria\},
#>   year = \{2026\},
#>   doi = \{10.32614/R.manuals\},
#>   url = \{https://www.R-project.org/\},
#> \}
#> 
#> @Manual\{,
#>   title = \{yaml: Methods to Convert R Data to YAML and Back\},
#>   author = \{Jeremy Stephens and Kirill Simonov\},
#>   year = \{2025\},
#>   note = \{R package version 2.3.12\},
#>   url = \{https://CRAN.R-project.org/package=yaml\},
#>   doi = \{10.32614/CRAN.package.yaml\},
#> \}

# cff_pers, cff_pers_lst
toBibtex(two_persons)
#> person, A and \{An entity\}
}\if{html}{\out{</div>}}
}

}
}
\references{
\itemize{
\item Wickham H (2019). "S3." In \emph{Advanced R}, 2nd edition.
Chapman and Hall/CRC. \doi{10.1201/9781351201315},
\url{https://adv-r.hadley.nz/s3.html}.
}
}
\seealso{
Coercing between \strong{R} classes with \strong{S3 Methods}:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=as_cff]{as_cff()}},
\code{\link[=as_cff_person]{as_cff_person()}}
}
\concept{s3method}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_read_bib_text.R
\encoding{UTF-8}
\name{cff_read_bib_text}
\alias{cff_read_bib_text}
\title{Read BibTeX markup as a \code{\link{cff_ref_lst}} object}
\usage{
cff_read_bib_text(x, encoding = "UTF-8", ...)
}
\arguments{
\item{x}{A \code{character} vector with one or more complete BibTeX entries.}

\item{encoding}{Encoding to be assumed for \code{x}. See \code{\link[=readLines]{readLines()}}.}

\item{...}{Arguments passed to \code{\link[=cff_read_bib]{cff_read_bib()}}.}
}
\value{
An object of classes \code{\link[=cff_ref_lst]{cff_ref_lst, cff}} according to the
\code{definitions.references} specified in
the \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Citation File Format schema}.
Each element of the \code{cff_ref_lst} object has classes
\code{\link[=cff_ref]{cff_ref, cff}}.
}
\description{
Convert a \code{character} string representing a BibTeX entry into a
\code{\link{cff_ref_lst}} object.
}
\details{
This function writes \code{x} to a temporary \verb{*.bib} file and reads it using
\code{\link[=cff_read_bib]{cff_read_bib()}}.

This function requires \CRANpkg{bibtex} (>= 0.5.0) and uses
\code{\link[bibtex:read.bib]{bibtex::read.bib()}} for parsing.
}
\examples{
if (requireNamespace("bibtex", quietly = TRUE)) {
  x <- c(
    "@book{einstein1921,
      title        = {Relativity: The Special and the General Theory},
      author       = {Einstein, Albert},
      year         = 1920,
      publisher    = {Henry Holt and Company},
      address      = {London, United Kingdom},
      isbn         = 9781587340925
  }",
    "@misc{misc-full,
      title        = {Handing out random pamphlets in airports},
      author       = {Joe-Bob Missilany},
      year         = 1984,
      month        = oct,
      note         = {This is a full MISC entry},
      howpublished = {Handed out at O'Hare}
  }"
  )

  cff_read_bib_text(x)
}
}
\seealso{
\code{\link[=cff_read_bib]{cff_read_bib()}} for reading \verb{*.bib} files.

Other functions for working with BibTeX format:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=cff_read]{cff_read()}},
\code{\link[=cff_write_bib]{cff_write_bib()}},
\code{\link[=encoded_utf_to_latex]{encoded_utf_to_latex()}}

Other functions for reading external files:
\code{\link[=cff_read]{cff_read()}}
}
\concept{bibtex}
\concept{reading}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\encoding{UTF-8}
\name{cff_parse_citation}
\alias{cff_parse_citation}
\title{Previous API: Parse a \code{bibentry} to \code{cff}}
\usage{
cff_parse_citation(bib)
}
\arguments{
\item{bib}{A \code{bibentry} object.}
}
\value{
A \code{bibentry} in format \code{\link{cff}}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Please use \code{\link[=as_cff.bibentry]{as_cff.bibentry()}} method
}
\examples{

bib <- citation("base")
bib

# To cff
bib_to_cff <- as_cff(bib)
bib_to_cff

}
\seealso{
\code{\link[=as_cff.bibentry]{as_cff.bibentry()}}

Other deprecated functions:
\code{\link[=cff_extract_to_bibtex]{cff_extract_to_bibtex()}},
\code{\link[=cff_from_bibtex]{cff_from_bibtex()}},
\code{\link[=cff_parse_person]{cff_parse_person()}},
\code{\link[=write_bib]{write_bib()}}
}
\concept{deprecated}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_write.R
\encoding{UTF-8}
\name{cff_write}
\alias{cff_write}
\title{Write a \code{CITATION.cff} file}
\usage{
cff_write(
  x,
  outfile = "CITATION.cff",
  keys = list(),
  cff_version = "1.2.0",
  gh_keywords = TRUE,
  r_citation = FALSE,
  dependencies = TRUE,
  validate = TRUE,
  verbose = TRUE,
  authors_roles = c("aut", "cre"),
  encoding = "UTF-8"
)
}
\arguments{
\item{x}{The source used to generate
the \code{\link{cff}} object. It can be:
\itemize{
\item A missing value, which retrieves the \code{DESCRIPTION} file from your
in-development \strong{R} package.
\item An existing \code{\link{cff}} object.
\item The name of an installed package (\code{"jsonlite"}).
\item Path to a \code{DESCRIPTION} file (\code{"./DESCRIPTION"}).
}}

\item{outfile}{The name and path of the \code{CITATION.cff} to be created.}

\item{keys}{List of additional keys to add to the \code{\link{cff}} object. See
\code{\link[=cff_modify]{cff_modify()}}.}

\item{cff_version}{The Citation File Format schema version that the
\code{CITATION.cff} file adheres to for providing the citation metadata.}

\item{gh_keywords}{Logical \code{TRUE/FALSE}. If the package is hosted on
GitHub, would you like to add the repo topics as keywords?}

\item{r_citation}{Logical \code{TRUE/FALSE}. When \code{TRUE}, the \strong{R} package
citation (for example, \code{inst/CITATION}) is created or updated.
\strong{No backup copy is created}. For more control, use
\code{\link[=cff_write_citation]{cff_write_citation()}}.}

\item{dependencies}{Logical \code{TRUE/FALSE}. Should the dependencies of your
package be added to the \code{references} CFF key?}

\item{validate}{Logical \code{TRUE/FALSE}. Should the new file be validated
using \code{cff_validate()}?}

\item{verbose}{Logical \code{TRUE/FALSE}. When \code{TRUE}, the function displays
informative messages.}

\item{authors_roles}{Roles to be considered as authors of the package when
generating the \code{CITATION.cff} file. See \strong{Details}.}

\item{encoding}{The name of the encoding to be assumed. Default is \code{"UTF-8"},
but it can be any other value as accepted by \code{\link[=iconv]{iconv()}}, such as
\code{"ASCII//TRANSLIT"}.}
}
\value{
A \code{CITATION.cff} file and an (invisible) \code{cff} object.
}
\description{
\strong{This is the core function of the package and likely to be the only one
you would need when developing a package}.

This function writes out a \code{CITATION.cff} file for a given package. This
function is basically a wrapper around \code{\link[=cff_create]{cff_create()}} to both create the
\code{\link{cff}} object and write it out to a YAML-formatted file in
one command.
}
\details{
For details of \code{authors_roles} see \code{\link[=cff_create]{cff_create()}}.

When creating and writing a \code{CITATION.cff} for the first time, the function
adds the pattern \code{"^CITATION\\.cff$"} to your \code{.Rbuildignore} file.
}
\examples{
\donttest{
tmpfile <- tempfile(fileext = ".cff")
cff_obj <- cff_write("jsonlite", outfile = tmpfile)

cff_obj

# Force clean-up
file.remove(tmpfile)
}
}
\seealso{
\href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Guide to Citation File Format schema version 1.2.0}.
This function unifies the workflow \code{\link[=cff_create]{cff_create()}} + \code{\link[=cff_validate]{cff_validate()}} +
write a file.

Other functions for creating external files:
\code{\link[=cff_write_bib]{cff_write_bib()}}
}
\concept{writing}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_validate.R
\encoding{UTF-8}
\name{cff_validate}
\alias{cff_validate}
\title{Validate a \code{CITATION.cff} file or a \code{\link{cff}} object}
\usage{
cff_validate(x = "CITATION.cff", verbose = TRUE)
}
\arguments{
\item{x}{This is expected to be either a full \code{cff} object created
with \code{\link[=cff_create]{cff_create()}} or the path to a \code{CITATION.cff} file to be validated.
In the case of a \verb{*.cff} file it would read with \code{\link[=cff_read_cff_citation]{cff_read_cff_citation()}}.}

\item{verbose}{Logical \code{TRUE/FALSE}. When \code{TRUE}, the function displays
informative messages.}
}
\value{
A message indicating the result of the validation and an invisible value
\code{TRUE/FALSE}. On error, the result has an attribute \code{"errors"}
containing the error summary (see \strong{Examples} and \code{\link[=attr]{attr()}}).
}
\description{
Validate a \code{CITATION.cff} file or a \code{\link{cff}} object using the corresponding \href{https://github.com/citation-file-format/citation-file-format/blob/main/schema.json}{validation schema}.
}
\examples{
\donttest{
# Full .cff example
cff_validate(system.file("examples/CITATION_complete.cff", package = "cffr"))

# Validate a cffr object
cffr <- cff_create("jsonlite")
class(cffr)
cff_validate(cffr)
}

# .cff with errors
err_f <- system.file("examples/CITATION_error.cff", package = "cffr")
# Can manipulate the errors as data frame
res <- try(cff_validate(err_f))

isTRUE(res)
isFALSE(res)

# Detailed results
df_errors <- attr(res, "errors")

df_errors[, c("field", "message")]

# If a CITATION file is supplied (note that it is not a .cff file), it
# throws an error
try(cff_validate(system.file("CITATION", package = "cffr")))
}
\seealso{
\href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Guide to Citation File Format schema version 1.2.0}.

\code{\link[jsonvalidate:json_validate]{jsonvalidate::json_validate()}}, which is the function that performs the
validation.

Other core functions of \CRANpkg{cffr}:
\code{\link[=cff]{cff()}},
\code{\link[=cff_create]{cff_create()}},
\code{\link[=cff_modify]{cff_modify()}}
}
\concept{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_write_misc.R
\encoding{UTF-8}
\name{cff_write_bib}
\alias{cff_write_bib}
\alias{cff_write_citation}
\title{Export \strong{R} objects to different file types}
\usage{
cff_write_bib(
  x,
  file = tempfile(fileext = ".bib"),
  append = FALSE,
  verbose = TRUE,
  ascii = FALSE,
  ...
)

cff_write_citation(
  x,
  file = tempfile("CITATION_"),
  append = FALSE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{\link{bibentry}} or a \code{\link{cff}} object.}

\item{file}{Name of the file to be created. If \code{NULL}, the lines are
displayed instead.}

\item{append}{Logical. Should entries be appended to an existing file?}

\item{verbose}{Logical. Display informative messages.}

\item{ascii}{Logical. Should entries be written using ASCII characters only?}

\item{...}{
  Arguments passed on to \code{\link{as_bibentry.cff}}, \code{\link{as_bibentry.cff_ref}}, \code{\link{as_bibentry.cff_ref_lst}}
  \describe{
    \item{\code{what}}{Fields to extract from a full \code{cff} object. The value could be:
\itemize{
\item \code{preferred}: This would create a single entry with the main citation
info of the package (key \code{preferred-citation}).
\item \code{references}: Extract all the entries of \code{references} key.
\item \code{all}: A combination of the previous two options. This would extract
both the \code{preferred-citation} and the \code{references} key.
}

See \code{vignette("crosswalk", package = "cffr")}.}
  }}
}
\value{
Writes the corresponding file specified on the \code{file} argument.
}
\description{
Export \strong{R} objects representing citations to specific file formats:
\itemize{
\item \code{\link[=cff_write_bib]{cff_write_bib()}} creates a \code{.bib} file.
\item \code{\link[=cff_write_citation]{cff_write_citation()}} creates an \strong{R} citation file as described in
Section 1.9 of \emph{Writing R Extensions} (R Core Team 2023).
}
}
\details{
When \code{x} is a \code{cff} object, it is converted to BibTeX using
\code{\link[=toBibtex.cff]{toBibtex.cff()}}.

For security reasons, if the file already exists, the function creates
a backup copy in the same directory.
}
\examples{

bib <- bibentry("Misc",
  title = "My title",
  author = "Fran Pérez"
)

my_temp_bib <- tempfile(fileext = ".bib")

cff_write_bib(bib, file = my_temp_bib)

cat(readLines(my_temp_bib), sep = "\n")

cff_write_bib(bib, file = my_temp_bib, ascii = TRUE, append = TRUE)

cat(readLines(my_temp_bib), sep = "\n")

# Create a CITATION file

# Use a system file
f <- system.file("examples/preferred-citation-book.cff", package = "cffr")
a_cff <- cff_read(f)

out <- file.path(tempdir(), "CITATION")
cff_write_citation(a_cff, file = out)

# Check by reading, use meta object
meta <- packageDescription("cffr")
meta$Encoding <- "UTF-8"

utils::readCitationFile(out, meta)
}
\references{
\itemize{
\item R Core Team (2023). \emph{Writing R Extensions}.
\url{https://cran.r-project.org/doc/manuals/r-release/R-exts.html}
}
}
\seealso{
\code{vignette("bibtex_cff", "cffr")}, \code{\link[knitr:write_bib]{knitr::write_bib()}} and the
following packages:
\itemize{
\item \CRANpkg{bibtex}.
\item \CRANpkg{RefManageR}
\item \CRANpkg{rbibutils}
}

Other functions for working with BibTeX format:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=cff_read]{cff_read()}},
\code{\link[=cff_read_bib_text]{cff_read_bib_text()}},
\code{\link[=encoded_utf_to_latex]{encoded_utf_to_latex()}}

Other functions for creating external files:
\code{\link[=cff_write]{cff_write()}}
}
\concept{bibtex}
\concept{writing}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_modify.R
\encoding{UTF-8}
\name{cff_modify}
\alias{cff_modify}
\title{Modify a \code{\link{cff}} object}
\usage{
cff_modify(x, ...)
}
\arguments{
\item{x}{A \code{\link{cff}} object.}

\item{...}{Named arguments used to modify \code{x}. See also the \code{...}
argument in \code{\link[=cff]{cff()}}.}
}
\value{
A \code{\link{cff}} object.
}
\description{
Add new keys or modify existing ones in a \code{\link{cff}} object.
}
\details{
Keys provided in \code{...} override the corresponding key in \code{x}.

You can add additional keys not detected by \code{\link[=cff_create]{cff_create()}} using
the \code{keys} argument. A list of valid keys can be retrieved with
\code{\link[=cff_schema_keys]{cff_schema_keys()}}. For details, refer to
\href{https://github.com/citation-file-format/citation-file-format/blob/main/schema-guide.md}{Guide to Citation File Format schema version 1.2.0}.
for additional details.
}
\examples{
x <- cff()
x

cff_validate(x)

x_mod <- cff_modify(x,
  contact = as_cff_person("A contact"),
  message = "This overwrites fields",
  title = "New Title",
  abstract = "New abstract",
  doi = "10.21105/joss.03900"
)

x_mod

cff_validate(x_mod)

}
\seealso{
This function is a wrapper of \code{\link[utils:modifyList]{utils::modifyList()}}.

See \code{\link[=cff]{cff()}} for creating \code{\link{cff}} objects from scratch.

Other core functions of \CRANpkg{cffr}:
\code{\link[=cff]{cff()}},
\code{\link[=cff_create]{cff_create()}},
\code{\link[=cff_validate]{cff_validate()}}
}
\concept{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff_gha_update.R
\encoding{UTF-8}
\name{cff_gha_update}
\alias{cff_gha_update}
\title{Install a \CRANpkg{cffr} GitHub Action}
\usage{
cff_gha_update(path = ".", overwrite = FALSE)
}
\arguments{
\item{path}{Project root directory.}

\item{overwrite}{Logical. If the action already exists, should it be
overwritten?}
}
\value{
Invisible, this function is called by its side effects.
}
\description{
This function installs a \href{https://github.com/features/actions}{GitHub Action} on your repository. The action
updates your \code{CITATION.cff} when any of these events occur:
\itemize{
\item You publish a new release of the package.
\item Your \code{DESCRIPTION} or \code{inst/CITATION} file is modified.
\item The action can be run manually.
}
}
\details{
Triggers on your action can be modified, see \href{https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows}{Events that trigger workflows}.
}
\examples{
\dontrun{
cff_gha_update()
}
}
\seealso{
Other Git/GitHub helpers provided by \CRANpkg{cffr}:
\code{\link{cff_git_hook}}
}
\concept{git}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cff.R
\encoding{UTF-8}
\name{cff}
\alias{cff}
\title{Create \code{\link{cff}} objects from direct inputs}
\usage{
cff(path, ...)
}
\arguments{
\item{path}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} \code{path} is no longer supported,
use \code{\link[=cff_read_cff_citation]{cff_read_cff_citation()}} instead.}

\item{...}{Named arguments to be used for creating a \code{\link{cff}} object. If no
arguments are supplied (the default behavior), a minimal valid \code{cff}
object is created.}
}
\value{
A \code{\link{cff}} object. Under the hood, a \code{cff} object is a regular \code{\link{list}}
object with a special \code{\link[=print.cff]{print}} method.
}
\description{
A class and utility methods for reading, creating and holding CFF
information. See \link{cff_class} to learn more about \code{cff} objects.
}
\details{
\code{cff()} converts \verb{_} in the argument name to \code{-}. For example,
\code{cff_version = "1.2.0"} is converted to \code{cff-version = "1.2.0"}.

Valid arguments are those specified on \code{\link[=cff_schema_keys]{cff_schema_keys()}}:
\itemize{
\item \code{cff-version}
\item \code{message}
\item \code{type}
\item \code{license}
\item \code{title}
\item \code{version}
\item \code{doi}
\item \code{identifiers}
\item \code{abstract}
\item \code{authors}
\item \code{preferred-citation}
\item \code{repository}
\item \code{repository-artifact}
\item \code{repository-code}
\item \code{commit}
\item \code{url}
\item \code{date-released}
\item \code{contact}
\item \code{keywords}
\item \code{references}
\item \code{license-url}
}
}
\examples{
# Blank cff
cff()

# Use custom params
test <- cff(
  title = "Manipulating files",
  keywords = c("A", "new", "list", "of", "keywords"),
  authors = as_cff_person("New author")
)
test
\donttest{
# Would fail
cff_validate(test)
}

# Modify with cff_create
new <- cff_create(test, keys = list(
  "cff_version" = "1.2.0",
  message = "A blank file"
))
new

# Would pass
cff_validate(new)

}
\seealso{
Other core functions of \CRANpkg{cffr}:
\code{\link[=cff_create]{cff_create()}},
\code{\link[=cff_modify]{cff_modify()}},
\code{\link[=cff_validate]{cff_validate()}}
}
\concept{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_cff.R
\encoding{UTF-8}
\name{as_cff}
\alias{as_cff}
\alias{as_cff.default}
\alias{as_cff.list}
\alias{as_cff.person}
\alias{as_cff.bibentry}
\alias{as_cff.Bibtex}
\alias{as.cff}
\title{Coerce lists, \code{person} and \code{bibentry} objects to \code{\link{cff}}}
\usage{
as_cff(x, ...)

\method{as_cff}{default}(x, ...)

\method{as_cff}{list}(x, ...)

\method{as_cff}{person}(x, ...)

\method{as_cff}{bibentry}(x, ...)

\method{as_cff}{Bibtex}(x, ...)
}
\arguments{
\item{x}{A \code{person}, \code{bibentry} or other object that could be coerced to a
list.}

\item{...}{Additional arguments to be passed on to other methods.}
}
\value{
\itemize{
\item \code{as_cff.person()} returns an object with classes
\code{\link[=cff_pers_lst]{cff_pers_lst, cff}}.
\item \code{as_cff.bibentry()} and \code{as_cff.Bibtex()} return an object with classes
\code{\link[=cff_ref_lst]{cff_ref_lst, cff}}.
\item The remaining methods return an object of class \code{cff}. However, if
\code{x} has a structure compatible with \code{definitions.person},
\code{definitions.entity} or \code{definitions.reference}, the object has the
corresponding subclass.
}

Learn more about the \CRANpkg{cffr} class system in \link{cff_class}.
}
\description{
\code{as_cff()} turns an existing list-like \strong{R} object into a so-called
\code{\link{cff}}, a list with class \code{cff}, with the corresponding
\link[=cff_class]{sub-class} if applicable.

\code{as_cff} is an S3 generic, with methods for:
\itemize{
\item \code{person} objects as produced by \code{\link[utils:person]{utils::person()}}.
\item \code{bibentry} objects as produced by \code{\link[utils:bibentry]{utils::bibentry()}}.
\item \code{Bibtex} objects as produced by \code{\link[=toBibtex]{toBibtex()}}.
\item Default: Other inputs are first coerced with \code{\link[=as.list]{as.list()}}.
}
}
\details{
For \code{as_cff.bibentry()} and \code{as_cff.Bibtex()}, see
\code{vignette("bibtex_cff", "cffr")} to understand how the mapping is performed.

\code{\link[=as_cff_person]{as_cff_person()}} is preferred over \code{as_cff.person()}, since it can handle
\code{character} inputs such as \code{"Davis, Jr., Sammy"}. For \code{person} objects both
functions are similar.
}
\examples{

# Convert a list to "cff" object
cffobj <- as_cff(list(
  "cff-version" = "1.2.0",
  title = "Manipulating files"
))

class(cffobj)

# Nice display thanks to yaml package
cffobj

# bibentry method
a_cit <- citation("cffr")[[1]]

a_cit

as_cff(a_cit)

# Bibtex method
a_bib <- toBibtex(a_cit)

a_bib

as_cff(a_cit)

}
\seealso{
\itemize{
\item \code{\link[=cff]{cff()}}: Create a full \code{cff} object from scratch.
\item \code{\link[=cff_modify]{cff_modify()}}: Modify a \code{cff} object.
\item \code{\link[=cff_create]{cff_create()}}: Create a \code{cff} object for an \strong{R} package.
\item \code{\link[=cff_read]{cff_read()}}: Create a \code{cff} object from an external file.
\item \code{\link[=as_cff_person]{as_cff_person()}}: Recommended way to create persons in CFF format.
}

Learn more about the \CRANpkg{cffr} class system in \link{cff_class}.

Coercing between \strong{R} classes with \strong{S3 Methods}:
\code{\link[=as_bibentry]{as_bibentry()}},
\code{\link[=as_cff_person]{as_cff_person()}},
\code{\link{cff_class}}
}
\concept{s3method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\encoding{UTF-8}
\name{cff_parse_person}
\alias{cff_parse_person}
\alias{cff_parse_person_bibtex}
\title{Previous API: Parse a \code{person} to \code{\link{cff}}}
\usage{
cff_parse_person(person)

cff_parse_person_bibtex(person)
}
\arguments{
\item{person}{It can be either:
\itemize{
\item A \code{person} or list of \code{person} object created with \code{\link[utils:person]{utils::person()}}.
\item A \code{character} object or vector representing a person or persons.
}}
}
\value{
A person in format \code{cff}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Please use \code{\link[=as_cff_person]{as_cff_person()}}
}
\examples{
# Create a person object
a_person <- person(
  given = "First", family = "Author",
  role = c("aut", "cre"),
  email = "first.last@example.com", comment = c(
    ORCID = "0000-0001-8457-4658",
    affiliation = "An affiliation"
  )
)

a_person

cff_person <- as_cff_person(a_person)

cff_person

# Back to person object with S3 Method
as.person(cff_person)

# Parse a string
a_str <- paste0(
  "Julio Iglesias <fake@email.com> ",
  "(<https://orcid.org/0000-0001-8457-4658>)"
)
as_cff_person(a_str)

# Several persons
persons <- c(person("Clark", "Kent"), person("Lois", "Lane"))

as_cff_person(persons)

# Or you can use BibTeX style if you prefer

x <- "Frank Sinatra and Dean Martin and Davis, Jr., Sammy and Joey Bishop"

as_cff_person(x)

as_cff_person("Herbert von Karajan")
}
\seealso{
\code{\link[=as_cff_person]{as_cff_person()}}

Other deprecated functions:
\code{\link[=cff_extract_to_bibtex]{cff_extract_to_bibtex()}},
\code{\link[=cff_from_bibtex]{cff_from_bibtex()}},
\code{\link[=cff_parse_citation]{cff_parse_citation()}},
\code{\link[=write_bib]{write_bib()}}
}
\concept{deprecated}
\keyword{internal}
